@@ -136,27 +136,50 @@ def toggleControlAction(*args, **kwargs):
     return ActionFactory(ToggleControlPanelAction, *args, **kwargs)
 
 
-class ActionProvider(object):
+class ActionProvider:
     """The ``ActionProvider`` class is intended to be used as a base class for
     classes which contain actions. The :func:`action` and :func:`toggleAction`
     functions can be used as decorators on class methods, to denote them as
     actions.
     """
 
+
+    def __init__(self, overlayList, displayCtx):
+        """Create an ``ActionProvider``.
+
+        :arg overlayList: The :class:`.OverlayList`.
+        :arg displayCtx:  The :class:`.DisplayContext`.
+        """
+        self.__overlayList = overlayList
+        self.__displayCtx  = displayCtx
+
+
+    @property
+    def overlayList(self):
+        """Return a reference to the :class:`.OverlayList`. """
+        return self.__overlayList
+
+
+    @property
+    def displayCtx(self):
+        """Return a reference to the :class:`.DisplayContext`. """
+        return self.__displayCtx
+
+
     def destroy(self):
         """Must be called when this ``ActionProvider`` is no longer needed.
         Calls the :meth:`Action.destroy` method of all ``Action`` instances.
         """
+
+        self.__overlayList = None
+        self.__displayCtx  = None
+
         for name, actionz in ActionProvider.getActions(self):
 
             # Entries in getActions may be (None, None)
             if actionz is None:
                 continue
 
-            # Or may be ('groupName', [('actionName', action), ...])
-            elif isinstance(actionz, list):
-                actionz = [a[1] for a in actionz]
-
             # Or may be ('actionName', action)
             else:
                 actionz = [actionz]
@@ -194,16 +217,10 @@ class ActionProvider(object):
         Sub-classes may wish to override this method to enforce a specific
         ordering of their actions.
 
-        .. note:: The list returned by this method may contain:
-
-                    - Entries equal to ``(None, None)``. This is used as a
-                      hint for GUIs which display action widgets/menu items to
+        .. note:: The list returned by this method may contain entries equal
+                      to ``(None, None)``. This is used as a hint for the
+                      :class:`.FSLeyesFrame`, which creates menu items, to
                       indicate that a separator should be inserted.
-
-                    - Entries equal to ``(name, [(name, Action), ...])``.
-                      This is used as a hint for GUIs that the nested list
-                      of actions should appear as a sub-menu with the
-                      specified ``name``.
         """
 
         acts = []
@@ -217,10 +234,11 @@ class ActionProvider(object):
         return acts
 
 
-class ActionFactory(object):
-    """The ``ActionFactory`` is used by the :func:`action` and
-    :func:`toggleAction` decorators. Its job is to create :class:`Action`
-    instances for :class:`ActionProvider` instances.
+class ActionFactory:
+    """The ``ActionFactory`` is used by the :func:`action`,
+    :func:`toggleAction`, and :func:`toggleControlPanelAction` decorators.
+    Its job is to create :class:`Action` instances for :class:`ActionProvider`
+    instances.
 
 
     .. warning:: This class contains difficult-to-understand code. Read up
@@ -336,9 +354,9 @@ class ActionFactory(object):
         """
 
         if self.__func is not None:
-            log.warn('ActionFactory.__call__ was called, but function is '
-                     'alreday set ({})! I\'m really confused.'.format(
-                         self.__func.__name__))
+            log.warning('ActionFactory.__call__ was called, but function is '
+                        'alreday set (%s)! I\'m really confused.',
+                        self.__func.__name__)
 
         self.__func = func
         return self
@@ -359,11 +377,21 @@ class ActionFactory(object):
 
         else:
 
+            # first argument will be the
+            # instance (the "self" argument)
+            func = functools.partial(self.__func, instance)
+            func = functools.update_wrapper(func, self.__func)
+
+            args = [instance.overlayList, instance.displayCtx]
+            if self.__actionType is ToggleControlPanelAction:
+                args.append(instance)
+
             # Create an Action for the instance
             action = self.__actionType(
-                self.__func,
-                instance,
+                *args,
                 *self.__args,
+                func=func,
+                instance=instance,
                 **self.__kwargs)
 
             # and replace this ActionFactory
