@@ -10,10 +10,14 @@ monkey-patched into the :class:`.FSLeyesFrame` class.
 """
 
 
+import            os
 import os.path as op
 
+import            wx
+
 import                      fsleyes
 import fsleyes.actions   as actions
+import fsleyes.strings   as strings
 from   fsleyes.frame import FSLeyesFrame
 
 
@@ -21,7 +25,6 @@ from   fsleyes.frame import FSLeyesFrame
 def addViewPanel(self, vpType, **kwargs):
     """Function shared by the add*Panel functions below. """
     vp = self.addViewPanel(vpType, **kwargs)
-    self.viewPanelDefaultLayout(vp)
     vp.SetFocus()
     return vp
 
@@ -127,36 +130,79 @@ def toggleOverlayVisibility(self, *args, **kwargs):
     display.enabled = not display.enabled
 
 
+def toggleOverlaysVisibility(self, *args, **kwargs):
+    """Toggles visibility of all images except the first image on the list. """
+
+    overlayList = self.overlayList
+
+    if len(overlayList) in (0, 1):
+        return
+
+    viewPanel = self.focusedViewPanel
+
+    if viewPanel is None: displayCtx = self     .displayCtx
+    else:                 displayCtx = viewPanel.displayCtx
+
+    for i in range(1,len(overlayList)):
+        o = overlayList[displayCtx.overlayOrder[i]]
+        display = displayCtx.getDisplay(o)
+        display.enabled = not display.enabled
+
+
 def openHelp(self, *args, **kwargs):
     """Opens FSLeyes help in a web browser. """
 
-    url = op.join(fsleyes.assetDir, 'userdoc', 'index.html')
-
     import fsleyes_widgets.utils.webpage as webpage
+    url = 'https://open.win.ox.ac.uk/pages/fsl/fsleyes/fsleyes/userdoc/'
+    webpage.openPage(url)
 
-    # Show locally stored help files
-    if op.exists(url):
-        webpage.openFile(url)
-    else:
-        url = 'https://users.fmrib.ox.ac.uk/~paulmc/fsleyes/userdoc/'
-        webpage.openPage(url)
+
+def setFSLDIR(self, *args, **kwargs):
+    """Opens a directory dialog allowing the user to select a value for
+    ``$FSLDIR``.
+    """
+
+    import wx
+    from   fsl.utils.platform import platform
+    import fsl.utils.settings as     settings
+
+    fsldir = platform.fsldir
+
+    if fsldir is None:
+        fsldir = os.getcwd()
+
+    msg = strings.titles[self, 'setFSLDIR']
+
+    dlg = wx.DirDialog(self,
+                       message=msg,
+                       defaultPath=fsldir,
+                       style=wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST)
+
+    if dlg.ShowModal() != wx.ID_OK:
+        return
+
+    fsldir                 = dlg.GetPath()
+    platform.fsldir        = fsldir
+    settings.write('fsldir', fsldir)
 
 
 def closeFSLeyes(self, *args, **kwargs):
     """Closes FSLeyes. """
-    self.Close()
+    wx.CallAfter(self.Close)
 
 
-FSLeyesFrame.addOrthoPanel           = actions.action(addOrthoPanel)
-FSLeyesFrame.addLightBoxPanel        = actions.action(addLightBoxPanel)
-FSLeyesFrame.addScene3DPanel         = actions.action(addScene3DPanel)
-FSLeyesFrame.addTimeSeriesPanel      = actions.action(addTimeSeriesPanel)
-FSLeyesFrame.addHistogramPanel       = actions.action(addHistogramPanel)
-FSLeyesFrame.addPowerSpectrumPanel   = actions.action(addPowerSpectrumPanel)
-FSLeyesFrame.addShellPanel           = actions.action(addShellPanel)
-FSLeyesFrame.removeFocusedViewPanel  = actions.action(removeFocusedViewPanel)
-FSLeyesFrame.selectNextOverlay       = actions.action(selectNextOverlay)
-FSLeyesFrame.selectPreviousOverlay   = actions.action(selectPreviousOverlay)
-FSLeyesFrame.toggleOverlayVisibility = actions.action(toggleOverlayVisibility)
-FSLeyesFrame.openHelp                = actions.action(openHelp)
-FSLeyesFrame.closeFSLeyes            = actions.action(closeFSLeyes)
+FSLeyesFrame.addOrthoPanel            = actions.action(addOrthoPanel)
+FSLeyesFrame.addLightBoxPanel         = actions.action(addLightBoxPanel)
+FSLeyesFrame.addScene3DPanel          = actions.action(addScene3DPanel)
+FSLeyesFrame.addTimeSeriesPanel       = actions.action(addTimeSeriesPanel)
+FSLeyesFrame.addHistogramPanel        = actions.action(addHistogramPanel)
+FSLeyesFrame.addPowerSpectrumPanel    = actions.action(addPowerSpectrumPanel)
+FSLeyesFrame.addShellPanel            = actions.action(addShellPanel)
+FSLeyesFrame.removeFocusedViewPanel   = actions.action(removeFocusedViewPanel)
+FSLeyesFrame.selectNextOverlay        = actions.action(selectNextOverlay)
+FSLeyesFrame.selectPreviousOverlay    = actions.action(selectPreviousOverlay)
+FSLeyesFrame.toggleOverlayVisibility  = actions.action(toggleOverlayVisibility)
+FSLeyesFrame.toggleOverlaysVisibility = actions.action(toggleOverlaysVisibility)
+FSLeyesFrame.openHelp                 = actions.action(openHelp)
+FSLeyesFrame.setFSLDIR                = actions.action(setFSLDIR)
+FSLeyesFrame.closeFSLeyes             = actions.action(closeFSLeyes)
