@@ -38,7 +38,7 @@ import fsl.utils.settings           as fslsettings
 import fsleyes_widgets.utils.status as status
 import fsleyes.autodisplay          as autodisplay
 import fsleyes.strings              as strings
-import fsleyes.overlay              as fsloverlay
+import fsleyes.data                 as dutils
 from . import                          base
 
 
@@ -57,10 +57,7 @@ class LoadOverlayAction(base.Action):
         :arg displayCtx:  The :class:`.DisplayContext`.
         :arg frame:       The :class:`.FSLeyesFrame`.
         """
-        base.Action.__init__(self, self.__loadOverlay)
-
-        self.__overlayList = overlayList
-        self.__displayCtx  = displayCtx
+        base.Action.__init__(self, overlayList, displayCtx, self.__loadOverlay)
 
 
     def __loadOverlay(self):
@@ -74,24 +71,23 @@ class LoadOverlayAction(base.Action):
         of each new overlay.
         """
 
-        def onLoad(overlays):
+        def onLoad(paths, overlays):
 
             if len(overlays) == 0:
                 return
 
-            self.__overlayList.extend(overlays)
+            self.overlayList.extend(overlays)
 
-            self.__displayCtx.selectedOverlay = \
-                self.__displayCtx.overlayOrder[-1]
+            self.displayCtx.selectedOverlay = self.displayCtx.overlayOrder[-1]
 
-            if self.__displayCtx.autoDisplay:
+            if self.displayCtx.autoDisplay:
                 for overlay in overlays:
                     autodisplay.autoDisplay(overlay,
-                                            self.__overlayList,
-                                            self.__displayCtx)
+                                            self.overlayList,
+                                            self.displayCtx)
 
         interactiveLoadOverlays(onLoad=onLoad,
-                                inmem=self.__displayCtx.loadInMemory)
+                                inmem=self.displayCtx.loadInMemory)
 
 
 def makeWildcard(allowedExts=None, descs=None):
@@ -99,31 +95,37 @@ def makeWildcard(allowedExts=None, descs=None):
     the the displayed file types to supported overlay file types.
     """
 
-    import fsl.data.image      as fslimage
-    import fsl.data.mghimage   as fslmgh
-    import fsl.data.vtk        as fslvtk
-    import fsl.data.gifti      as fslgifti
-    import fsl.data.freesurfer as fslfs
+    import fsl.data.image          as fslimage
+    import fsl.data.mghimage       as fslmgh
+    import fsl.data.vtk            as fslvtk
+    import fsl.data.gifti          as fslgifti
+    import fsl.data.freesurfer     as fslfs
+    import fsl.data.bitmap         as fslbmp
+    import fsleyes.data.tractogram as tractogram
 
     # Hack - the wx wildcard logic doesn't support
     # files with multiple extensions (e.g. .nii.gz).
-    # So I'm adding support for '.gz' and '.gii'
-    # extensions here.
+    # So I'm adding support for the '.gz' extension
+    # here.
     fsfiles = [op.splitext(f)[1] for f in fslfs.CORE_GEOMETRY_FILES]
     if allowedExts is None:
-        allowedExts  = (fslimage.ALLOWED_EXTENSIONS  +
-                        fslvtk  .ALLOWED_EXTENSIONS  +
-                        fslmgh  .ALLOWED_EXTENSIONS  +
-                        fslgifti.ALLOWED_EXTENSIONS  +
-                        fsfiles                      +
-                        ['.gz', '.gii'])
+        allowedExts  = (fslimage   .ALLOWED_EXTENSIONS +
+                        fslvtk     .ALLOWED_EXTENSIONS +
+                        fslmgh     .ALLOWED_EXTENSIONS +
+                        fslgifti   .ALLOWED_EXTENSIONS +
+                        tractogram .ALLOWED_EXTENSIONS +
+                        fslbmp     .BITMAP_EXTENSIONS  +
+                        fsfiles                        +
+                        ['.gz'])
     if descs is None:
-        descs        = (fslimage.EXTENSION_DESCRIPTIONS     +
-                        fslvtk  .EXTENSION_DESCRIPTIONS     +
-                        fslmgh  .EXTENSION_DESCRIPTIONS     +
-                        fslgifti.EXTENSION_DESCRIPTIONS     +
-                        fslfs   .CORE_GEOMETRY_DESCRIPTIONS +
-                        ['Compressed images', 'GIFTI surfaces'])
+        descs        = (fslimage   .EXTENSION_DESCRIPTIONS     +
+                        fslvtk     .EXTENSION_DESCRIPTIONS     +
+                        fslmgh     .EXTENSION_DESCRIPTIONS     +
+                        fslgifti   .EXTENSION_DESCRIPTIONS     +
+                        tractogram .EXTENSION_DESCRIPTIONS     +
+                        fslbmp     .BITMAP_DESCRIPTIONS        +
+                        fslfs      .CORE_GEOMETRY_DESCRIPTIONS +
+                        ['Compressed images'])
 
     exts  = ['*{}'.format(ext) for ext in allowedExts]
     exts  = [';'.join(exts)]        + exts
@@ -167,8 +169,13 @@ def loadOverlays(paths,
                     later on as the default load directory.
 
     :arg onLoad:    Optional function to call when all overlays have been
-                    loaded. Must accept one parameter - a list containing
-                    the overlays that were loaded.
+                    loaded. Must accept two parameters:
+
+                     - a list of indices, one for each overlay, into the
+                       ``paths`` parameter, indicating, for each overlay, the
+                       path from which it was loaded.
+
+                     - a list of the overlays that were loaded
 
     :arg inmem:     If ``True``, all :class:`.Image` overlays are
                     force-loaded into memory. Otherwise, large compressed
@@ -177,15 +184,16 @@ def loadOverlays(paths,
     :arg blocking:  Defaults to ``False``. If ``True``, overlays are loaded
                     immediately (and the ``onLoad`` function is called
                     directly. Otherwise, overlays and the ``onLoad`` are loaded
-                    loaded/called on the :func:`.idle.idle` loop.
+                    and called on the :func:`.idle.idle` loop.
 
     :returns:       If ``blocking is False`` (the default), returns ``None``.
                     Otherwise returns a list containing the loaded overlay
                     objects.
     """
 
-    import fsl.data.image as fslimage
-    import fsl.data.mesh  as fslmesh
+    import fsl.data.image  as fslimage
+    import fsl.data.mesh   as fslmesh
+    import fsl.data.bitmap as fslbmp
 
     # The default load function updates
     # the dialog window created above
@@ -202,11 +210,11 @@ def loadOverlays(paths,
             e)
 
     # A function which loads a single overlay
-    def loadPath(path):
+    def loadPath(path, idx):
 
         loadFunc(path)
 
-        dtype, path = fsloverlay.guessDataSourceType(path)
+        dtype, _, path = dutils.guessType(path)
 
         if dtype is None:
             errorFunc(path, strings.messages['loadOverlays.unknownType'])
@@ -217,13 +225,16 @@ def loadOverlays(paths,
 
         try:
             if   issubclass(dtype, fslimage.Image):
-                overlay = loadImage(dtype, path, inmem=inmem)
+                loaded = loadImage(dtype, path, inmem=inmem)
             elif issubclass(dtype, fslmesh.Mesh):
-                overlay = dtype(path, fixWinding=True)
+                loaded = [dtype(path, fixWinding=True)]
+            elif issubclass(dtype, fslbmp.Bitmap):
+                loaded = [dtype(path).asImage()]
             else:
-                overlay = dtype(path)
+                loaded = [dtype(path)]
 
-            overlays.append(overlay)
+            overlays.extend(loaded)
+            pathIdxs.extend([idx] * len(loaded))
 
         except Exception as e:
             errorFunc(path, e)
@@ -237,10 +248,11 @@ def loadOverlays(paths,
     def realOnLoad(*a):
 
         if saveDir and len(paths) > 0:
-            fslsettings.write('loadSaveOverlayDir', op.dirname(paths[-1]))
+            ovlDir = op.abspath(op.dirname(paths[-1]))
+            fslsettings.write('loadSaveOverlayDir', ovlDir)
 
         if onLoad is not None:
-            onLoad(overlays)
+            onLoad(pathIdxs, overlays)
 
     # If loadFunc or errorFunc are explicitly set to
     # None, use these no-op load/error functions
@@ -252,12 +264,13 @@ def loadOverlays(paths,
     if loadFunc  == 'default': loadFunc  = defaultLoadFunc
     if errorFunc == 'default': errorFunc = defaultErrorFunc
 
+    pathIdxs = []
     overlays = []
     funcs    = []
 
     # Load the images
-    for path in paths:
-        funcs.append(lambda p=path: loadPath(p))
+    for idx, path in enumerate(paths):
+        funcs.append(lambda p=path, i=idx: loadPath(p, i))
     funcs.append(realOnLoad)
 
     for func in funcs:
@@ -275,17 +288,21 @@ def loadImage(dtype, path, inmem=False):
     kept on disk, and the initial image data range may be calculated
     from the whole image, or from a sample.
 
+    This function returns a sequence, most likely containing a single
+    :class:`.Image` instance. But in some circumstances, more than one
+    :class:`.Image` will be created and returned.
+
     :arg dtype: Overlay type (``Image``, or a sub-class of ``Image``).
     :arg path:  Path to the overlay file.
     :arg inmem: If ``True``, ``Image`` overlays are loaded into memory.
+    :returns:   A sequence of :class:`.Image` instances that were loaded.
     """
 
-    rangethres = fslsettings.read('fsleyes.overlay.rangethres', 419430400)
-    idxthres   = fslsettings.read('fsleyes.overlay.idxthres',   1073741824)
+    import fsl.data.image            as fslimage
+    import fsleyes.data.imagewrapper as imagewrapper
 
-    # We're going to load the file
-    # twice - first to get its
-    # dimensions, and then for real.
+    # We're going to load the file twice - first to
+    # get its dimensions/data type, and then for real.
     #
     # TODO It is annoying that you have to create a 'dtype'
     #      instance twice, as e.g. the MelodicImage does a
@@ -295,40 +312,39 @@ def loadImage(dtype, path, inmem=False):
     #      (e.g. ./filtered_func.ica/ turned into
     #      ./filtered_func.ica/melodic_IC) so that you can
     #      just create a fsl.data.Image, or a nib.Nifti1Image.
-    image = dtype(path,
-                  loadData=False,
-                  calcRange=False,
-                  indexed=False,
-                  threaded=False)
-    nbytes = np.prod(image.shape) * image.dtype.itemsize
+    image  = dtype(path)
+    shape  = image.shape
+    nbytes = np.prod(shape) * image.niftiDataTypeSize / 8
     image  = None
 
-    # If the file is compressed (gzipped),
-    # index the file if its compressed size
-    # is greater than the index threshold.
-    indexed = nbytes > idxthres
-    image   = dtype(path,
-                    loadData=inmem,
-                    calcRange=False,
-                    indexed=indexed,
-                    threaded=indexed)
+    # If the file is a large 4D image, we will use an
+    # ImageWrapper to manage acccess to the data. The
+    # ImageWrapper will incrementally update the
+    # known data range of an image as more data is
+    # read from disk.
+    useWrapper = fslsettings.read('fsleyes.overlay.usewrapper', 536870912)
+    useWrapper = all((not inmem,
+                      len(shape) > 3,
+                      nbytes >= useWrapper))
+    if useWrapper: wrapper = imagewrapper.ImageWrapper(threaded=True)
+    else:          wrapper = None
 
-    # If the image is bigger than the
-    # index threshold, keep it on disk.
-    if inmem or (not indexed):
-        log.debug('Loading {} into memory'.format(path))
-        image.loadData()
-    else:
-        log.debug('Keeping {} on disk'.format(path))
+    image = dtype(path, loadMeta=True, dataMgr=wrapper)
 
-    # If the image size is less than the range
-    # threshold, calculate the full data range
-    # now. Otherwise calculate the data range
-    # from a sample. This is handled by the
-    # Image.calcRange method.
-    image.calcRange(rangethres)
+    # Force-load the full image data array
+    # into memory. See the Image.data method.
+    if inmem:
+        image.data
 
-    return image
+    # If using an image wrapper, read a
+    # sample of data to force the wrapper
+    # to initialise its known data range
+    if wrapper is not None:
+        wrapper.setImage(image.nibImage)
+        with wrapper.unthreaded():
+            wrapper[..., 0]
+
+    return [image]
 
 
 def interactiveLoadOverlays(fromDir=None, dirdlg=False, **kwargs):
@@ -380,7 +396,7 @@ def interactiveLoadOverlays(fromDir=None, dirdlg=False, **kwargs):
                             style=wx.FD_OPEN | wx.FD_MULTIPLE)
 
     if dlg.ShowModal() != wx.ID_OK:
-        return []
+        return
 
     if dirdlg: paths = [dlg.GetPath()]
     else:      paths =  dlg.GetPaths()
