@@ -32,6 +32,8 @@ import                        textwrap
 import                        functools
 import                        collections
 
+import nibabel             as nib
+
 import fsl.utils.settings  as fslsettings
 import fsleyes.strings     as strings
 import fsleyes.version     as version
@@ -55,11 +57,8 @@ class RunScriptAction(base.Action):
         :arg displayCtx:  The top-level :class:`.DisplayContext`.
         :arg overlayList: The :class:`.FSLeyesFrame`.
         """
-        base.Action.__init__(self, self.__doAction)
-
-        self.__frame       = frame
-        self.__overlayList = overlayList
-        self.__displayCtx  = displayCtx
+        base.Action.__init__(self, overlayList, displayCtx, self.__doAction)
+        self.__frame = frame
 
 
     def __doAction(self, script=None):
@@ -100,8 +99,8 @@ class RunScriptAction(base.Action):
         # error if it crashes
         try:
             runScript(self.__frame,
-                      self.__overlayList,
-                      self.__displayCtx,
+                      self.overlayList,
+                      self.displayCtx,
                       script)
 
         except Exception as e:
@@ -199,6 +198,7 @@ def fsleyesScriptEnvironment(frame, overlayList, displayCtx):
     import fsl.data.gifti                       as fslgifti
     import fsl.data.freesurfer                  as fslfs
     import fsl.wrappers                         as wrappers
+    import fsl.wrappers.wrapperutils            as wutils
     import fsl.utils.fslsub                     as fslsub
 
     def load(filename):
@@ -207,7 +207,7 @@ def fsleyesScriptEnvironment(frame, overlayList, displayCtx):
         from . import                 loadoverlay
         import fsleyes.autodisplay as autodisplay
 
-        def onLoad(overlays):
+        def onLoad(paths, overlays):
 
             if len(overlays) == 0:
                 return
@@ -227,17 +227,19 @@ def fsleyesScriptEnvironment(frame, overlayList, displayCtx):
     def scaledVoxels():
         """Display all NIFTI images in true scaled voxels (but
         with a radiological/neurological flip). """
-        for o in overlayList:
-            if isinstance(o, fslimage.Nifti):
-                displayCtx.getOpts(o).transform = 'pixdim-flip'
+        print('This function will be removed in a future version of '
+              'FSLeyes. You can achieve the same result by setting '
+              'displayCtx.displaySpace = "fslview".')
+        displayCtx.displaySpace = 'fslview'
 
 
     def trueScaledVoxels():
         """Display all NIFTI images in true scaled voxels (without
         any radiological/neurological flip). """
-        for o in overlayList:
-            if isinstance(o, fslimage.Nifti):
-                displayCtx.getOpts(o).transform = 'pixdim'
+        print('This function will be removed in a future version of '
+              'FSLeyes. You can achieve the same result by setting '
+              'displayCtx.displaySpace = "scaledVoxel".')
+        displayCtx.displaySpace = 'scaledVoxel'
 
     def rawVoxels():
         """Display all NIFTI images in raw voxels. """
@@ -332,51 +334,49 @@ def fsleyesScriptEnvironment(frame, overlayList, displayCtx):
         ('output',             fslsub.output),
     ))
 
-
-    # We are assuming that all callable
-    # things in the wrappers module
-    # are decorated with one of the
-    # @fileOrImage or @fileOrArray
-    # decorators, found in wrapperutils.
+    # Most fsl.wrappers function use the
+    # fsl.wrappers.wrapperutils.fileOrImage
+    # decorator, which allows outputs to be
+    # auto-loaded. We apply another decorator
+    # to have those loaded results auto-added
+    # to the overlayList.
     def loadOutputDecorator(func):
         def wrapper(*args, **kwargs):
 
-            # All wrapper functions return a dict
-            # with an attribute called "output".
             result = func(*args, **kwargs)
 
-            # Submitted as a cluster job?
-            # The output contains the job ID.
-            if 'submit' in kwargs:
-                return result.output
+            # The decorated functions we're after
+            # return a value of type FoT.Results.
+            # Anything else is passed through.
+            if not isinstance(result, wutils.FileOrThing.Results):
+                return result
 
-            # Called directly? The output
-            # contains stdout/stderr.
-            stdout, stderr = result.output
-
-            print(stdout)
-            print(stderr, file=sys.stderr)
-
-            # Any image arguments which were
-            # specified as LOAD are loaded
-            # into FSLeyes.
+            # Any loaded images are
+            # loaded into FSLeyes.
             for name, val in result.items():
+                if isinstance(val, nib.Nifti1Image):
+                    val = fslimage.Image(val)
                 if isinstance(val, fslimage.Image):
                     overlayList.append(val)
+                    displayCtx.getDisplay(val).name = name
 
-            # We return nothing, as images
-            # will either have been loaded
-            # or saved to disk, and stdout
-            # got printed above.
+            return result
 
         return functools.update_wrapper(wrapper, func)
 
+
+    # a few exceptions that we do not
+    # want the loadOutputDecorator
+    # to be applied to
+    exclude = ('LOAD', 'fslstats', 'slicer', 'fslmaths')
+
     for att in dir(wrappers):
+
         val = getattr(wrappers, att)
         if att[0] == '_' or isinstance(val, types.ModuleType):
             continue
 
-        if callable(val):
+        if (att not in exclude) and callable(val):
             val = loadOutputDecorator(val)
         _locals[att] = val
 
@@ -402,14 +402,19 @@ def fsleyesShellHelpText(_globals, _locals):
         'LOAD' : 'Load the output from a FSL wrapper function',
     }
 
-    localVars  = list(_locals.keys())
-    localDescs = [_locals[k].__doc__
-                  if k not in overrideDocs
-                  else overrideDocs[k]
-                  for k in localVars]
+    localVars  = []
+    localDescs = []
 
-    descWidth   = 60
-    varWidth    = max([len(v) for v in localVars])
+    for name, obj in _locals.items():
+        doc = overrideDocs.get(name, obj.__doc__)
+        if doc is None:
+            doc = 'n/a'
+
+        localVars .append(name)
+        localDescs.append(doc)
+
+    descWidth  = 60
+    varWidth   = max([len(v) for v in localVars])
 
     localDescs = [d[:descWidth + 30]   for d in localDescs]
     localDescs = [d.replace('\n', ' ') for d in localDescs]
