@@ -11,17 +11,12 @@ if a new version of FSLeyes is available.
 
 import logging
 
-from six.moves.urllib import request
+import urllib.request as request
+import ssl
+import json
 
 import wx
-
-# The HyperLinkCtrl is in wx.adv in wxPython/Phoenix
-try:
-    import wx.adv as wxadv
-
-# But it is in wx in wxpython 3.
-except ImportError:
-    wxadv = wx
+import wx.adv as wxadv
 
 import fsl.version                  as fslversion
 
@@ -34,15 +29,14 @@ from . import                          base
 log = logging.getLogger(__name__)
 
 
-_FSLEYES_URL = 'https://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FSLeyes'
+_FSLEYES_URL = 'https://fsl.fmrib.ox.ac.uk/fsl/docs/#/utilities/fsleyes'
 """A url to direct the user towards to download the latest version of FSLeyes.
 """
 
 
-_FSLEYES_VERSION_URL = 'https://fsl.fmrib.ox.ac.uk/'\
-                       'fsldownloads/fsleyes/version.txt'
-"""A url which points to a text file that contains the most recently released
-FSLeyes version number.
+_FSLEYES_VERSION_URL = 'https://api.anaconda.org/package/conda-forge/fsleyes'
+"""A url which points to a JSON file that contains information about the
+FSLeyes package on conda-forge.
 """
 
 
@@ -53,9 +47,10 @@ class UpdateCheckAction(base.Action):
     """
 
 
-    def __init__(self):
+    def __init__(self, overlayList, displayCtx):
         """Create an ``UpdateCheckAction``. """
-        base.Action.__init__(self, self.__checkForUpdates)
+        base.Action.__init__(
+            self, overlayList, displayCtx, self.__checkForUpdates)
 
 
     def __checkForUpdates(self,
@@ -82,13 +77,22 @@ class UpdateCheckAction(base.Action):
         errMsg   = strings.messages[self, 'newVersionError']
         errTitle = strings.titles[  self, 'newVersionError']
 
-        with status.reportIfError(errTitle, errMsg, report=showErrorMessage):
+        with status.reportIfError(errTitle,
+                                  errMsg,
+                                  raiseError=False,
+                                  report=showErrorMessage):
 
             log.debug('Checking for FSLeyes updates ({})'.format(
                 _FSLEYES_VERSION_URL))
 
-            f        = request.urlopen(_FSLEYES_VERSION_URL)
-            latest   = f.read().decode('utf-8').strip()
+            ctx                = ssl.create_default_context()
+            ctx.check_hostname = False
+            ctx.verify_mode    = ssl.CERT_NONE
+
+            with request.urlopen(_FSLEYES_VERSION_URL, context=ctx) as f:
+                info   = json.loads(f.read().decode('utf-8'))
+                latest = info['latest_version']
+
             current  = version.__version__
             upToDate = fslversion.compareVersions(latest,
                                                   current,
@@ -152,6 +156,8 @@ class UrlDialog(wx.Dialog):
         ok  = wx.Button(    self, label='Ok', id=wx.ID_OK)
         msg = wx.StaticText(self, label=msg)
 
+        self.__ok = ok
+
         if urlMsg is not None:
             urlMsg = wx.StaticText(self, label=urlMsg)
         if url is not None:
@@ -183,3 +189,9 @@ class UrlDialog(wx.Dialog):
         self.SetSizer(sizer)
         self.Layout()
         self.Fit()
+
+
+    @property
+    def ok(self):
+        """Return a reference to the OK button. """
+        return self.__ok
