@@ -19,14 +19,20 @@ FSLeyes colour maps and lookup tables are stored in the following locations:
    - ``[assetsbase]/luts/``
    - ``[settingsbase]/colourmaps/``
    - ``[settingsbase]/luts/``
+   - ``[sitebase]/colourmaps/``
+   - ``[sitebase]/luts/``
 
+where:
 
-where ``[assetsbase]`` is the location of the FSLeyes assets directory (see
-the :attr:`fsleyes.assetDir` attribute), and ``[settingsbase]`` is the base
-directory use by the :mod:`fsl.utils.settings` module for storing FSLeyes
-application settings and data. "Built-in" colour maps and lookup tables are
-stored underneath ``[assetsbase]``, and user-added ones are stored under
-``[settingsbase]``.
+ - ``[assetsbase]`` is the location of the FSLeyes assets directory, containing
+   built-in colourmaps and lookup tables (see the :attr:`fsleyes.assetDir`
+   attribute)
+ - ``[settingsbase]`` is the base directory use by the
+   :mod:`fsl.utils.settings` module for storing FSLeyes application settings
+   and data, and user-added colourmaps and lookup tables.
+ - ``[sitebase]`` is a site-specific configuration directory, specified by
+   the ``$FSLEYES_SITE_CONFIG_DIR`` environment variable, which may contain
+   site-specific colourmaps and lookup tables.
 
 
 When :func:`init` is called, it searches in the above locations, and attempts
@@ -83,17 +89,17 @@ The following functions are available for managing and accessing colour maps:
 Display name and ordering
 ^^^^^^^^^^^^^^^^^^^^^^^^^
 
+FSLeyes comes with a file at ``[assetsbase]/colourmaps/order.txt``, which
+contains a list of colour map names and identifiers, defining the order in
+which the colour maps should be displayed to the user. Any colour maps which
+are present in ``[assetsbase]/colourmaps/``, but are not listed in the
+``order.txt``, file will be appended to the end of the list, and their name
+will be derived from the file name.
 
-For built-in colour maps, a file named ``[assetsbase]/colourmaps/order.txt``
-is assumed to contain a list of colour map names, and colour map identifiers,
-defining the order in which the colour maps should be displayed to the
-user. Any colour maps which are present in ``[assetsbase]/colourmaps/``, but
-are not listed in the ``order.txt``, file will be appended to the end of the
-list, and their name will be derived from the file name.
-
-
-If a file ``[settingsbase]/colourmaps/order.txt`` exists, then it will be used
-in place of the ``order.txt`` file in ``[assetsbase]``.
+If a file called ``[settingsbase]/colourmaps/order.txt`` or
+``[sitebase]/colourmaps/order.txt`` exists in the user or site configuration
+directory, then it will be used in place of the built-in
+``[assetsbase]/colourmaps/order.txt`` file.
 
 
 User-added colour maps
@@ -105,14 +111,11 @@ persistent setting called ``fsleyes.colourmaps``, which is a dictionary of ``{
 cmapid : displayName }`` mappings. The ``cmapid`` is typically the display
 name, converted to lower case, with spaces replaced with underscores. A
 user-added colour map with id ``cmapid`` will be saved as
-``[settingsbase]/colourmaps/cmapid.cmap``.  All user-added colour maps wll be
-displayed after all built-in colour maps, and their order cannot be
-customised. Any user-added colour map files which are not present in the
-``fsleyes.colourmaps`` dictionary will be given a display name the same as
-the colour map ID (which is taken from the file name).
+``[settingsbase]/colourmaps/cmapid.cmap``.
 
 
-Installing a user-added colour map is a two-step process:
+Installing a user-added colour map is a two-step process - this is implemented
+in the :class:`.LoadColourMapAction` class:
 
  1. First, the colour map must be *registered*, via the
     :func:`registerColourMap` function. This adds the colour map to the
@@ -123,6 +126,12 @@ Installing a user-added colour map is a two-step process:
     permanently.
 
 
+Alternatively, colour map files can be copied directly into
+``[settingsbase]/colourmaps/``, and their name specified in
+``[settingsbase]/colourmaps/order.txt`` as outlined above. In this case, the
+colour maps will be loaded automatically via :func:`init`.
+
+
 -------------
 Lookup tables
 -------------
@@ -146,13 +155,12 @@ This list of label, colour, and name mappings is used to create a
 :class:`LookupTable` instance, which can be used to access the colours and
 names associated with each label value.
 
-
 Once created, ``LookupTable`` instances may be modified - labels can be
 added/removed, and the name/colour of existing labels can be modified.  The
 :func:`.installLookupTable` method will install a new lookup table, or save
 any changes made to an existing one.
 
-Built-in and user-added lookup tables are managed in the same manner as
+Built-in user- and site-added lookup tables are managed in the same manner as
 described for colour maps above.  The following functions are available to
 access and manage :class:`LookupTable` instances:
 
@@ -183,14 +191,8 @@ for querying installed colour maps and lookup tables,
    :nosignatures:
 
    fileType
-   getCmapDir
-   getLutDir
-   scanBuiltInCmaps
-   scanBuiltInLuts
-   scanUserAddedCmaps
-   scanUserAddedLuts
-   makeValidMapKey
-   isValidMapKey
+   getCmapDirs
+   getLutDirs
 
 The following functions may be used for calculating the relationship between a
 data display range and brightness/contrast scales, and generating/manipulating
@@ -204,6 +206,8 @@ colours:
    briconToDisplayRange
    briconToScaleOffset
    applyBricon
+   darken
+   brighten
    randomColour
    randomBrightColour
    randomDarkColour
@@ -211,8 +215,8 @@ colours:
 """
 
 
-import itertools as it
 import functools as ft
+import operator  as ops
 import os.path   as op
 import              os
 import              bisect
@@ -222,31 +226,49 @@ import              colorsys
 
 import numpy             as np
 import matplotlib        as mpl
-import matplotlib.cm     as mplcm
 import matplotlib.colors as colors
 
-import fsl.version            as fslversion
 import fsl.utils.settings     as fslsettings
 import fsl.utils.notifier     as notifier
 import fsl.data.vest          as vest
 import fsleyes_props          as props
 import                           fsleyes
 import fsleyes.displaycontext as fsldisplay
+import fsleyes.utils          as utils
 
 
 log = logging.getLogger(__name__)
 
 
-def getCmapDir():
-    """Returns the directory in which all built-in colour map files are stored.
+def _getMapDirs(subDir):
+    """Used by the :func:`getCmapDirs` and :func:`getLutDirs` functions.
+    Returns a list of all base directories within which colourmap/lookup table
+    files might be found, in the order of precedence (i.e. files in the first
+    directory should take precedence over subsequent directories).
     """
-    return op.join(fsleyes.assetDir, 'colourmaps')
+    # cmaps/luts can be stored in the following locations:
+    #   - The FSLeyes assets directory (built-ins)
+    #   - The FSLEYES_SITE_CONFIG_DIR (site-added)
+    #   - The FSLeyes settings directory (user-added)
+    mapDirs = [fslsettings.settings.configDir,
+               os.environ.get('FSLEYES_SITE_CONFIG_DIR', ''),
+               fsleyes.assetDir]
+    mapDirs = [op.join(d, subDir) for d in mapDirs]
+    return [d for d in mapDirs if op.isdir(d)]
 
 
-def getLutDir():
-    """Returns the directory in which all built-in lookup table files are stored.
+def getCmapDirs():
+    """Returns a list of directories within which all colour map files can be
+    found.
     """
-    return op.join(fsleyes.assetDir, 'luts')
+    return _getMapDirs('colourmaps')
+
+
+def getLutDirs():
+    """Returns a list of directories within which all lookup table files can be
+    found.
+    """
+    return _getMapDirs('luts')
 
 
 def _walk(dirname, suffix):
@@ -254,118 +276,59 @@ def _walk(dirname, suffix):
     specified ``suffix``.
     """
     matches = []
-    for dirpath, dirnames, filenames in os.walk(dirname):
+    for dirpath, _, filenames in os.walk(dirname):
         for filename in filenames:
             if filename.endswith(suffix):
                 matches.append(op.join(dirpath, filename))
     return matches
 
 
-def _find(mapid, dirname):
-    """Finds the file associated with the given (built-in) colour map or lookup
-    table ID.
+def _scanMaps(mapDir, mapFiles):
+    """Used by the :func:`scanColourMaps` and :func:`scanLookupTables`
+    functions. Converts a sequence of colourmap or lookup table file paths
+    into IDs. Returns a dictionary containing ``{mapID : mapFile}`` mappings
+    """
+    mapFiles = [f for f in mapFiles if op.basename(f) != 'order.txt']
+    mapIDs   = [op.relpath(f, mapDir)  for f in mapFiles]
+    mapIDs   = [m.replace(op.sep, '_') for m in mapIDs]
+    mapIDs   = [op.splitext(m)[0]      for m in mapIDs]
+    mapIDs   = [m.lower()              for m in mapIDs]
+    return dict(sorted(zip(mapIDs, mapFiles)))
+
+
+def _scanMapDirs(mapDirs, suffixes):
+    """Used by the :func:`scanColourMaps` and :func:`scanLookupTables`
+    functions. Searches the given ``mapDir`` for all files with any of the
+    given ``suffixes``.
     """
 
-    divs     = [i for i, c in enumerate(mapid) if c == '_']
-    pathopts = it.chain(*[it.combinations(divs, n)
-                          for n in range(len(divs) + 1)])
+    maps = {}
 
-    for pathidxs in pathopts:
+    # loop through dirs in reverse so
+    # that files in the first dir will
+    # override files in latter dirs
+    for mapDir in reversed(mapDirs):
+        mapFiles = [_walk(mapDir, s) for s in suffixes]
+        mapFiles = ft.reduce(ops.add, mapFiles)
+        maps.update(_scanMaps(mapDir, mapFiles))
 
-        path = mapid
-        for i in pathidxs:
-            path = path[:i] + op.sep + path[i + 1:]
-
-        path = op.join(dirname, path)
-
-        if op.exists(path):
-            return path
-
-    raise ValueError('Cannot find {} in {}'.format(mapid, dirname))
-
-
-def scanBuiltInCmaps():
-    """Returns a list of IDs for all built-in colour maps. """
-
-    basedir = getCmapDir()
-    cmapIDs = _walk(basedir, '.cmap')
-    cmapIDs = [op.splitext(i)[0]      for i in cmapIDs]
-    cmapIDs = [op.relpath(i, basedir) for i in cmapIDs]
-    cmapIDs = [i.replace(op.sep, '_') for i in cmapIDs]
-
-    return list(sorted(cmapIDs))
-
-
-def scanBuiltInLuts():
-    """Returns a list of IDs for all built-in lookup tables. """
-
-    basedir = getLutDir()
-    lutIDs  = _walk(basedir, '.lut')
-    lutIDs  = [op.splitext(i)[0]      for i in lutIDs]
-    lutIDs  = [op.relpath(i, basedir) for i in lutIDs]
-    lutIDs  = [i.replace(op.sep, '_') for i in lutIDs]
-
-    return list(sorted(lutIDs))
-
-
-def scanUserAddedCmaps():
-    """Returns a list of IDs for all user-added colour maps. """
-
-    cmapFiles = fslsettings.listFiles('colourmaps/*.cmap')
-    cmapFiles = [op.basename(f)    for f in cmapFiles]
-    cmapIDs   = [op.splitext(f)[0] for f in cmapFiles]
-    cmapIDs   = [m.lower()         for m in cmapIDs]
-
-    return list(sorted(cmapIDs))
-
-
-def scanUserAddedLuts():
-    """Returns a list of IDs for all user-added lookup tables. """
-
-    lutFiles = fslsettings.listFiles('luts/*.lut')
-    lutFiles = [op.basename(f)    for f in lutFiles]
-    lutIDs   = [op.splitext(f)[0] for f in lutFiles]
-    lutIDs   = [m.lower()         for m in lutIDs]
-
-    return list(sorted(lutIDs))
-
-
-def makeValidMapKey(name):
-    """Turns the given string into a valid key for use as a colour map
-    or lookup table identifier.
-    """
-
-    valid = string.ascii_lowercase + string.digits + '_-'
-    key   = name.lower().replace(' ', '_')
-    key   = ''.join([c for c in key if c in valid])
-
-    return key
-
-
-def isValidMapKey(key):
-    """Returns ``True`` if the given string is a valid key for use as a colour
-    map or lookup table identifier, ``False`` otherwise. A valid key comprises
-    lower case letters, numbers, underscores and hyphens.
-    """
-
-    valid = string.ascii_lowercase + string.digits + '_-'
-    return all(c in valid for c in key)
+    return maps
 
 
 def scanColourMaps():
-    """Scans the colour maps directories, and returns a list containing the
-    names of all colour maps contained within. This function may be called
-    before :func:`init`.
+    """Scans the colour maps directories, and returns a dictionary of ``{id :
+    filepath}`` containing the IDs and files of all colour maps contained
+    within. This function may be called before :func:`init`.
     """
-    return scanBuiltInCmaps() + scanUserAddedCmaps()
+    return _scanMapDirs(getCmapDirs(), ['.cmap', '.txt'])
 
 
 def scanLookupTables():
-    """Scans the lookup tables directories, and returns a list containing the
-    names of all lookup tables contained within. This function may be called
-    before :func:`init`.
+    """Scans the lookup table directories, and returns a dictionary of ``{id :
+    filepath}`` containing the IDs and files of all lookup tables contained
+    within. This function may be called before :func:`init`.
     """
-    return scanBuiltInLuts() + scanUserAddedLuts()
+    return _scanMapDirs(getLutDirs(), ['.lut', '.txt'])
 
 
 _cmaps = None
@@ -442,40 +405,29 @@ def init(force=False):
     # way, so we loop over all of
     # these lists, doing colour maps
     # first, then luts second.
-    mapTypes    = ['cmap',               'lut']
-    builtinDirs = [getCmapDir(),         getLutDir()]
-    userDirs    = ['colourmaps',         'luts']
-    allBuiltins = [scanBuiltInCmaps(),   scanBuiltInLuts()]
-    allUsers    = [scanUserAddedCmaps(), scanUserAddedLuts()]
-    registers   = [_cmaps,               _luts]
+    registers  = [_cmaps,           _luts]
+    mapTypes   = ['cmap',           'lut']
+    allMapDirs = [getCmapDirs(),    getLutDirs()]
+    allMaps    = [scanColourMaps(), scanLookupTables()]
 
-    for mapType, builtinDir, userDir, builtinIDs, userIDs, register in zip(
-            mapTypes, builtinDirs, userDirs, allBuiltins, allUsers, registers):
-
-        builtinFiles = ['{}.{}'.format(m, mapType) for m in builtinIDs]
-        builtinFiles = [_find(m, builtinDir)       for m in builtinFiles]
-        userFiles    = ['{}.{}'.format(m, mapType) for m in userIDs]
-        userFiles    = [op.join(userDir, m)        for m in userFiles]
-        userFiles    = [fslsettings.filePath(m)    for m in userFiles]
-
-        allIDs   = builtinIDs   + userIDs
-        allFiles = builtinFiles + userFiles
-        allFiles = dict(zip(allIDs, allFiles))
+    for mapType, mapDirs, maps, register in zip(
+            mapTypes, allMapDirs, allMaps, registers):
 
         # Read order/display names from order.txt -
-        # if an order.txt file exists in the user
-        # dir, it takes precednece over built-in
-        # order/display names.
-        #
-        # User-added display names may also be in
-        # fslsettings. Any user-added maps with
-        # the same ID as a builtin will override
-        # the builtin.
-        builtinOrder = op.join(builtinDir, 'order.txt')
-        userOrder    = op.join(userDir,    'order.txt')
+        # an order.txt file may exist in any of the
+        # map directories - the first that is found
+        # takes precedence (see _getMapDirs).
+        names = None
+        for mapDir in mapDirs:
+            ot = op.join(mapDir, 'order.txt')
+            if op.exists(ot):
+                names = readOrderTxt(ot)
+                break
 
-        if op.exists(userOrder): names = readOrderTxt(userOrder)
-        else:                    names = readOrderTxt(builtinOrder)
+        # This should never happen, unless the built-
+        # in order.txt is deleted for some reason
+        if names is None:
+            names = {}
 
         names.update(readDisplayNames(mapType))
 
@@ -486,7 +438,7 @@ def init(force=False):
         # and their name is just set to the
         # ID (which is equal to the file name
         # prefix).
-        for mid in allIDs:
+        for mid in maps.keys():
             if mid not in names:
                 names[mid] = mid
 
@@ -498,23 +450,24 @@ def init(force=False):
             # might contain obsolete/invalid
             # names, so we ignore keyerrors
             try:
-                mapFile = allFiles[mapID]
+                mapFile = maps[mapID]
             except KeyError:
                 continue
 
             try:
                 kwargs = {'key' : mapID, 'name' : mapName}
 
-                if   mapType == 'cmap': registerColourMap(  mapFile, **kwargs)
-                elif mapType == 'lut':  registerLookupTable(mapFile, **kwargs)
+                if   mapType == 'cmap':
+                    mapID = registerColourMap(mapFile, **kwargs)
+                elif mapType == 'lut':
+                    registerLookupTable(mapFile, **kwargs)
 
                 register[mapID].installed    = True
                 register[mapID].mapObj.saved = True
 
             except Exception as e:
-                log.warn('Error processing custom {} '
-                         'file {}: {}'.format(mapType, mapFile, str(e)),
-                         exc_info=True)
+                log.warning('Error processing custom %s file {mapFile}: %s',
+                            mapType, e, exc_info=True)
 
 
 def registerColourMap(cmapFile,
@@ -548,35 +501,33 @@ def registerColourMap(cmapFile,
     :returns:         The key that the ``ColourMap`` was registered under.
     """
 
-    if key is not None and not isValidMapKey(key):
-        raise ValueError('{} is not a valid colour map identifier'.format(key))
+    if key is not None and not utils.isValidMapKey(key):
+        raise ValueError(f'{key} is not a valid colour map identifier')
 
     if key is None:
         key = op.splitext(op.basename(cmapFile))[0]
-        key = makeValidMapKey(key)
+        key = utils.makeValidMapKey(key)
 
     if name        is None: name        = key
     if overlayList is None: overlayList = []
 
+    # We cannot override built-in mpl colourmaps.
+    # If a FSLeyes colourmap clashes with a built-in,
+    # we register it as "fsleyes_{key}"
+    if key in mpl.colormaps:
+        key = f'fsleyes_{key}'
+
     data = loadColourMapFile(cmapFile)
-    cmap = colors.ListedColormap(data, key)
+    cmap = colors.ListedColormap(data, name=key)
 
     log.debug('Loading and registering custom '
-              'colour map: {}'.format(cmapFile))
-
-    # matplotlib 3.3.4 and newer will raise an error
-    # when trying to register a colour map with the
-    # same name as a built-in, unless override_builtin
-    # (also added in that version) is set to True.
-    if fslversion.compareVersions(mpl.__version__, '3.3.4') > 0:
-        mplcm.register_cmap(key, cmap, override_builtin=True)
-    else:
-        mplcm.register_cmap(key, cmap)
+              'colour map: %s', cmapFile)
 
+    mpl.colormaps.register(cmap, name=key, force=True)
     _cmaps[key] = _Map(key, name, cmap, cmapFile, False)
 
     log.debug('Patching DisplayOpts instances and class '
-              'to support new colour map {}'.format(key))
+              'to support new colour map %s', key)
 
     # A list of all DisplayOpts colour map properties.
     # n.b. We can't simply list the ColourMapOpts class
@@ -586,11 +537,11 @@ def registerColourMap(cmapFile,
     # TODO Any new DisplayOpts sub-types which have a
     #      colour map will need to be patched here
     cmapProps = []
-    cmapProps.append((fsldisplay.VolumeOpts, 'cmap'))
-    cmapProps.append((fsldisplay.VolumeOpts, 'negativeCmap'))
-    cmapProps.append((fsldisplay.VectorOpts, 'cmap'))
-    cmapProps.append((fsldisplay.MeshOpts,   'cmap'))
-    cmapProps.append((fsldisplay.MeshOpts,   'negativeCmap'))
+    cmapProps.append((fsldisplay.VolumeOpts,      'cmap'))
+    cmapProps.append((fsldisplay.VolumeOpts,      'negativeCmap'))
+    cmapProps.append((fsldisplay.NiftiVectorOpts, 'cmap'))
+    cmapProps.append((fsldisplay.MeshOpts,        'cmap'))
+    cmapProps.append((fsldisplay.MeshOpts,        'negativeCmap'))
 
     # Update the colour map properties
     # for any existing instances
@@ -654,13 +605,13 @@ def registerLookupTable(lut,
 
         if key is None:
             key = op.splitext(op.basename(lutFile))[0]
-            key = makeValidMapKey(key)
+            key = utils.makeValidMapKey(key)
 
         if name is None:
             name = key
 
         log.debug('Loading and registering custom '
-                  'lookup table: {}'.format(lutFile))
+                  'lookup table: %s', lutFile)
 
         lut = LookupTable(key, name, lutFile)
 
@@ -678,7 +629,7 @@ def registerLookupTable(lut,
     _luts[key] = _Map(key, name, lut, lutFile, False)
 
     log.debug('Patching LabelOpts classes to support '
-              'new LookupTable {}'.format(key))
+              'new LookupTable %s', key)
 
     # See similar situation in the registerColourMap
     # function above. All DisplayOpts classes which
@@ -837,9 +788,9 @@ def installColourMap(key):
     #      this.
     data = cmap.mapObj.colors
 
-    destFile = op.join('colourmaps', '{}.cmap'.format(key))
+    destFile = op.join('colourmaps', f'{key}.cmap')
 
-    log.debug('Installing colour map {} to {}'.format(key, destFile))
+    log.debug('Installing colour map %s to %s', key, destFile)
 
     # Numpy under python 3 will break if
     # we give it a file opened with mode='wt'.
@@ -862,11 +813,11 @@ def installLookupTable(key):
 
     # keyerror if not registered
     lut      = _luts[key]
-    destFile = op.join('luts', '{}.lut'.format(key))
+    destFile = op.join('luts', f'{key}.lut')
     destFile = fslsettings.filePath(destFile)
     destDir  = op.dirname(destFile)
 
-    log.debug('Installing lookup table {} to {}'.format(key, destFile))
+    log.debug('Installing lookup table %s to %s', key, destFile)
 
     if not op.exists(destDir):
         os.makedirs(destDir)
@@ -928,7 +879,7 @@ def fileType(fname):
         except ValueError:
             pass
 
-    raise ValueError('Cannot determine type of {}'.format(fname))
+    raise ValueError(f'Cannot determine type of {fname}')
 
 
 def loadColourMapFile(fname, aslut=False):
@@ -981,7 +932,7 @@ def loadLookupTableFile(fname):
     # Accept cmap file, auto-generate labels/names
     if fileType(fname) in ('vest', 'cmap'):
         lut   = loadColourMapFile(fname, aslut=True)
-        names = ['{}'.format(int(l)) for l in lut[:, 0]]
+        names = [str(int(l)) for l in lut[:, 0]]
         return lut, names
 
     # Otherwise assume a FSLeyes lut file
@@ -992,6 +943,7 @@ def loadLookupTableFile(fname):
         struclut = np.genfromtxt(
             f,
             usecols=(0, 1, 2, 3),
+            comments='#',
             dtype={
                 'formats' : (int, float, float, float),
                 'names'   : ('label', 'r', 'g', 'b')})
@@ -1012,8 +964,11 @@ def loadLookupTableFile(fname):
         f.seek(0)
         names = []
         for i, line in enumerate(f):
+            line = line.strip()
+            if line == '' or line.startswith('#'):
+                continue
             tkns = line.split(None, 4)
-            if len(tkns) < 5: name = '{}'.format(int(lut[i, 0]))
+            if len(tkns) < 5: name = str(int(lut[i, 0]))
             else:             name = tkns[4].strip()
             names.append(name)
 
@@ -1146,7 +1101,7 @@ def applyBricon(rgb, brightness, contrast):
 
     :arg contrast:   A contrast level in the range ``[0, 1]``.
     """
-    rgb       = np.array(rgb)
+    rgb       = np.array(rgb, dtype=float)
     oneColour = len(rgb.shape) == 1
     rgb       = rgb.reshape(-1, rgb.shape[-1])
 
@@ -1165,6 +1120,20 @@ def applyBricon(rgb, brightness, contrast):
     else:         return rgb
 
 
+def darken(rgb, amount=0.05):
+    """Darken the given rgb colour(s) by the given amount. Short-hand
+    for ``applyBricon(rgb, 0.5 - amount, 0.5)``.
+    """
+    return applyBricon(rgb, 0.5 - amount, 0.5)
+
+
+def brighten(rgb, amount=0.05):
+    """Brighten the given rgb colour(s) by the given amount. Short-hand
+    for ``applyBricon(rgb, 0.5 + amount, 0.5)``.
+    """
+    return applyBricon(rgb, 0.5 + amount, 0.5)
+
+
 def randomColour():
     """Generates a random RGB colour. """
     values = [randomColour.random.rand() for i in range(3)]
@@ -1189,8 +1158,7 @@ def randomBrightColour():
 
 def randomDarkColour():
     """Generates a random saturated and darkened RGB colour."""
-
-    return applyBricon(randomBrightColour(), 0.35, 0.5)
+    return darken(randomBrightColour(), 0.15)
 
 
 def complementaryColour(rgb):
@@ -1272,7 +1240,7 @@ class _Map:
         :arg name:        The display name of the colour map/lookup table.
 
         :arg mapObj:      The colourmap/lut object, either a
-                          :class:`matplotlib.colors..Colormap`, or a
+                          ``matplotlib.colors.Colormap``, or a
                           :class:`LookupTable` instance.
 
         :arg mapFile:     The file from which this map was loaded,
@@ -1399,10 +1367,8 @@ class LutLabel(props.HasProperties):
 
     def __str__(self):
         """Returns a string representation of this ``LutLabel``."""
-        return '{}: {} / {} ({})'.format(self.value,
-                                         self.internalName,
-                                         self.colour,
-                                         self.enabled)
+        return f'{self.value}: {self.internalName} / ' \
+               f'{self.colour} ({self.enabled})'
 
 
     def __repr__(self):
@@ -1452,7 +1418,7 @@ class LookupTable(notifier.Notifier):
         """Create a ``LookupTable``.
 
         :arg key:     The identifier for this ``LookupTable``. Must be
-                      a valid key (see :func:`isValidMapKey`).
+                      a valid key (see :func:`.isValidMapKey`).
 
         :arg name:    The display name for this ``LookupTable``.
 
@@ -1462,13 +1428,13 @@ class LookupTable(notifier.Notifier):
                       :meth:`insert` methods.
         """
 
-        if not isValidMapKey(key):
-            raise ValueError('{} is not a valid lut identifier'.format(key))
+        if not utils.isValidMapKey(key):
+            raise ValueError(f'{key} is not a valid lut identifier')
 
         self.key      = key
         self.name     = name
         self.__labels = []
-        self.__name   = 'LookupTable({})_{}'.format(self.name, id(self))
+        self.__name   = f'LookupTable({self.name})_{id(self)}'
 
         # The LUT is loaded now, but parsed
         # lazily on first access
@@ -1609,13 +1575,11 @@ class LookupTable(notifier.Notifier):
         """
         if not isinstance(value, (int, np.integer)) or \
            value < 0 or value > 65535:
-            raise ValueError('Lookup table values must be '
-                             '16 bit unsigned integers ({}: {}).'.format(
-                                 type(value), value))
+            raise ValueError('Lookup table values must be 16 bit unsigned '
+                             f'integers ({type(value)}: {value}).')
 
         if self.get(value) is not None:
-            raise ValueError('Value {} is already in '
-                             'lookup table'.format(value))
+            raise ValueError(f'Value {value} is already in lookup table')
 
         label = LutLabel(value, name, colour, enabled)
         label.addGlobalListener(self.__name, self.__labelChanged)
@@ -1659,7 +1623,7 @@ class LookupTable(notifier.Notifier):
                 tkns   = [value, colour[0], colour[1], colour[2], name]
                 line   = ' '.join(map(str, tkns))
 
-                f.write('{}\n'.format(line))
+                f.write(f'{line}\n')
 
         self.saved = True
 
