@@ -14,13 +14,14 @@ import collections
 
 import wx
 
-import fsl.data.image                as fslimage
-import fsleyes_props                 as props
-import fsleyes.controls.controlpanel as ctrlpanel
-import fsleyes.views.canvaspanel     as canvaspanel
-import fsleyes.gl                    as fslgl
-import fsleyes.tooltips              as fsltooltips
-import fsleyes.strings               as strings
+import fsl.data.image                        as fslimage
+import fsleyes_props                         as props
+import fsleyes.controls.controlpanel         as ctrlpanel
+import fsleyes.views.canvaspanel             as canvaspanel
+import fsleyes.gl                            as fslgl
+import fsleyes.plugins                       as plugins
+import fsleyes.tooltips                      as fsltooltips
+import fsleyes.strings                       as strings
 
 
 class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
@@ -122,7 +123,11 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
             ('bgColour',     props.Widget('bgColour')),
             ('fgColour',     props.Widget('fgColour')),
             ('cursorColour', props.Widget('cursorColour')),
-            ('highDpi', props.Widget('highDpi')),
+            ('performance',
+             props.Widget('performance',
+                          spin=False,
+                          showLimits=False,
+                          labels=strings.choices['SceneOpts.performance'])),
             ('showColourBar', props.Widget('showColourBar')),
             ('colourBarLabelSide',
              props.Widget('colourBarLabelSide',
@@ -144,21 +149,22 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
                                        spin=True,
                                        showLimits=False))))
 
-        # The highDpi setting is
-        # not always relevant
-        if not fslgl.WXGLCanvasTarget.canToggleHighDPI():
-            sceneOptsProps.pop('highDpi')
-
         def _displaySpaceOptionName(opt):
-
             if isinstance(opt, fslimage.Nifti):
                 return opt.name
             else:
                 return strings.choices['DisplayContext.displaySpace'][opt]
 
+        def _onDisplaySpace(newDS):
+            if isinstance(newDS, fslimage.Nifti):
+                newDS = 'ref'
+            displayCtx.defaultDisplaySpace = newDS
+
         displayCtxProps = collections.OrderedDict((
             ('displaySpace',
-             props.Widget('displaySpace', labels=_displaySpaceOptionName)),
+             props.Widget('displaySpace',
+                          labels=_displaySpaceOptionName,
+                          onUser=_onDisplaySpace)),
             ('radioOrientation', props.Widget('radioOrientation'))))
 
         orthoOptsProps = collections.OrderedDict((
@@ -181,9 +187,17 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
             ('zoom',
              props.Widget('zoom', showLimits=False, spin=False)),
             ('sliceSpacing',   props.Widget('sliceSpacing', showLimits=False)),
+            ('sliceOverlap',   props.Widget('sliceOverlap', showLimits=False)),
             ('zrange',         props.Widget('zrange',       showLimits=False)),
+            ('sampleSlices',   _genLightboxSampleWidget),
+            ('reverseSlices',  props.Widget('reverseSlices')),
+            ('reverseOverlap', props.Widget('reverseOverlap')),
             ('highlightSlice', props.Widget('highlightSlice')),
-            ('showGridLines',  props.Widget('showGridLines'))))
+            ('showGridLines',  props.Widget('showGridLines')),
+            ('labelSpace',     props.Widget(
+                'labelSpace',
+                labels=strings.choices['LightBoxOpts.labelSpace'])),
+        ))
 
         scene3dOptsProps = collections.OrderedDict((
             ('zoom',          props.Widget('zoom', showLimits=False)),
@@ -210,6 +224,7 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
             panelProps = scene3dOptsProps
 
             # We hide some options in 3D
+            sceneOptsProps .pop('performance')
             displayCtxProps.pop('displaySpace')
             displayCtxProps.pop('radioOrientation')
 
@@ -264,7 +279,7 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
         for propName, dispProp in panelProps.items():
 
             if callable(dispProp):
-                widget = dispProp(widgets, opts)
+                widget = dispProp(canvasPanel, widgets, opts)
             else:
                 widget = props.buildGUI(widgets, opts, dispProp)
 
@@ -281,7 +296,7 @@ class CanvasSettingsPanel(ctrlpanel.SettingsPanel):
         widgets.Expand(panelGroup)
 
 
-def _genLightPosWidget(parent, opts):
+def _genLightPosWidget(canvasPanel, parent, opts):
     """Generates a widget for the :attr:`.Scene3DOpts.lightPos` property.
     """
     px, py, pz = props.makeListWidgets(
@@ -297,3 +312,33 @@ def _genLightPosWidget(parent, opts):
     sizer.Add(py, flag=wx.EXPAND)
     sizer.Add(pz, flag=wx.EXPAND)
     return sizer
+
+
+def _genLightboxSampleWidget(canvasPanel, parent, opts):
+    """Generates a widget for the :attr:`.LightBoxCanvasOpts.sampleSlices`
+    property, and a button that opens a :class:`.LightboxSampleDiaglog`.
+    """
+
+    # sampleSlices dropdown
+    sample = props.makeWidget(
+        parent,
+        opts,
+        'sampleSlices',
+        labels=strings.choices['LightBoxOpts.sampleSlices'])
+
+    # button which opens the lightbox sample dialog
+    btn = wx.Button(parent)
+    btn.SetLabel(strings.labels[CanvasSettingsPanel, 'lightboxsample'])
+
+    LightBoxSampleAction = plugins.lookupTool('LightBoxSampleAction')
+    action = LightBoxSampleAction(canvasPanel.overlayList,
+                                  canvasPanel.displayCtx,
+                                  canvasPanel)
+
+    action.bindToWidget(parent, wx.EVT_BUTTON, btn)
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+    sizer.Add(sample, flag=wx.EXPAND, proportion=1)
+    sizer.Add(btn,    flag=wx.EXPAND)
+
+    return sizer
