@@ -393,13 +393,15 @@ class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
         """Creates and returns a collection of controls for editing properties
         of the given :class:`.MaskOpts` instance.
         """
-        thresSpec  = self.__widgetSpecs[opts, 'threshold']
-        colourSpec = self.__widgetSpecs[opts, 'colour']
+        thresSpec   = self.__widgetSpecs[opts, 'threshold']
+        colourSpec  = self.__widgetSpecs[opts, 'colour']
+        outlineSpec = self.__widgetSpecs[opts, 'outline']
 
-        thresWidget  = props.buildGUI(self, opts, thresSpec)
-        colourWidget = props.buildGUI(self, opts, colourSpec)
+        thresWidget   = props.buildGUI(self, opts, thresSpec)
+        colourWidget  = props.buildGUI(self, opts, colourSpec)
+        outlineWidget = props.buildGUI(self, opts, outlineSpec)
 
-        tools = [thresWidget, colourWidget]
+        tools = [thresWidget, colourWidget, outlineWidget]
 
         return tools, tools
 
@@ -670,8 +672,10 @@ class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
             :attr:`.VectorOpts.clipImage`, and other :class:`.Image`-based
             choice properties.
             """
-            if img is None: return 'None'
-            else:           return self.displayCtx.getDisplay(img).name
+            if (img is None) or (img not in self.overlayList):
+                return 'None'
+            else:
+                return self.displayCtx.getDisplay(img).name
 
         def _pathLabel(p):
             if p is None: return 'None'
@@ -684,7 +688,10 @@ class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
             if data is None:
                 return 'None'
             elif isinstance(data, fslimage.Image):
-                return self.displayCtx.getDisplay(data).name
+                if data in self.overlayList:
+                    return self.displayCtx.getDisplay(data).name
+                else:
+                    return 'None'
             else:
                 return op.basename(data)
 
@@ -760,6 +767,12 @@ class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
                 'colour',
                 size=(24, 24),
                 tooltip=_TOOLTIPS['MaskOpts.colour']),
+            'MaskOpts.outline'      : props.Widget(
+                'outline',
+                tooltip=_TOOLTIPS['MaskOpts.outline'],
+                icon=[icons.findImageFile('outline24'),
+                      icons.findImageFile('filled24')],
+                toggle=True),
 
             'LabelOpts.lut'     : props.Widget(
                 'lut',
@@ -792,7 +805,8 @@ class OverlayDisplayToolBar(ctrlpanel.ControlToolBar):
                 showLimits=False,
                 spin=False,
                 tooltip=_TOOLTIPS['MeshOpts.outlineWidth'],
-                enabledWhen=lambda i: i.outline),
+                dependencies=['outline', 'vertexData'],
+                enabledWhen=lambda i, o, vd: o or (vd is not None)),
             'MeshOpts.vertexSet' : props.Widget(
                 'vertexSet',
                 labels=_pathLabel,
@@ -914,6 +928,7 @@ _TOOLTIPS = td.TypeDict({
     'ComplexOpts.component' : fsltooltips.properties['ComplexOpts.component'],
 
     'MaskOpts.threshold' : fsltooltips.properties['MaskOpts.threshold'],
+    'MaskOpts.outline'   : fsltooltips.properties['MaskOpts.outline'],
     'MaskOpts.colour'    : fsltooltips.properties['MaskOpts.colour'],
 
     'LabelOpts.lut'          : fsltooltips.properties['LabelOpts.lut'],
