@@ -28,6 +28,9 @@ import            functools
 import            wx
 
 import fsl.utils.idle                 as idle
+import fsl.data.mesh                  as fslmesh
+import fsl.data.image                 as fslimage
+import fsleyes.data.tractogram        as tractogram
 import fsleyes_props                  as props
 import fsleyes_widgets                as fwidgets
 import fsleyes_widgets.imagepanel     as imagepanel
@@ -79,7 +82,11 @@ def getWidgetSpecs(target, displayCtx, threedee=False):
     if sdicts is None:
         return {}
 
-    return functools.reduce(_merge_dicts, sdicts)
+    # TypeDict.get sorts fromfrom sub-class (first) to
+    # base-class (last). As we want sub-class settings
+    # to have priority, we merge the dicts in recerse
+    # order.
+    return functools.reduce(_merge_dicts, reversed(sdicts))
 
 
 def get3DWidgetSpecs(target, displayCtx):
@@ -90,7 +97,7 @@ def get3DWidgetSpecs(target, displayCtx):
     if sdicts is None:
         return {}
 
-    return functools.reduce(_merge_dicts, sdicts)
+    return functools.reduce(_merge_dicts, reversed(sdicts))
 
 
 def _getThing(target, prefix, thingDict, *args, **kwargs):
@@ -158,9 +165,8 @@ def _initPropertyList_VolumeOpts(threedee):
              'interpolateCmaps',
              'invert',
              'invertClipping',
-             'linkLowRanges',
-             'linkHighRanges',
-             'modulateAlpha',
+             'custom_linkRanges',
+             'custom_modulateAlpha',
              'clipImage',
              'modulateImage',
              'displayRange',
@@ -169,7 +175,7 @@ def _initPropertyList_VolumeOpts(threedee):
              'custom_overrideDataRange']
 
     if threedee:
-        plist.remove('modulateAlpha')
+        plist.remove('custom_modulateAlpha')
         plist.remove('clipImage')
         plist.remove('modulateImage')
         plist.remove('modulateRange')
@@ -199,11 +205,13 @@ def _initPropertyList_MaskOpts(threedee):
             'outlineWidth']
 
 
-def _initPropertyList_VectorOpts(threedee):
-    return ['colourImage',
+def _initPropertyList_NiftiVectorOpts(threedee):
+    return ['normaliseColour',
+            'colourImage',
             'modulateImage',
             'clipImage',
             'cmap',
+            'colourRange',
             'clippingRange',
             'modulateRange',
             'modulateMode',
@@ -217,8 +225,7 @@ def _initPropertyList_VectorOpts(threedee):
 
 
 def _initPropertyList_RGBVectorOpts(threedee):
-    return ['interpolation',
-            'unitLength']
+    return ['interpolation']
 
 
 def _initPropertyList_LineVectorOpts(threedee):
@@ -245,6 +252,7 @@ def _initPropertyList_MeshOpts(threedee):
              'custom_vertexSet',
              'custom_vertexData',
              'vertexDataIndex',
+             'interpolation',
              'custom_lut',
              'custom_cmap',
              'cmapResolution',
@@ -253,9 +261,8 @@ def _initPropertyList_MeshOpts(threedee):
              'invert',
              'invertClipping',
              'discardClipped',
-             'linkLowRanges',
-             'linkHighRanges',
-             'modulateAlpha',
+             'custom_linkRanges',
+             'custom_modulateAlpha',
              'modulateData',
              'displayRange',
              'clippingRange',
@@ -271,7 +278,7 @@ def _initPropertyList_MeshOpts(threedee):
 
 
 def _init3DPropertyList_MeshOpts():
-    return ['flatShading', 'wireframe']
+    return ['wireframe']
 
 
 def _initPropertyList_GiftiOpts(threedee):
@@ -317,8 +324,7 @@ def _initPropertyList_MIPOpts(threedee):
             'interpolateCmaps',
             'invert',
             'invertClipping',
-            'linkLowRanges',
-            'linkHighRanges',
+            'custom_linkRanges',
             'displayRange',
             'clippingRange',
             'window',
@@ -339,6 +345,42 @@ def _initPropertyList_VolumeRGBOpts(threedee):
             'suppressMode']
 
 
+def _initPropertyList_TractogramOpts(threedee):
+    plist = ['lineWidth',
+             'resolution',
+             'subsample']
+
+    if not threedee:
+        plist.append('sliceWidth')
+        plist.append('pseudo3D')
+        plist.append('custom_zclipping')
+
+    plist += ['refImage',
+              'coordSpace',
+              'custom_colourMode',
+              'clipMode',
+              'custom_cmap',
+              'cmapResolution',
+              'gamma',
+              'logScale',
+              'interpolateCmaps',
+              'invert',
+              'invertClipping',
+              'custom_linkRanges',
+              'custom_modulateAlpha',
+              'displayRange',
+              'clippingRange',
+              'modulateRange',
+              'xColour',
+              'yColour',
+              'zColour',
+              'suppressX',
+              'suppressY',
+              'suppressZ',
+              'suppressMode']
+    return plist
+
+
 def _initWidgetSpec_Display(displayCtx, threedee):
     return {
         'name'        : props.Widget('name'),
@@ -369,14 +411,17 @@ def _initWidgetSpec_ColourMapOpts(displayCtx, threedee):
             slider=True,
             spin=True,
             showLimits=False),
-        'interpolateCmaps' : props.Widget('interpolateCmaps'),
-        'invert'           : props.Widget('invert'),
-        'invertClipping'   : props.Widget('invertClipping'),
-        'linkLowRanges'    : props.Widget('linkLowRanges'),
-        'linkHighRanges'   : props.Widget('linkHighRanges'),
-        'modulateAlpha'    : props.Widget('modulateAlpha'),
-        'logScale'         : props.Widget('logScale'),
-        'gamma'            : props.Widget(
+        'interpolateCmaps'     : props.Widget('interpolateCmaps'),
+        'invert'               : props.Widget('invert'),
+        'invertClipping'       : props.Widget('invertClipping'),
+        'linkLowRanges'        : props.Widget('linkLowRanges'),
+        'linkHighRanges'       : props.Widget('linkHighRanges'),
+        'custom_linkRanges'    : _ColourMapOpts_linkRangesWidget,
+        'logScale'             : props.Widget('logScale'),
+        'custom_modulateAlpha' : _ColourMapOpts_modulateAlphaWidget,
+        'modulateAlpha'        : props.Widget('modulateAlpha'),
+        'invertModulateAlpha'  : props.Widget('invertModulateAlpha'),
+        'gamma'                : props.Widget(
             'gamma',
             showLimits=False,
             slider=True,
@@ -385,18 +430,21 @@ def _initWidgetSpec_ColourMapOpts(displayCtx, threedee):
             'displayRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
                     strings.choices['ColourMapOpts.displayRange.max']]),
         'clippingRange'  : props.Widget(
             'clippingRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
                     strings.choices['ColourMapOpts.displayRange.max']]),
         'modulateRange'  : props.Widget(
             'modulateRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             dependencies=['modulateAlpha'],
             enabledWhen=lambda i, ma : ma,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
@@ -407,8 +455,10 @@ def _initWidgetSpec_ColourMapOpts(displayCtx, threedee):
 def _initWidgetSpec_VolumeOpts(displayCtx, threedee):
 
     def imageName(img):
-        if img is None: return 'None'
-        else:           return displayCtx.getDisplay(img).name
+        if img is None or img not in displayCtx.overlayList:
+            return 'None'
+        else:
+            return displayCtx.getDisplay(img).name
 
     return {
         'custom_volume'            : _NiftiOpts_VolumeWidget,
@@ -454,8 +504,7 @@ def _init3DWidgetSpec_VolumeOpts(displayCtx):
                                            showLimits=False),
         'blendFactor'       : props.Widget('blendFactor',
                                            showLimits=False),
-        'blendByIntensity'  : props.Widget('blendByIntensity',
-                                           showLimits=False),
+        'blendByIntensity'  : props.Widget('blendByIntensity'),
         'resolution'        : props.Widget('resolution',
                                            showLimits=False),
         'numClipPlanes'     : props.Widget('numClipPlanes',
@@ -502,7 +551,7 @@ def _initWidgetSpec_MaskOpts(displayCtx, threedee):
             labels=strings.choices['VolumeOpts.interpolation']),
         'colour'       : props.Widget('colour'),
         'invert'       : props.Widget('invert'),
-        'threshold'    : props.Widget('threshold', showLimits=False),
+        'threshold'    : props.Widget('threshold', showLimits=False, spinWidth=8),
         'outline'      : props.Widget('outline'),
         'outlineWidth' : props.Widget('outlineWidth', showLimits=False),
     }
@@ -528,10 +577,12 @@ def _initWidgetSpec_LabelOpts(displayCtx, threedee):
     }
 
 
-def _initWidgetSpec_VectorOpts(displayCtx, threedee):
+def _initWidgetSpec_NiftiVectorOpts(displayCtx, threedee):
     def imageName(img):
-        if img is None: return 'None'
-        else:           return displayCtx.getDisplay(img).name
+        if (img is None) or (img not in displayCtx.overlayList):
+            return 'None'
+        else:
+            return displayCtx.getDisplay(img).name
 
     return {
         'colourImage'   : props.Widget(
@@ -552,6 +603,7 @@ def _initWidgetSpec_VectorOpts(displayCtx, threedee):
             'clippingRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['VectorOpts.clippingRange.min'],
                     strings.choices['VectorOpts.clippingRange.max']],
             dependencies=['clipImage'],
@@ -560,10 +612,20 @@ def _initWidgetSpec_VectorOpts(displayCtx, threedee):
             'modulateRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['VectorOpts.modulateRange.min'],
                     strings.choices['VectorOpts.modulateRange.max']],
             dependencies=['colourImage', 'modulateImage'],
             enabledWhen=lambda o, ci, mi: ci is None and mi is not None),
+        'colourRange' : props.Widget(
+            'colourRange',
+            showLimits=False,
+            slider=True,
+            spinWidth=8,
+            labels=[strings.choices['VectorOpts.colourRange.min'],
+                    strings.choices['VectorOpts.colourRange.max']],
+            dependencies=['colourImage'],
+            enabledWhen=lambda o, ci: ci is not None),
         'modulateMode' : props.Widget(
             'modulateMode',
             labels=strings.choices['VectorOpts.modulateMode'],
@@ -598,6 +660,7 @@ def _initWidgetSpec_VectorOpts(displayCtx, threedee):
             dependencies=['colourImage'],
             labels=strings.choices['VectorOpts.suppressMode'],
             enabledWhen=lambda o, ci: ci is None),
+        'normaliseColour' : props.Widget('normaliseColour'),
     }
 
 
@@ -612,11 +675,16 @@ def _initWidgetSpec_RGBVectorOpts(displayCtx, threedee):
 
 def _initWidgetSpec_LineVectorOpts(displayCtx, threedee):
     return {
-        'directed'    : props.Widget('directed'),
-        'unitLength'  : props.Widget('unitLength'),
-        'orientFlip'  : props.Widget('orientFlip'),
-        'lineWidth'   : props.Widget('lineWidth',   showLimits=False),
-        'lengthScale' : props.Widget('lengthScale', showLimits=False),
+        'directed'     : props.Widget('directed'),
+        'unitLength'   : props.Widget('unitLength'),
+        'orientFlip'   : props.Widget('orientFlip'),
+        'lineWidth'    : props.Widget('lineWidth',   showLimits=False),
+        'lengthScale'  : props.Widget('lengthScale', showLimits=False),
+        'modulateMode' : props.Widget(
+            'modulateMode',
+            labels=strings.choices['LineVectorOpts.modulateMode'],
+            dependencies=['colourImage', 'modulateImage'],
+            enabledWhen=lambda o, ci, mi: ci is None and mi is not None),
     }
 
 
@@ -627,13 +695,13 @@ def _initWidgetSpec_TensorOpts(displayCtx, threedee):
         'tensorResolution' : props.Widget(
             'tensorResolution',
             showLimits=False,
-            spin=False,
+            spin=True,
             labels=[strings.choices['TensorOpts.tensorResolution.min'],
                     strings.choices['TensorOpts.tensorResolution.max']]),
         'tensorScale'      : props.Widget(
             'tensorScale',
             showLimits=False,
-            spin=False),
+            spin=True),
     }
 
 def _initWidgetSpec_SHOpts(displayCtx, threedee):
@@ -699,8 +767,10 @@ def _initWidgetSpec_SHOpts(displayCtx, threedee):
 def _initWidgetSpec_MeshOpts(displayCtx, threedee):
 
     def imageName(img):
-        if img is None: return 'None'
-        else:           return displayCtx.getDisplay(img).name
+        if (img is None) or (img not in displayCtx.overlayList):
+            return 'None'
+        else:
+            return displayCtx.getDisplay(img).name
 
     def pathName(vdata):
         if vdata is None: return 'None'
@@ -709,6 +779,14 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
     def colourEnabledWhen(opts, vdata, useLut):
         return (vdata is not None) and (not useLut)
 
+    def interpEnabledWhen(opts, vdata):
+        return vdata is not None
+
+    interpKwargs = {
+        'dependencies' : ['vertexData'],
+        'enabledWhen'  : interpEnabledWhen
+    }
+
     colourKwargs = {
         'dependencies' : ['vertexData', 'useLut'],
         'enabledWhen'  : colourEnabledWhen
@@ -727,8 +805,8 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
         'refImage'     : props.Widget('refImage', labels=imageName),
         'coordSpace'   : props.Widget(
             'coordSpace',
-            enabledWhen=lambda o, ri: ri != 'none',
-            labels=strings.choices['MeshOpts.coordSpace'],
+            enabledWhen=lambda o, ri: ri is not None,
+            labels=strings.choices['RefImageOpts.coordSpace'],
             dependencies=['refImage']),
         'colour'       : props.Widget('colour'),
         'custom_vertexData' : _MeshOpts_vertexDataWidget,
@@ -781,6 +859,10 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
         'interpolateCmaps' : props.Widget(
             'interpolateCmaps',
             **colourKwargs),
+        'interpolation'  : props.Widget(
+            'interpolation',
+            labels=strings.choices['VolumeOpts.interpolation'],
+            **interpKwargs),
         'invert'           : props.Widget(
             'invert',
             **colourKwargs),
@@ -797,6 +879,7 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
             'displayRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
                     strings.choices['ColourMapOpts.displayRange.max']],
             **colourKwargs),
@@ -804,6 +887,7 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
             'clippingRange',
             showLimits=False,
             slider=True,
+            spinWidth=8,
             labels=[strings.choices['ColourMapOpts.displayRange.min'],
                     strings.choices['ColourMapOpts.displayRange.max']],
             dependencies=['vertexData'],
@@ -815,13 +899,7 @@ def _initWidgetSpec_MeshOpts(displayCtx, threedee):
 
 
 def _init3DWidgetSpec_MeshOpts(displayCtx):
-    return {
-        'wireframe' : props.Widget('wireframe'),
-        'flatShading' : props.Widget(
-            'flatShading',
-            dependencies=['vertexData'],
-            enabledWhen=lambda o, vd: vd is not None),
-    }
+    return {'wireframe' : props.Widget('wireframe')}
 
 
 def _initWidgetSpec_GiftiOpts(displayCtx, threedee):
@@ -857,7 +935,8 @@ def _initWidgetSpec_MIPOpts(displayCtx, threedee):
         'window' : props.Widget(
             'window',
             showLimits=False,
-            slider=True),
+            slider=True,
+            spinWidth=8),
         'minimum'  : props.Widget('minimum'),
         'absolute' : props.Widget('absolute'),
     }
@@ -892,6 +971,110 @@ def _initWidgetSpec_VolumeRGBOpts(displayCtx, threedee):
     }
 
 
+def _initWidgetSpec_TractogramOpts(displayCtx, threedee):
+    def cmodeName(data):
+        if data == 'orientation':
+            return 'Orientation'
+        if data is None:
+            return 'None'
+        elif isinstance(data, fslimage.Image):
+            if data in displayCtx.overlayList:
+                return displayCtx.getDisplay(data).name
+            else:
+                return 'None'
+        else:
+            return op.basename(data)
+
+    def imageName(img):
+        if (img is None) or (img not in displayCtx.overlayList):
+            return 'None'
+        else:
+            return displayCtx.getDisplay(img).name
+
+    cmapOpts    = dict(dependencies=['colourMode'],
+                       enabledWhen=lambda o, cm: cm != 'orientation')
+    clipOpts    = dict(dependencies=['clipMode'],
+                       enabledWhen=lambda o, cm: cm is not None)
+    orientOpts  = dict(dependencies=['colourMode'],
+                       enabledWhen=lambda o, cm: cm == 'orientation')
+    sliderOpts  = dict(spin=True, slider=True, showLimits=False)
+    zclipOpts   = dict(dependencies=['pseudo3D'],
+                       enabledWhen=lambda o, p: p)
+    clipdirOpts = dict(dependencies=['pseudo3D'],
+                       enabledWhen=lambda o, p: p,
+                       labels=strings.choices['TractogramOpts.clipdir'])
+
+    return {
+        'colourMode'        : props.Widget('colourMode', labels=cmodeName),
+        'clipMode'          : props.Widget('clipMode',   labels=cmodeName),
+        'custom_colourMode' : _TractogramOpts_colourModeWidget,
+        'xColour'           : props.Widget('xColour',      **orientOpts),
+        'yColour'           : props.Widget('yColour',      **orientOpts),
+        'zColour'           : props.Widget('zColour',      **orientOpts),
+        'suppressX'         : props.Widget('suppressX',    **orientOpts),
+        'suppressY'         : props.Widget('suppressY',    **orientOpts),
+        'suppressZ'         : props.Widget('suppressZ',    **orientOpts),
+        'suppressMode'      : props.Widget('suppressMode', **orientOpts),
+        'lineWidth'         : props.Widget('lineWidth',    **sliderOpts),
+        'sliceWidth'        : props.Widget('sliceWidth',   **sliderOpts),
+        'resolution'        : props.Widget('resolution',   **sliderOpts),
+        'subsample'         : props.Widget('subsample',    **sliderOpts),
+        'pseudo3D'          : props.Widget('pseudo3D'),
+        'xclipdir'          : props.Widget('xclipdir',     **clipdirOpts),
+        'yclipdir'          : props.Widget('yclipdir',     **clipdirOpts),
+        'zclipdir'          : props.Widget('zclipdir',     **clipdirOpts),
+        'custom_zclipping'  : _TractogramOpts_zclippingWidget,
+
+        # We override the ColourMapOpts definitions
+        # for custom enabledWhen behaviour.
+        'useNegativeCmap'  : props.Widget('useNegativeCmap',  **cmapOpts),
+        'interpolateCmaps' : props.Widget('interpolateCmaps', **cmapOpts),
+        'invert'           : props.Widget('invert',           **cmapOpts),
+        'logScale'         : props.Widget('logScale',         **cmapOpts),
+        'invertClipping'   : props.Widget('invertClipping',   **cmapOpts),
+        'linkLowRanges'    : props.Widget('linkLowRanges',    **cmapOpts),
+        'linkHighRanges'   : props.Widget('linkHighRanges',   **cmapOpts),
+
+        'refImage'     : props.Widget('refImage', labels=imageName),
+        'coordSpace'   : props.Widget(
+            'coordSpace',
+            enabledWhen=lambda o, ri: ri is not None,
+            labels=strings.choices['RefImageOpts.coordSpace'],
+            dependencies=['refImage']),
+
+        'cmap' : props.Widget(
+            'cmap',
+            labels=fslcm.getColourMapLabel,
+            **cmapOpts),
+        'negativeCmap' : props.Widget(
+            'negativeCmap',
+            labels=fslcm.getColourMapLabel,
+            **cmapOpts),
+        'gamma' : props.Widget(
+            'gamma',
+            **sliderOpts,
+            **cmapOpts),
+        'cmapResolution' : props.Widget(
+            'cmapResolution',
+            **sliderOpts,
+            **cmapOpts),
+        'displayRange' : props.Widget(
+            'displayRange',
+            showLimits=False,
+            slider=True,
+            labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                    strings.choices['ColourMapOpts.displayRange.max']],
+            **cmapOpts),
+        'clippingRange' : props.Widget(
+            'clippingRange',
+            showLimits=False,
+            slider=True,
+            labels=[strings.choices['ColourMapOpts.displayRange.min'],
+                    strings.choices['ColourMapOpts.displayRange.max']],
+            **clipOpts),
+    }
+
+
 def _ColourMapOpts_ColourMapWidget(
         target,
         parent,
@@ -916,9 +1099,9 @@ def _ColourMapOpts_ColourMapWidget(
 
     loadAction.bindToWidget(panel, wx.EVT_BUTTON, loadButton)
 
-    cmap       = getWidgetSpecs(target, threedee)['cmap']
-    negCmap    = getWidgetSpecs(target, threedee)['negativeCmap']
-    useNegCmap = getWidgetSpecs(target, threedee)['useNegativeCmap']
+    cmap       = getWidgetSpecs(target, displayCtx, threedee)['cmap']
+    negCmap    = getWidgetSpecs(target, displayCtx, threedee)['negativeCmap']
+    useNegCmap = getWidgetSpecs(target, displayCtx, threedee)['useNegativeCmap']
 
     cbpanel    = imagepanel.ImagePanel(parent)
     cbpanel.SetMinSize((-1, 30))
@@ -973,6 +1156,60 @@ def _ColourMapOpts_ColourMapWidget(
     return sizer, [cmap, negCmap, useNegCmap]
 
 
+def _ColourMapOpts_modulateAlphaWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains widgets for the
+    :attr:`.ColourMapOpts.modulateAlpha` and
+    :attr:`.ColourMapOpts.invertModulateAlpha` properties.
+    """
+    mod = getWidgetSpecs(target, displayCtx, threedee)['modulateAlpha']
+    inv = getWidgetSpecs(target, displayCtx, threedee)['invertModulateAlpha']
+
+    mod = props.buildGUI(parent, target, mod)
+    inv = props.buildGUI(parent, target, inv)
+
+    inv.SetLabel('Invert')
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    sizer.Add(mod, flag=wx.EXPAND, proportion=1)
+    sizer.Add(inv, flag=wx.EXPAND, proportion=1)
+
+    return sizer, [mod, inv]
+
+
+def _ColourMapOpts_linkRangesWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains widgets for the
+    :attr:`.ColourMapOpts.linkLowRanges` and
+    :attr:`.ColourMapOpts.linkHighRanges` properties.
+    """
+    lo = getWidgetSpecs(target, displayCtx, threedee)['linkLowRanges']
+    hi = getWidgetSpecs(target, displayCtx, threedee)['linkHighRanges']
+
+    lo = props.buildGUI(parent, target, lo)
+    hi = props.buildGUI(parent, target, hi)
+
+    lo.SetLabel('Low')
+    hi.SetLabel('High')
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+    sizer.Add(lo, flag=wx.EXPAND, proportion=1)
+    sizer.Add(hi, flag=wx.EXPAND, proportion=1)
+
+    return sizer, [lo, hi]
+
+
 def _NiftiOpts_VolumeWidget(
         target,
         parent,
@@ -984,8 +1221,8 @@ def _NiftiOpts_VolumeWidget(
     :attr:`.NiftiOpts.volume` and :attr:`.NiftiOpts.volumeDim` properties.
     """
 
-    volume    = getWidgetSpecs(target, threedee)['volume']
-    volumeDim = getWidgetSpecs(target, threedee)['volumeDim']
+    volume    = getWidgetSpecs(target, displayCtx, threedee)['volume']
+    volumeDim = getWidgetSpecs(target, displayCtx, threedee)['volumeDim']
 
     volume    = props.buildGUI(parent, target, volume)
     volumeDim = props.buildGUI(parent, target, volumeDim)
@@ -1018,8 +1255,8 @@ def _VolumeOpts_OverrideDataRangeWidget(
     """
 
     # Override data range widget
-    enable   = getWidgetSpecs(target, threedee)['enableOverrideDataRange']
-    ovrRange = getWidgetSpecs(target, threedee)['overrideDataRange']
+    enable   = getWidgetSpecs(target, displayCtx, threedee)['enableOverrideDataRange']
+    ovrRange = getWidgetSpecs(target, displayCtx, threedee)['overrideDataRange']
 
     enable   = props.buildGUI(parent, target, enable)
     ovrRange = props.buildGUI(parent, target, ovrRange)
@@ -1109,7 +1346,10 @@ def _MeshOpts_vertexDataWidget(
     data.
     """
 
-    loadAction = loadvdata.LoadVertexDataAction(overlayList, displayCtx)
+    loadAction = loadvdata.LoadVertexDataAction(overlayList,
+                                                displayCtx,
+                                                'vertexData',
+                                                fslmesh.Mesh)
     loadButton = wx.Button(parent)
     loadButton.SetLabel(strings.labels[panel, 'loadVertexData'])
 
@@ -1117,7 +1357,7 @@ def _MeshOpts_vertexDataWidget(
 
     sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-    vdata = getWidgetSpecs(target, threedee)['vertexData']
+    vdata = getWidgetSpecs(target, displayCtx, threedee)['vertexData']
     vdata = props.buildGUI(parent, target, vdata)
 
     sizer.Add(vdata,      flag=wx.EXPAND, proportion=1)
@@ -1141,7 +1381,8 @@ def _MeshOpts_vertexSetWidget(
 
     loadAction = loadvdata.LoadVertexDataAction(overlayList,
                                                 displayCtx,
-                                                vertices=True)
+                                                'vertices',
+                                                fslmesh.Mesh)
     loadButton = wx.Button(parent)
     loadButton.SetLabel(strings.labels[panel, 'loadVertices'])
 
@@ -1149,7 +1390,7 @@ def _MeshOpts_vertexSetWidget(
 
     sizer = wx.BoxSizer(wx.HORIZONTAL)
 
-    vdata = getWidgetSpecs(target, threedee)['vertexSet']
+    vdata = getWidgetSpecs(target, displayCtx, threedee)['vertexSet']
     vdata = props.buildGUI(parent, target, vdata)
 
     sizer.Add(vdata,      flag=wx.EXPAND, proportion=1)
@@ -1170,8 +1411,8 @@ def _MeshOpts_LutWidget(
     """
 
     # enable lut widget
-    lut    = getWidgetSpecs(target, threedee)['lut']
-    enable = getWidgetSpecs(target, threedee)['useLut']
+    lut    = getWidgetSpecs(target, displayCtx, threedee)['lut']
+    enable = getWidgetSpecs(target, displayCtx, threedee)['useLut']
 
     lut    = props.buildGUI(parent, target, lut)
     enable = props.buildGUI(parent, target, enable)
@@ -1182,3 +1423,71 @@ def _MeshOpts_LutWidget(
     sizer.Add(lut,    flag=wx.EXPAND, proportion=1)
 
     return sizer, [enable, lut]
+
+
+def _TractogramOpts_zclippingWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel containing widgets for changing the
+    :attr:`.TractogramOpts.xclipdir` and related properties.
+    """
+
+    xclipdir  = getWidgetSpecs(target, displayCtx, threedee)['xclipdir']
+    yclipdir  = getWidgetSpecs(target, displayCtx, threedee)['yclipdir']
+    zclipdir  = getWidgetSpecs(target, displayCtx, threedee)['zclipdir']
+
+    xclipdir  = props.buildGUI(parent, target, xclipdir)
+    yclipdir  = props.buildGUI(parent, target, yclipdir)
+    zclipdir  = props.buildGUI(parent, target, zclipdir)
+
+    xcliplbl  = wx.StaticText(parent, label='X')
+    ycliplbl  = wx.StaticText(parent, label='Y')
+    zcliplbl  = wx.StaticText(parent, label='Z')
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    sizer.Add(xcliplbl,  flag=wx.EXPAND)
+    sizer.Add(xclipdir,  flag=wx.EXPAND, proportion=1)
+    sizer.Add(ycliplbl,  flag=wx.EXPAND)
+    sizer.Add(yclipdir,  flag=wx.EXPAND, proportion=1)
+    sizer.Add(zcliplbl,  flag=wx.EXPAND)
+    sizer.Add(zclipdir,  flag=wx.EXPAND, proportion=1)
+
+    return sizer, [xclipdir, yclipdir, zclipdir]
+
+
+
+def _TractogramOpts_colourModeWidget(
+        target,
+        parent,
+        panel,
+        overlayList,
+        displayCtx,
+        threedee):
+    """Builds a panel which contains a widget for controlling the
+    :attr:`.TractogramOpts.colourMode` property, and also has a button
+    which opens a file dialog, allowing the user to load vertex data files.
+    """
+
+    loadAction = loadvdata.LoadVertexDataAction(overlayList,
+                                                displayCtx,
+                                                'vertexData',
+                                                tractogram.Tractogram)
+    loadButton = wx.Button(parent)
+    loadButton.SetLabel(strings.labels[panel, 'loadVertexData'])
+
+    loadAction.bindToWidget(panel, wx.EVT_BUTTON, loadButton)
+
+    sizer = wx.BoxSizer(wx.HORIZONTAL)
+
+    cmode = getWidgetSpecs(target, displayCtx, threedee)['colourMode']
+    cmode = props.buildGUI(parent, target, cmode)
+
+    sizer.Add(cmode,      flag=wx.EXPAND, proportion=1)
+    sizer.Add(loadButton, flag=wx.EXPAND)
+
+    return sizer, [cmode]
