@@ -10,8 +10,10 @@ displays a list of all overlays currently in the :class:`.OverlayList`.
 
 
 import logging
+import colorsys
 
-import wx
+import          wx
+import numpy as np
 
 import fsl.data.image                as fslimage
 import fsl.utils.idle                as idle
@@ -20,7 +22,7 @@ import fsleyes_props                 as props
 import fsleyes_widgets.bitmaptoggle  as bmptoggle
 import fsleyes_widgets.elistbox      as elistbox
 
-import fsleyes.panel                 as fslpanel
+import fsleyes.controls.controlpanel as ctrlpanel
 import fsleyes.icons                 as icons
 import fsleyes.autodisplay           as autodisplay
 import fsleyes.strings               as strings
@@ -33,7 +35,7 @@ import fsleyes.actions.removeoverlay as removeoverlay
 log = logging.getLogger(__name__)
 
 
-class OverlayListPanel(fslpanel.FSLeyesPanel):
+class OverlayListPanel(ctrlpanel.ControlPanel):
     """The ``OverlayListPanel`` displays all overlays in the
     :class:`.OverlayList`, and allows the user to add, remove, and re-order
     overlays. An ``OverlayListPanel`` looks something like this:
@@ -56,11 +58,19 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
     """
 
 
+    @staticmethod
+    def defaultLayout():
+        """Returns a dictionary containing layout settings to be passed to
+        :class:`.ViewPanel.togglePanel`.
+        """
+        return {'location' : wx.BOTTOM}
+
+
     def __init__(self,
                  parent,
                  overlayList,
                  displayCtx,
-                 frame,
+                 viewPanel,
                  showVis=True,
                  showGroup=True,
                  showSave=True,
@@ -75,7 +85,7 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
 
         :arg displayCtx:      A :class:`.DisplayContext` instance.
 
-        :arg frame:           The :class:`.FSLeyesFrame` instance.
+        :arg viewPanel:       The :class:`.ViewPanel` instance.
 
         :arg showVis:         If ``True`` (the default), a button will be shown
                               alongside each overlay, allowing the user to
@@ -111,8 +121,8 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
         if filterFunc is None:
             filterFunc = defaultFilter
 
-        fslpanel.FSLeyesPanel.__init__(
-            self, parent, overlayList, displayCtx, frame)
+        ctrlpanel.ControlPanel.__init__(
+            self, parent, overlayList, displayCtx, viewPanel)
 
         self.__showVis         = showVis
         self.__showGroup       = showGroup
@@ -181,7 +191,7 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
     def destroy(self):
         """Must be called when this ``OverlayListPanel`` is no longer needed.
         Removes some property listeners, and calls
-        :meth:`.FSLeyesPanel.destroy`.
+        :meth:`.ControlPanel.destroy`.
         """
 
         self.overlayList.removeListener('overlays',        self.name)
@@ -197,7 +207,7 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
         self.__filterFunc = None
         self.__listBox.Clear()
 
-        fslpanel.FSLeyesPanel.destroy(self)
+        ctrlpanel.ControlPanel.destroy(self)
 
 
     def __selectedOverlayChanged(self, *a):
@@ -301,7 +311,7 @@ class OverlayListPanel(fslpanel.FSLeyesPanel):
         Calls the :func:`.loadoverlay.interactiveLoadOverlays` method.
         """
 
-        def onLoad(overlays):
+        def onLoad(paths, overlays):
             if len(overlays) == 0:
                 return
 
@@ -383,32 +393,6 @@ class ListItemWidget(wx.Panel):
     """
 
 
-    enabledFG  = '#000000'
-    """This colour is used as the foreground (text) colour for overlays
-    where their :attr:`.Display.enabled` property is ``True``.
-    """
-
-
-    disabledFG = '#888888'
-    """This colour is used as the foreground (text) colour for overlays
-    where their :attr:`.Display.enabled` property is ``False``.
-    """
-
-
-    unsavedDefaultBG = '#ffeeee'
-    """This colour is used as the default background colour for
-    :class:`.Image` overlays with an :attr:`.Image.saved` property
-    of ``False``.
-    """
-
-
-    unsavedSelectedBG = '#ffcdcd'
-    """This colour is used as the background colour for :class:`.Image`
-    overlays with an :attr:`.Image.saved` property of ``False``, when
-    they are selected in the :class:`OverlayListPanel`.
-    """
-
-
     def __init__(self,
                  parent,
                  overlay,
@@ -444,6 +428,27 @@ class ListItemWidget(wx.Panel):
         """
         wx.Panel.__init__(self, parent)
 
+        self.enabledFG  = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOWTEXT)
+        self.disabledFG = wx.SystemSettings.GetColour(wx.SYS_COLOUR_GRAYTEXT)
+
+        # give unsaved overlays reddish colours
+        isdark = False
+
+        # GetAppearance added in wxwidgets 3.1.3 (~=wxpython 4.1.0)
+        if hasattr(wx.SystemSettings, 'GetAppearance'):
+            isdark = wx.SystemSettings.GetAppearance().IsDark()
+        else:
+            # Guess whether we have a light or dark theme
+            colour = wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW).Get()
+            isdark = all(c < 128 for c in colour[:3])
+
+        if isdark:
+            self.unsavedDefaultBG  = '#443333'
+            self.unsavedSelectedBG = '#551111'
+        else:
+            self.unsavedDefaultBG  = '#ffeeee'
+            self.unsavedSelectedBG = '#ffcdcd'
+
         self.__overlay         = overlay
         self.__display         = display
         self.__displayCtx      = displayCtx
@@ -569,11 +574,11 @@ class ListItemWidget(wx.Panel):
         idx     = self.__listBox.IndexOf(self.__overlay)
         enabled = self.__visibility.GetValue()
 
-        with props.suppress(self.__display, 'enabled', self.__name):
+        with props.skip(self.__display, 'enabled', self.__name):
             self.__display.enabled = enabled
 
-        if enabled: fgColour = ListItemWidget.enabledFG
-        else:       fgColour = ListItemWidget.disabledFG
+        if enabled: fgColour = self.enabledFG
+        else:       fgColour = self.disabledFG
 
         self.__listBox.SetItemForegroundColour(idx, fgColour)
 
@@ -588,8 +593,8 @@ class ListItemWidget(wx.Panel):
 
         enabled = self.__display.enabled
 
-        if enabled: fgColour = ListItemWidget.enabledFG
-        else:       fgColour = ListItemWidget.disabledFG
+        if enabled: fgColour = self.enabledFG
+        else:       fgColour = self.disabledFG
 
         self.__visibility.SetValue(enabled)
         self.__listBox.SetItemForegroundColour(idx, fgColour)
@@ -640,8 +645,8 @@ class ListItemWidget(wx.Panel):
         else:
             self.__listBox.SetItemBackgroundColour(
                 idx,
-                ListItemWidget.unsavedDefaultBG,
-                ListItemWidget.unsavedSelectedBG),
+                self.unsavedDefaultBG,
+                self.unsavedSelectedBG),
 
         tooltip = self.__overlay.dataSource
         if tooltip is None:
