@@ -8,12 +8,13 @@
 *FSLeyes control* panel for controlling a :class:`.PowerSpectrumPanel`.
 """
 
+import wx
 
 import fsleyes_props                        as props
-
 import fsleyes.tooltips                     as fsltooltips
-import fsleyes.plotting.powerspectrumseries as powerspectrumseries
 import fsleyes.strings                      as strings
+import fsleyes.plotting.powerspectrumseries as powerspectrumseries
+import fsleyes.views.powerspectrumpanel     as powerspectrumpanel
 from . import plotcontrolpanel              as plotcontrol
 
 
@@ -22,6 +23,33 @@ class PowerSpectrumControlPanel(plotcontrol.PlotControlPanel):
     which allows the user to control a :class:`.PowerSpectrumPanel`.
     """
 
+
+    @staticmethod
+    def supportedViews():
+        """Overrides :meth:`.ControlMixin.supportedViews`. The
+        ``PowerSpectrumControlPanel`` is only intended to be added to
+        :class:`.PowerSpectrumPanel` views.
+        """
+        return [powerspectrumpanel.PowerSpectrumPanel]
+
+
+    @staticmethod
+    def supportSubClasses():
+        """Overrides :meth:`.ControlPanel.supportSubClasses`. Returns
+        ``False`` - the ``PowerSpectrumToolBar`` is only intended to be
+        used with the :class:`.PowerSpectrumPanel`.
+        """
+        return False
+
+
+    @staticmethod
+    def defaultLayout():
+        """Returns a dictionary containing layout settings to be passed to
+        :class:`.ViewPanel.togglePanel`.
+        """
+        return {'location' : wx.RIGHT}
+
+
     def __init__(self, *args, **kwargs):
         """Create a ``PowerSpectrumControlPanel``. All arguments are passed
         through to the :meth:`.PlotControlPanel.__init__` method.
@@ -29,7 +57,7 @@ class PowerSpectrumControlPanel(plotcontrol.PlotControlPanel):
 
         plotcontrol.PlotControlPanel.__init__(self, *args, **kwargs)
 
-        psPanel = self.getPlotPanel()
+        psPanel = self.plotPanel
         psPanel.addListener('plotMelodicICs',
                             self.name,
                             self.__plotMelodicICsChanged)
@@ -40,7 +68,7 @@ class PowerSpectrumControlPanel(plotcontrol.PlotControlPanel):
         longer needed. Removes some property listeners and calls the
         :meth:`.PlotControlPanel.destroy` method.
         """
-        psPanel = self.getPlotPanel()
+        psPanel = self.plotPanel
         psPanel.removeListener('plotMelodicICs', self.name)
         plotcontrol.PlotControlPanel.destroy(self)
 
@@ -51,7 +79,7 @@ class PowerSpectrumControlPanel(plotcontrol.PlotControlPanel):
         Adds some widgets for controlling the :class:`.PowerSpectrumPanel`.
         """
 
-        psPanel    = self.getPlotPanel()
+        psPanel    = self.plotPanel
         widgetList = self.getWidgetList()
         allWidgets = []
         psProps    = ['plotFrequencies',
@@ -72,26 +100,37 @@ class PowerSpectrumControlPanel(plotcontrol.PlotControlPanel):
         return allWidgets
 
 
-    def generateDataSeriesWidgets(self, ps, groupName):
+    def generateCustomDataSeriesWidgets(self, ps, groupName):
         """Overrides :meth:`.PlotControlPanel.generateDataSeriesWidgets`.
         Adds some widgets for controlling :class:`.PowerSpectrumSeries`
         instances.
         """
-        allWidgets = plotcontrol.PlotControlPanel.generateDataSeriesWidgets(
-            self, ps, groupName)
-
-        if not isinstance(ps, powerspectrumseries.PowerSpectrumSeries):
-            return
 
         widgetList = self.getWidgetList()
+        allWidgets = []
+
         varNorm    = props.makeWidget(widgetList, ps, 'varNorm')
 
         widgetList.AddWidget(varNorm,
                              displayName=strings.properties[ps, 'varNorm'],
                              tooltip=strings.properties[ps, 'varNorm'],
                              groupName=groupName)
+        allWidgets.append(varNorm)
 
-        return allWidgets + [varNorm]
+        if isinstance(ps, powerspectrumseries.ComplexPowerSpectrumSeries):
+            for propName in ['zeroOrderPhaseCorrection',
+                             'firstOrderPhaseCorrection',
+                             'plotReal', 'plotImaginary',
+                             'plotMagnitude', 'plotPhase']:
+                widg = props.makeWidget(widgetList, ps, propName)
+                widgetList.AddWidget(
+                    widg,
+                    displayName=strings.properties[ps, propName],
+                    tooltip=fsltooltips.properties[ps, propName],
+                    groupName=groupName)
+                allWidgets.append(widg)
+
+        return allWidgets
 
 
     def __plotMelodicICsChanged(self, *a):
