@@ -72,11 +72,12 @@ new one accordingly.  The following ``DisplayOpts`` sub-classes exist:
 .. autosummary::
    :nosignatures:
 
-   ~fsleyes.displaycontext.volumeopts.NiftiOpts
+   ~fsleyes.displaycontext.niftiopts.NiftiOpts
    ~fsleyes.displaycontext.volumeopts.VolumeOpts
+   ~fsleyes.displaycontext.volumeopts.VolumeRGBOpts
+   ~fsleyes.displaycontext.volumeopts.ComplexOpts
    ~fsleyes.displaycontext.volume3dopts.Volume3DOpts
    ~fsleyes.displaycontext.maskopts.MaskOpts
-   ~fsleyes.displaycontext.vectoropts.VectorOpts
    ~fsleyes.displaycontext.vectoropts.RGBVectorOpts
    ~fsleyes.displaycontext.vectoropts.LineVectorOpts
    ~fsleyes.displaycontext.meshopts.MeshOpts
@@ -85,6 +86,8 @@ new one accordingly.  The following ``DisplayOpts`` sub-classes exist:
    ~fsleyes.displaycontext.labelopts.LabelOpts
    ~fsleyes.displaycontext.tensoropts.TensorOpts
    ~fsleyes.displaycontext.shopts.SHOpts
+   ~fsleyes.displaycontext.mipopts.MIPOpts
+   ~fsleyes.displaycontext.tractogramopts.TractogramOpts
 
 
 --------------
@@ -143,6 +146,7 @@ define *scene* options:
 import itertools as it
 
 import fsleyes_widgets.utils.typedict as td
+import fsl.utils.path                 as fslpath
 import fsl.data.constants             as constants
 
 from .               import display
@@ -155,11 +159,14 @@ from .orthoopts      import OrthoOpts
 from .lightboxopts   import LightBoxOpts
 from .scene3dopts    import Scene3DOpts
 from .colourmapopts  import ColourMapOpts
-from .volumeopts     import NiftiOpts
+from .niftiopts      import NiftiOpts
 from .volumeopts     import VolumeOpts
+from .volumeopts     import ComplexOpts
+from .volumeopts     import VolumeRGBOpts
 from .volume3dopts   import Volume3DOpts
 from .maskopts       import MaskOpts
 from .vectoropts     import VectorOpts
+from .vectoropts     import NiftiVectorOpts
 from .vectoropts     import RGBVectorOpts
 from .vectoropts     import LineVectorOpts
 from .meshopts       import MeshOpts
@@ -168,16 +175,22 @@ from .freesurferopts import FreesurferOpts
 from .labelopts      import LabelOpts
 from .tensoropts     import TensorOpts
 from .shopts         import SHOpts
+from .mipopts        import MIPOpts
+from .tractogramopts import TractogramOpts
 
 from .displaycontext import InvalidOverlayError
 
 
 OVERLAY_TYPES = td.TypeDict({
 
+    'Bitmap'         : ['rgb', 'volume'],
     'Image'          : ['volume',     'mask',  'rgbvector',
-                        'linevector', 'label', 'sh', 'tensor'],
+                        'linevector', 'label', 'sh',
+                        'tensor',     'mip',   'rgb',
+                        'complex'],
     'Mesh'           : ['mesh'],
     'DTIFitTensor'   : ['tensor', 'rgbvector', 'linevector'],
+    'Tractogram'     : ['tractogram']
 })
 """This dictionary provides a mapping between all overlay classes,
 and the possible values that the :attr:`Display.overlayType` property
@@ -196,17 +209,21 @@ dictionary for more details.
 
 
 DISPLAY_OPTS_MAP = td.TypeDict({
-    'Nifti.volume'        : VolumeOpts,
-    'Nifti.rgbvector'     : RGBVectorOpts,
-    'Nifti.linevector'    : LineVectorOpts,
-    'Nifti.mask'          : MaskOpts,
-    'Nifti.label'         : LabelOpts,
-    'Nifti.tensor'        : TensorOpts,
-    'Nifti.sh'            : SHOpts,
-    'Mesh.mesh'           : MeshOpts,
-    'VTKMesh.mesh'        : MeshOpts,
-    'GiftiMesh.mesh'      : GiftiOpts,
-    'FreesurferMesh.mesh' : FreesurferOpts,
+    'Nifti.volume'          : VolumeOpts,
+    'Nifti.rgbvector'       : RGBVectorOpts,
+    'Nifti.linevector'      : LineVectorOpts,
+    'Nifti.mask'            : MaskOpts,
+    'Nifti.label'           : LabelOpts,
+    'Nifti.tensor'          : TensorOpts,
+    'Nifti.sh'              : SHOpts,
+    'Nifti.mip'             : MIPOpts,
+    'Nifti.rgb'             : VolumeRGBOpts,
+    'Nifti.complex'         : ComplexOpts,
+    'Mesh.mesh'             : MeshOpts,
+    'VTKMesh.mesh'          : MeshOpts,
+    'GiftiMesh.mesh'        : GiftiOpts,
+    'FreesurferMesh.mesh'   : FreesurferOpts,
+    'Tractogram.tractogram' : TractogramOpts,
 })
 """This dictionary provides a mapping between each (overlay type,
 :attr:`.Display.overlayType`) pair, and the :class:`DisplayOpts` subclass
@@ -220,25 +237,49 @@ def getOverlayTypes(overlay):
     might adjust the returned list based on properties of the overlay.
     """
 
-    import fsl.data.image as fslimage
-    from . import            shopts
+    import fsl.data.image  as fslimage
+    import fsl.data.bitmap as fslbmp
+    from . import             shopts
 
     possibleTypes = list(OVERLAY_TYPES[overlay])
 
     if not isinstance(overlay, fslimage.Image):
         return possibleTypes
 
-    shape = overlay.shape
+    shape     = overlay.shape
+    ndims     = len(shape)
+    nvals     = overlay.nvals
+    iscomplex = overlay.iscomplex
 
     # Could this image be a vector image?
-    couldBeVector = len(shape) == 4 and shape[-1] == 3
+    couldBeVector = ((ndims == 4 and shape[-1] == 3) or
+                     (ndims == 3 and nvals     == 3))
+
+    # Could this image be a RGB(A) image?
+    couldBeRGB = (nvals in (3, 4)) or (ndims == 4 and shape[-1] in (3, 4))
 
     # Or could it be a SH or tensor image?
-    couldBeTensor = len(shape) == 4 and shape[-1] == 6
-    couldBeSH     = len(shape) == 4 and shape[-1] in shopts.SH_COEFFICIENT_TYPE
+    couldBeTensor = ndims == 4 and shape[-1] == 6
+    couldBeSH     = ndims == 4 and shape[-1] in shopts.SH_COEFFICIENT_TYPE
+
+    # Special cases
+
+    # If the image is complex, make complex
+    # the default overlay type
+    possibleTypes.remove('complex')
+    if iscomplex:
+        possibleTypes.insert(0, 'complex')
+
+    # If the overlay looks like a 2D image
+    # (jpg, etc), make rgb the default type
+    if couldBeRGB:
+        if fslpath.hasExt(overlay.dataSource, fslbmp.BITMAP_EXTENSIONS):
+            # bitmaps are loaded as Images, so
+            # the OVERLAY_TYPES lookup above
+            # would have returned the types for
+            # Image overlays
+            possibleTypes = list(OVERLAY_TYPES['Bitmap'])
 
-    # Special cases:
-    #
     # If the overlay looks like a vector image,
     # and its nifti intent code is set as such,
     # make rgbvector the default overlay type
@@ -251,13 +292,16 @@ def getOverlayTypes(overlay):
 
     # Otherwise, remove the vector options
     else:
-
         try:               possibleTypes.remove('rgbvector')
         except ValueError: pass
 
         try:               possibleTypes.remove('linevector')
         except ValueError: pass
 
+    if not couldBeRGB:
+        try:               possibleTypes.remove('rgb')
+        except ValueError: pass
+
     if not couldBeSH:
         try:               possibleTypes.remove('sh')
         except ValueError: pass
