@@ -11,15 +11,16 @@
 
 import logging
 
-import fsleyes_props      as props
-import fsleyes.actions    as actions
-import fsleyes.colourmaps as fslcm
+import fsl.transform.affine as affine
+import fsleyes_props        as props
+import fsleyes.actions      as actions
+import fsleyes.colourmaps   as fslcm
 
 
 log = logging.getLogger(__name__)
 
 
-class ColourMapOpts(object):
+class ColourMapOpts:
     """The ``ColourMapOpts`` class is a mixin for use with
     :class:`.DisplayOpts` sub-classes. It provides properties and logic
     for displaying overlays which are coloured according to some data values.
@@ -48,6 +49,8 @@ class ColourMapOpts(object):
       4. Call :meth:`updateDataRange` whenever the data driving the colouring
          changes.
 
+      5. Call :meth:`destroy` when your instance is being destroyed.
+
 
     The ``ColourMapOpts`` class links the :attr:`.Display.brightness` and
     :attr:`.Display.contrast` properties to its own :attr:`displayRange`
@@ -106,7 +109,7 @@ class ColourMapOpts(object):
     """
 
 
-    cmap = props.ColourMap()
+    cmap = props.ColourMap(prefix='fsleyes_')
     """The colour map, a :class:`matplotlib.colors.Colourmap` instance."""
 
 
@@ -114,8 +117,6 @@ class ColourMapOpts(object):
     """Gamma correction factor - exponentially weights the :attr:`cmap`
     and :attr:`negCmap` towards the low or high ends.
 
-    Ignored if :attr:`logScale` is enabled.
-
     This property takes values between -1 and +1. The exponential weight
     that should actually be used to apply gamma correction should be derived
     as follows:
@@ -148,7 +149,7 @@ class ColourMapOpts(object):
     """
 
 
-    negativeCmap = props.ColourMap()
+    negativeCmap = props.ColourMap(prefix='fsleyes_')
     """A second colour map, used if :attr:`useNegativeCmap` is ``True``.
     When active, the :attr:`cmap` is used to colour positive values, and
     the :attr:`negativeCmap` is used to colour negative values.
@@ -196,7 +197,15 @@ class ColourMapOpts(object):
     """
 
 
-    modulateRange = props.Bounds(ndims=1)
+    invertModulateAlpha = props.Boolean(default=False)
+    """If ``True``, the effect of :attr:`modulateAlpha` is inverted. Regions
+    with a value near to the low :attr:`modulateRange` will have an alpha
+    near 1, and regions with a value near to the high :attr:`modulateRange`
+    will have an alpha near 0.
+    """
+
+
+    modulateRange = props.Bounds(ndims=1, clamped=False)
     """Range used to determine how much to modulate :attr:`.Display.alpha`
     by, when :attr:`modulateAlpha` is active.
     """
@@ -220,7 +229,7 @@ class ColourMapOpts(object):
 
 
     def __init__(self):
-        """Create a ``ColourMapOpts`` instance. This must be called
+        """Initialise a ``ColourMapOpts`` instance. This must be called
         *after* the :meth:`.DisplayOpts.__init__` method.
         """
 
@@ -383,7 +392,7 @@ class ColourMapOpts(object):
         range is always the same as the data range, this method does not
         need to be overridden.
 
-        Otherwise, if the modulate ange differs from the data range (see
+        Otherwise, if the modulate range differs from the data range (see
         e.g. the :attr:`.VolumeOpts.modulateImage` property), this method must
         return the modulate range as a ``(min, max)`` tuple.
 
@@ -394,6 +403,32 @@ class ColourMapOpts(object):
         return None
 
 
+    def modulateScaleOffset(self, xform=False):
+        """Returns a scale and offset which can be used transforms a value
+        from the data range into a value between 0 and 1, according to the
+        current :attr:`modulateRange` and :attr:`invertModulateAlpha`
+        settings.
+
+        If ``xform is True``, a 4x4 affine containing the scale and offset
+        values is returned.
+        """
+
+        modlo, modhi = self.modulateRange
+        mrange       = modhi - modlo
+        if mrange == 0:
+            scale  = 1
+            offset = 0
+        elif not self.invertModulateAlpha:
+            scale  = 1      / mrange
+            offset = -modlo / mrange
+        else:
+            scale  = -1    / mrange
+            offset = modhi / mrange
+
+        if xform: return affine.scaleOffsetXform(scale, offset)
+        else:     return scale, offset
+
+
     @actions.action
     def resetDisplayRange(self):
         """Resets the :attr:`displayRange`, :attr:`clippingRange`, and
@@ -446,6 +481,14 @@ class ColourMapOpts(object):
         if modRange  is not None: mrmin, mrmax = modRange
         else:                     mrmin, mrmax = drmin, drmax
 
+        # If the display and clipping ranges differ,
+        # it probably makes sense to de-couple them
+        if (crmin, crmax) != (drmin, drmax):
+            if self.propertyIsEnabled('linkLowRanges'):
+                self.linkLowRanges  = False
+            if self.propertyIsEnabled('linkHighRanges'):
+                self.linkHighRanges = False
+
         # Clipping works on >= and <=, so we add
         # a small offset to the display range limits
         # (which are equal to the clipping limiits)
