@@ -12,8 +12,6 @@ which encapsulate overlay display settings.
 import logging
 import inspect
 
-import deprecation
-
 import fsl.data.image                 as fslimage
 import fsl.data.constants             as constants
 import fsleyes_props                  as props
@@ -74,7 +72,7 @@ class Display(props.SyncableHasProperties):
                  overlayList,
                  displayCtx,
                  parent=None,
-                 overlayType=None):
+                 **kwa):
         """Create a :class:`Display` for the specified overlay.
 
         :arg overlay:     The overlay object.
@@ -88,10 +86,17 @@ class Display(props.SyncableHasProperties):
         :arg parent:      A parent ``Display`` instance - see
                           :mod:`props.syncable`.
 
-        :arg overlayType: Initial overlay type - see the :attr:`overlayType`
-                          property.
+        All other keyword arguments are assumed to be ``name=value`` pairs,
+        containing initial property values, for both this ``Display``, and
+        the initially created :class:`DisplayOpts` instance. For the latter,
+        it is assumed that any properties specified are appropriate for the
+        initial overlay type.
         """
 
+        dispProps     = self.getAllProperties()[0]
+        initDispProps = {n : v for n, v in kwa.items() if n     in dispProps}
+        initOptProps  = {n : v for n, v in kwa.items() if n not in dispProps}
+
         self.__overlay     = overlay
         self.__overlayList = overlayList
         self.__displayCtx  = displayCtx
@@ -107,11 +112,6 @@ class Display(props.SyncableHasProperties):
         log.debug('Enabling overlay types for {}: '.format(overlay, ovlTypes))
         ovlTypeProp.setChoices(ovlTypes, instance=self)
 
-        # Override the default overlay
-        # type if it has been specified
-        if overlayType is not None:
-            self.overlayType = overlayType
-
         # Call the super constructor after our own
         # initialisation, in case the provided parent
         # has different property values to our own,
@@ -130,7 +130,10 @@ class Display(props.SyncableHasProperties):
             # Initial sync state between this
             # Display and the parent Display
             # (if this Display has a parent)
-            state=displayCtx.syncOverlayDisplay)
+            state=displayCtx.syncOverlayDisplay,
+
+            # set initial display property values
+            **initDispProps)
 
         # When the overlay type changes, the property
         # values of the DisplayOpts instance for the
@@ -148,7 +151,13 @@ class Display(props.SyncableHasProperties):
         # LabelOpts) - the values of all common
         # properties are copied to the new
         # DisplayOpts instance.
-        self.__oldOptValues = td.TypeDict()
+        self.__oldOptProps = td.TypeDict()
+
+        # Initial DisplayOpt property values
+        # are used in the first call to
+        # __makeDisplayOpts, and then cleared
+        # afterwards.
+        self.__initOptProps = initOptProps
 
         # Set up listeners after caling Syncable.__init__,
         # so the callbacks don't get called during
@@ -177,20 +186,6 @@ class Display(props.SyncableHasProperties):
             log.debug('{}.del ({})'.format(type(self).__name__, id(self)))
 
 
-    @deprecation.deprecated(deprecated_in='0.14.3',
-                            removed_in='1.0.0',
-                            details='Use overlay instead')
-    def getOverlay(self):
-        """Deprecated - use :meth:`overlay` instead."""
-        return self.__overlay
-
-
-    @property
-    def overlay(self):
-        """Returns the overlay associated with this ``Display`` instance."""
-        return self.__overlay
-
-
     def destroy(self):
         """This method must be called when this ``Display`` instance
         is no longer needed.
@@ -206,13 +201,22 @@ class Display(props.SyncableHasProperties):
 
         self.detachAllFromParent()
 
-        self.__displayOpts = None
-        self.__overlayList = None
-        self.__displayCtx  = None
-        self.__overlay     = None
+        self.__oldOptProps  = None
+        self.__initOptProps = None
+        self.__displayOpts  = None
+        self.__overlayList  = None
+        self.__displayCtx   = None
+        self.__overlay      = None
 
 
-    def getDisplayOpts(self):
+    @property
+    def overlay(self):
+        """Returns the overlay associated with this ``Display`` instance."""
+        return self.__overlay
+
+
+    @property
+    def opts(self):
         """Return the :class:`.DisplayOpts` instance associated with this
         ``Display``, which contains overlay type specific display settings.
 
@@ -241,30 +245,47 @@ class Display(props.SyncableHasProperties):
         value of the :attr:`overlayType` property.
 
         The :data:`.displaycontext.DISPLAY_OPTS_MAP` dictionary defines the
-        mapping between :attr:`overlayType` values, and ``DisplayOpts``
-        sub-class types.
+        mapping between overlay types and :attr:`overlayType` values, and
+        ``DisplayOpts`` sub-class types.
         """
 
         if self.getParent() is None:
             oParent = None
         else:
-            oParent = self.getParent().getDisplayOpts()
+            oParent = self.getParent().opts
+
+        initOptProps        = self.__initOptProps
+        self.__initOptProps = None
+
+        if initOptProps is None:
+            initOptProps = {}
 
         from . import DISPLAY_OPTS_MAP
 
-        optType = DISPLAY_OPTS_MAP[self.overlayType]
+        optType = DISPLAY_OPTS_MAP[self.__overlay, self.overlayType]
 
         log.debug('Creating {} instance (synced: {}) for overlay '
                   '{} ({})'.format(optType.__name__,
                                    self.__displayCtx.syncOverlayDisplay,
                                    self.__overlay, self.overlayType))
 
+        volProps  = optType.getVolumeProps()
+        allProps  = optType.getAllProperties()[0]
+        initState = {}
+
+        for p in allProps:
+            if p in volProps:
+                initState[p] = self.__displayCtx.syncOverlayVolume
+            else:
+                initState[p] = self.__displayCtx.syncOverlayDisplay
+
         return optType(self.__overlay,
                        self,
                        self.__overlayList,
                        self.__displayCtx,
                        parent=oParent,
-                       state=self.__displayCtx.syncOverlayDisplay)
+                       state=initState,
+                       **initOptProps)
 
 
     def __findOptBaseType(self, optType, optName):
@@ -310,7 +331,7 @@ class Display(props.SyncableHasProperties):
             log.debug('Saving {}.{} = {} [{} {}]'.format(
                 base, propName, val, type(opts).__name__, id(self)))
 
-            self.__oldOptValues[base, propName] = val
+            self.__oldOptProps[base, propName] = val
 
 
     def __restoreOldDisplayOpts(self):
@@ -325,11 +346,14 @@ class Display(props.SyncableHasProperties):
         for propName in opts.getAllProperties()[0]:
 
             try:
-                value = self.__oldOptValues[opts, propName]
+                value = self.__oldOptProps[opts, propName]
 
                 if not hasattr(opts, propName):
                     continue
 
+                if not opts.propertyIsEnabled(propName):
+                    continue
+
                 log.debug('Restoring {}.{} = {} [{}]'.format(
                     type(opts).__name__, propName, value, id(self)))
 
@@ -344,7 +368,7 @@ class Display(props.SyncableHasProperties):
         that the :class:`DisplayOpts` instance is of the correct type.
         """
         self.__saveOldDisplayOpts()
-        self.getDisplayOpts()
+        self.opts
         self.__restoreOldDisplayOpts()
 
 
@@ -376,6 +400,11 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
                     instances.
     ``name``        A unique name for this ``DisplayOpts`` instance.
     =============== ======================================================
+
+
+    .. warning:: :class:`DisplayOpts` sub-classes must not define any
+                 properties with the same name as any of the :class:`Display`
+                 properties.
     """
 
 
@@ -413,13 +442,11 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
 
         self.__overlay     = overlay
         self.__display     = display
-        self.__overlayList = overlayList
-        self.__displayCtx  = displayCtx
         self.__overlayType = display.overlayType
         self.__name        = '{}_{}'.format(type(self).__name__, id(self))
 
         props.SyncableHasProperties.__init__(self, **kwargs)
-        actions.ActionProvider     .__init__(self)
+        actions.ActionProvider     .__init__(self, overlayList, displayCtx)
 
         log.debug('{}.init [DC: {}] ({})'.format(
             type(self).__name__, id(displayCtx), id(self)))
@@ -431,6 +458,31 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
             log.debug('{}.del ({})'.format(type(self).__name__, id(self)))
 
 
+    def destroy(self):
+        """This method must be called when this ``DisplayOpts`` instance
+        is no longer needed.
+
+        If a subclass overrides this method, the subclass implementation
+        must call this method, **after** performing its own clean up.
+        """
+        actions.ActionProvider.destroy(self)
+
+        self.detachAllFromParent()
+
+        self.__overlay = None
+        self.__display = None
+
+
+    @classmethod
+    def getVolumeProps(cls):
+        """Intended to be overridden by sub-classes as needed.  Returns a list
+        of property names which control the currently displayed
+        volume/timepoint for 4D overlays. The default implementation returns
+        an empty list.
+        """
+        return []
+
+
     @property
     def overlay(self):
         """Return the overlay associated with this ``DisplayOpts`` object.
@@ -446,21 +498,6 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
         return self.__display
 
 
-    @property
-    def overlayList(self):
-        """Return the :class:`.OverlayList` that contains all overlays.
-        """
-        return self.__overlayList
-
-
-    @property
-    def displayCtx(self):
-        """Return the :class:`.DisplayContext` that is managing this
-        ``DisplayOpts`` object.
-        """
-        return self.__displayCtx
-
-
     @property
     def overlayType(self):
         """Return the type of this ``DisplayOpts`` object (the value of
@@ -475,28 +512,12 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
         return self.__name
 
 
-    def destroy(self):
-        """This method must be called when this ``DisplayOpts`` instance
-        is no longer needed.
-
-        If a subclass overrides this method, the subclass implementation
-        must call this method, **after** performing its own clean up.
-        """
-        actions.ActionProvider.destroy(self)
-
-        self.detachAllFromParent()
-
-        self.__overlay     = None
-        self.__display     = None
-        self.__overlayList = None
-        self.__displayCtx  = None
-
-
-    def getReferenceImage(self):
+    @property
+    def referenceImage(self):
         """Return the reference image associated with this ``DisplayOpts``
         instance.
 
-        Some non-volumetric overlay types (e.g. the :class:`.TriangleMesh` -
+        Some non-volumetric overlay types (e.g. the :class:`.Mesh` -
         see :class:`.MeshOpts`) may have a *reference* :class:`.Nifti` instance
         associated with them, allowing the overlay to be localised in the
         coordinate space defined by the :class:`.Nifti`. The
@@ -534,25 +555,21 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
             :meth:`.Image.getOrientation`)
         """
 
-        refImage = self.getReferenceImage()
+        refImage = self.referenceImage
 
+        # No reference image (e.g. an unassociated mesh)
         if refImage is None:
             return ('??????', [constants.ORIENT_UNKNOWN] * 3)
 
-        opts = self.displayCtx.getOpts(refImage)
-
+        dctx    = self.displayCtx
+        opts    = dctx.getOpts(refImage)
         xorient = None
         yorient = None
         zorient = None
 
-        # If we are displaying in voxels/scaled voxels,
-        # and this image is not the current display
-        # image, then we do not show anatomical
-        # orientation labels, as there's no guarantee
-        # that all of the loaded overlays are in the
-        # same orientation, and it can get confusing.
-        if opts.transform in ('id', 'pixdim', 'pixdim-flip') and \
-           self.displayCtx.displaySpace != refImage:
+        # Scaled voxels - display data orientation
+        # labels instead of anatomical labels
+        if dctx.displaySpace == 'scaledVoxel':
             xlo = 'Xmin'
             xhi = 'Xmax'
             ylo = 'Ymin'
@@ -560,12 +577,12 @@ class DisplayOpts(props.SyncableHasProperties, actions.ActionProvider):
             zlo = 'Zmin'
             zhi = 'Zmax'
 
-        # Otherwise we assume that all images
-        # are aligned to each other, so we
-        # estimate the current image's orientation
-        # in the display coordinate system
+        # Otherwise we display anatomical orientation
+        # labels for the currently selected overlay
+        # in the current display coordinate system
         else:
-
+            # Estimate a mapping between each axis of
+            # the display and world coordinate systems
             xform   = opts.getTransform('display', 'world')
             xorient = refImage.getOrientation(0, xform)
             yorient = refImage.getOrientation(1, xform)
