@@ -10,16 +10,20 @@ general display settings for displaying the overlays in a
 """
 
 
+import os.path as op
 import sys
 import logging
 import contextlib
+from collections import defaultdict
 
 import numpy        as np
 import numpy.linalg as npla
 
-import        fsl.data.image as fslimage
-import        fsleyes_props  as props
-from . import group          as dcgroup
+import fsl.data.image               as fslimage
+import fsl.utils.path               as fslpath
+import fsleyes_props                as props
+import fsleyes.data                 as dsutils
+import fsleyes.displaycontext.group as dcgroup
 
 
 log = logging.getLogger(__name__)
@@ -56,8 +60,6 @@ class DisplayContext(props.SyncableHasProperties):
         getDisplay
         getOpts
         getReferenceImage
-        displayToWorld
-        worldToDisplay
         displaySpaceIsRadiological
         selectOverlay
         getSelectedOverlay
@@ -128,8 +130,15 @@ class DisplayContext(props.SyncableHasProperties):
     a group of overlays which share display properties.
 
     By default there is one overlay group, to which all new overlays are
-    initially added. The :class:`.OverlayListPanel` allows the user to
-    add/remove overlays from this default group.
+    initially added (unless :attr:`groupOverlays.` is ``False``). The
+    :class:`.OverlayListPanel` allows the user to add/remove overlays from
+    this default group.
+    """
+
+
+    groupOverlays = props.Boolean(default=True)
+    """Boolean which controls whether all newly added overlays are added
+    to the deafult :attr:`overlayGroup`.
     """
 
 
@@ -165,12 +174,12 @@ class DisplayContext(props.SyncableHasProperties):
     """
 
 
-    displaySpace = props.Choice(('world', ))
+    displaySpace = props.Choice(('world', 'scaledVoxel', 'fslview'))
     """The *space* in which overlays are displayed. This property defines the
     display coordinate system for this ``DisplayContext``. When it is changed,
     the :attr:`.NiftiOpts.transform` property of all :class:`.Nifti` overlays
-    in the :class:`.OverlayList` is updated. It has two settings, described
-    below. The options for this property are dynamically added by
+    in the :class:`.OverlayList` is updated. It can be set to one of the values
+    described below. The options for this property are dynamically added by
     :meth:`__updateDisplaySpaceOptions`.
 
     1. **World** space (a.k.a. ``'world'``)
@@ -179,7 +188,24 @@ class DisplayContext(props.SyncableHasProperties):
        their affine transformation matrix - the :attr:`.NiftiOpts.transform`
        property for every ``Nifti`` overlay is set to ``affine``.
 
-    2. **Reference image** space
+    2. **Scaled-voxel** space (a.k.a. ``'scaledVoxel'``)
+
+       All :class:`.Nifti` overlays are displayed in a scaled-voxel coordinate
+       system, with origin set to the centre of voxel ``(0, 0, 0)``, and
+       voxels scaled by image pixdims. This is accomplished by setting the
+       :attr:`.NiftiOpts.transform` property for every ``Nifti`` overlay to
+       ``pixdim``.
+
+    3. **FSLView** space (a.k.a. ``'fslview'``)
+
+       Equivalent to ``'scaledVoxel'``, but with a L/R flip applied to all
+       ``Nifti`` overlays which have a voxel-to-world transformation with
+       a positive determinant. The flip is applied to the first voxel axis,
+       which is assumed to correspond to left-right. This is accomplished by
+       setting the :attr:`.NiftiOpts.transform` property for every ``Nifti``
+       overlay to ``pixdim-flip``.
+
+    4. **Reference image** space
 
        A single :class:`.Nifti` overlay is selected as a *reference* image,
        and is displayed in scaled voxel space (with a potential L/R flip for
@@ -197,6 +223,14 @@ class DisplayContext(props.SyncableHasProperties):
 
     The :meth:`defaultDisplaySpace` can be used to control how the
     ``displaySpace`` is initialised.
+
+    ..note:: This setting updates the :attr:`.NiftiOpts.transform` property for
+             every :class:`.Nifti` overlay in the :class:`.OverlayList`.  It is
+             possible to modify the ``NiftiOpts.transform`` property for
+             individual overlays independently of the ``displaySpace``, but
+             doing so is likely to break assumptions regarding image
+             orientation (e.g. radiological vs neurological display, and
+             anatomical labels in the ortho view).
     """
 
 
@@ -237,7 +271,42 @@ class DisplayContext(props.SyncableHasProperties):
     """
 
 
-    def __init__(self, overlayList, parent=None, defaultDs='ref', **kwargs):
+    showOrientationWarnings = props.Boolean(default=True)
+    """If ``False``, do not display warnings when displayed anatomical
+    labels may be incorrect.
+    """
+
+
+    autoNameOverlays = props.Boolean(default=False)
+    """Defaults to ``False``. If ``True``, overlays may be automatically
+    renamed (their :attr:`.Display.name` property changed) to prevent
+    duplicates.
+
+    Only on-disk overlays are renamed. Note that the names of manually
+    renamed overlays will be  overwritten when this setting is enabled.
+    """
+
+
+    preserveSelectedOverlay = props.Boolean(default=False)
+    """Whether or not to preserve the currently selected overlay when the
+    :class:`.OverlayList` changes.
+
+    As the :attr:`selectedOverlay` is an integer index, changes to the
+    :class:`.OverlayList` may change the overlay to which it refers. If this
+    property is ``True``, the selected overlay is preserved on
+    :class:`.OverlayList` changes.
+
+    This property is not to be used externally - use the
+    :meth:`preserveSelection` method instead.
+    """
+
+
+    def __init__(self,
+                 overlayList,
+                 parent=None,
+                 defaultDs='ref',
+                 displayType=None,
+                 **kwargs):
         """Create a ``DisplayContext``.
 
         :arg overlayList: An :class:`.OverlayList` instance.
@@ -253,14 +322,19 @@ class DisplayContext(props.SyncableHasProperties):
                           :class:`.Nifti` overlay. Otherwise (``'world'``),
                           the display space will be set to ``'world'``.
 
+        :arg displayType: String describing how this DisplayContext is going
+                          to be used. This is not used internally, but may be
+                          interrogated by :class:`.Display` and
+                          :class:`.DisplayOpts` instances. Can be accessed
+                          through the :meth:`displayType` property.
+
         All other arguments are passed through to the ``SyncableHasProperties``
         constructor, in addition to the following:
 
-          - The ``syncOverlayDisplay``,, ``syncOverlayVolume``, ``location``,
-            and ``bounds`` properties are added to the ``nobind`` argument
-
-          - The ``overlayGroups``, ``autoDisplay`` and ``loadInMemory``
-            properties are added to the ``nounbind`` argument.
+         - The ``syncOverlayDisplay``,, ``syncOverlayVolume``, ``location``,
+           and ``bounds`` properties are added to the ``nobind`` argument -
+           The ``overlayGroups``, ``groupOverlays``, ``autoDisplay`` and
+           ``loadInMemory`` properties are added to the ``nounbind`` argument.
         """
 
         kwargs = dict(kwargs)
@@ -273,6 +347,8 @@ class DisplayContext(props.SyncableHasProperties):
                          'location',
                          'bounds'])
         nounbind.extend(['overlayGroups',
+                         'groupOverlays',
+                         'preserveSelectedOverlay',
                          'autoDisplay',
                          'loadInMemory'])
 
@@ -284,7 +360,8 @@ class DisplayContext(props.SyncableHasProperties):
         props.SyncableHasProperties.__init__(self, **kwargs)
 
         self.__overlayList = overlayList
-        self.__name        = '{}_{}'.format(self.__class__.__name__, id(self))
+        self.__displayType = displayType
+        self.__name        = f'{self.__class__.__name__}_{id(self)}'
         self.__child       = parent is not None
 
         # When the first overlay(s) is/are
@@ -302,21 +379,23 @@ class DisplayContext(props.SyncableHasProperties):
 
         # If this is the first child DC, we
         # need to initialise the display
-        # space and location. If there is
-        # already a child DC, then we have
-        # (probably) inherited initial
-        # settings.
+        # location. If there is already a
+        # child DC, then we have (probably)
+        # inherited initial settings.
         if self.__child:
-            self.__initDS = (len(parent.getChildren()) - 1) == 0
+            self.__initDisplay = (len(parent.getChildren()) - 1) == 0
 
 
         # While the DisplayContext may refer to
         # multiple overlay groups, we are currently
         # using just one, allowing the user to specify
         # a set of overlays for which their display
-        # properties are 'locked'.
+        # properties are 'linked'. See the
+        # fsleyes.displaycontext.group module.
         if not self.__child:
-            lockGroup = dcgroup.OverlayGroup(self, overlayList)
+            if self.groupOverlays: group = overlayList
+            else:                  group = []
+            lockGroup = dcgroup.OverlayGroup(self, group)
             self.overlayGroups.append(lockGroup)
 
         # This dict contains the Display
@@ -354,7 +433,7 @@ class DisplayContext(props.SyncableHasProperties):
         # is important - check it out
         self.__overlayListChanged()
 
-        log.debug('{}.init ({})'.format(type(self).__name__, id(self)))
+        log.debug('%s.init (%s)', type(self).__name__, id(self))
 
 
     def __del__(self):
@@ -410,6 +489,19 @@ class DisplayContext(props.SyncableHasProperties):
         return self.__overlayList is None
 
 
+    @property
+    def overlayList(self):
+        """Return the :class:`.OverlayList`. """
+        return self.__overlayList
+
+
+    @property
+    def displayType(self):
+        """Returns the ``displayType`` that was passed to :meth:`__init__`.
+        """
+        return self.__displayType
+
+
     def getDisplay(self, overlay, **kwargs):
         """Returns the :class:`.Display` instance for the specified overlay
         (or overlay index).
@@ -495,50 +587,6 @@ class DisplayContext(props.SyncableHasProperties):
         return self.getOpts(overlay).referenceImage
 
 
-    def displayToWorld(self, dloc, *args, **kwargs):
-        """Transforms the given coordinates from the display coordinate
-        system into the world coordinate system.
-
-        .. warning:: If any :attr:`.NiftiOpts.transform` properties have
-                     been modified manually, this method will return invalid
-                     results.
-
-        All other arguments are passed to the
-        :meth:`.NiftiOpts.transformCoords` method.
-        """
-
-        displaySpace = self.displaySpace
-
-        if displaySpace == 'world' or len(self.__overlayList) == 0:
-            return dloc
-
-        opts = self.getOpts(displaySpace)
-
-        return opts.transformCoords(dloc, 'display', 'world', *args, **kwargs)
-
-
-    def worldToDisplay(self, wloc, *args, **kwargs):
-        """Transforms the given coordinates from the world coordinate
-        system into the display coordinate system.
-
-        .. warning:: If any :attr:`.NiftiOpts.transform` properties have
-                     been modified manually, this method will return invalid
-                     results.
-
-        All other arguments are passed to the
-        :meth:`.NiftiOpts.transformCoords` method.
-        """
-
-        displaySpace = self.displaySpace
-
-        if displaySpace == 'world' or len(self.__overlayList) == 0:
-            return wloc
-
-        opts = self.getOpts(displaySpace)
-
-        return opts.transformCoords(wloc, 'world', 'display', *args, **kwargs)
-
-
     def displaySpaceIsRadiological(self):
         """Returns ``True`` if the current :attr:`displaySpace` aligns with
         a radiological orientation. A radiological orientation is one in
@@ -552,21 +600,43 @@ class DisplayContext(props.SyncableHasProperties):
         if len(self.__overlayList) == 0:
             return True
 
+        opts  = None
         space = self.displaySpace
 
-        # Display space is either 'world', or an image.
-        # We assume that 'world' is an RAS coordinate
-        # system which, if transferred directly to a
-        # display coordinate system, would result in a
-        # neurological view (left on left, right on
-        # right).
+        # Display space is one of 'world', 'scaledVoxel',
+        # 'fslview', or a reference image.  We assume
+        # that 'world' is an RAS coordinate system which,
+        # if transferred directly to a display coordinate
+        # system, would result in a neurological view
+        # (left on left, right on right).
         if space == 'world':
             return False
-        else:
-            opts  = self.getOpts(space)
-            xform = opts.getTransform('pixdim-flip', 'display')
 
-            return npla.det(xform) > 0
+        # If space == scaledVoxel or fslview, we decide
+        # based on the currently selected overlay.
+        # Otherwise (reference image), we decide based on
+        # the ref image.
+        elif space == 'scaledVoxel':
+            space    = self.getSelectedOverlay()
+            srcSpace = 'pixdim'
+        elif space == 'fslview':
+            space    = self.getSelectedOverlay()
+            srcSpace = 'pixdim-flip'
+        else:
+            srcSpace = 'pixdim-flip'
+
+        # Use the FSL / FLIRT convention - if the affine
+        # determinant is negative, assume neurological
+        # storage order.
+        if space is not None:
+            ref = self.getOpts(space).referenceImage
+            if ref is not None:
+                opts  = self.getOpts(space)
+                xform = opts.getTransform(srcSpace, 'display')
+                return npla.det(xform) > 0
+
+        # no nifti overlays loaded
+        return False
 
 
     def selectOverlay(self, overlay):
@@ -589,6 +659,25 @@ class DisplayContext(props.SyncableHasProperties):
         return self.__overlayList[self.selectedOverlay]
 
 
+    @contextlib.contextmanager
+    def preserveSelection(self):
+        """Context manager to be used while changing the :class:`.OverlayList`.
+        Preserves the currently selected overlay if possible.
+        Note that this is applied to *all* ``DisplayContext`` instances,
+        not just the one on which this method is called.
+        """
+
+        # Guard against re-entrancy
+        needChange = not self.preserveSelectedOverlay
+        try:
+            if needChange:
+                self.preserveSelectedOverlay = True
+            yield
+        finally:
+            if needChange:
+                self.preserveSelectedOverlay = False
+
+
     def getOverlayOrder(self, overlay):
         """Returns the order in which the given overlay (or an index into
         the :class:`.OverlayList` list) should be displayed
@@ -691,10 +780,11 @@ class DisplayContext(props.SyncableHasProperties):
     def defaultDisplaySpace(self, ds):
         """Sets the :meth:`defaultDisplaySpace`.
 
-        :arg ds: Either ``'ref'`` or ``'world'``.
+        :arg ds: Either ``'ref'``, ``'scaledVoxel'``, ``'fslview'`` or
+        ``'world'``.
         """
-        if ds not in ('world', 'ref'):
-            raise ValueError('Invalid default display space: {}'.format(ds))
+        if ds not in ('world', 'scaledVoxel', 'ref', 'fslview'):
+            raise ValueError(f'Invalid default display space: {ds}')
         self.__defaultDisplaySpace = ds
 
 
@@ -793,11 +883,16 @@ class DisplayContext(props.SyncableHasProperties):
                         opts.detachFromParent('transform')
 
             # All new overlays are initially
-            # added to the default overlay group.
+            # added to the default overlay group
+            # (unless the groupOverlays property
+            # is False).
             # This only needs to be done on the
             # master DC, as the overlayGroups are
             # the same across all DCs.
-            if new and not self.__child and len(self.overlayGroups) > 0:
+            if new                and \
+               not self.__child   and \
+               self.groupOverlays and \
+               len(self.overlayGroups) > 0:
                 self.overlayGroups[0].addOverlay(overlay)
 
         # Ensure that the displaySpace
@@ -805,6 +900,10 @@ class DisplayContext(props.SyncableHasProperties):
         # with the overlay list.
         self.__updateDisplaySpaceOptions()
 
+        # Make sure each overlay has a unique name
+        if self.autoNameOverlays:
+            self.__renameOverlays()
+
         # The rest of the stuff only
         # needs to be done on child DCs
         if not self.__child:
@@ -815,6 +914,7 @@ class DisplayContext(props.SyncableHasProperties):
         # than len(overlayList)-1. selectedOverlay
         # is always synchronised, so we only
         # need to do this on the parent DC.
+        oldSelIdx = self.selectedOverlay
         nOverlays = len(self.__overlayList)
         if nOverlays > 0:
             self.setAttribute('selectedOverlay',
@@ -823,52 +923,36 @@ class DisplayContext(props.SyncableHasProperties):
         else:
             self.setAttribute('selectedOverlay', 'maxval', 0)
 
+        # Preserve the selected overlay if needed
+        if self.preserveSelectedOverlay:
+            oldList = self.__overlayList.overlays.getLast()
+            if oldSelIdx < len(oldList):
+                selOvl = oldList[oldSelIdx]
+                if selOvl in self.__overlayList:
+                    self.selectOverlay(selOvl)
+
         # Ensure that the overlayOrder
         # property is valid
         self.__syncOverlayOrder()
 
-        # If the overlay list was empty,
-        # and is now non-empty, we need
-        # to initialise the display space
-        # and the display location
-        initDS        = self.__initDS                      and \
-                        np.all(np.isclose(self.bounds, 0)) and \
-                        len(self.__overlayList) > 0
-        self.__initDS = len(self.__overlayList) == 0
-
-        # Initialise the display space. We
-        # have to do this before updating
-        # image transforms, and updating
-        # the display bounds
-        if initDS:
-
-            displaySpace = 'world'
-
-            if self.defaultDisplaySpace == 'ref':
-                for overlay in self.__overlayList:
-                    if isinstance(overlay, fslimage.Nifti):
-                        displaySpace = overlay
-                        break
-
-            with props.skip(self, 'displaySpace', self.__name):
-                self.displaySpace = displaySpace
-
         # Initialise the transform property
         # of any Image overlays which have
-        # just been added to the list,
-        oldList = self.__overlayList.getLastValue('overlays')[:]
-        for overlay in self.__overlayList:
-            if isinstance(overlay, fslimage.Nifti) and \
-               (overlay not in oldList):
-                self.__setTransform(overlay)
+        # just been added to the list, and
+        # ensure that the display bounds
+        # are up to date - displaySpaceChanged
+        # performs both of these steps.
+        self.__displaySpaceChanged()
 
-        # Ensure that the bounds
-        # property is accurate
-        self.__updateBounds()
+        # If the overlay list was empty,
+        # and is now non-empty, we need
+        # to initialise the display location
+        initDisplay        = self.__initDisplay and \
+                             len(self.__overlayList) > 0
+        self.__initDisplay = len(self.__overlayList) == 0
 
         # Initialise the display location to
         # the centre of the display bounds
-        if initDS:
+        if initDisplay:
             b = self.bounds
             self.location.xyz = [
                 b.xlo + b.xlen / 2.0,
@@ -887,15 +971,70 @@ class DisplayContext(props.SyncableHasProperties):
         """
 
         choiceProp = self.getProp('displaySpace')
-        choices    = []
+        ovlList    = self.__overlayList
+        choices    = [o for o in ovlList if isinstance(o, fslimage.Nifti)]
 
-        for overlay in self.__overlayList:
-            if isinstance(overlay, fslimage.Nifti):
-                choices.append(overlay)
+        choices.extend(('world', 'scaledVoxel', 'fslview'))
 
-        choices.append('world')
+        # If the current displaySpace setting is no
+        # longer valid (e.g. image has been removed),
+        # update the displaySpace to a sensible default
+        oldVal  = self.displaySpace
+        newVal  = None
+        default = self.defaultDisplaySpace
 
-        choiceProp.setChoices(choices, instance=self)
+        # oldVal not in choices means that
+        # it was a reference image which has
+        # been removed
+        if default != 'ref':
+            newVal = default
+        elif oldVal not in ovlList:
+            for c in choices:
+                if isinstance(c, fslimage.Nifti):
+                    newVal = c
+                    break
+
+        # This method is called from overlayListChanged,
+        # which will invoke the displaySpaceChanged
+        # listener manually. The parent displaycontext
+        # doesn't register a listener, hence ignoreInvalid
+        with props.skip(self, 'displaySpace', self.__name,
+                        ignoreInvalid=True):
+            choiceProp.setChoices(choices, instance=self, newChoice=newVal)
+
+
+    def __renameOverlays(self):
+        """Called by :meth:`__overlayListChanged` if :attr:`autoNameOverlays`
+        is ``True``. Makes sure that all on-disk overlays have a unique name.
+        In-memory overlay (e.g. mask images created in the editor) are not
+        renamed.
+        """
+
+        # Only rename on-disk overlays
+        ondisk = [o for o in self.overlayList if o.dataSource is not None]
+
+        # Group overlays by name
+        groups = defaultdict(list)
+        for o in ondisk:
+            display = self.getDisplay(o)
+            groups[dsutils.overlayName(o)].append(o)
+
+        # Identify groups of overlays with the same name
+        groups = {n : g for n, g in groups.items() if len(g) > 1}
+
+        # Rename each overlay according to its
+        # unique location in the file system
+        for groupname, group in groups.items():
+
+            ondiskpaths = [op.abspath(o.dataSource) for o in ondisk]
+            ondiskbase  = fslpath.commonBase(ondiskpaths)
+
+            for ovl in group:
+                ovldir       = op.dirname(op.abspath(ovl.dataSource))
+                ovlname      = op.join(ovldir, groupname)
+                ovlname      = op.relpath(ovlname, ondiskbase)
+                display      = self.getDisplay(ovl)
+                display.name = ovlname
 
 
     def __setTransform(self, image):
@@ -920,9 +1059,11 @@ class DisplayContext(props.SyncableHasProperties):
         # get called before we have registered a
         # listener on the bounds property.
         with props.skip(opts, 'bounds', self.__name, ignoreInvalid=True):
-            if   space == 'world':  opts.transform = 'affine'
-            elif image is space:    opts.transform = 'pixdim-flip'
-            else:                   opts.transform = 'reference'
+            if   space == 'world':       opts.transform = 'affine'
+            elif space == 'scaledVoxel': opts.transform = 'pixdim'
+            elif space == 'fslview':     opts.transform = 'pixdim-flip'
+            elif image is space:         opts.transform = 'pixdim-flip'
+            else:                        opts.transform = 'reference'
 
 
     def __displaySpaceChanged(self, *a):
@@ -964,7 +1105,6 @@ class DisplayContext(props.SyncableHasProperties):
         if len(self.overlayOrder) == len(self.__overlayList):
             return
 
-        #
         # NOTE: The following logic assumes that operations
         #       which modify the overlay list will only do
         #       one of the following:
@@ -978,9 +1118,14 @@ class DisplayContext(props.SyncableHasProperties):
         oldList  = self.__overlayList.getLastValue('overlays')[:]
         oldOrder = self.overlayOrder[:]
 
-        # If the overlay order was just the
-        # list order, preserve that ordering
-        if self.overlayOrder[:] == list(range(len(oldList))):
+        # If the overlay order was not valid prior
+        # to this call (can happen during initialisation),
+        # just set the overlay order to the list order.
+        #
+        # If the overlay order was just the list order,
+        # preserve that ordering
+        if (len(self.overlayOrder) != len(oldList)) or \
+           (self.overlayOrder[:] == list(range(len(oldList)))):
             self.overlayOrder[:] = list(range(len(self.__overlayList)))
 
         # If overlays have been added to
@@ -1132,7 +1277,7 @@ class DisplayContext(props.SyncableHasProperties):
 
         for ovl in self.__overlayList:
 
-            display = self.__displays[ovl]
+            display = self.getDisplay(ovl)
             opts    = display.opts
             lo      = opts.bounds.getLo()
             hi      = opts.bounds.getHi()
@@ -1185,14 +1330,26 @@ class DisplayContext(props.SyncableHasProperties):
                     self.location = self.worldLocation
             return
 
-        ref  = self.displaySpace
+        if self.displaySpace == 'scaledVoxel':
+            ref      = self.getSelectedOverlay()
+            srcSpace = 'pixdim'
+        elif self.displaySpace == 'fslview':
+            ref      = self.getSelectedOverlay()
+            srcSpace = 'pixdim-flip'
+        else:
+            ref      = self.displaySpace
+            srcSpace = 'display'
+
+        if ref is None:
+            return
+
         opts = self.getOpts(ref)
 
         if dest == 'world':
             with props.skip(self, 'location', self.__name):
                 self.worldLocation = opts.transformCoords(
-                    self.location, 'display', 'world')
+                    self.location, srcSpace, 'world')
         else:
             with props.skip(self, 'worldLocation', self.__name):
                 self.location = opts.transformCoords(
-                    self.worldLocation, 'world', 'display')
+                    self.worldLocation, 'world', srcSpace)
