@@ -63,29 +63,36 @@ class OverlayGroup(props.HasProperties):
 
 
     _groupBindings = td.TypeDict({
-        'Display'        : [],
-        'NiftiOpts'      : ['volume',
-                            'volumeDim'],
-        'VolumeOpts'     : ['interpolation'],
-        'Volume3DOpts'   : ['numSteps',
-                            'numInnerSteps',
-                            'resolution',
-                            'numClipPlanes',
-                            'clipMode',
-                            'clipPosition',
-                            'clipAzimuth',
-                            'clipInclination'],
-        'LabelOpts'      : [],
-        'MeshOpts'       : ['refImage',
-                            'coordSpace'],
-        'VectorOpts'     : ['suppressX',
-                            'suppressY',
-                            'suppressZ',
-                            'suppressMode'],
-        'LineVectorOpts' : [],
-        'RGBVectorOpts'  : ['interpolation'],
-        'TensorOpts'     : ['lighting',
-                            'tensorResolution'],
+        'Display'         : [],
+        'NiftiOpts'       : ['volume',
+                             'volumeDim'],
+        'VolumeOpts'      : ['interpolation'],
+        'Volume3DOpts'    : ['numSteps',
+                             'numInnerSteps',
+                             'resolution',
+                             'numClipPlanes',
+                             'clipMode',
+                             'clipPosition',
+                             'clipAzimuth',
+                             'clipInclination'],
+        'LabelOpts'       : [],
+        'MeshOpts'        : ['refImage',
+                             'coordSpace',
+                             'outline',
+                             'outlineWidth',
+                             'wireframe'],
+        'NiftiVectorOpts' : ['orientFlip'],
+        'LineVectorOpts'  : ['lineWidth'],
+        'RGBVectorOpts'   : ['interpolation'],
+        'VectorOpts'      : ['suppressX',
+                             'suppressY',
+                             'suppressZ',
+                             'suppressMode'],
+        'TensorOpts'      : ['lighting',
+                             'tensorResolution'],
+        'TractogramOpts'  : ['lineWidth',
+                             'resolution',
+                             'subsample']
     })
     """This dictionary defines the properties which are bound across
     :class:`.Display` instances :class:`.DisplayOpts` sub-class instances, for
@@ -108,13 +115,15 @@ class OverlayGroup(props.HasProperties):
             NiftiOpts,
             VolumeOpts,
             Volume3DOpts,
-            MaskOpts,
-            VectorOpts,
-            RGBVectorOpts,
-            LineVectorOpts,
-            MeshOpts,
             LabelOpts,
-            TensorOpts)
+            MeshOpts,
+            MaskOpts,
+            LineVectorOpts,
+            NiftiVectorOpts,
+            RGBVectorOpts,
+            VectorOpts,
+            TensorOpts,
+            TractogramOpts)
 
         self.__displayCtx  = displayCtx
         self.__overlayList = overlayList
