@@ -45,7 +45,7 @@ class MIPOpts(cmapopts.ColourMapOpts, niftiopts.NiftiOpts):
     """
 
 
-    interpolation = props.Choice(('none', 'linear', 'spline'))
+    interpolation = props.Choice(('none', 'linear', 'spline', 'true_spline'))
     """How the value shown at a real world location is derived from the
     corresponding data value(s). ``none`` is equivalent to nearest neighbour
     interpolation.
@@ -63,8 +63,10 @@ class MIPOpts(cmapopts.ColourMapOpts, niftiopts.NiftiOpts):
         # spline interpolation
         if float(fslgl.GL_COMPATIBILITY) < 2.1:
             interp = self.getProp('interpolation')
-            interp.removeChoice('spline', instance=self)
-            interp.updateChoice('linear', instance=self, newAlt=['spline'])
+            interp.removeChoice('spline',      instance=self)
+            interp.removeChoice('true_spline', instance=self)
+            interp.updateChoice('linear',      instance=self,
+                                newAlt=['spline', 'true_spline'])
 
         niftiopts.NiftiOpts    .__init__(self, *args, **kwargs)
         cmapopts .ColourMapOpts.__init__(self)
@@ -100,7 +102,7 @@ class MIPOpts(cmapopts.ColourMapOpts, niftiopts.NiftiOpts):
         """
 
         d2tmat  = self.getTransform('display', 'texture')
-        xform   = affine.concat(d2tmat, viewmat)
+        xform   = affine.concat(d2tmat, viewmat.T)
         cdir    = np.array([0, 0, 1])
         cdir    = affine.transform(cdir, xform, vector=True)
         cdir    = affine.normalise(cdir)
