@@ -27,15 +27,17 @@ in one of several ways:
 
  **scaled voxels**              (a.k.a. ``pixdim``) The image data voxel
                                 coordinates are scaled by the ``pixdim`` values
-                                stored in the NIFTI header.
-
+                                stored in the NIFTI header. The origin is
+                                fixed at the centre of voxel ``(0, 0, 0)``.
 
  **radioloigcal scaled voxels** (a.k.a. ``pixdim-flip``) The image data voxel
-                                coordinates are scaled by the ``pixdim`` values
-                                stored in the NIFTI header and, if the image
-                                appears to be stored in neurological order,
-                                the X (left-right) axis is inverted.
-
+                                coordinates are scaled by the ``pixdim``
+                                values stored in the NIFTI header and, if the
+                                image appears to be stored in neurological
+                                order, the X (left-right) axis is
+                                inverted. The origin is fixed at the centre of
+                                voxel ``(0, 0, 0)`` (or ``(X-1, 0, 0)`` for
+                                inverted images).
 
  **world**                      (a.k.a. ``affine``) The image data voxel
                                 coordinates are transformed by the
@@ -73,7 +75,7 @@ oriented according to this space, so if the input data is in neurological
 orientation, these vectors need to be inverted along the x axis.
 
 
-http://fsl.fmrib.ox.ac.uk/fsl/fslwiki/FLIRT/FAQ#What_is_the_format_of_the_matrix_used_by_FLIRT.2C_and_how_does_it_relate_to_the_transformation_parameters.3F
+https://fsl.fmrib.ox.ac.uk/fsl/docs/#/registration/flirt/faq?id=what-is-the-format-of-the-matrix-used-by-flirt-and-how-does-it-relate-to-the-transformation-parameters
 
 
 What is a voxel?
@@ -261,6 +263,7 @@ class NiftiOpts(fsldisplay.DisplayOpts):
             # (if it is an overlay)
             self.__xforms    = {}
             self.__dsOverlay = None
+            self.__displaySpaceChanged(refresh=False)
             self.__setupTransforms()
             self.__transformChanged()
             self.__volumeDimChanged()
@@ -373,13 +376,14 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         self.bounds[:] = [lo[0], hi[0], lo[1], hi[1], lo[2], hi[2]]
 
 
-    def __displaySpaceChanged(self, *a):
+    def __displaySpaceChanged(self, *a, **kwa):
         """Called when the :attr:`.DisplayContext.displaySpace` property
         changes.  Re-generates transformation matrices, and re-calculates
         the display :attr:`bounds` (via calls to :meth:`__setupTransforms` and
         :meth:`__transformChanged`).
         """
 
+        refresh      = kwa.pop('refresh', True)
         displaySpace = self.displayCtx.displaySpace
 
         if self.__dsOverlay is not None:
@@ -396,9 +400,10 @@ class NiftiOpts(fsldisplay.DisplayOpts):
                                       self.__displaySpaceTransformChanged,
                                       topic='transform')
 
-        self.__setupTransforms()
-        if self.transform == 'reference':
-            self.__transformChanged()
+        if refresh:
+            self.__setupTransforms()
+            if self.transform == 'reference':
+                self.__transformChanged()
 
 
     def __displayXformChanged(self, *a):
@@ -433,9 +438,9 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         shape = np.array(image.shape[:3])
 
         voxToIdMat      = np.eye(4)
-        voxToPixdimMat  = np.diag(list(image.pixdim[:3]) + [1.0])
-        voxToPixFlipMat = image.voxToScaledVoxMat
-        voxToWorldMat   = image.voxToWorldMat
+        voxToPixdimMat  = image.getAffine('voxel', 'scaled')
+        voxToPixFlipMat = image.getAffine('voxel', 'fsl')
+        voxToWorldMat   = image.getAffine('voxel', 'world')
         voxToWorldMat   = affine.concat(self.displayXform, voxToWorldMat)
         ds              = self.displayCtx.displaySpace
 
@@ -443,11 +448,15 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         # on the value of displaySpace
         if ds == 'world':
             voxToRefMat = voxToWorldMat
+        elif ds == 'scaledVoxel':
+            voxToRefMat = voxToPixdimMat
+        elif ds == 'fslview':
+            voxToRefMat = voxToPixFlipMat
         elif ds is self.overlay:
             voxToRefMat = voxToPixFlipMat
         else:
-            voxToRefMat = affine.concat(ds.voxToScaledVoxMat,
-                                        ds.worldToVoxMat,
+            voxToRefMat = affine.concat(ds.getAffine('voxel', 'fsl'),
+                                        ds.getAffine('world', 'voxel'),
                                         voxToWorldMat)
 
         # When going from voxels to textures,
@@ -455,7 +464,7 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         # the note on coordinate systems at
         # the top of this file).
         voxToTexMat        = affine.scaleOffsetXform(tuple(1.0 / shape),
-                                                        tuple(0.5 / shape))
+                                                     tuple(0.5 / shape))
 
         idToVoxMat         = affine.invert(voxToIdMat)
         idToPixdimMat      = affine.concat(voxToPixdimMat,  idToVoxMat)
@@ -550,7 +559,7 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         return ['volume', 'volumeDim']
 
 
-    def getTransform(self, from_, to, xform=None):
+    def getTransform(self, from_, to):
         """Return a matrix which may be used to transform coordinates
         from ``from_`` to ``to``. Valid values for ``from_`` and ``to``
         are:
@@ -589,27 +598,19 @@ class NiftiOpts(fsldisplay.DisplayOpts):
                         suitable for looking up voxel values when stored as
                         an OpenGL texture.
         =============== ======================================================
-
-
-        If the ``xform`` parameter is provided, and one of ``from_`` or ``to``
-        is ``display``, the value of ``xform`` is used instead of the current
-        value of :attr:`transform`.
         """
 
         if not self.__child:
             raise RuntimeError('getTransform cannot be called on '
                                'a parent NiftiOpts instance')
 
-        if xform is None:
-            xform = self.transform
-
-        if   from_ == 'display': from_ = xform
+        if   from_ == 'display': from_ = self.transform
         elif from_ == 'world':   from_ = 'affine'
         elif from_ == 'voxel':   from_ = 'id'
         elif from_ == 'pixflip': from_ = 'pixdim-flip'
         elif from_ == 'ref':     from_ = 'reference'
 
-        if   to    == 'display': to    = xform
+        if   to    == 'display': to    = self.transform
         elif to    == 'world':   to    = 'affine'
         elif to    == 'voxel':   to    = 'id'
         elif to    == 'pixflip': to    = 'pixdim-flip'
@@ -835,6 +836,18 @@ class NiftiOpts(fsldisplay.DisplayOpts):
         return vox
 
 
+    def setIndex(self, indices):
+        """Sets the indexes of all non-spatial dimensions. The :attr:`volume`
+        property is also updated.
+        """
+        if len(indices) != self.overlay.ndim - 3:
+            raise ValueError(
+                f'Wrong number of indices - {self.display.name} '
+                f'has {self.overlay.ndim-3} non-spatial dimensions')
+        self.setAttribute('volume', 'cache', indices)
+        self.volume = indices[self.volumeDim]
+
+
     def index(self, slc=None, atVolume=True):
         """Given a slice object ``slc``, which indexes into the X, Y, and Z
         dimensions, fills it to slice every dimension of the image, using
