@@ -57,7 +57,7 @@ class OrthoOpts(sceneopts.SceneOpts):
 
 
     showLocation = props.Choice(('no', 'X', 'Y', 'Z'))
-    """If not ``'no'`` labels showing the current location, in voxel and
+    """If not ``'no'``, labels showing the current location, in voxel and
     world coordinatees, will be shown on the selected canvas.
     """
 
@@ -112,7 +112,7 @@ class OrthoOpts(sceneopts.SceneOpts):
         """
         sceneopts.SceneOpts.__init__(self, *args, **kwargs)
 
-        name = '{}_{}'.format(type(self).__name__, id(self))
+        name = f'{type(self).__name__}_{id(self)}'
 
         self.addListener('zoom', name, self.__onZoom)
 
@@ -126,3 +126,16 @@ class OrthoOpts(sceneopts.SceneOpts):
         self.xzoom = self.zoom
         self.yzoom = self.zoom
         self.zzoom = self.zoom
+
+
+    def _onPerformanceChange(self, *a):
+        """Overrides :meth:`.SceneOpts._onPerformanceChange`. Changes the value
+        of the :attr:`renderMode` property according to the performance setting.
+        """
+
+        if   self.performance == 3: self.renderMode = 'onscreen'
+        elif self.performance == 2: self.renderMode = 'offscreen'
+        elif self.performance == 1: self.renderMode = 'prerender'
+
+        log.debug('Performance settings changed: '
+                  'renderMode=%s', self.renderMode)
