@@ -52,3 +52,7 @@ class Scene3DOpts(sceneopts.SceneOpts):
         self.fgColour = (0,   1,   0)
 
         sceneopts.SceneOpts.__init__(self, *args, **kwargs)
+
+
+    def _onPerformanceChange(self, *a):
+        """Overrides :meth:`.SceneOpts._onPerformanceChange`. Does nothing. """
