@@ -49,12 +49,11 @@ maximum SH order that was used in generating the coefficients.
 """
 
 
-class SHOpts(vectoropts.VectorOpts):
-    """The ``SHOpts`` is used for rendering class for rendering :class:`.Image`
-    instances which contain fibre orientation distributions (FODs) in the form
-    of spherical harmonic (SH) coefficients. A ``SHOpts`` instance will be
-    used for ``Image`` overlays with a :attr:`.Displaty.overlayType` set to
-    ``'sh'``.
+class SHOpts(vectoropts.NiftiVectorOpts):
+    """The ``SHOpts`` is used for rendering :class:`.Image` instances which
+    contain fibre orientation distributions (FODs) in the form of spherical
+    harmonic (SH) coefficients. A ``SHOpts`` instance will be used for
+    ``Image`` overlays with a :attr:`.Displaty.overlayType` set to ``'sh'``.
 
 
     A collection of pre-calculated SH basis function parameters are stored in
@@ -128,7 +127,7 @@ class SHOpts(vectoropts.VectorOpts):
 
     def __init__(self, *args, **kwargs):
 
-        vectoropts.VectorOpts.__init__(self, *args, **kwargs)
+        vectoropts.NiftiVectorOpts.__init__(self, *args, **kwargs)
 
         ncoefs           = self.overlay.shape[3]
         shType, maxOrder = SH_COEFFICIENT_TYPE.get(ncoefs)
@@ -187,7 +186,6 @@ class SHOpts(vectoropts.VectorOpts):
         ftype, _   = SH_COEFFICIENT_TYPE[ncoefs]
         fname     = op.join(
             fsleyes.assetDir,
-            'assets',
             'sh',
             '{}_coef_{}_{}.txt'.format(ftype, resolution, order))
 
@@ -205,7 +203,6 @@ class SHOpts(vectoropts.VectorOpts):
         """
         fname = op.join(
             fsleyes.assetDir,
-            'assets',
             'sh',
             'vert_{}.txt'.format(self.shResolution))
 
@@ -219,7 +216,6 @@ class SHOpts(vectoropts.VectorOpts):
         """
         fname = op.join(
             fsleyes.assetDir,
-            'assets',
             'sh',
             'face_{}.txt'.format(self.shResolution))
 
