@@ -56,7 +56,7 @@ class VolumeOpts(cmapopts.ColourMapOpts,
     """
 
 
-    interpolation = props.Choice(('none', 'linear', 'spline'))
+    interpolation = props.Choice(('none', 'linear', 'spline', 'true_spline'))
     """How the value shown at a real world location is derived from the
     corresponding data value(s). ``none`` is equivalent to nearest neighbour
     interpolation.
@@ -65,11 +65,16 @@ class VolumeOpts(cmapopts.ColourMapOpts,
 
     @classmethod
     def getInitialDisplayRange(cls):
-        """This class method returns a tuple containing ``(low, high)``
-        percentile values which are used to set the initial values for the
-        :attr:`.ColourMapOpts.displayRange` and
-        :attr:`.ColourMapOpts.clippingRange` properties. If the initial
-        display range has not yet been set (via the
+        """This class method returns a tuple containing values which are used
+        to set the initial values for the :attr:`.ColourMapOpts.displayRange`
+        and :attr:`.ColourMapOpts.clippingRange` properties.
+
+        The returned tuple contains:
+         - The low and high display range values (as a tuple)
+         - ``True`` if the values should be interpreted as percentiles,
+           or ``False`` if they should be interpreted as raw intensities.
+
+        If the initial display range has not yet been set (via the
         :meth:`setInitialDisplayRange` method), ``None`` is returned.
         """
         try:
@@ -79,26 +84,21 @@ class VolumeOpts(cmapopts.ColourMapOpts,
 
 
     @classmethod
-    def setInitialDisplayRange(cls, drange):
+    def setInitialDisplayRange(cls, drange, percentiles=False):
         """Sets the initial values for the :attr:`.ColourMapOpts.displayRange`
         and :attr:`.ColourMapOpts.clippingRange` to be used for new
         :class:`VolumeOpts` instances.
 
-        :arg drange: A tuple containing ``(low, high)`` display range values
-                     as percentiles of the image data range. May be ``None``,
-                     in which case the initial display range will be set to the
-                     image data range.
-        """
+        :arg drange:      A tuple containing ``(low, high)`` display range
+                          values. May be ``None``, in which case the initial
+                          display range will be set to the image data range.
 
+        :arg percentiles: If ``True``, interpret ``drange`` as percentiles of
+                          the data range of each image, rather than raw
+                          intensities.
+        """
         if drange is not None:
-            low, high = drange
-            if not all((low < high,
-                        low >= 0,
-                        low <= 100,
-                        high >= 0,
-                        high <= 100)):
-                raise ValueError('Invalid initial display '
-                                 'range: {}'.format(drange))
+            drange = (drange, percentiles)
 
         cls.__initialDisplayRange = drange
 
@@ -120,8 +120,10 @@ class VolumeOpts(cmapopts.ColourMapOpts,
         # spline interpolation
         if float(fslgl.GL_COMPATIBILITY) < 2.1:
             interp = self.getProp('interpolation')
-            interp.removeChoice('spline', instance=self)
-            interp.updateChoice('linear', instance=self, newAlt=['spline'])
+            interp.removeChoice('spline',      instance=self)
+            interp.removeChoice('true_spline', instance=self)
+            interp.updateChoice('linear',      instance=self,
+                                newAlt=['spline', 'true_spline'])
 
         # Interpolation cannot be unbound
         # between VolumeOpts instances. This is
@@ -159,7 +161,8 @@ class VolumeOpts(cmapopts.ColourMapOpts,
 
         # Default to spline interpolation
         # if we are displaying in 3D
-        if displayCtx.displayType == '3D':
+        if displayCtx.displayType == '3D' and \
+           vol3dopts.Volume3DOpts.enableInterpolation:
             self.interpolation = 'spline'
 
         # Some things only happen
@@ -197,15 +200,17 @@ class VolumeOpts(cmapopts.ColourMapOpts,
 
         if self.__registered and drange is not None:
 
-            if   overlay.ndim == 3: sample = overlay[:]
-            elif overlay.ndim == 4: sample = overlay[..., 0]
+            drange, percentiles = drange
 
-            drange = np.percentile(sample[sample != 0], drange)
-            crange = [drange[0], overlay.dataRange[1]]
+            if percentiles:
+                if   overlay.ndim == 3: sample = overlay[:]
+                elif overlay.ndim == 4: sample = overlay[..., 0]
+                drange = np.clip(drange, 0, 100)
+                drange = np.percentile(sample[sample != 0], drange)
 
             self.displayRange  = drange
             self.modulateRange = drange
-            self.clippingRange = crange
+            self.clippingRange = [drange[0], overlay.dataRange[1]]
 
         # If this is not a RGB(A) image, disable
         # the channel property. If it's a RGB
@@ -329,10 +334,22 @@ class VolumeOpts(cmapopts.ColourMapOpts,
         returns ``None``.
         """
 
+        # No clipping image set - defer to
+        # cmap opts behaviour, which is to
+        # use the main data range
         if self.clipImage is None:
             return cmapopts.ColourMapOpts.getClippingRange(self)
-        else:
-            return self.clipImage.dataRange
+
+        dmin, dmax = self.clipImage.dataRange
+
+        # If negcmap is enabled, we modulate according
+        # to the absolute value of the clip image.
+        # This will make sense in the vast majority of
+        # cases (e.g. clipping by a stats image)
+        if self.useNegativeCmap:
+            dmin, dmax = 0, np.max(np.abs((dmin, dmax)))
+
+        return dmin, dmax
 
 
     def getModulateRange(self):
@@ -340,11 +357,17 @@ class VolumeOpts(cmapopts.ColourMapOpts,
         If a :attr:`.modulateImage` is set, returns its data range. Otherwise
         returns ``None``.
         """
+        # see inline comments in getClippingRange
 
         if self.modulateImage is None:
             return cmapopts.ColourMapOpts.getModulateRange(self)
-        else:
-            return self.modulateImage.dataRange
+
+        dmin, dmax = self.modulateImage.dataRange
+
+        if self.useNegativeCmap:
+            dmin, dmax = 0, np.max(np.abs((dmin, dmax)))
+
+        return dmin, dmax
 
 
     def __dataRangeChanged(self, *a):
@@ -513,8 +536,10 @@ class VolumeRGBOpts(niftiopts.NiftiOpts):
         # spline interpolation
         if float(fslgl.GL_COMPATIBILITY) < 2.1:
             interp = self.getProp('interpolation')
-            interp.removeChoice('spline', instance=self)
-            interp.updateChoice('linear', instance=self, newAlt=['spline'])
+            interp.removeChoice('spline',      instance=self)
+            interp.removeChoice('true_spline', instance=self)
+            interp.updateChoice('linear', instance=self,
+                                newAlt=['spline', 'true_spline'])
 
         niftiopts.NiftiOpts.__init__(self,
                                      overlay,
