@@ -9,13 +9,13 @@ selection of voxels in a 3D :class:`.Image`.
 """
 
 import logging
-import collections
+import collections.abc as abc
 
-import numpy                       as np
-import scipy.ndimage.measurements  as ndimeas
+import numpy          as np
+import scipy.ndimage  as ndimage
 
-import fsl.utils.notifier          as notifier
-import fsleyes.gl.routines         as glroutines
+import fsl.utils.notifier  as notifier
+import fsleyes.gl.routines as glroutines
 
 
 log = logging.getLogger(__name__)
@@ -48,7 +48,6 @@ class Selection(notifier.Notifier):
        :nosignatures:
 
        setSelection
-       replaceSelection
        addToSelection
        removeFromSelection
 
@@ -101,12 +100,18 @@ class Selection(notifier.Notifier):
 
         self.__image              = image
         self.__display            = display
-        self.__opts               = display.opts
         self.__clear              = True
         self.__lastChangeOffset   = None
         self.__lastChangeOldBlock = None
         self.__lastChangeNewBlock = None
 
+        # We keep track of regions of the selection
+        # that have been modified, as a sequence of
+        # (xlo, ylo, zlo, xhi, yhi, zhi) values.
+        # This is used by the getBoundedSelection
+        # method,
+        self.__dirty = None
+
         if selection is None:
             selection = np.zeros(image.shape[:3], dtype=np.uint8)
 
@@ -117,13 +122,24 @@ class Selection(notifier.Notifier):
 
         self.__selection = selection
 
-        log.debug('{}.init ({})'.format(type(self).__name__, id(self)))
+        log.debug('%s.init (%s)', type(self).__name__, id(self))
 
 
     def __del__(self):
         """Prints a log message."""
         if log:
-            log.debug('{}.del ({})'.format(type(self).__name__, id(self)))
+            log.debug('%s.del (%s)', type(self).__name__, id(self))
+
+
+    @property
+    def shape(self):
+        """Returns the selection shape. """
+        return self.__selection.shape
+
+
+    def __getitem__(self, key):
+        """Convenience wrapper around ``self.getSelection().__getitem__``. """
+        return self.__selection.__getitem__(key)
 
 
     def getSelection(self):
@@ -201,21 +217,6 @@ class Selection(notifier.Notifier):
         self.__updateSelectionBlock(block, offset, combine)
 
 
-    def replaceSelection(self, block, offset, combine=False):
-        """Copies the given ``block`` into the selection, starting at
-        ``offset``.
-
-        :arg block:   A ``numpy.uint8`` array containing a selection.
-
-        :arg offset:  Voxel coordinates specifying the block location.
-
-        :arg combine: Combine this change with the previous stored change (see
-                      :meth:`__storeChange`).
-        """
-
-        self.__updateSelectionBlock(block, offset, combine)
-
-
     def addToSelection(self, block, offset, combine=False):
         """Adds the selection (via a boolean OR operation) in the given
         ``block`` to the current selection, starting at ``offset``.
@@ -261,23 +262,29 @@ class Selection(notifier.Notifier):
         Returns a tuple containing the region, as a ``numpy.uint8`` array, and
         the coordinates specifying its location in the full :attr:`selection`
         array.
-
-        .. warning:: This method is slow, and in many cases it may be
-                     faster simply to access the full selection array.
         """
 
-        xs, ys, zs = np.where(self.__selection > 0)
+        # If this method is called when a dirty
+        # region has not been saved (e.g. after
+        # a call to clearSelection), we fall
+        # back to manually calculating it, which
+        # is quite slow.
+        if self.__dirty is None:
+            xs, ys, zs = self.__selection.nonzero()
 
-        if len(xs) == 0:
-            return np.array([]).reshape(0, 0, 0), (0, 0, 0)
+            if len(xs) == 0:
+                xlo = ylo = zlo = xhi = yhi = zhi = 0
+            else:
+                xlo = int(xs.min())
+                ylo = int(ys.min())
+                zlo = int(zs.min())
+                xhi = int(xs.max() + 1)
+                yhi = int(ys.max() + 1)
+                zhi = int(zs.max() + 1)
 
-        xlo = int(xs.min())
-        ylo = int(ys.min())
-        zlo = int(zs.min())
-        xhi = int(xs.max() + 1)
-        yhi = int(ys.max() + 1)
-        zhi = int(zs.max() + 1)
+            self.__dirty = xlo, ylo, zlo, xhi, yhi, zhi
 
+        xlo, ylo, zlo, xhi, yhi, zhi = self.__dirty
         selection = self.__selection[xlo:xhi, ylo:yhi, zlo:zhi]
 
         return selection, (xlo, ylo, zlo)
@@ -305,7 +312,7 @@ class Selection(notifier.Notifier):
         fRestrict = fixSlices(restrict)
         offset    = [r.start if r.start is not None else 0 for r in fRestrict]
 
-        log.debug('Clearing selection ({}): {}'.format(id(self), fRestrict))
+        log.debug('Clearing selection (%s): %s', id(self), fRestrict)
 
         block                       = np.array(self.__selection[fRestrict])
         self.__selection[fRestrict] = False
@@ -322,6 +329,15 @@ class Selection(notifier.Notifier):
         if restrict is None:
             self.__clear = True
 
+        # Always clear the dirty region - in
+        # theory we could adjust the dirty
+        # region by the restrict slices (if
+        # provided), but this is awkward to
+        # do. The getBoundedSelection method
+        # will resort to np.ndarray.nonzero
+        # if the dirty region is not set.
+        self.__dirty = None
+
         self.notify()
 
 
@@ -376,11 +392,11 @@ class Selection(notifier.Notifier):
 
             if log.getEffectiveLevel() == logging.DEBUG:
                 log.debug('Replacing previously stored change with: '
-                          '[({}, {}), ({}, {}), ({}, {})] ({} selected)'
-                          .format(offset[0], offset[0] + old.shape[0],
-                                  offset[1], offset[1] + old.shape[1],
-                                  offset[2], offset[2] + old.shape[2],
-                                  new.sum()))
+                          '[(%s, %s), (%s, %s), (%s, %s)] (%s selected)',
+                          offset[0], offset[0] + old.shape[0],
+                          offset[1], offset[1] + old.shape[1],
+                          offset[2], offset[2] + old.shape[2],
+                          new.sum())
 
             self.__lastChangeOldBlock = old
             self.__lastChangeNewBlock = new
@@ -429,15 +445,15 @@ class Selection(notifier.Notifier):
         # to make indexing easier. The last/current
         # slice objects are defined relative to the
         # combined space of both.
-        cmbSlices  = [slice(lo, hi) for lo, hi in cmbIdxs]
+        cmbSlices  = tuple([slice(lo, hi) for lo, hi in cmbIdxs])
 
-        lastSlices = [slice(lLo - cmLo, lHi - cmLo)
-                      for ((lLo, lHi), (cmLo, cmHi))
-                      in zip(lastIdxs, cmbIdxs)]
+        lastSlices = tuple([slice(lLo - cmLo, lHi - cmLo)
+                            for ((lLo, lHi), (cmLo, cmHi))
+                            in zip(lastIdxs, cmbIdxs)])
 
-        currSlices = [slice(cuLo - cmLo, cuHi - cmLo)
-                      for ((cuLo, cuHi), (cmLo, cmHi))
-                      in zip(currIdxs, cmbIdxs)]
+        currSlices = tuple([slice(cuLo - cmLo, cuHi - cmLo)
+                            for ((cuLo, cuHi), (cmLo, cmHi))
+                            in zip(currIdxs, cmbIdxs)])
 
         cmbOld    = np.array(self.__selection[cmbSlices])
         cmbNew    = np.array(cmbOld)
@@ -448,21 +464,21 @@ class Selection(notifier.Notifier):
 
         if log.getEffectiveLevel() == logging.DEBUG:
             log.debug('Combining changes: '
-                      '[({}, {}), ({}, {}), ({}, {})] ({} selected) + '
-                      '[({}, {}), ({}, {}), ({}, {})] ({} selected) = '
-                      '[({}, {}), ({}, {}), ({}, {})] ({} selected)'.format(
-                          lastIdxs[0][0], lastIdxs[0][1],
-                          lastIdxs[1][0], lastIdxs[1][1],
-                          lastIdxs[2][0], lastIdxs[2][1],
-                          lcNew.sum(),
-                          currIdxs[0][0], currIdxs[0][1],
-                          currIdxs[1][0], currIdxs[1][1],
-                          currIdxs[2][0], currIdxs[2][1],
-                          new.sum(),
-                          cmbIdxs[0][0], cmbIdxs[0][1],
-                          cmbIdxs[1][0], cmbIdxs[1][1],
-                          cmbIdxs[2][0], cmbIdxs[2][1],
-                          cmbNew.sum()))
+                      '[(%s, %s), (%s, %s), (%s, %s)] (%s selected) + '
+                      '[(%s, %s), (%s, %s), (%s, %s)] (%s selected) = '
+                      '[(%s, %s), (%s, %s), (%s, %s)] (%s selected)',
+                      lastIdxs[0][0], lastIdxs[0][1],
+                      lastIdxs[1][0], lastIdxs[1][1],
+                      lastIdxs[2][0], lastIdxs[2][1],
+                      lcNew.sum(),
+                      currIdxs[0][0], currIdxs[0][1],
+                      currIdxs[1][0], currIdxs[1][1],
+                      currIdxs[2][0], currIdxs[2][1],
+                      new.sum(),
+                      cmbIdxs[0][0], cmbIdxs[0][1],
+                      cmbIdxs[1][0], cmbIdxs[1][1],
+                      cmbIdxs[2][0], cmbIdxs[2][1],
+                      cmbNew.sum())
 
         self.__lastChangeOldBlock = cmbOld
         self.__lastChangeNewBlock = cmbNew
@@ -513,7 +529,8 @@ class Selection(notifier.Notifier):
                   and offset of this array into the full selection image.
         """
 
-        data = self.__image[self.__opts.index()]
+        opts = self.__display.opts
+        data = self.__image[opts.index()]
         block, offset = selectByValue(data,
                                       seedLoc,
                                       precision,
@@ -521,7 +538,7 @@ class Selection(notifier.Notifier):
                                       local,
                                       restrict)
 
-        self.replaceSelection(block, offset, combine)
+        self.setSelection(block, offset, combine)
 
         return block, offset
 
@@ -657,11 +674,22 @@ class Selection(notifier.Notifier):
             offset,
             combine)
 
-        log.debug('Updating selection ({}) block [{}:{}, {}:{}, {}:{}]'.format(
-            id(self), xlo, xhi, ylo, yhi, zlo, zhi))
+        log.debug('Updating selection (%i) block [%i:%i, %i:%i, %i:%i]',
+                  id(self), xlo, xhi, ylo, yhi, zlo, zhi)
 
         self.__selection[xlo:xhi, ylo:yhi, zlo:zhi] = block
 
+        # Save/update the dirty region - the region
+        # of the selection that has been modified
+        # and therefore is potentially non-zero.
+        if self.__dirty is None:
+            self.__dirty = [xlo, ylo, zlo, xhi, yhi, zhi]
+        else:
+            self.__dirty[:3] = np.min([self.__dirty[:3],
+                                       [xlo, ylo, zlo]], axis=0)
+            self.__dirty[3:] = np.max([self.__dirty[3:],
+                                       [xhi, yhi, zhi]], axis=0)
+
         self.__clear = False
 
         self.notify()
@@ -698,7 +726,7 @@ def fixSlices(slices):
         if s is None:
             slices[i] = slice(None)
 
-    return slices
+    return tuple(slices)
 
 
 def selectByValue(data,
@@ -749,7 +777,7 @@ def selectByValue(data,
     # turn it into the third.
     if searchRadius is None:
         searchRadius = np.array([0, 0, 0])
-    elif not isinstance(searchRadius, collections.Sequence):
+    elif not isinstance(searchRadius, abc.Sequence):
         searchRadius = np.array([searchRadius] * 3)
 
     searchRadius = np.ceil(searchRadius)
@@ -783,7 +811,7 @@ def selectByValue(data,
 
         data         = data[restrict]
         shape        = data.shape
-        searchOffset = [start for start in [xs, ys, zs]]
+        searchOffset = [xs, ys, zs]
         seedLoc      = [sl - so for sl, so in zip(seedLoc, searchOffset)]
 
     # No search radius - search
@@ -835,7 +863,7 @@ def selectByValue(data,
 
         # Extract the search space, and
         # create the ellipsoid mask
-        searchSpace  = data[slices]
+        searchSpace  = data[tuple(slices)]
         searchOffset = [so + r[0] for so, r in zip(searchOffset, ranges)]
         searchMask   = dists <= 1
 
@@ -856,7 +884,7 @@ def selectByValue(data,
     # If local is not True, any same or similar
     # values are part of the selection
     if local:
-        hits, _   = ndimeas.label(hits)
+        hits, _   = ndimage.label(hits)
         seedLabel = hits[seedLoc[0], seedLoc[1], seedLoc[2]]
         hits      = hits == seedLabel
 
@@ -891,25 +919,36 @@ def selectLine(shape,
                  full image. If the
     """
 
-    from_   = np.array(from_)
-    to      = np.array(to)
+    from_ = np.array(from_)
+    to    = np.array(to)
+
+    if not isinstance(boxSize, abc.Sequence):
+        boxSize = [boxSize] * 3
 
     # The boxSize is specified in scaled
-    # voxels, so we need to calculate the
-    # distance between the two voxels,
-    # and the required number of points,
-    # in scaled voxels as well.
+    # voxels, and may either be a scalar or
+    # a sequence. We start by adjusting the
+    # box size so that it is an integer
+    # multiple of the voxel dimensions (of
+    # at least length 1), so that all voxels
+    # between from_ and to will be covered
+    # by our interpolated points, below.
+    boxSize = np.array(boxSize)
+    dims    = np.array(dims)
+    boxSize = np.floor(boxSize / dims)
+    boxSize = np.clip(boxSize, 1, None)
+    boxSize = dims * boxSize
+
+    # Now we need to calculate the distance
+    # between the to/from voxels, and the
+    # required number of points, in scaled
+    # voxels as well.
     length  = np.sqrt(np.sum((from_ * dims - to * dims) ** 2))
 
-    # box size can either be
-    # a scalar or a sequence.
-    # We add 2 to the number
-    # of points for the from_
-    # and to locations.
-    if isinstance(boxSize, collections.Sequence):
-        npoints = int(np.ceil(length / min(boxSize))) + 2
-    else:
-        npoints = int(np.ceil(length / boxSize)) + 2
+    # We add 2 to the number of points
+    # to account for the from_ and to
+    # locations.
+    npoints = int(np.ceil(length / min(boxSize))) + 2
 
     # Create a bunch of interpolated
     # points between from_ and to
@@ -956,7 +995,7 @@ def selectLine(shape,
 
     # Allocate a selection block
     # which will contain the line
-    block = np.zeros(size, dtype=np.bool)
+    block = np.zeros(size, dtype=bool)
 
     # Generate a voxel block
     # at each point
