@@ -174,11 +174,12 @@ class FSLeyesFrame(wx.Frame):
         font.SetWeight(wx.FONTWEIGHT_NORMAL)
         self.SetFont(font)
 
-        self.__overlayList = overlayList
-        self.__displayCtx  = displayCtx
-        self.__name        = '{}_{}'.format(type(self).__name__, id(self))
-        self.__mainPanel   = wx.Panel(self)
-        self.__statusBar   = wx.StaticText(self)
+        self.__overlayList   = overlayList
+        self.__displayCtx    = displayCtx
+        self.__name          = f'{type(self).__name__}_{id(self)}'
+        self.__mainPanel     = wx.Panel(self)
+        self.__statusBar     = wx.StaticText(self)
+        self.__closeHandlers = closeHandlers
 
         # Even though the FSLeyesFrame does not allow
         # panels to be floated, I am specifying the
@@ -212,7 +213,7 @@ class FSLeyesFrame(wx.Frame):
         #
         # This problem does not occur with the aero/
         # whidbey guides.
-        self.__auiManager  = aui.AuiManager(
+        self.__auiManager = aui.AuiManager(
             self.__mainPanel,
             agwFlags=(aui.AUI_MGR_RECTANGLE_HINT          |
                       aui.AUI_MGR_NO_VENETIAN_BLINDS_FADE |
@@ -317,15 +318,6 @@ class FSLeyesFrame(wx.Frame):
 
         self.__auiManager.Bind(aui.EVT_AUI_PANE_CLOSE, self.__onViewPanelClose)
 
-        # Event handlers are called in order of
-        # last bound first, first bound last.
-        # Our __onClose handler may veto the
-        # close (if the user cancels it), so
-        # we have to bind our handler last.
-        if closeHandlers is not None:
-            for h in closeHandlers:
-                self.Bind(wx.EVT_CLOSE, h)
-
         self.Bind(wx.EVT_CLOSE, self.__onClose)
 
         overlayList.addListener('overlays',
@@ -470,8 +462,10 @@ class FSLeyesFrame(wx.Frame):
     def addViewPanel(
             self,
             panelCls      : Type[viewpanel.ViewPanel],
-            title         : str  = None,
-            defaultLayout : bool = True,
+            title         : str   = None,
+            defaultLayout : bool  = True,
+            location      : int   = None,
+            size          : float = None,
             **kwargs) -> viewpanel.ViewPanel:
         """Adds a new :class:`.ViewPanel` to the centre of the frame, and a
         menu item allowing the user to configure the view.
@@ -486,13 +480,29 @@ class FSLeyesFrame(wx.Frame):
                             :meth:`viewPanelDefaultLayout` is called, to add
                             a default set of control panels to the view.
 
+        :arg location:      View location - one of ``wx.TOP``, ``wx.BOTTOM``,
+                            ``wx.LEFT``, or ``wx.RIGHT``. If not provided, the
+                            static :meth:`.ViewPanel.defaultLocation` method
+                            is called. If that method returns ``None``, a
+                            default layout strategy is used. This argument is
+                            ignored if there are no view panels already open,
+                            in which case the view is placed in the centre of
+                            the frame.
+
+        :arg size:          View size, as a proportion between 0 and 1 of the
+                            current frame size. Interpreted as width/height
+                            depending on the view location, and handled in
+                            the same manner as ``location``.
+
         :returns: The newly created ``ViewPanel``.
 
         All other arguments are passed to the ``__init__`` method of the child
         :class:`.DisplayContext` that is created for the new view.
         """
-        import fsleyes.views.plotpanel  as plotpanel
-        import fsleyes.views.shellpanel as shellpanel
+        if location not in (None, wx.TOP, wx.BOTTOM, wx.LEFT, wx.RIGHT):
+            raise ValueError(f'Invalid value for location: {location}')
+        if (size is not None) and (not 0 < size <= 1):
+            raise ValueError(f'Invalid value for size: {size}')
 
         if title is None:
             title = panelCls.title()
@@ -507,21 +517,20 @@ class FSLeyesFrame(wx.Frame):
         # The PaneInfo Name contains the panel
         # class name - this is used for saving
         # and restoring layouts .
-        name  = '{} {}'.format(panelCls.__name__, panelId)
-        title = '{} {}'.format(title,             panelId)
+        name  = f'{panelCls.__name__} {panelId}'
+        title = f'{title} {panelId}'
 
         childDC = displaycontext.DisplayContext(
             self.__overlayList,
             parent=self.__displayCtx,
             displayType=panelCls.displayType(),
+            selectedOverlay=self.__displayCtx.selectedOverlay,
             **kwargs)
 
         panel = panelCls(self.__mainPanel, self.__overlayList, childDC, self)
 
-        log.debug('Created new {} ({}) with DisplayContext {}'.format(
-            panelCls.__name__,
-            id(panel),
-            id(childDC)))
+        log.debug('Created new %s (%s) with DisplayContext %s',
+                  panelCls.__name__, id(panel), id(childDC))
 
         paneInfo = (aui.AuiPaneInfo()
                     .Name(name)
@@ -553,22 +562,16 @@ class FSLeyesFrame(wx.Frame):
         # If this is not the first view panel,
         # give it a sensible initial size.
         if panelId > 1:
-            width, height = self.GetClientSize().Get()
 
-            # PlotPanels are initially
-            # placed along the bottom
-            if isinstance(panel, plotpanel.PlotPanel):
-                paneInfo.Bottom().BestSize(width, height // 3)
+            width, height      = self.GetClientSize().Get()
+            loc, width, height = self.viewPanelLocationAndSize(
+                panelCls, width, height, location, size)
 
-            # As are ShellPanels, albeit
-            # a bit smaller
-            elif isinstance(panel, shellpanel.ShellPanel):
-                paneInfo.Bottom().BestSize(width, height // 5)
-
-            # Other panels (e.g. CanvasPanels)
-            # are placed on the right
-            else:
-                paneInfo.Right().BestSize(width // 2, height)
+            if   loc == wx.TOP:    paneInfo.Top()
+            elif loc == wx.BOTTOM: paneInfo.Bottom()
+            elif loc == wx.LEFT:   paneInfo.Left()
+            elif loc == wx.RIGHT:  paneInfo.Right()
+            paneInfo.BestSize(width, height)
 
         self.__viewPanels.append(panel)
         self.__viewPanelDCs[     panel] = childDC
@@ -590,6 +593,64 @@ class FSLeyesFrame(wx.Frame):
         return panel
 
 
+    @staticmethod
+    def viewPanelLocationAndSize(
+            panelCls      : Type[viewpanel.ViewPanel],
+            width         : int   = None,
+            height        : int   = None,
+            location      : int   = None,
+            size          : float = None):
+        """Called by :meth:`addViewPanel`. Calculates the width, height, and
+        location to be used for the new view.
+
+        :arg panelCls: The :class:`.ViewPanel` type being added.
+        :arg width:    Current frame width.
+        :arg height:   Current frame height.
+        :arg location: View location, passed to ``addViewPanel``.
+        :arg size:     View size, passed to ``addViewPanel``.
+
+        :returns:      A tuple containing
+                        - the location code (e.g. ``wx.TOP``, etc)
+                        - Width in pixels
+                        - Height in pixels
+        """
+
+        import fsleyes.views.plotpanel  as plotpanel
+        import fsleyes.views.shellpanel as shellpanel
+
+        # Get the panel's default location/size
+        # if not specified by caller
+        defaultLoc = panelCls.defaultLocation()
+
+        # Use a default strategy for views that
+        # don't implement defaultLocation
+        if defaultLoc is None:
+            # PlotPanels are initially placed along
+            # the bottom
+            if issubclass(panelCls, plotpanel.PlotPanel):
+                defaultLoc = (wx.BOTTOM, 0.333)
+
+            # As are ShellPanels, albeit
+            # a bit smaller
+            elif issubclass(panelCls, shellpanel.ShellPanel):
+                defaultLoc = (wx.BOTTOM, 0.2)
+
+            # Other panels (e.g. CanvasPanels)
+            # are placed on the right
+            else:
+                defaultLoc = (wx.RIGHT, 0.5)
+
+        # If the caller specified location/
+        # proportion, use them over the default
+        if location is None: location = defaultLoc[0]
+        if size     is None: size     = defaultLoc[1]
+
+        if   location in (wx.TOP, wx.BOTTOM): height = int(height * size)
+        elif location in (wx.LEFT, wx.RIGHT): width  = int(width  * size)
+
+        return location, width, height
+
+
     def viewPanelDefaultLayout(self, viewPanel : viewpanel.ViewPanel):
         """After a :class:`.ViewPanel` is added via the view menu, this method
         is called to perform some basic initialisation on the panel. This
@@ -682,6 +743,13 @@ class FSLeyesFrame(wx.Frame):
         for action, item in items:
             action.unbindWidget(item)
 
+            # Some actions here are bound to instance
+            # methods, and so cannot be re-created -
+            # we don't destroy actions which invoke a
+            # function that is owned by something else.
+            if action.instance is action:
+                action.destroy()
+
         # all items from self.__toolMenuActions
         # will also be returned by GetMenuItems,
         # along with other things (e.g. section
@@ -747,7 +815,7 @@ class FSLeyesFrame(wx.Frame):
             shortcut = shortcuts.actions.get((target, actionName))
 
             if shortcut is not None:
-                title = '{}\t{}'.format(title, shortcut)
+                title = f'{title}\t{shortcut}'
 
             if isinstance(actionObj, actions.ToggleAction):
                 itemType = wx.ITEM_CHECK
@@ -1039,11 +1107,8 @@ class FSLeyesFrame(wx.Frame):
                 if key is panel:
                     shortcutList.pop(key)
 
-        log.debug('Destroying {} ({}) and '
-                  'associated DisplayContext ({})'.format(
-                      type(panel).__name__,
-                      id(panel),
-                      id(dctx)))
+        log.debug('Destroying %s (%s) and associated DisplayContext (%s)',
+                  type(panel).__name__, id(panel), id(dctx))
 
         # Remove the view panel menu,
         # and make sure that the tools
@@ -1274,9 +1339,9 @@ class FSLeyesFrame(wx.Frame):
                 size     = self.GetSize().Get()
                 position = self.GetScreenPosition().Get()
 
-                log.debug('Saving size: {}'    .format(size))
-                log.debug('Saving position: {}'.format(position))
-                log.debug('Saving layout: {}'  .format(layout))
+                log.debug('Saving size: %s',     size)
+                log.debug('Saving position: %s', position)
+                log.debug('Saving layout: %s',   layout)
 
                 fslsettings.write('fsleyes.frame.size',     size)
                 fslsettings.write('fsleyes.frame.position', position)
@@ -1286,51 +1351,56 @@ class FSLeyesFrame(wx.Frame):
                 fslsettings.delete('fsleyes.frame.position')
                 fslsettings.delete('fsleyes.frame.layout')
 
-        # The close is going ahead. We assume that this
-        # EVT_CLOSE handler is the first that is called,
-        # and can thus control whether any the event
-        # gets propagated to other handlers.
-        ev.Skip()
+        # Disable the idle loop, so that the cleanup
+        # operations below have immediate effect
+        with idle.idleLoop.synchronous():
 
-        # It's nice to explicitly clean
-        # up our FSLeyesPanels, otherwise
-        # they'll probably complain
-        for vp in self.__viewPanels:
-            self.__onViewPanelClose(panel=vp, displaySync=False)
+            self.overlayList.clear()
 
-        self.__auiManager.UnInit()
-        self.__auiManager._frame = None
-        self.__auiManager        = None
+            # It's nice to explicitly clean
+            # up our FSLeyesPanels, otherwise
+            # they'll probably complain
+            for vp in list(self.__viewPanels):
+                self.__auiManager.DetachPane(vp)
+                self.__onViewPanelClose(panel=vp, displaySync=False)
 
-        # (not created) self.__overlayMenuActions
+            self.__auiManager.UnInit()
+            self.__auiManager._frame = None
+            self.__auiManager        = None
 
-        # Cleanly destroy all
-        # menu action objects
-        allactions = []
-        allactions.extend([a for a, _ in self.__viewMenuActions])
-        allactions.extend([a for a, _ in self.__layoutMenuActions])
-        allactions.extend([a for a, _ in self.__overlayMenuActions])
-        allactions.extend([a for a, _ in self.__toolsMenuActions])
-        allactions.extend(self.__menuActions.values())
-        for vpactions in self.__viewPanelMenuActions.values():
-            allactions.extend([a for a, _ in vpactions])
+            # Cleanly destroy all
+            # menu action objects
+            allactions = []
+            allactions.extend([a for a, _ in self.__viewMenuActions])
+            allactions.extend([a for a, _ in self.__layoutMenuActions])
+            allactions.extend([a for a, _ in self.__overlayMenuActions])
+            allactions.extend([a for a, _ in self.__toolsMenuActions])
+            allactions.extend(self.__menuActions.values())
+            for vpactions in self.__viewPanelMenuActions.values():
+                allactions.extend([a for a, _ in vpactions])
 
-        for action in allactions:
-            # actions associated with some object
-            # (e.g. a ViewPanel) will have already
-            # been destroyed by them
-            if not action.destroyed:
-                action.destroy()
+            for action in allactions:
+                # actions associated with some object
+                # (e.g. a ViewPanel) will have already
+                # been destroyed by them
+                if not action.destroyed:
+                    action.destroy()
 
-        self.__layoutMenuActions    = None
-        self.__viewPanelMenuActions = None
-        self.__toolsMenuActions     = None
-        self.__menuActions          = None
+            self.__layoutMenuActions    = None
+            self.__viewPanelMenuActions = None
+            self.__toolsMenuActions     = None
+            self.__menuActions          = None
 
-        # Deregister loadoverlay listener
-        # (registered in __init__)
-        import fsleyes.actions.loadoverlay as loadoverlay
-        loadoverlay.recentPathManager.deregister(self.__name)
+            # Deregister loadoverlay listener
+            # (registered in __init__)
+            import fsleyes.actions.loadoverlay as loadoverlay
+            loadoverlay.recentPathManager.deregister(self.__name)
+
+            if self.__closeHandlers is not None:
+                for h in self.__closeHandlers:
+                    h()
+
+            self.Destroy()
 
 
     def __restoreState(self, restore):
@@ -1402,14 +1472,14 @@ class FSLeyesFrame(wx.Frame):
             if ratio < 0.9:
 
                 log.debug('Intersection of saved frame area with available '
-                          'display area is too small ({}) - reverting to '
-                          'default frame size/position'.format(ratio))
+                          'display area is too small (%s) - reverting to '
+                          'default frame size/position', ratio)
 
                 size     = None
                 position = None
 
         if size is not None:
-            log.debug('Restoring previous size: {}'.format(size))
+            log.debug('Restoring previous size: %s', size)
             self.SetSize(size)
 
         else:
@@ -1419,18 +1489,18 @@ class FSLeyesFrame(wx.Frame):
             size    = list(wx.Display(0).GetGeometry().GetSize())
             size[0] = round(size[0] * 0.9)
             size[1] = round(size[1] * 0.9)
-            log.debug('Setting default frame size: {}'.format(size))
+            log.debug('Setting default frame size: %s', size)
             self.SetSize(size)
 
         if position is not None:
-            log.debug('Restoring previous position: {}'.format(position))
+            log.debug('Restoring previous position: %s', position)
             self.SetPosition(position)
         else:
             self.Centre()
 
         if restore:
             if layout is not None:
-                log.debug('Restoring previous layout: {}'.format(layout))
+                log.debug('Restoring previous layout: %s', layout)
 
                 try:
                     layouts.applyLayout(
@@ -1477,6 +1547,7 @@ class FSLeyesFrame(wx.Frame):
         # get the built-in app menu
         elif haveAppMenu:
             fsleyesMenu = menuBar.OSXGetAppleMenu()
+            fsleyesMenu.SetTitle('FSLeyes')
 
         # On OSX/wxPython, we fudge
         # things a bit - see below.
@@ -1592,9 +1663,9 @@ class FSLeyesFrame(wx.Frame):
         for action, title, shortcut, wxid in fsleyesActions:
 
             if shortcut is None: shortcut = ''
-            else:                shortcut = '\t{}'.format(shortcut)
+            else:                shortcut = f'\t{shortcut}'
 
-            title = '{}{}'.format(title, shortcut)
+            title = f'{title}{shortcut}'
 
             item = menu.Append(wxid, title)
             action.bindToWidget(self, wx.EVT_MENU, item)
@@ -1639,7 +1710,7 @@ class FSLeyesFrame(wx.Frame):
             shortcut = shortcuts.actions.get(action)
 
             if shortcut is not None:
-                title = '{}\t{}'.format(title, shortcut)
+                title = f'{title}\t{shortcut}'
 
             menuItem  = menu.Append(wx.ID_ANY, title)
             actionObj = action(self.__overlayList, self.__displayCtx, self)
@@ -1738,10 +1809,10 @@ class FSLeyesFrame(wx.Frame):
             shortcut = shortcuts.actions.get((self, action.__name__))
 
             if shortcut is None: shortcut = ''
-            else:                shortcut = '\t{}'.format(shortcut)
+            else:                shortcut = f'\t{shortcut}'
 
-            title = '{}{}'.format(strings.actions[self, action.__name__],
-                                  shortcut)
+            title = strings.actions[self, action.__name__]
+            title = f'{title}{shortcut}'
 
             item = menu.Append(wx.ID_ANY, title)
             action.bindToWidget(self, wx.EVT_MENU, item)
@@ -1786,12 +1857,12 @@ class FSLeyesFrame(wx.Frame):
         # Add a menu item to load each built-in layouts
         for layout in builtIns:
 
-            title    = strings.layouts.get(layout, layout)
+            title    = layouts.getLayoutTitle(layout)
             shortcut = shortcuts.actions.get((self, 'layouts', layout),
                                              None)
 
             if shortcut is not None:
-                title = '{}\t{}'.format(title, shortcut)
+                title = f'{title}\t{shortcut}'
 
             menuItem = menu.Append(wx.ID_ANY, title)
 
@@ -1806,8 +1877,8 @@ class FSLeyesFrame(wx.Frame):
         # Add a menu item to load each saved layout
         for layout in saved:
 
-            menuItem  = menu.Append(
-                wx.ID_ANY, strings.layouts.get(layout, layout))
+            title     = layouts.getLayoutTitle(layout)
+            menuItem  = menu.Append(wx.ID_ANY, title)
             actionObj = LoadLayoutAction(
                 self.overlayList, self.displayCtx, self, layout)
             actionObj.bindToWidget(self, wx.EVT_MENU, menuItem)
@@ -1857,7 +1928,8 @@ class FSLeyesFrame(wx.Frame):
                        SaveOverlayAction,
                        ReloadOverlayAction,
                        RemoveOverlayAction,
-                       'toggleOverlayVisibility']
+                       'toggleOverlayVisibility',
+                       'toggleOverlaysVisibility']
 
         for action in fileActions:
 
@@ -1887,7 +1959,7 @@ class FSLeyesFrame(wx.Frame):
                 actionObj = action(self.__overlayList, self.__displayCtx, self)
 
             if shortcut is not None:
-                title = '{}\t{}'.format(title, shortcut)
+                title = f'{title}\t{shortcut}'
 
             menuItem = menu.Append(wx.ID_ANY, title)
             actionObj.bindToWidget(self, wx.EVT_MENU, menuItem)
@@ -1942,7 +2014,7 @@ class FSLeyesFrame(wx.Frame):
             shortcut = shortcuts.actions.get(cls)
 
             if shortcut is not None:
-                name = '{}\t{}'.format(name, shortcut)
+                name = f'{name}\t{shortcut}'
 
             menuItem = menu.Append(wx.ID_ANY, name)
             actionObj.bindToWidget(self, wx.EVT_MENU, menuItem)
@@ -2060,9 +2132,10 @@ class FSLeyesFrame(wx.Frame):
 
         haveOverlays = overlay is not None
 
-        self.selectNextOverlay      .enabled = haveOverlays
-        self.selectPreviousOverlay  .enabled = haveOverlays
-        self.toggleOverlayVisibility.enabled = haveOverlays
+        self.selectNextOverlay       .enabled = haveOverlays
+        self.selectPreviousOverlay   .enabled = haveOverlays
+        self.toggleOverlayVisibility .enabled = haveOverlays
+        self.toggleOverlaysVisibility.enabled = haveOverlays
 
         if not self.__haveMenu:
             return
