@@ -190,17 +190,28 @@ import sys
 import logging
 import platform
 
-import fsl.utils.idle                     as idle
-import fsl.version                        as fslversion
+from   fsl.utils                      import idle
 from   fsl.utils.platform import platform as fslplatform
+import fsl.version                        as fslversion
 import fsleyes_widgets                    as fwidgets
-import fsleyes.gl.highdpi                 as highdpi
+from   fsleyes.utils                  import lazyimport
+
+import OpenGL  # noqa
 
 
 log = logging.getLogger(__name__)
 
 
-import OpenGL  # noqa
+GL = lazyimport('OpenGL.GL', 'fsleyes.gl.GL')
+"""This attribute contains a reference to the ``OpenGL.GL`` module. It is
+available to other parts of the code base as an alternative to importing
+the ``OpenGL.GL`` module directly.
+
+This module is lazily imported, because importing the ``OpenGL.GL`` module
+causes the GL backend (GLX, EGL, etc) to be selected and frozen (i.e. once
+selected, it cannot be changed). This is because we need an open window
+before we know which back end needs to be selected.
+"""
 
 
 # Make PyOpenGL throw an error, instead of implicitly
@@ -237,45 +248,73 @@ GL_RENDERER = None
 """
 
 
-def _selectPyOpenGLPlatform():
-    """Pyopengl sometimes doesn't select a suitable platform, so in some
-    circumstances we need to force things (but not if ``PYOPENGL_PLATFORM``
-    is already set in the environment).
+def selectPyOpenGLPlatform(offscreen=False):
+    """Set the ``PYOPENGL_PLATFORM`` environment variable. Called by the
+    :class:`GLContext` class during initialisation.  Not intended to be called
+    from any other location.
+
+    1. If ``PYOPENGL_PLATFORM`` is already set, this function does nothing.
+    2. If ``offscreen=True``, or we appear to be running on a headless system,
+       ``PYOPENGL_PLATFORM`` is set to ``'osmesa'``.
+    3. If running on a system other than Linux, this function does nothing.
+
+    When running on Linux/Wayland, PyOpenGL will select EGL as its backend
+    platform, which is not always the correct choice, as this depends on which
+    backend wxPython/wxWidgets has been compiled to use.  So in some
+    circumstances we need to force PyOpenGL to select a different platform, by
+    setting the ``PYOPENGL_PLATFORM`` environment variable.
+
+    Identifying the correct value is annoyingly complicated, and involves
+    determining whether wxPython/wxWidgets was compiled to use EGL or GLX for
+    its ``GLCanvas``.
+
+    For certain wxPython builds, this is easy:
+      - All GTK2 versions of wxPython/wxWidgets use GLX.
+      - All versions of wxPython older than 4.1.1 (wxWidgets ~= 3.1.4) use GLX.
+
+    Versions of wxPython 4.1.1 or newer may have been compiled to use either
+    GLX or EGL, and there is no direct means of knowing which. This function
+    uses a heuristic - testing whether the ``GLX_ARB_multisample`` extension
+    is available - it should be available when GLX has been initialised, but
+    not for systems using EGL.
+
+    Note that this function must be called after a wxPython frame and GL
+    context have been created (via :func:`fsleyes.gl.getGLContext`), but
+    before import of any PyOpenGL modules (with the exception of the top-level
+    ``OpenGL`` package). The ``PYOPENGL_PLATFORM`` variable must be set before
+    any ``OpenGL`` sub-modules are imported, and the selected platform cannot
+    be changed after import.
     """
+
+    offscreen = offscreen or (not fwidgets.canHaveGui())
+    plat      = fslplatform.os.lower()
+
+    # Do nothing if PYOPENGL_PLATFORM is already set
     if 'PYOPENGL_PLATFORM' in os.environ:
         return
 
-    # If no display, osmesa on all platforms
-    if not fwidgets.canHaveGui():
+    # Do nothing on windows
+    if plat == 'windows':
+        return
+
+    # If no display, osmesa on macOS/linux
+    if offscreen:
+        log.debug('Setting PYOPENGL_PLATFORM to osmesa')
         os.environ['PYOPENGL_PLATFORM'] = 'osmesa'
         return
 
     # We only need special handling on linux
-    if not fslplatform.os.lower() == 'linux':
+    if plat != 'linux':
         return
 
-    wxver  = fwidgets.wxVersion()
-    wxplat = fwidgets.wxPlatform()
+    # Test whether GLX_ARB_multisample is available -
+    # if it is, we assume that GLX has been initialised
+    ctx = getGLContext()
+    if ctx.canvas.IsExtensionSupported(b'GLX_ARB_multisample'): plat = 'x11'
+    else:                                                       plat = 'egl'
 
-    # Don't know what version of wxpython
-    # we have - don't do anything
-    if wxver is None:
-        return
-
-    # GTK3 versions of wxpython>=4.1.1 use EGL
-    # for GL initialisation. All older wxpython
-    # versions, and all GTK2 versions, use GLX.
-    #
-    # PyOpenGL>=3.1.6 will use EGL if it detects
-    # that it is running under Wayland.  But this
-    # breaks things if we have a GTK2 wxpython.
-    if wxplat == fwidgets.WX_GTK2:
-        os.environ['PYOPENGL_PLATFORM'] = 'x11'
-    elif fslversion.compareVersions(wxver, '4.1.1') >= 0:
-        os.environ['PYOPENGL_PLATFORM'] = 'egl'
-
-
-_selectPyOpenGLPlatform()
+    log.debug('Setting PYOPENGL_PLATFORM to %s', plat)
+    os.environ['PYOPENGL_PLATFORM'] = plat
 
 
 def glIsSoftwareRenderer():
@@ -387,7 +426,6 @@ def bootstrap(glVersion=None):
                     version will be used.
     """
 
-    import OpenGL.GL             as gl
     import fsleyes.gl.gl14       as gl14
     import fsleyes.gl.gl21       as gl21
     import fsleyes.gl.gl33       as gl33
@@ -399,7 +437,7 @@ def bootstrap(glVersion=None):
         return
 
     if glVersion is None:
-        glver = gl.glGetString(gl.GL_VERSION).decode('latin1').split()[0]
+        glver = GL.glGetString(GL.GL_VERSION).decode('latin1').split()[0]
         major, minor = [int(v) for v in glver.split('.')][:2]
     else:
         major, minor = glVersion
@@ -433,9 +471,9 @@ def bootstrap(glVersion=None):
 
         if not all(hasExtension(e, 2.1) for e in exts):
             log.warning('One of these OpenGL extensions is '
-                        'not available: [{}]. Falling back '
-                        'to an older OpenGL implementation.'
-                        .format(', '.join(exts)))
+                        'not available: [%s]. Falling back '
+                        'to an older OpenGL implementation.',
+                        ', '.join(exts))
             verstr = '1.4'
             glpkg  = gl14
 
@@ -467,9 +505,9 @@ def bootstrap(glVersion=None):
         dc.OVERLAY_TYPES['Image']       .remove('tensor')
         dc.OVERLAY_TYPES['Image']       .remove('mip')
 
-    renderer = gl.glGetString(gl.GL_RENDERER).decode('latin1')
-    log.debug('Using OpenGL {} implementation with renderer {}'.format(
-        verstr, renderer))
+    renderer = GL.glGetString(GL.GL_RENDERER).decode('latin1')
+    log.debug('Using OpenGL %s implementation with renderer %s',
+              verstr, renderer)
 
     # Import GL version-specific sub-modules
     globjects = ['glvolume', 'glrgbvolume', 'glrgbvector', 'gllinevector',
@@ -477,7 +515,7 @@ def bootstrap(glVersion=None):
                  'gltractogram']
 
     for globj in globjects:
-        modname = '{}_funcs'.format(globj)
+        modname = f'{globj}_funcs'
         setattr(thismod, modname, getattr(glpkg, modname, None))
 
     thismod.GL_VERSION       = str(glVersion)
@@ -496,7 +534,7 @@ def bootstrap(glVersion=None):
                   'lowering default performance settings.')
 
         import fsleyes.displaycontext as dc
-        dc.SceneOpts.highDpi = False
+        dc.SceneOpts.performance.setAttribute(None, 'default', 1)
 
 
 def getGLContext(**kwargs):
@@ -677,6 +715,10 @@ class GLContext:
         # dictated by PYOPENGL_PLATFORM.
         # Otherewise we use wx if possible.
         if offscreen and (osmesa or (not canHaveGui)):
+            # Make sure PyOpenGL is set to use
+            # osmesa as its backend before
+            # creating an osmesa GL context
+            selectPyOpenGLPlatform(offscreen=True)
             self.__createOSMesaContext()
             ready()
             return
@@ -717,6 +759,11 @@ class GLContext:
 
             self.__createWXGLContext(requestVersion=requestVersion)
 
+            # Once a GL context has been created, we
+            # can determine whether PyOpenGL should
+            # be using EGL or GLX
+            selectPyOpenGLPlatform()
+
             # If we've created and started
             # our own loop, kill it
             if self.__ownApp:
@@ -729,10 +776,8 @@ class GLContext:
                     ready()
 
                 except Exception as e:
-                    log.warning('GLContext callback function raised '
-                                '{}: {}'.format(type(e).__name__,
-                                                str(e)),
-                                                exc_info=True)
+                    log.warning('GLContext callback function raised %s: %s',
+                                type(e).__name__, str(e), exc_info=True)
                     if raiseErrors:
                         raise e
 
@@ -758,6 +803,14 @@ class GLContext:
             self.__app.MainLoop()
 
 
+    @property
+    def canvas(self):
+        """Return a reference to the ``wx.glcanvas.GLCanvas`` that was used
+        to create the GL context.
+        """
+        return self.__canvas
+
+
     def destroy(self):
         """Called by the module-level :func:`shutdown` function. If this is an
         on-screen context, the dummy canvas and frame that were created at
@@ -905,10 +958,7 @@ class GLContext:
         #  2. In 4.1.1, the GLContext interface changed
         #     so that we have to create and pass a
         #     GLContextAttrs object
-        #  3. Step 2 works on macOS, but on linux only
-        #     seems to work with GTK3/wayland/EGL builds
-        #     of wxpython.
-        #  4. In order to find out whether we can use
+        #  3. In order to find out whether we can use
         #     OpenGL 3.3, we have to create a GLContext,
         #     and check whether it isOK().
         #
@@ -928,12 +978,15 @@ class GLContext:
                 coreAttrs = wxgl.GLContextAttrs()
                 coreAttrs.OGLVersion(3, 3)
                 coreAttrs.CoreProfile()
+                coreAttrs.PlatformDefaults()
                 coreAttrs.EndList()
                 candidates.append({'ctxAttrs' : coreAttrs})
+
             # Request compat profile if we
             # can't request a core profile.
             compatAttrs = wxgl.GLContextAttrs()
             compatAttrs.CompatibilityProfile()
+            compatAttrs.PlatformDefaults()
             compatAttrs.EndList()
             candidates.append({'ctxAttrs' : compatAttrs})
 
@@ -944,7 +997,7 @@ class GLContext:
 
         log.debug('Creating wx.GLContext')
 
-        for  candidate in candidates:
+        for candidate in candidates:
             if other is not None:
                 ctx = wxgl.GLContext(target, other=other, **candidate)
             else:
@@ -954,6 +1007,7 @@ class GLContext:
                 break
             if ctx.IsOK():
                 break
+
         else:
             raise RuntimeError('Cannot create GL context')
 
@@ -973,7 +1027,6 @@ class GLContext:
         ``__context``.
         """
 
-        import OpenGL.GL              as gl
         import OpenGL.raw.osmesa.mesa as osmesa
         import OpenGL.arrays          as glarrays
 
@@ -983,10 +1036,10 @@ class GLContext:
         # for the off-screen context.
         buffer  = glarrays.GLubyteArray.zeros((640, 480, 4))
         context = osmesa.OSMesaCreateContextExt(
-            gl.GL_RGBA, 8, 4, 0, None)
+            GL.GL_RGBA, 8, 4, 0, None)
         osmesa.OSMesaMakeCurrent(context,
                                  buffer,
-                                 gl.GL_UNSIGNED_BYTE,
+                                 GL.GL_UNSIGNED_BYTE,
                                  640,
                                  480)
 
@@ -1015,7 +1068,7 @@ class OffScreenCanvasTarget:
                 id(self)))
 
 
-    def _setGLContext(self):
+    def setGLContext(self):
         """Configures the GL context to render to this canvas. """
         return getGLContext().setTarget(self)
 
@@ -1083,18 +1136,12 @@ class OffScreenCanvasTarget:
         pass
 
 
-    def EnableHighDPI(self):
-        """Does nothing. This canvas is for static (i.e. unchanging) rendering.
-        """
-        pass
-
-
     def draw(self):
         """Calls the :meth:`_draw` method, which must be provided by
         subclasses.
         """
 
-        self._setGLContext()
+        self.setGLContext()
         self._initGL()
         self.__target.shape = self.__width, self.__height
 
@@ -1109,7 +1156,7 @@ class OffScreenCanvasTarget:
         :meth:`draw`).
         """
 
-        self._setGLContext()
+        self.setGLContext()
         return self.__target.getBitmap()
 
 
@@ -1139,14 +1186,6 @@ class WXGLCanvasTarget:
     """
 
 
-    @staticmethod
-    def canToggleHighDPI():
-        """Return ``True`` if high-DPI scaling can be toggled, ``False``
-        otherwise.
-        """
-        return highdpi.needsToggling()
-
-
     @staticmethod
     def displayAttributes():
         """Used within ``__init__`` methods of ``WXGLCanvasTarget``
@@ -1207,6 +1246,7 @@ class WXGLCanvasTarget:
         """Create a ``WXGLCanvasTarget``. """
 
         import wx
+        import fsleyes.gl.textures.rendertexture as rendertexture
 
         context = getGLContext()
 
@@ -1219,19 +1259,26 @@ class WXGLCanvasTarget:
            'software' in GL_RENDERER.lower():
 
             log.debug('Creating separate GL context for '
-                      'WXGLCanvasTarget {}'.format(id(self)))
+                      'WXGLCanvasTarget %s', id(self))
 
             context = GLContext(other=context, target=self)
 
+        self.__name              = f'{type(self).__name__}.{id(self)}'
         self.__glReady           = False
         self.__freezeDraw        = False
         self.__freezeSwapBuffers = False
         self.__context           = context
 
-        if highdpi.needsToggling():
-            self.__dpiscale = 1.0
-        else:
-            self.__dpiscale = self.GetContentScaleFactor()
+        # configure the GL context for rendering
+        # to this canvas (must be done before any
+        # calls to gl functions)
+        self.setGLContext()
+
+        # All renders are directed to an off-screen
+        # frame buffer, which is then drawn to the
+        # main frame buffer. This makes screenshots
+        # much more reliable.
+        self.__fbo = rendertexture.RenderTexture(f'fbo_{id(self)}', 'cd')
 
         self.Bind(wx.EVT_PAINT,            self.__onPaint)
         self.Bind(wx.EVT_ERASE_BACKGROUND, self.__onEraseBackground)
@@ -1241,8 +1288,11 @@ class WXGLCanvasTarget:
         """Must be called when this WXGLCanvasTarget is no longer in use.
         Clears the GL rendering context target.
         """
+        self.__fbo.destroy()
         self.__context.setTarget(None)
+
         self.__context = None
+        self.__fbo     = None
 
 
     @property
@@ -1253,6 +1303,22 @@ class WXGLCanvasTarget:
         return self.__context is None
 
 
+    def __scheduleRefresh(self):
+        """Called internally whenever a request is made to refresh the canvas.
+
+        Schedules the refresh on the ``wx`` idle loop, in a way that minimises
+        redundant refreshes.
+        """
+        def doRefresh():
+            # The canvas may have been destroyed between the
+            # refresh being scheduled, and it being executed
+            if fwidgets.isalive(self):
+                self.__realDraw()
+
+        if not self.__freezeDraw:
+            idle.idle(doRefresh, name=self.__name, skipIfQueued=True)
+
+
     def __onEraseBackground(self, ev):
         """Called on ``wx.EVT_ERASE_BACKGROUND`` events. Does nothing. """
         pass
@@ -1262,16 +1328,10 @@ class WXGLCanvasTarget:
         """Called on ``wx.EVT_PAINT`` events. Schedules :meth:`Refresh`
         to be called on the idle loop.
         """
-
-        def doRefresh():
-            if fwidgets.isalive(self):
-                self.Refresh()
-
         # GL canvases do need to be refreshed
         # on EVT_PAINT events. If they are not,
         # the canvas will be corrupted.
-        if not self.__freezeDraw:
-            idle.idle(doRefresh)
+        self.__scheduleRefresh()
 
 
     def _initGL(self):
@@ -1337,15 +1397,31 @@ class WXGLCanvasTarget:
         # honours FreezeDraw and FreezeSwapBuffers.
         subClassDraw = self._draw
 
-        import OpenGL.GL as gl
+        import fsleyes.gl.routines as glroutines
 
         def drawWrapper(*a, **kwa):
 
             if self.destroyed:
                 return
 
+            # Bail out of premature draw calls
+            width, height = self.GetScaledSize()
+            if width == 0 or height == 0:
+                return
+
             if not self.__freezeDraw:
-                subClassDraw(*a, **kwa)
+
+                # Set the offscreen fbo size to the
+                # canvas window size and draw to it
+                self.__fbo.shape = width, height
+                with self.__fbo.target(0, 1, (0, 0, 0), (1, 1, 1)):
+                    subClassDraw(*a, **kwa)
+
+                # Draw the fbo contents to the main
+                # frame buffer
+                GL.glViewport(0, 0, width, height)
+                glroutines.clear((0, 0, 0, 0))
+                self.__fbo.drawOnBounds(0, -1, 1, -1, 1, 0, 1)
 
             if not self.__freezeSwapBuffers:
 
@@ -1354,7 +1430,7 @@ class WXGLCanvasTarget:
                 # set as the target for the GL
                 # context.
                 if not self.__freezeDraw:
-                    self._setGLContext()
+                    self.setGLContext()
                 self.SwapBuffers()
 
             # If not swapping the front/back
@@ -1363,7 +1439,7 @@ class WXGLCanvasTarget:
             # operations, otherwise we may lose
             # them.
             elif not self.__freezeDraw:
-                gl.glFlush()
+                GL.glFlush()
 
         def doInit(*a):
 
@@ -1391,7 +1467,16 @@ class WXGLCanvasTarget:
             wx.CallAfter(doInit)
 
 
-    def _setGLContext(self):
+    def IsShownOnScreen(self):
+        """Overrides ``wx.Window.IsShownOnScreen``. On Windows, delegates to
+        ``IsShown``.
+        """
+        if fslplatform.os.lower() == 'windows':
+            return super().IsShown()
+        return super().IsShownOnScreen()
+
+
+    def setGLContext(self):
         """Configures the GL context for drawing to this canvas.
 
         This method should be called before any OpenGL operations related to
@@ -1400,8 +1485,8 @@ class WXGLCanvasTarget:
         if not (fwidgets.isalive(self) and self.IsShownOnScreen()):
             return False
 
-        log.debug('Setting context target to {} ({})'.format(
-            type(self).__name__, id(self)))
+        log.debug('Setting context target to %s (%s)',
+                  type(self).__name__, id(self))
 
         return self.__context.setTarget(self)
 
@@ -1411,31 +1496,9 @@ class WXGLCanvasTarget:
         return self.GetClientSize().Get()
 
 
-    def GetContentScaleFactor(self):
-        """Overrides ``wx.Window.GetContentScaleFactor``.
-
-        Calls the base class implementation, except where wxpython 3.0.2.0 is
-        being used, as the method does not exist in that version. In this
-        case, 1.0 is returned.
-        """
-        try:
-            return float(super().GetContentScaleFactor())
-        except AttributeError:
-            return 1.0
-
-
     def GetScale(self):
         """Returns the current DPI scaling factor. """
-
-        scale = self.GetContentScaleFactor()
-
-        # Reset scaling factor in case the canvas
-        # window has moved between displays with
-        # different scaling factors
-        if scale == 1 and self.__dpiscale != 1:
-            self.EnableHighDPI(False)
-
-        return self.__dpiscale
+        return self.GetContentScaleFactor()
 
 
     def GetScaledSize(self):
@@ -1450,7 +1513,7 @@ class WXGLCanvasTarget:
 
     def Refresh(self, *a):
         """Triggers a redraw via the :meth:`_draw` method. """
-        self.__realDraw()
+        self.__scheduleRefresh()
 
 
     def FreezeDraw(self):
@@ -1482,61 +1545,60 @@ class WXGLCanvasTarget:
         only if ``FreezeSwapBuffers`` is not active.
         """
         if not self.__freezeSwapBuffers:
-            if self._setGLContext():
+            if self.setGLContext():
                 super(WXGLCanvasTarget, self).SwapBuffers()
 
 
-    def EnableHighDPI(self, enable=True):
-        """Attempts to enable/disable high-resolution rendering.
-        """
-
-        # We don't necessarily need to
-        # enable high-DPI support
-        if not highdpi.needsToggling():
-            return
-
-        if not self._setGLContext():
-            return
-
-        self.__dpiscale = 1.0
-
-        # If the display can't scale,
-        # (scale == 1) there's no point
-        # in enabling it.
-        scale  = self.GetContentScaleFactor()
-        enable = enable and scale > 1
-
-        # on macOS, we have to set
-        # scaling on the GL canvas
-        highdpi.toggle(self, enable)
-
-        if enable: self.__dpiscale = scale
-        else:      self.__dpiscale = 1.0
-
-
     def getBitmap(self):
         """Return a (width*height*4) shaped numpy array containing the
         rendered scene as an RGBA bitmap.
         """
-        import OpenGL.GL as gl
-        import numpy     as np
+        return self.__fbo.getBitmap()
 
-        self._setGLContext()
 
-        width, height = self.GetScaledSize()
 
-        # Make sure we're reading
-        # from the front buffer
-        gl.glReadBuffer(gl.GL_FRONT_LEFT)
+def glTypeName(gltype):
+    """Returns the name of a GL type.
 
-        bmp = gl.glReadPixels(
-            0, 0,
-            width, height,
-            gl.GL_RGBA,
-            gl.GL_UNSIGNED_BYTE)
+    Given an OpenGL type identifier, e.g. ``OpenGL.GL.GL_FLOAT``, returns
+    the type name as a string, e.g. ``'GL_FLOAT'``.
 
-        bmp = np.frombuffer(bmp, dtype=np.uint8)
-        bmp = bmp.reshape((height, width, 4))
-        bmp = np.flipud(bmp)
+    This function is only intended to be used for log messages and debugging.
+    """
 
-        return bmp
+    import OpenGL.GL.ARB.texture_float as arbtf
+
+    return {
+        GL.GL_UNSIGNED_BYTE       : 'GL_UNSIGNED_BYTE',
+        GL.GL_UNSIGNED_SHORT      : 'GL_UNSIGNED_SHORT',
+        GL.GL_FLOAT               : 'GL_FLOAT',
+
+        GL.GL_ALPHA               : 'GL_ALPHA',
+        GL.GL_RED                 : 'GL_RED',
+        GL.GL_LUMINANCE           : 'GL_LUMINANCE',
+        GL.GL_RGB                 : 'GL_RGB',
+        GL.GL_RGBA                : 'GL_RGBA',
+
+        GL.GL_LUMINANCE8          : 'GL_LUMINANCE8',
+        GL.GL_LUMINANCE16         : 'GL_LUMINANCE16',
+
+        # These non-standard types are only
+        # used when OpenGL <= 2.1 is in use,
+        arbtf.GL_LUMINANCE16F_ARB : 'GL_LUMINANCE16F',
+        arbtf.GL_LUMINANCE32F_ARB : 'GL_LUMINANCE32F',
+
+        GL.GL_R32F                : 'GL_R32F',
+
+        GL.GL_ALPHA8              : 'GL_ALPHA8',
+
+        GL.GL_R8                  : 'GL_R8',
+        GL.GL_R16                 : 'GL_R16',
+
+        GL.GL_RGB8                : 'GL_RGB8',
+        GL.GL_RGB16               : 'GL_RGB16',
+        GL.GL_RGB32F              : 'GL_RGB32F',
+
+        GL.GL_RGBA8               : 'GL_RGBA8',
+        GL.GL_RGBA16              : 'GL_RGBA16',
+        GL.GL_RGBA32F             : 'GL_RGBA32F',
+    }[gltype]
