@@ -35,11 +35,12 @@ following annotation types are defined:
 import time
 import logging
 
-import numpy       as np
-import OpenGL.GL   as gl
+import numpy as np
 
 import fsl.transform.affine     as affine
+
 import fsleyes_props            as props
+from   fsleyes.utils        import lazyimport
 import fsleyes.gl               as fslgl
 import fsleyes.gl.globject      as globject
 import fsleyes.gl.shaders       as shaders
@@ -50,6 +51,9 @@ import fsleyes.gl.text          as gltext
 import fsleyes.gl.textures.data as texdata
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 log = logging.getLogger(__name__)
 
 
@@ -167,66 +171,56 @@ class Annotations(props.HasProperties):
         return shader
 
 
-    def line(self, *args, **kwargs):
-        """Queues a line for drawing - see the :class:`Line` class. """
+    def __create(self, atype, *args, **kwargs):
+        """Used by the annotation creation methods below. Creates and
+        enqueues an annotation of type ``atype``.
+        """
+        self.__canvas.setGLContext()
         hold  = kwargs.pop('hold',  False)
         fixed = kwargs.pop('fixed', True)
-        obj   = Line(self, *args, **kwargs)
+        obj   = atype(self, *args, **kwargs)
         return self.obj(obj, hold, fixed)
 
 
+    def line(self, *args, **kwargs):
+        """Queues a line for drawing - see the :class:`Line` class. """
+        return self.__create(Line, *args, **kwargs)
+
+
     def arrow(self, *args, **kwargs):
         """Queues an arrow for drawing - see the :class:`Arrow` class. """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = Arrow(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(Arrow, *args, **kwargs)
 
 
     def point(self, *args, **kwargs):
         """Queues a point for drawing - see the :class:`Point` class. """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = Point(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(Point, *args, **kwargs)
 
 
     def rect(self, *args, **kwargs):
         """Queues a rectangle for drawing - see the :class:`Rectangle` class.
         """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = Rect(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(Rect, *args, **kwargs)
 
 
     def ellipse(self, *args, **kwargs):
         """Queues a circle for drawing - see the :class:`Ellipse` class.
         """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = Ellipse(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(Ellipse, *args, **kwargs)
 
 
     def selection(self, *args, **kwargs):
         """Queues a selection for drawing - see the :class:`VoxelSelection`
         class.
         """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = VoxelSelection(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(VoxelSelection, *args, **kwargs)
 
 
     def text(self, *args, **kwargs):
         """Queues a text annotation for drawing - see the
         :class:`TextAnnotation` class.
         """
-        hold  = kwargs.pop('hold',  False)
-        fixed = kwargs.pop('fixed', True)
-        obj   = TextAnnotation(self, *args, **kwargs)
-        return self.obj(obj, hold, fixed)
+        return self.__create(TextAnnotation, *args, **kwargs)
 
 
     def obj(self, obj, hold=False, fixed=True):
@@ -1179,7 +1173,7 @@ class VoxelSelection(AnnotationObject):
     def __init__(self,
                  annot,
                  selection,
-                 opts,
+                 overlay,
                  offsets=None,
                  **kwargs):
         """Create a ``VoxelSelection`` annotation.
@@ -1190,8 +1184,8 @@ class VoxelSelection(AnnotationObject):
         :arg selection: A :class:`.selection.Selection` instance which defines
                         the voxels to be highlighted.
 
-        :arg opts:      A :class:`.NiftiOpts` instance which is used
-                        for its voxel-to-display transformation matrices.
+        :arg overlay:   A :class:`.Nifti` instance which is used for its
+                        voxel-to-display transformation matrices.
 
         :arg offsets:   If ``None`` (the default), the ``selection`` must have
                         the same shape as the image data being
@@ -1208,9 +1202,10 @@ class VoxelSelection(AnnotationObject):
             offsets = [0, 0, 0]
 
         self.__selection = selection
-        self.__opts      = opts
+        self.__overlay   = overlay
         self.__offsets   = offsets
-        texName = '{}_{}'.format(type(self).__name__, id(selection))
+
+        texName = f'{type(self).__name__}_{id(selection)}'
 
         ndims = texdata.numTextureDims(selection.shape)
 
@@ -1254,11 +1249,12 @@ class VoxelSelection(AnnotationObject):
             glresources.delete(self.__texture.name)
 
         if self.__shader is not None:
+
             self.__shader.destroy()
 
+        self.__overlay = None
         self.__shader  = None
         self.__texture = None
-        self.__opts    = None
 
 
     @property
@@ -1275,7 +1271,7 @@ class VoxelSelection(AnnotationObject):
         """
 
         xax, yax, zax = axes
-        opts          = self.__opts
+        opts          = self.annot.canvas.displayCtx.getOpts(self.__overlay)
         texture       = self.__texture
         shape         = self.__selection.getSelection().shape
         displayToVox  = opts.getTransform('display', 'voxel')
