@@ -17,23 +17,30 @@ colour bar is created.
 
 import logging
 
-import numpy     as np
-import OpenGL.GL as gl
+import numpy as np
 
-import fsleyes_props                         as props
-import fsleyes.controls.colourbar            as cbar
-import fsleyes.gl.textures                   as textures
+import fsleyes_props              as props
+from   fsl.utils              import idle
+import fsleyes.controls.colourbar as cbar
+from   fsleyes.gl             import textures
+import fsleyes.gl.routines        as glroutines
+from   fsleyes.utils          import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 class ColourBarCanvas(props.HasProperties):
     """Contains logic to render a colour bar as an OpenGL texture. """
 
 
-    highDpi = props.Boolean(default=False)
-    """Scale colour bar canvas for high-resolution screens. """
+    barSize = props.Percentage(default=100)
+    """Size of the colour bar along its major axis, as a proportion of
+    the available space.
+    """
 
 
     def __init__(self, overlayList, displayCtx):
@@ -44,9 +51,9 @@ class ColourBarCanvas(props.HasProperties):
         self.__tex  = None
         self.__name = '{}_{}'.format(self.__class__.__name__, id(self))
         self.__cbar = cbar.ColourBar(overlayList, displayCtx)
-        self.__cbar.register(self.__name, self.__updateTexture)
+        self.__cbar.register(self.__name, self.updateColourBarTexture)
 
-        self.addListener('highDpi', self.__name, self.__highDpiChanged)
+        self.addListener('barSize', self.__name, self.updateColourBarTexture)
 
 
     @property
@@ -57,10 +64,16 @@ class ColourBarCanvas(props.HasProperties):
         return self.__cbar
 
 
-    def __updateTexture(self, *a):
+    def updateColourBarTexture(self, *a):
         """Called whenever the colour bar texture needs to be updated. """
-        self._genColourBarTexture()
-        self.Refresh()
+
+        def update():
+            self.__genColourBarTexture()
+            self.Refresh()
+
+        name = '{}_updateColourBarTexture'.format(id(self))
+
+        idle.idle(update, name=name, skipIfQueued=True)
 
 
     def _initGL(self):
@@ -70,15 +83,7 @@ class ColourBarCanvas(props.HasProperties):
 
         Generates the colour bar texture.
         """
-        self._genColourBarTexture()
-
-
-    def __highDpiChanged(self, *a):
-        """Called when the :attr:`highDpi` property changes. Calls the
-        :meth:`.GLCanvasTarget.EnableHighDPI` method.
-        """
-        self.EnableHighDPI(self.highDpi)
-        self.__updateTexture()
+        self.__genColourBarTexture()
 
 
     def destroy(self):
@@ -88,17 +93,22 @@ class ColourBarCanvas(props.HasProperties):
         """
         self.__cbar.deregister(self.__name)
         self.__cbar.destroy()
-        self.__tex.destroy()
+
+        if self.__tex is not None:
+            self.__tex.destroy()
+
+        self.removeListener('barSize', self.__name)
+
         self.__tex  = None
         self.__cbar = None
 
 
-    def _genColourBarTexture(self):
+    def __genColourBarTexture(self):
         """Retrieves a colour bar bitmap from the :class:`.ColourBar`, and
         copies it to a :class:`.Texture2D`.
         """
 
-        if not self._setGLContext():
+        if not self.setGLContext():
             return
 
         # we may have already
@@ -111,45 +121,49 @@ class ColourBarCanvas(props.HasProperties):
         if w < 50 or h < 50:
             return
 
+        if self.__cbar.orientation == 'vertical': h = h * self.barSize / 100.0
+        else:                                     w = w * self.barSize / 100.0
+
         scale  = self.GetScale()
         bitmap = self.__cbar.colourBar(w, h, scale)
 
         # The bitmap has shape W*H*4, but
         # Texture2D instances need it in
         # shape 4*W*H
-        bitmap = np.fliplr(bitmap).transpose([2, 0, 1])
-
         if bitmap is None:
             return
 
-        if self.__tex is None:
-            self.__tex = textures.Texture2D('{}_{}'.format(
-                type(self).__name__, id(self)), gl.GL_LINEAR)
+        bitmap = np.fliplr(bitmap).transpose([2, 0, 1])
 
-        self.__tex.setData(bitmap)
-        self.__tex.refresh()
+        if self.__tex is None:
+            self.__tex = textures.Texture2D(
+                f'{type(self).__name__}_{id(self)}', interp=gl.GL_LINEAR)
+
+        self.__tex.set(data=bitmap)
 
 
     def _draw(self):
         """Renders the colour bar texture using all available canvas space."""
 
-        if self.__tex is None or not self._setGLContext():
+        if self.__tex is None or not self.setGLContext():
             return
 
         width, height = self.GetScaledSize()
+        xform         = glroutines.ortho2D(0, 1, 0, 1, -1, 1)
 
         # viewport
         gl.glViewport(0, 0, width, height)
-        gl.glMatrixMode(gl.GL_PROJECTION)
-        gl.glLoadIdentity()
-        gl.glOrtho(0, 1, 0, 1, -1, 1)
-        gl.glMatrixMode(gl.GL_MODELVIEW)
-        gl.glLoadIdentity()
+        glroutines.clear(self.__cbar.bgColour)
 
-        gl.glClearColor(*self.__cbar.bgColour)
-        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT)
-        gl.glEnable(gl.GL_BLEND)
-        gl.glBlendFunc(gl.GL_SRC_ALPHA, gl.GL_ONE_MINUS_SRC_ALPHA)
-        gl.glShadeModel(gl.GL_FLAT)
+        xmin, xmax = 0, 1
+        ymin, ymax = 0, 1
+        off        = (100 - self.barSize) / 100.0
 
-        self.__tex.drawOnBounds(0, 0, 1, 0, 1, 0, 1)
+        if self.colourBar.orientation == 'vertical':
+            ymin += off / 2.0
+            ymax -= off / 2.0
+        else:
+            xmin += off / 2.0
+            xmax -= off / 2.0
+
+        self.__tex.drawOnBounds(0, xmin, xmax, ymin, ymax, 0, 1, xform)
