@@ -49,6 +49,8 @@ def compileShaders(self):
     if self.shader is not None:
         self.shader.destroy()
 
+    opts = self.opts
+
     if self.threedee: frag = 'glvolume_3d'
     else:             frag = 'glvolume'
 
@@ -69,22 +71,19 @@ def compileShaders(self):
 
     if self.threedee:
 
-        if   self.opts.clipMode == 'intersection': clipMode = 1
-        elif self.opts.clipMode == 'union':        clipMode = 2
-        elif self.opts.clipMode == 'complement':   clipMode = 3
-        else:                                      clipMode = 0
+        if   opts.clipMode == 'intersection': clipMode = 1
+        elif opts.clipMode == 'union':        clipMode = 2
+        elif opts.clipMode == 'complement':   clipMode = 3
+        else:                                 clipMode = 0
 
-        constants['numSteps']        = self.opts.numInnerSteps
+        numSteps = min((opts.numInnerSteps, opts.getNumSteps()))
+        constants['numSteps']        = numSteps
         constants['clipMode']        = clipMode
         constants['numClipPlanes']   = self.opts.numClipPlanes
         texes[    'startingTexture'] = 5
         texes[    'depthTexture']    = 6
 
-    self.shader = shaders.ARBPShader(vertSrc,
-                                     fragSrc,
-                                     shaders.getShaderDir(),
-                                     texes,
-                                     constants)
+    self.shader = shaders.ARBPShader(vertSrc, fragSrc, texes, constants)
 
 
 def updateShaderState(self):
@@ -106,7 +105,7 @@ def updateShaderState(self):
     # into a value between 0 and 1, suitable
     # for looking up an appropriate colour
     # in the 1D colour map texture.
-    voxValXform = affine.concat(self.colourTexture.getCoordinateTransform(),
+    voxValXform = affine.concat(self.cmapTexture.getCoordinateTransform(),
                                 self.imageTexture.voxValXform)
     voxValXform = [voxValXform[0, 0], voxValXform[0, 3], 0, 0]
 
@@ -117,21 +116,25 @@ def updateShaderState(self):
     imageIsClip = 1 if opts.clipImage is None else -1
 
     # modalpha not applied in 3D
-    modAlpha    = 1 if opts.modulateAlpha         else -1
-    imageIsMod  = 1 if opts.modulateImage is None else -1
-    modXform    = self.getModulateValueXform()
+    modAlpha     = 1 if opts.modulateAlpha         else -1
+    imageIsMod   = 1 if opts.modulateImage is None else -1
+    fullModXform = self.getModulateValueXform()
 
     imgXform = self.imageTexture.invVoxValXform
-    if opts.clipImage is None: clipXform = imgXform
-    else:                      clipXform = self.clipTexture.invVoxValXform
+    if opts.clipImage:     clipXform = self.clipTexture.invVoxValXform
+    else:                  clipXform = imgXform
+    if opts.modulateImage: modXform  = self.modulateTexture.invVoxValXform
+    else:                  modXform  = imgXform
 
-    clipLo  = opts.clippingRange[0] * clipXform[0, 0] + clipXform[0, 3]
-    clipHi  = opts.clippingRange[1] * clipXform[0, 0] + clipXform[0, 3]
-    texZero = 0.0                   * imgXform[ 0, 0] + imgXform[ 0, 3]
+    clipLo   = opts.clippingRange[0] * clipXform[0, 0] + clipXform[0, 3]
+    clipHi   = opts.clippingRange[1] * clipXform[0, 0] + clipXform[0, 3]
+    texZero  = 0.0                   * imgXform[ 0, 0] + imgXform[ 0, 3]
+    modZero  = 0.0                   * modXform[ 0, 0] + modXform[ 0, 3]
+    clipZero = 0.0                   * clipXform[0, 0] + clipXform[0, 3]
 
     clipping = [clipLo, clipHi, invClip, imageIsClip]
-    modulate = [modXform[0, 0], modXform[0, 3], modAlpha, imageIsMod]
-    negCmap  = [useNegCmap, texZero, modAlpha, 0]
+    modulate = [fullModXform[0, 0], fullModXform[0, 3], modAlpha, imageIsMod]
+    negCmap  = [useNegCmap, texZero, clipZero, modZero]
 
     # disable clip image/modalpha for 3D
     if self.threedee:
@@ -162,11 +165,10 @@ def updateShaderState(self):
     return changed
 
 
-def preDraw(self, xform=None, bbox=None):
+def preDraw(self):
     """Prepares to draw a slice from the given :class:`.GLVolume` instance. """
 
     self.shader.load()
-    self.shader.loadAtts()
 
     if isinstance(self, glvolume.GLVolume):
         clipCoordXform = self.getAuxTextureXform('clip')
@@ -175,58 +177,68 @@ def preDraw(self, xform=None, bbox=None):
         self.shader.setVertParam('modCoordXform',  modCoordXform)
 
 
-def draw2D(self, zpos, axes, xform=None, bbox=None):
+def draw2D(self, canvas, zpos, axes, xform=None):
     """Draws a 2D slice of the image at the given Z location. """
 
-    vertices, voxCoords, texCoords = self.generateVertices2D(
+    shader                 = self.shader
+    bbox                   = canvas.viewport
+    projmat                = canvas.projectionMatrix
+    viewmat                = canvas.viewMatrix
+    vertices, _, texCoords = self.generateVertices2D(
         zpos, axes, bbox=bbox)
 
-    if xform is not None:
-        vertices = affine.transform(vertices, xform)
+    if xform is None: xform = affine.concat(projmat, viewmat)
+    else:             xform = affine.concat(projmat, viewmat, xform)
 
-    vertices = np.array(vertices, dtype=np.float32).ravel('C')
+    vertices = affine.transform(vertices, xform)
+    vertices = np.asarray(vertices, dtype=np.float32).ravel('C')
 
     # Voxel coordinates are calculated
-    # in the vertex program
-    self.shader.setAtt('texCoord', texCoords)
+    # in the vertex program, so we
+    # only pass texCoords and vertices
+    shader.setAtt('texCoord', texCoords)
 
-    with glroutines.enabled((gl.GL_VERTEX_ARRAY)):
+    with shader.loadedAtts(), glroutines.enabled((gl.GL_VERTEX_ARRAY)):
         gl.glVertexPointer(3, gl.GL_FLOAT, 0, vertices)
         gl.glDrawArrays(gl.GL_TRIANGLES, 0, 6)
 
 
-def draw3D(self, xform=None, bbox=None):
+def draw3D(self, canvas, xform=None):
     """Draws the image in 3D on the canvas.
 
     :arg self:    The :class:`.GLVolume` object which is managing the image
                   to be drawn.
 
+    :arg canvas:  The canvas being drawn on
+
     :arg xform:   A 4*4 transformation matrix to be applied to the vertex
                   data.
-
-    :arg bbox:    An optional bounding box.
     """
     opts    = self.opts
-    canvas  = self.canvas
     display = self.display
     shader  = self.shader
     shape   = self.image.shape
     proj    = canvas.projectionMatrix
+    mvpmat  = canvas.mvpMatrix
+    mvmat   = canvas.viewMatrix
+    bbox    = canvas.viewport
     src     = self.renderTexture1
     dest    = self.renderTexture2
     w, h    = src.shape
 
-    vertices, voxCoords, texCoords = self.generateVertices3D(bbox)
-    rayStep, texform               = opts.calculateRayCastSettings(xform, proj)
+    if xform is not None:
+        mvpmat = affine.concat(mvpmat, xform)
+        mvmat  = affine.concat(mvmat,  xform)
+
+    vertices, _, texCoords = self.generateVertices3D(bbox)
+    rayStep, texform       = opts.calculateRayCastSettings(mvmat, proj)
 
     rayStep = affine.transformNormal(
         rayStep, self.imageTexture.texCoordXform(shape))
     texform = affine.concat(
         texform, self.imageTexture.invTexCoordXform(shape))
 
-    if xform is not None:
-        vertices = affine.transform(vertices, xform)
-
+    vertices = affine.transform(vertices, mvpmat)
     vertices = np.array(vertices, dtype=np.float32).ravel('C')
 
     outerLoop  = opts.getNumOuterSteps()
@@ -253,9 +265,18 @@ def draw3D(self, xform=None, bbox=None):
     # Disable blending - we want each
     # loop to replace the contents of
     # the texture, not blend into it!
-    with glroutines.enabled((gl.GL_VERTEX_ARRAY)), \
+    with shader.loadedAtts(), \
+         glroutines.enabled((gl.GL_VERTEX_ARRAY)), \
          glroutines.disabled((gl.GL_BLEND)):
 
+        # The depth value for a fragment will
+        # not necessary be set at the same
+        # time that the fragment colour is
+        # set, so we need to use <= for depth
+        # testing so that unset depth values
+        # do not cause depth clipping.
+        gl.glDepthFunc(gl.GL_LEQUAL)
+
         for i in range(outerLoop):
 
             settings    = list(settings)
@@ -280,35 +301,42 @@ def draw3D(self, xform=None, bbox=None):
 
             dest, src = src, dest
 
-    shader.unloadAtts()
+    gl.glDepthFunc(gl.GL_LESS)
+
     shader.unload()
 
     self.renderTexture1 = src
     self.renderTexture2 = dest
+    self.drawClipPlanes(xform=xform)
 
 
-def drawAll(self, axes, zposes, xforms):
+
+def drawAll(self, canvas, axes, zposes, xforms):
     """Draws mutltiple slices of the given image at the given Z position,
     applying the corresponding transformation to each of the slices.
     """
 
     nslices   = len(zposes)
+    shader    = self.shader
+    projmat   = canvas.projectionMatrix
+    viewmat   = canvas.viewMatrix
     vertices  = np.zeros((nslices * 6, 3), dtype=np.float32)
     texCoords = np.zeros((nslices * 6, 3), dtype=np.float32)
     indices   = np.arange(nslices * 6,     dtype=np.uint32)
 
     for i, (zpos, xform) in enumerate(zip(zposes, xforms)):
 
-        v, vc, tc = self.generateVertices2D(zpos, axes)
+        xform    = affine.concat(projmat, viewmat, xform)
+        v, _, tc = self.generateVertices2D(zpos, axes)
 
         vertices[ i * 6: i * 6 + 6, :] = affine.transform(v, xform)
         texCoords[i * 6: i * 6 + 6, :] = tc
 
     vertices = vertices.ravel('C')
 
-    self.shader.setAtt('texCoord', texCoords)
+    shader.setAtt('texCoord', texCoords)
 
-    with glroutines.enabled((gl.GL_VERTEX_ARRAY)):
+    with shader.loadedAtts(), glroutines.enabled((gl.GL_VERTEX_ARRAY)):
         gl.glVertexPointer(3, gl.GL_FLOAT, 0, vertices)
         gl.glDrawElements(gl.GL_TRIANGLES,
                           nslices * 6,
@@ -316,12 +344,8 @@ def drawAll(self, axes, zposes, xforms):
                           indices)
 
 
-def postDraw(self, xform=None, bbox=None):
+def postDraw(self):
     """Cleans up the GL state after drawing from the given :class:`.GLVolume`
     instance.
     """
-    self.shader.unloadAtts()
     self.shader.unload()
-
-    if self.threedee:
-        self.drawClipPlanes(xform=xform)
