@@ -10,6 +10,7 @@ types in an OpenGL 2.1 compatible manner.
 
 
 from . import glvolume_funcs       # noqa
+from . import glrgbvolume_funcs    # noqa
 from . import glrgbvector_funcs    # noqa
 from . import gllinevector_funcs   # noqa
 from . import glmesh_funcs         # noqa
@@ -17,3 +18,5 @@ from . import glmask_funcs         # noqa
 from . import gllabel_funcs        # noqa
 from . import gltensor_funcs       # noqa
 from . import glsh_funcs           # noqa
+from . import glmip_funcs          # noqa
+from . import gltractogram_funcs   # noqa
