@@ -20,13 +20,14 @@ import fsleyes.gl.textures       as textures
 import fsleyes.gl.routines       as glroutines
 import fsleyes.gl.resources      as glresources
 import fsleyes.gl.shaders.filter as glfilter
-from . import                       glimageobject
+import fsleyes.gl.glimageobject  as glimageobject
+import fsleyes.gl.globject       as globject
 
 
 log = logging.getLogger(__name__)
 
 
-class GLMask(glimageobject.GLImageObject):
+class GLMask(glimageobject.GLImageObject, globject.GLObject):
     """The ``GLMask`` class encapsulates logic to render an :class:`.Image`
     instance as a binary mask in OpenGL.
 
@@ -86,11 +87,12 @@ class GLMask(glimageobject.GLImageObject):
         :arg displayCtx:  The :class:`.DisplayContext` managing the scene.
         :arg threedee:    2D or 3D rendering
         """
-        glimageobject.GLImageObject.__init__(self,
-                                             image,
-                                             overlayList,
-                                             displayCtx,
-                                             threedee)
+        globject.GLObject.__init__(self,
+                                   image,
+                                   overlayList,
+                                   displayCtx,
+                                   threedee)
+        glimageobject.GLImageObject.__init__(self)
 
         # The shader attribute will be created
         # by the glmask_funcs module
@@ -220,6 +222,7 @@ class GLMask(glimageobject.GLImageObject):
 
         opts     = self.opts
         texName  = '{}_{}' .format(type(self).__name__, id(self.image))
+        interp   = opts.interpolation
         unsynced = (opts.getParent() is None or
                     not opts.isSyncedToParent('volume'))
 
@@ -233,8 +236,11 @@ class GLMask(glimageobject.GLImageObject):
             self.imageTexture.deregister(self.name)
             glresources.delete(self.imageTexture.name)
 
-        if opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                            interp = gl.GL_LINEAR
+        if interp == 'true_spline': prefilter = textures.splineFilter
+        else:                       prefilter = None
+
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
 
         self.imageTexture = glresources.get(
             texName,
@@ -242,6 +248,7 @@ class GLMask(glimageobject.GLImageObject):
             texName,
             self.image,
             interp=interp,
+            prefilter=prefilter,
             volume=opts.index()[3:],
             notify=False)
 
@@ -315,8 +322,7 @@ class GLMask(glimageobject.GLImageObject):
             gl.glClear(gl.GL_COLOR_BUFFER_BIT)
             fslgl.glmask_funcs.draw2D(self, rtex, zpos, axes, xform)
 
-        if xform is None: xform = affine.concat(projmat, viewmat)
-        else:             xform = affine.concat(projmat, viewmat, xform)
+        xform = affine.concat(projmat, viewmat)
 
         # Run the texture through an edge detection
         # filter, drawing the result to screen
@@ -390,9 +396,15 @@ class GLMask(glimageobject.GLImageObject):
         """Called when the :attr:`.MaskOpts.interpolation` changes. Updates the
         image texture.
         """
-        if self.opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                                 interp = gl.GL_LINEAR
-        self.imageTexture.set(interp=interp)
+        interp = self.opts.interpolation
+
+        if interp == 'true_spline': prefilter = textures.splineFilter
+        else:                       prefilter = None
+
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
+
+        self.imageTexture.set(interp=interp, prefilter=prefilter)
 
 
     def __imageTextureChanged(self, *a):
