@@ -8,21 +8,21 @@
 RGB(A) :class:`.Image` overlays.
 """
 
+import numpy                    as np
+import OpenGL.GL                as gl
 
-import numpy                as np
-import OpenGL.GL            as gl
+import fsl.utils.idle           as idle
+import fsl.transform.affine     as affine
 
-import fsl.utils.idle       as idle
-import fsl.transform.affine as affine
-
-import fsleyes.gl           as fslgl
-import fsleyes.gl.routines  as glroutines
-import fsleyes.gl.textures  as textures
-import fsleyes.gl.resources as glresources
-from . import                  glimageobject
+import fsleyes.gl               as fslgl
+import fsleyes.gl.routines      as glroutines
+import fsleyes.gl.textures      as textures
+import fsleyes.gl.resources     as glresources
+import fsleyes.gl.glimageobject as glimageobject
+import fsleyes.gl.globject      as globject
 
 
-class GLRGBVolume(glimageobject.GLImageObject):
+class GLRGBVolume(glimageobject.GLImageObject, globject.GLObject):
     """The ``GLRGBVolume`` class is used to render RGB(A) :class:`.Image`
     overlays. The RGB(A) value at each voxel is directly used as the colour
     for that voxel.
@@ -37,11 +37,12 @@ class GLRGBVolume(glimageobject.GLImageObject):
         :arg displayCtx:  The :class:`.DisplayContext` managing the scene.
         :arg threedee:    2D or 3D rendering
         """
-        glimageobject.GLImageObject.__init__(self,
-                                             image,
-                                             overlayList,
-                                             displayCtx,
-                                             threedee)
+        globject.GLObject.__init__(self,
+                                   image,
+                                   overlayList,
+                                   displayCtx,
+                                   threedee)
+        glimageobject.GLImageObject.__init__(self)
 
         self.shader       = None
         self.imageTexture = None
@@ -150,9 +151,13 @@ class GLRGBVolume(glimageobject.GLImageObject):
 
         texName = '{}_{}' .format(type(self).__name__, id(self.image))
         nvals   = self.overlay.nvals
+        interp  = self.opts.interpolation
 
-        if self.opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                                 interp = gl.GL_LINEAR
+        if interp == 'true_spline': prefilter = textures.splineFilter
+        else:                       prefilter = None
+
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
 
         if nvals == 1:
             nvals = self.overlay.shape[-1]
@@ -164,6 +169,7 @@ class GLRGBVolume(glimageobject.GLImageObject):
             self.overlay,
             nvals=nvals,
             interp=interp,
+            prefilter=prefilter,
             notify=False)
 
         self.imageTexture.register(self.name, self.__imageTextureChanged)
@@ -174,12 +180,15 @@ class GLRGBVolume(glimageobject.GLImageObject):
         Updates the image texture.
         """
 
-        opts = self.opts
+        interp = self.opts.interpolation
 
-        if opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                            interp = gl.GL_LINEAR
+        if interp == 'true_spline': prefilter = textures.splineFilter
+        else:                       prefilter = None
 
-        self.imageTexture.set(interp=interp)
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
+
+        self.imageTexture.set(interp=interp, prefilter=prefilter)
 
 
     def __imageTextureChanged(self, *a):
@@ -238,7 +247,7 @@ class GLRGBVolume(glimageobject.GLImageObject):
 
 
     def draw2D(self, canvas, *args, **kawrgs):
-        """Calls :func:`.glrgbvolume_funcs.draw2D`. """
+        """Calls ``glrgbvolume_funcs.draw2D``. """
         with glroutines.enabled((gl.GL_CULL_FACE)):
             gl.glPolygonMode(gl.GL_FRONT_AND_BACK, gl.GL_FILL)
             gl.glCullFace(gl.GL_BACK)
@@ -247,7 +256,7 @@ class GLRGBVolume(glimageobject.GLImageObject):
 
 
     def drawAll(self, *args, **kawrgs):
-        """Calls :func:`.glrgbvolume_funcs.drawAll`. """
+        """Calls ``glrgbvolume_funcs.drawAll``. """
         fslgl.glrgbvolume_funcs.drawAll(self, *args, **kawrgs)
 
 
