@@ -14,6 +14,7 @@ coefficients which represent fibre orientation distributions (FODs).  The
 
 
 import               logging
+import               warnings
 
 import numpy      as np
 
@@ -86,7 +87,7 @@ class GLSH(glvector.GLVectorBase):
     """
 
 
-    def __init__(self, image, displayCtx, canvas, threedee):
+    def __init__(self, image, overlayList, displayCtx, threedee):
         """Create a ``GLSH`` object.
 
 
@@ -95,19 +96,23 @@ class GLSH(glvector.GLVectorBase):
         instances, and sets up shaders.
 
 
-        :arg image:      The :class:`.Image` instance
-        :arg displayCtx: The :class:`.DisplayContext` managing the scene.
-        :arg canvas:     The canvas doing the drawing.
-        :arg threedee:   Set up for 2D or 3D rendering.
+        :arg image:       The :class:`.Image` instance
+        :arg overlayList: The :class:`.OverlayList`
+        :arg displayCtx:  The :class:`.DisplayContext` managing the scene.
+        :arg threedee:    Set up for 2D or 3D rendering.
         """
 
         self.shader     = None
         self.radTexture = None
 
+        # These are updated in the
+        # __shStateChanged method.
+        self.__shParams = None
+
         # This texture gets updated on
         # draw calls, so we want it to
         # run on the main thread.
-        self.radTexture = textures.Texture3D('{}_radTexture'.format('fuck'),
+        self.radTexture = textures.Texture3D('{}_radTexture'.format(id(self)),
                                              threaded=False)
 
         # Usin preinit here (see GLVectorBase)
@@ -118,8 +123,8 @@ class GLSH(glvector.GLVectorBase):
         glvector.GLVectorBase.__init__(
             self,
             image,
+            overlayList,
             displayCtx,
-            canvas,
             threedee,
             preinit=self.compileShaders,
             init=self.__shStateChanged)
@@ -142,6 +147,27 @@ class GLSH(glvector.GLVectorBase):
         glvector.GLVectorBase.destroy(self)
 
 
+    def ready(self):
+        """Overrides :class:`.GLVectorBase.ready`. Returns ``True`` when this
+        ``GLSH`` is ready to be drawn.
+        """
+        return self.__shParams is not None \
+            and glvector.GLVectorBase.ready(self)
+
+
+    def getDataResolution(self, xax, yax, width, height):
+        """Overrides :meth:`.GLVector.getDataResolution`. Returns a pixel
+        resolution suitable for off-screen rendering of this ``GLSH``.
+        """
+
+        res       = list(glvector.GLVector.getDataResolution(
+            self, xax, yax, width, height))
+        res[xax] *= 20
+        res[yax] *= 20
+
+        return res
+
+
     def addListeners(self):
         """Overrides :meth:`.GLVectorBase.addListeners`.
 
@@ -162,8 +188,9 @@ class GLSH(glvector.GLVectorBase):
         opts.addListener('size',            name, self.updateShaderState)
         opts.addListener('lighting',        name, self.updateShaderState)
         opts.addListener('orientFlip',      name, self.updateShaderState)
-        opts.addListener('radiusThreshold', name, self.notify)
         opts.addListener('colourMode',      name, self.updateShaderState)
+        opts.addListener('radiusThreshold', name, self.notify)
+        opts.addListener('normalise',       name, self.notify)
 
 
     def removeListeners(self):
@@ -181,8 +208,9 @@ class GLSH(glvector.GLVectorBase):
         opts.removeListener('size',            name)
         opts.removeListener('lighting',        name)
         opts.removeListener('orientFlip',      name)
-        opts.removeListener('radiusThreshold', name)
         opts.removeListener('colourMode',      name)
+        opts.removeListener('radiusThreshold', name)
+        opts.removeListener('normalise',       name)
 
 
     def compileShaders(self, *a):
@@ -314,6 +342,10 @@ class GLSH(glvector.GLVectorBase):
         which all radii are less than the threshold are removed from the
         ``voxels`` array.
 
+        If :attr:`.SHOpts.normalise` is ``True``, the radii within each voxel
+        are normalised to lie between 0 and 0.5, so that they fit within the
+        voxel.
+
         This function returns a tuple containing:
 
           - The ``voxels`` array. If ``SHOpts.radiusThreshold == 0``,
@@ -336,10 +368,8 @@ class GLSH(glvector.GLVectorBase):
                   (x >= shape[0]) | \
                   (y >= shape[1]) | \
                   (z >= shape[2])
-
-        x = np.array(x[~out], dtype=np.int32)
-        y = np.array(y[~out], dtype=np.int32)
-        z = np.array(z[~out], dtype=np.int32)
+        voxels  = np.asarray(voxels[~out, :], dtype=np.uint32)
+        x, y, z = voxels.T
 
         # The dot product of the SH parameters with
         # the SH coefficients for a single voxel gives
@@ -350,19 +380,27 @@ class GLSH(glvector.GLVectorBase):
         # *all* voxels.
         params = self.__shParams
         vols   = self.__coefVolumeMask()
-        coefs  = self.image.nibImage.get_data()[x, y, z, vols]
-        radii  = np.dot(params, coefs.T)
+        coefs  = self.image.data[x, y, z, vols]
+        radii  = np.dot(coefs, params.T)
 
         # Remove sub-threshold voxels/radii
         if opts.radiusThreshold > 0:
-            aboveThres = np.any(radii >= opts.radiusThreshold, axis=0)
-            radii      = np.array(radii[:, aboveThres])
-            voxels     = np.array(voxels[aboveThres, :])
+            aboveThres = np.any(radii >= opts.radiusThreshold, axis=1)
+            radii      = radii[ aboveThres, :]
+            voxels     = voxels[aboveThres, :]
 
         # No voxels - nothing to do
         if voxels.shape[0] == 0:
             return [], [0, 0, 0]
 
+        # Normalise within voxel if necessary
+        if opts.normalise:
+            rmin = radii.min(axis=1)
+            rmax = radii.max(axis=1)
+            with warnings.catch_warnings():
+                warnings.filterwarnings('ignore')
+                radii = ((radii.T - rmin) / (2 * (rmax - rmin))).T
+
         # The radii are interpreted as a 1D vector
         # containing the radii for every vertex
         # in every voxel. Because we're using an
@@ -408,12 +446,26 @@ class GLSH(glvector.GLVectorBase):
             radTexShape[imax] /= divisor
             radTexShape[imin] *= divisor
 
-        # Resize and reshape the
-        # radius array as needed
+        # Resize and reshape the radius array as needed.
+        # Radii starts off as a (voxels, vertices) array.
+        # We flatten it to 1D (by voxels, i.e. with
+        # vertices the fastest changing)
+        radii      = radii.reshape(-1)
         radTexSize = np.prod(radTexShape)
-        if radTexSize != radii.size:
-            radii.resize(radTexSize)
 
+        # if the texture size needs to be bigger,
+        # create a new array, and copy the radii
+        # data into it
+        if radTexSize != radii.size:
+            tmp = np.zeros(radTexSize, dtype=radii.dtype)
+            tmp[:radii.size] = radii
+            radii = tmp
+
+        # reshape to the texture size, making
+        # sure that the first dimension is the
+        # fastest changing (as this is what
+        # OpenGL requires). This ensures that
+        # the data remains contiguous in memory
         radii = radii.reshape(radTexShape, order='F')
 
         # Copy the data to the texture
@@ -428,11 +480,10 @@ class GLSH(glvector.GLVectorBase):
         ``False`` otherwise.
         """
         return (self.radTexture is not None and
-                self.radTexture.ready()     and
                 glvector.GLVectorBase.texturesReady(self))
 
 
-    def preDraw(self, xform=None, bbox=None):
+    def preDraw(self):
         """Overrides :meth:`.GLVectorBase.preDraw`.  Binds textures, and calls
         :func:`.glsh_funcs.preDraw`.
         """
@@ -442,27 +493,33 @@ class GLSH(glvector.GLVectorBase):
         # be called through draw, and if radTexture
         # is not the most recently bound texture,
         # the update will fail.
-        glvector.GLVectorBase.preDraw(self, xform, bbox)
+        glvector.GLVectorBase.preDraw(self)
         self.radTexture.bindTexture(gl.GL_TEXTURE4)
-        fslgl.glsh_funcs.preDraw(self, xform, bbox)
+        fslgl.glsh_funcs.preDraw(self)
 
 
     def draw2D(self, *args, **kwargs):
-        """Overrides :meth:`.GLObject.draw`. Calls :func:`.glsh_funcs.draw`.
+        """Overrides :meth:`.GLObject.draw2D`. Calls
+        :func:`.glsh_funcs.draw2D`.
         """
         fslgl.glsh_funcs.draw2D(self, *args, **kwargs)
 
 
     def draw3D(self, *args, **kwargs):
-        """Overrides :meth:`.GLObject.draw`. Calls :func:`.glsh_funcs.draw`.
+        """Does nothing."""
+
+
+    def drawAll(self, *args, **kwargs):
+        """Overrides :meth:`.GLObject.draw2D`. Calls
+        :func:`.glsh_funcs.drawAll`.
         """
-        fslgl.glsh_funcs.draw3D(self, *args, **kwargs)
+        fslgl.glsh_funcs.drawAll(self, *args, **kwargs)
 
 
-    def postDraw(self, xform=None, bbox=None):
+    def postDraw(self):
         """Overrides :meth:`.GLVectorBase.postDraw`.  Unbinds textures, and
         calls :func:`.glsh_funcs.postDraw`.
         """
-        glvector.GLVectorBase.postDraw(self, xform, bbox)
+        glvector.GLVectorBase.postDraw(self)
         self.radTexture.unbindTexture()
-        fslgl.glsh_funcs.postDraw(self, xform, bbox)
+        fslgl.glsh_funcs.postDraw(self)
