@@ -7,24 +7,25 @@
 """This module provides the :class:`GLVectorBase` and :class:`GLVector`
 classes. The ``GLVectorBase`` class encapsulate the logic for rendering
 overlays which contain directional data, and the ``GLVector`` class
-specifically conatins logic for displaying ``X*Y*Z*3`` :class:`.Image`
-overlays.
+specifically conatins logic for displaying :class:`.Image` overlays with
+shape ``X*Y*Z*3``, or of type ``NIFTI_TYPE_RGB24``.
 """
 
 
-import numpy               as np
-import OpenGL.GL           as gl
+import functools                as ft
 
-import fsl.data.image      as fslimage
-import fsl.utils.async     as async
-import fsl.utils.transform as transform
-import fsleyes.colourmaps  as fslcm
-from . import resources    as glresources
-from . import                 textures
-from . import                 glimageobject
+import numpy                    as np
+import OpenGL.GL                as gl
+
+import fsl.utils.idle           as idle
+import fsleyes.colourmaps       as fslcm
+import fsleyes.gl.resources     as glresources
+import fsleyes.gl.textures      as textures
+import fsleyes.gl.glimageobject as glimageobject
+import fsleyes.gl.globject      as globject
 
 
-class GLVectorBase(glimageobject.GLImageObject):
+class GLVectorBase(glimageobject.GLImageObject, globject.GLObject):
     """The :class:`GLVectorBase` class encapsulates the logic for rendering
     :class:`.Nifti` overlay types which represent directional data (and which
     are described by a :class:`.VectorOpts` instance).  The ``GLVectorBase``
@@ -49,7 +50,7 @@ class GLVectorBase(glimageobject.GLImageObject):
        are specified by the :attr:`.VectorOpts.xColour`,
        :attr:`.VectorOpts.yColour`, and :attr:`.VectorOpts.zColour`
        properties. If the image being displayed contains directional data
-       (i.e. is a ``X*Y*Z*3`` vector image), you should use the
+       (e.g. is a ``X*Y*Z*3`` vector image), you should use the
        :class:`GLVector` class.
 
      - Each voxel is coloured according to the values contained in another
@@ -58,9 +59,10 @@ class GLVectorBase(glimageobject.GLImageObject):
        :attr:`.VectorOpts.colourImage` and :attr:`.VectorOpts.cmap` properties.
 
 
-    In either case, the brightness of each vector colour may be modulated by
-    another image, specified by the :attr:`.VectorOpts.modulateImage`
-    property.  This modulation image is stored as a 3D single-channel
+    When coloured by orientation, the brightness or transparency of each
+    vector colour may be modulated by another image, specified by the
+    :attr:`.VectorOpts.modulateImage` and :attr:`.VectorOpts.modulateMode`
+    properties.  This modulation image is stored as a 3D single-channel
     :class:`.ImageTexture`.
 
 
@@ -86,8 +88,8 @@ class GLVectorBase(glimageobject.GLImageObject):
 
     def __init__(self,
                  overlay,
+                 overlayList,
                  displayCtx,
-                 canvas,
                  threedee,
                  init=None,
                  preinit=None):
@@ -103,70 +105,61 @@ class GLVectorBase(glimageobject.GLImageObject):
             instances, so the textures and geometry can be updated when
             necessary.
 
-        :arg overlay:        A :class:`.Nifti` object.
+        :arg overlay:     A :class:`.Nifti` object.
 
-        :arg displayCtx:     A :class:`.DisplayContext` object which describes
-                             how the overlay is to be displayed.
+        :arg overlayList: The :class:`.OverlayList`
 
-        :arg canvas:         The canvas doing the drawing.
+        :arg displayCtx:  A :class:`.DisplayContext` object which describes
+                          how the overlay is to be displayed.
 
-        :arg threedee:       2D or 3D rendering.
+        :arg threedee:    2D or 3D rendering.
 
-        :arg init:           An optional function to be called when all of the
-                             :class:`.ImageTexture` instances associated with
-                             this ``GLVectorBase`` have been initialised.
+        :arg init:        An optional function to be called when all of the
+                          :class:`.ImageTexture` instances associated with
+                          this ``GLVectorBase`` have been initialised.
 
-        :arg preinit:        An optional functiono be called after this
-                             ``GLVectorBase`` has configured itself, but
-                             *before* ``init`` is called. Used by
-                             :class:`GLVector`.
+        :arg preinit:     An optional functiono be called after this
+                          ``GLVectorBase`` has configured itself, but
+                          *before* ``init`` is called. Used by
+                          :class:`GLVector`.
         """
 
-        glimageobject.GLImageObject.__init__(self,
-                                             overlay,
-                                             displayCtx,
-                                             canvas,
-                                             threedee)
+        globject.GLObject.__init__(self,
+                                   overlay,
+                                   overlayList,
+                                   displayCtx,
+                                   threedee)
+        glimageobject.GLImageObject.__init__(self)
 
         name = self.name
-
-        self.cmapTexture     = textures.ColourMapTexture('{}_cm'.format(name))
-
-        self.shader          = None
-        self.modulateImage   = None
-        self.clipImage       = None
-        self.colourImage     = None
-        self.modulateOpts    = None
-        self.clipOpts        = None
-        self.colourOpts      = None
-        self.modulateTexture = None
-        self.clipTexture     = None
-        self.colourTexture   = None
-
-        # Make sure we are registered with the
-        # auxillary images if any of them are set.
         opts = self.opts
 
-        if opts.colourImage   is not None: self.registerAuxImage('colour')
-        if opts.modulateImage is not None: self.registerAuxImage('modulate')
-        if opts.clipImage     is not None: self.registerAuxImage('clip')
+        self.cmapTexture = textures.ColourMapTexture('{}_cm'.format(name))
+        self.shader      = None
+        self.auxmgr      = textures.AuxImageTextureManager(
+            self, colour=None, modulate=None, clip=None)
+
+        self.registerAuxImage('colour',   opts.colourImage)
+        self.registerAuxImage('modulate', opts.modulateImage)
+        self.registerAuxImage('clip',     opts.clipImage)
 
         self.addListeners()
+        self.refreshColourMapTexture()
 
         def initWrapper():
+            # Sometimes (e.g. during testing),
+            # this GLObject might get destroyed
+            # before this function is called
+            if self.destroyed:
+                return
             if init is not None:
                 init()
             self.notify()
 
-        self.refreshColourMapTexture()
-        self.refreshAuxTexture('modulate')
-        self.refreshAuxTexture('clip')
-        self.refreshAuxTexture('colour')
-
         if preinit is not None:
             preinit()
 
-        async.idleWhen(initWrapper, self.texturesReady)
+        idle.idleWhen(initWrapper, self.texturesReady)
 
 
     def destroy(self):
@@ -176,31 +169,39 @@ class GLVectorBase(glimageobject.GLImageObject):
         """
 
         self.cmapTexture.destroy()
-
-        for tex in (self.modulateTexture,
-                    self.clipTexture,
-                    self.colourTexture):
-            tex.deregister(self.name)
-            glresources.delete(tex.getTextureName())
-
+        self.auxmgr.destroy()
         self.removeListeners()
-        self.deregisterAuxImage('modulate')
-        self.deregisterAuxImage('clip')
-        self.deregisterAuxImage('colour')
 
-        self.modulateTexture = None
-        self.clipTexture     = None
-        self.colourTexture   = None
-        self.modulateImage   = None
-        self.clipImage       = None
-        self.colourImage     = None
-        self.modulateOpts    = None
-        self.clipOpts        = None
-        self.colourOpts      = None
+        self.cmapTexture = None
+        self.auxmgr      = None
 
         glimageobject.GLImageObject.destroy(self)
 
 
+    @property
+    def modulateTexture(self):
+        """Returns the :class:`.ImageTexture` for the
+        :attr:`.VectorOpts.modulateImage`.
+        """
+        return self.auxmgr.texture('modulate')
+
+
+    @property
+    def clipTexture(self):
+        """Returns the :class:`.ImageTexture` for the
+        :attr:`.VectorOpts.clipImage`.
+        """
+        return self.auxmgr.texture('clip')
+
+
+    @property
+    def colourTexture(self):
+        """Returns the :class:`.ImageTexture` for the
+        :attr:`.VectorOpts.colourImage`.
+        """
+        return self.auxmgr.texture('colour')
+
+
     def ready(self):
         """Returns ``True`` if this ``GLVectorBase`` is ready to be drawn,
         ``False`` otherwise.
@@ -209,15 +210,8 @@ class GLVectorBase(glimageobject.GLImageObject):
 
 
     def texturesReady(self):
-        """Returns ``True`` if all of the textures are ready, ``False``
-        otherwise.
-        """
-        return (self.modulateTexture is not None and
-                self.clipTexture     is not None and
-                self.colourTexture   is not None and
-                self.modulateTexture.ready()     and
-                self.clipTexture    .ready()     and
-                self.colourTexture  .ready())
+        """Calls :meth:`.AuxImageTextureManager.texturesReady`. """
+        return self.auxmgr.texturesReady()
 
 
     def addListeners(self):
@@ -240,12 +234,14 @@ class GLVectorBase(glimageobject.GLImageObject):
         opts   .addListener('suppressY',     name, self.asyncUpdateShaderState)
         opts   .addListener('suppressZ',     name, self.asyncUpdateShaderState)
         opts   .addListener('suppressMode',  name, self.asyncUpdateShaderState)
+        opts   .addListener('modulateMode',  name, self.asyncUpdateShaderState)
         opts   .addListener('cmap',          name, self.__cmapPropChanged)
         opts   .addListener('modulateImage', name, self.__modImageChanged)
         opts   .addListener('clipImage',     name, self.__clipImageChanged)
         opts   .addListener('colourImage',   name, self.__colourImageChanged)
         opts   .addListener('clippingRange', name, self.asyncUpdateShaderState)
         opts   .addListener('modulateRange', name, self.asyncUpdateShaderState)
+        opts   .addListener('colourRange',   name, self.__cmapPropChanged)
         opts   .addListener('transform',     name, self.notify)
 
 
@@ -269,6 +265,7 @@ class GLVectorBase(glimageobject.GLImageObject):
         opts   .removeListener('suppressY',     name)
         opts   .removeListener('suppressZ',     name)
         opts   .removeListener('suppressMode',  name)
+        opts   .removeListener('modulateMode',  name)
         opts   .removeListener('modulateImage', name)
         opts   .removeListener('clipImage',     name)
         opts   .removeListener('colourImage',   name)
@@ -299,7 +296,7 @@ class GLVectorBase(glimageobject.GLImageObject):
 
     def asyncUpdateShaderState(self, *args, **kwargs):
         """Calls :meth:`updateShaderState` and then :meth:`.Notifier.notify`, using
-        :func:`.async.idleWhen` function to make sure that it is only called
+        :func:`.idle.idleWhen` function to make sure that it is only called
         when :meth:`ready` returns ``True``.
         """
 
@@ -309,138 +306,10 @@ class GLVectorBase(glimageobject.GLImageObject):
             if self.updateShaderState() or alwaysNotify:
                 self.notify()
 
-        async.idleWhen(func,
-                       self.ready,
-                       name=self.name,
-                       skipIfQueued=True)
-
-
-    def registerAuxImage(self, which):
-        """Called when the :attr:`.VectorOpts.modulateImage`,
-        :attr:`.VectorOpts.clipImage`, or :attr:`.VectorOpts.colourImage`
-        properties change. Registers a listener with the
-        :attr:`.NiftiOpts.volume` property of the modulate/clip/colour image,
-        so the modulate/clip/colour textures can be updated when the image
-        volume changes.
-        """
-
-        imageAttr = '{}Image'  .format(which)
-        optsAttr  = '{}Opts'   .format(which)
-        texAttr   = '{}Texture'.format(which)
-
-        image = getattr(self.opts, imageAttr)
-        tex   = getattr(self,      texAttr)
-
-        if image is None or image == 'none':
-            image = None
-
-        setattr(self, optsAttr,  None)
-        setattr(self, imageAttr, image)
-
-        if image is None:
-            return
-
-        opts = self.opts.displayCtx.getOpts(image)
-
-        setattr(self, optsAttr, opts)
-
-        def volumeChange(*a):
-            tex.set(volume=opts.index()[3:])
-            self.asyncUpdateShaderState(alwaysNotify=True)
-
-        # We set overwrite=True, because
-        # the modulate/clip/colour images
-        # may be the same.
-        opts.addListener('volume',
-                         self.name,
-                         volumeChange,
-                         overwrite=True,
-                         weak=False)
-
-
-    def deregisterAuxImage(self, which):
-        """Called when the :attr:`.VectorOpts.modulateImage`,
-        :attr:`.VectorOpts.clipImage` or :attr:`.VectorOpts.colourImage`
-        properties change.  Deregisters the :attr:`.NiftiOpts.volume`
-        listener that was registered in :meth:`registerAuxImage`.
-        """
-
-        imageAttr = '{}Image'.format(which)
-        optsAttr  = '{}Opts' .format(which)
-
-        opts = getattr(self, optsAttr)
-
-        if opts is not None:
-            opts.removeListener('volume', self.name)
-
-        setattr(self, imageAttr, None)
-        setattr(self, optsAttr,  None)
-
-
-    def refreshAuxTexture(self, which, interp=gl.GL_NEAREST):
-        """Called when the :attr`.VectorOpts.modulateImage`,
-        :attr`.VectorOpts.clipImage`, or :attr`.VectorOpts.colourImage`
-        properties changes.  Reconfigures the modulation/clip/colour
-        :class:`.ImageTexture`. If no image is selected, a 'dummy' texture is
-        creatad, which contains all white values (and which result in the
-        auxillary textures having no effect).
-
-        The ``interp`` argument can be used to set the initial interpolation
-        type (``GL_NEAREST`` or ``GL_LINEAR``).
-        """
-
-        imageAttr = '{}Image'  .format(which)
-        optsAttr  = '{}Opts'   .format(which)
-        texAttr   = '{}Texture'.format(which)
-
-        image = getattr(self, imageAttr)
-        opts  = getattr(self, optsAttr)
-        tex   = getattr(self, texAttr)
-
-        if tex is not None:
-            tex.deregister(self.name)
-            glresources.delete(tex.getTextureName())
-
-        if image is None:
-
-            textureData    = np.zeros((5, 5, 5), dtype=np.uint8)
-            textureData[:] = 255
-            image          = fslimage.Image(textureData)
-            norm           = None
-
-        else:
-            norm = image.dataRange
-
-        texName = '{}_{}_{}_{}'.format(
-            type(self).__name__, id(self.image), id(image), which)
-
-        if opts is not None:
-            unsynced = (opts.getParent() is None or
-                        not opts.isSyncedToParent('volume'))
-
-            # TODO If unsynced, this GLVectorBase needs to
-            # update the mod/clip/colour textures whenever
-            # their volume property changes.
-            # Right?
-            if unsynced:
-                texName = '{}_unsync_{}'.format(texName, id(opts))
-
-        if opts is not None: volume = opts.index()[3:]
-        else:                volume = 0
-
-        tex = glresources.get(
-            texName,
-            textures.ImageTexture,
-            texName,
-            image,
-            normaliseRange=norm,
-            volume=volume,
-            notify=False,
-            interp=interp)
-
-        tex.register(self.name, self.__textureChanged)
-
-        setattr(self, texAttr, tex)
+        idle.idleWhen(func,
+                      self.ready,
+                      name=self.name,
+                      skipIfQueued=True)
 
 
     def refreshColourMapTexture(self, colourRes=256):
@@ -454,11 +323,8 @@ class GLVectorBase(glimageobject.GLImageObject):
         display = self.display
         opts    = self.opts
 
-        if self.colourImage is not None:
-            dmin, dmax = self.colourImage.dataRange
-
-        else:
-            dmin, dmax = 0.0, 1.0
+        if opts.colourImage is not None: dmin, dmax = opts.colourRange
+        else:                            dmin, dmax = 0.0, 1.0
 
         dmin, dmax = fslcm.briconToDisplayRange(
             (dmin, dmax),
@@ -470,53 +336,6 @@ class GLVectorBase(glimageobject.GLImageObject):
                              displayRange=(dmin, dmax))
 
 
-    def getVectorColours(self):
-        """Prepares the colours that represent each direction.
-
-
-        Returns:
-          - a ``numpy`` array of size ``(3, 4)`` containing the
-            RGBA colours that correspond to the ``x``, ``y``, and ``z``
-            vector directions.
-
-          - A ``numpy`` array of shape ``(4, 4)`` which encodes a scale
-            and offset to be applied to the vector value before it
-            is combined with the colours, encoding the current
-            brightness and contrast settings.
-        """
-        display = self.display
-        opts    = self.opts
-        bri     = display.brightness / 100.0
-        con     = display.contrast   / 100.0
-        alpha   = display.alpha      / 100.0
-
-        colours       = np.array([opts.xColour, opts.yColour, opts.zColour])
-        colours[:, 3] = alpha
-
-        if   opts.suppressMode == 'white':       suppress = [1, 1, 1, alpha]
-        elif opts.suppressMode == 'black':       suppress = [0, 0, 0, alpha]
-        elif opts.suppressMode == 'transparent': suppress = [0, 0, 0, 0]
-
-        # Transparent suppression
-        if opts.suppressX: colours[0, :] = suppress
-        if opts.suppressY: colours[1, :] = suppress
-        if opts.suppressZ: colours[2, :] = suppress
-
-        # Scale/offset for brightness/contrast.
-        # Note: This code is a duplicate of
-        # that found in ColourMapTexture.
-        lo, hi = fslcm.briconToDisplayRange((0, 1), bri, con)
-
-        if hi == lo: scale = 0.0000000000001
-        else:        scale = hi - lo
-
-        xform = np.identity(4, dtype=np.float32)
-        xform[0, 0] = 1.0 / scale
-        xform[0, 3] = -lo * xform[0, 0]
-
-        return colours, xform
-
-
     def getClippingRange(self):
         """Returns the :attr:`clippingRange`, suitable for use in the fragment
         shader. The returned values are transformed into the clip image
@@ -566,19 +385,10 @@ class GLVectorBase(glimageobject.GLImageObject):
         to transform texture coordinates from the vector image to the specified
         auxillary image (``'clip'``, ``'modulate'`` or ``'colour'``).
         """
-        opts     = self.opts
-        auxImage = getattr(self, '{}Image'.format(which), None)
-        auxOpts  = getattr(self, '{}Opts' .format(which), None)
-
-        if auxImage is None:
-            return np.eye(4)
-        else:
-            return transform.concat(
-                auxOpts.getTransform('display', 'texture'),
-                opts   .getTransform('texture', 'display'))
+        return self.auxmgr.textureXform(which)
 
 
-    def preDraw(self, xform=None, bbox=None):
+    def preDraw(self):
         """Must be called by subclass implementations.
 
         Ensures that all of the textures managed by this ``GLVectorBase`` are
@@ -591,7 +401,7 @@ class GLVectorBase(glimageobject.GLImageObject):
         self.cmapTexture    .bindTexture(gl.GL_TEXTURE3)
 
 
-    def postDraw(self, xform=None, bbox=None):
+    def postDraw(self):
         """Must be called by subclass implementations.
 
         Unbinds all of the textures managed by this ``GLVectorBase``.
@@ -612,40 +422,43 @@ class GLVectorBase(glimageobject.GLImageObject):
         self.asyncUpdateShaderState(alwaysNotify=True)
 
 
+    def registerAuxImage(self, which, image, onReady=None, **kwargs):
+        """Registers the given auxillary image with the
+        ``AuxImageTextureManager``.
+        """
+
+        self.auxmgr.texture(which).deregister(self.name)
+        self.auxmgr.registerAuxImage(which, image, **kwargs)
+        self.auxmgr.texture(which).register(self.name, self.__textureChanged)
+        if onReady is not None:
+            idle.idleWhen(onReady, self.auxmgr.texturesReady)
+
+
     def __colourImageChanged(self, *a):
         """Called when the :attr:`.VectorOpts.colourImage` changes. Registers
         with the new image, and refreshes textures as needed.
         """
-        self.deregisterAuxImage('colour')
-        self.registerAuxImage(  'colour')
-
-        def onRefresh():
+        def onReady():
             self.compileShaders()
             self.refreshColourMapTexture()
             self.asyncUpdateShaderState(alwaysNotify=True)
-
-        self.refreshAuxTexture('colour')
-        async.idleWhen(onRefresh, self.texturesReady)
+        self.registerAuxImage('colour', self.opts.colourImage, onReady)
 
 
     def __modImageChanged(self, *a):
         """Called when the :attr:`.VectorOpts.modulateImage` changes.
         Registers with the new image, and refreshes textures as needed.
         """
-        self.deregisterAuxImage('modulate')
-        self.registerAuxImage(  'modulate')
-        self.refreshAuxTexture( 'modulate')
-        self.asyncUpdateShaderState(alwaysNotify=True)
+        onReady = ft.partial(self.asyncUpdateShaderState, alwaysNotify=True)
+        self.registerAuxImage('modulate', self.opts.modulateImage, onReady)
 
 
     def __clipImageChanged(self, *a):
         """Called when the :attr:`.VectorOpts.clipImage` changes.
         Registers with the new image, and refreshes textures as needed.
         """
-        self.deregisterAuxImage('clip')
-        self.registerAuxImage(  'clip')
-        self.refreshAuxTexture( 'clip')
-        self.asyncUpdateShaderState(alwaysNotify=True)
+        onReady = ft.partial(self.asyncUpdateShaderState, alwaysNotify=True)
+        self.registerAuxImage('clip', self.opts.clipImage, onReady)
 
 
     def __textureChanged(self, *a):
@@ -659,8 +472,9 @@ class GLVectorBase(glimageobject.GLImageObject):
 
 class GLVector(GLVectorBase):
     """The ``GLVector`` class is a sub-class of :class:`GLVectorBase`, which
-    contains some additional logic for rendering :class:`.Image` overlays
-    with a shape ``X*Y*Z*3``, and which contain directional data.
+    contains some additional logic for rendering :class:`.Image` overlays with
+    a shape ``X*Y*Z*3``, or of type ``NIFTI_TYPE_RGB24``, and which contain
+    directional data.
 
 
     By default , the ``image`` overlay passed to :meth:`__init__` is assumed
@@ -673,6 +487,7 @@ class GLVector(GLVectorBase):
     where the ``R`` channel contains the ``x`` vector values, the ``G``
     channel the ``y`` values, and the ``B`` channel the ``z`` values.
 
+
     This texture is bound to texture unit  ``gl.GL_TEXTURE4`` in the
     :meth:`preDraw` method.
     """
@@ -682,8 +497,15 @@ class GLVector(GLVectorBase):
         optional, but if provided, must be passed as keyword arguments. All
         other arguments are passed through to :meth:`GLVectorBase.__init__`.
 
+        The ``image``, (or ``vectorImage``) argument is assumed to be an
+        :class:`.Image` instance of shape ``(X, Y, Z, 3)``, or of type
+        ``NIFTI_TYPE_RGB24``, which contains the vector data. If the former,
+        the vector data is assumed to be in the range ``[-1, 1]``. If the
+        latter, the vector data is, by definition, in the range ``[0, 255]``
+        - this is assumed to map directly to the range ``[-1, 1]``.
 
-        :arg vectorImage:    If ``None``, the ``image`` is assumed to be a 4D
+
+        :arg vectorImage:    If ``None``, the ``image`` is assumed to be an
                              :class:`.Image` instance which contains the
                              vector data. If this is not the case, the
                              ``vectorImage`` parameter can be used to specify
@@ -712,9 +534,24 @@ class GLVector(GLVectorBase):
         prefilter      = kwargs.pop('prefilter',      defaultPrefilter)
         prefilterRange = kwargs.pop('prefilterRange', None)
 
-        if len(vectorImage.shape) != 4 or vectorImage.shape[3] != 3:
-            raise ValueError('Image must be 4 dimensional, with 3 volumes '
-                             'representing the XYZ vector angles')
+        # Must be an image of shape (X, Y, Z, 3),
+        # or of type RGB24. If the latter, the
+        # test below will accept any numpy
+        # structured array with three values per
+        # voxel, but we assume elsewhere that
+        # those values are all of type np.uint8
+        # (and thus correspond to the
+        # NIFTI_TYPE_RGB24 type)
+        shape = vectorImage.shape
+        ndims = len(shape)
+        nvals = vectorImage.nvals
+        isRGB = (((ndims == 4) and (nvals == 1) and (shape[3] == 3)) or
+                 ((ndims == 3) and (nvals == 3)))
+
+        if not isRGB:
+            raise ValueError('Image must be 4 dimensional with 3 volumes '
+                             'representing the XYZ vector angles, or of '
+                             'type RGB24')
 
         self.vectorImage     = vectorImage
         self.imageTexture    = None
@@ -754,7 +591,7 @@ class GLVector(GLVectorBase):
 
         GLVectorBase.destroy(self)
         self.imageTexture.deregister(self.name)
-        glresources.delete(self.imageTexture.getTextureName())
+        glresources.delete(self.imageTexture.name)
 
         self.imageTexture = None
 
@@ -787,12 +624,7 @@ class GLVector(GLVectorBase):
 
         if self.imageTexture is not None:
             self.imageTexture.deregister(self.name)
-            glresources.delete(self.imageTexture.getTextureName())
-
-        # the fourth dimension (the vector directions)
-        # must be the fastest changing in the texture data
-        def realPrefilter(d):
-            return prefilter(d.transpose((3, 0, 1, 2)))
+            glresources.delete(self.imageTexture.name)
 
         self.imageTexture = glresources.get(
             texName,
@@ -802,24 +634,24 @@ class GLVector(GLVectorBase):
             nvals=3,
             interp=interp,
             normaliseRange=vecImage.dataRange,
-            prefilter=realPrefilter,
+            prefilter=prefilter,
             prefilterRange=prefilterRange,
             notify=False)
 
         self.imageTexture.register(self.name, self.__textureChanged)
 
 
-    def preDraw(self, xform=None, bbox=None):
+    def preDraw(self):
         """Overrides :meth:`GLVectorBase`. Binds the vector image texture.
         """
-        GLVectorBase.preDraw(self, xform, bbox)
+        GLVectorBase.preDraw(self)
         self.imageTexture.bindTexture(gl.GL_TEXTURE4)
 
 
-    def postDraw(self, xform=None, bbox=None):
+    def postDraw(self):
         """Overrides :meth:`GLVectorBase`. Unbinds the vector image texture.
         """
-        GLVectorBase.postDraw(self, xform, bbox)
+        GLVectorBase.postDraw(self)
         self.imageTexture.unbindTexture()
 
 
