@@ -9,7 +9,6 @@ encapsulates the data and logic required to render 2D slice of an
 :class:`.Image` instance.
 """
 
-
 import logging
 
 import numpy                     as np
@@ -20,15 +19,16 @@ import fsl.transform.affine      as affine
 import fsleyes.gl                as fslgl
 import fsleyes.gl.routines       as glroutines
 import fsleyes.gl.shaders.filter as glfilter
-from . import                       textures
-from . import                       glimageobject
-from . import resources          as glresources
+import fsleyes.gl.textures       as textures
+import fsleyes.gl.globject       as globject
+import fsleyes.gl.glimageobject  as glimageobject
+import fsleyes.gl.resources      as glresources
 
 
 log = logging.getLogger(__name__)
 
 
-class GLVolume(glimageobject.GLImageObject):
+class GLVolume(glimageobject.GLImageObject, globject.GLObject):
     """The ``GLVolume`` class is a :class:`.GLImageObject` which encapsulates
     the data and logic required to render  :class:`.Image` overlays in 2D and
     3D.
@@ -171,17 +171,15 @@ class GLVolume(glimageobject.GLImageObject):
                          image data.
     ``modulateTexture``  The :class:`.ImageTexture` which stores the
                          modulate image data.
-    ``colourTexture``    The :class:`.ColourMapTexture` used to store the
+    ``cmapTexture``      The :class:`.ColourMapTexture` used to store the
                          colour map.
-    ``negColourTexture`` The :class:`.ColourMapTexture` used to store the
+    ``negCmapTexture``   The :class:`.ColourMapTexture` used to store the
                          negative colour map.
     ``renderTexture1``   The first :class:`.RenderTexture` used for 3D
                          rendering.
     ``renderTexture2``   The first :class:`.RenderTexture` used for 3D
                          rendering.
-    ``texName``          A name used for the ``imageTexture``,
-                         ``colourTexture``, and ``negColourTexture`. The
-                         name for the latter is suffixed with ``'_neg'``.
+    ``texName``          A name used for the ``imageTexture``.
     ==================== ==================================================
     """
 
@@ -199,11 +197,12 @@ class GLVolume(glimageobject.GLImageObject):
         :arg threedee:    Set up for 2D or 3D rendering.
         """
 
-        glimageobject.GLImageObject.__init__(self,
-                                             image,
-                                             overlayList,
-                                             displayCtx,
-                                             threedee)
+        globject.GLObject.__init__(self,
+                                   image,
+                                   overlayList,
+                                   displayCtx,
+                                   threedee)
+        glimageobject.GLImageObject.__init__(self)
 
         # Create an image texture, clip texture, and a colour map texture
         #
@@ -221,16 +220,22 @@ class GLVolume(glimageobject.GLImageObject):
 
         # aux texture manager takes care
         # of clip+modulate textures
-        self.auxmgr = textures.AuxImageTextureManager(
-            self, clip=None, modulate=None)
-        self.auxmgr.registerAuxImage('clip',     self.opts.clipImage)
-        self.auxmgr.registerAuxImage('modulate', self.opts.modulateImage)
+        if self.opts.interpolation == 'none': interp = gl.GL_NEAREST
+        else:                                 interp = gl.GL_LINEAR
+        settings = {'interp' : interp}
 
-        # Refs to all of the texture objects.
-        self.imageTexture     = None
-        self.colourTexture    = textures.ColourMapTexture(self.texName)
-        self.negColourTexture = textures.ColourMapTexture(
-            '{}_neg'.format(self.texName))
+        self.auxmgr = textures.AuxImageTextureManager(
+            self,
+            clip=(self.opts.clipImage, settings),
+            modulate=(self.opts.modulateImage, settings))
+
+        # cmap manager takes care
+        # of cmap/negcmap textures
+        self.cmapmgr = textures.ColourMapTextureManager(self)
+
+        # we take care of the
+        # main image texture
+        self.imageTexture = None
 
         if self.threedee:
 
@@ -249,7 +254,6 @@ class GLVolume(glimageobject.GLImageObject):
         # See that method for details.
         self.__alwaysNotify = False
 
-        self.refreshColourTextures()
         self.refreshImageTexture()
 
         # Add listeners to this image so the view can be
@@ -282,14 +286,12 @@ class GLVolume(glimageobject.GLImageObject):
         self.imageTexture.deregister(self.name)
         glresources.delete(self.imageTexture.name)
 
-        self.colourTexture   .destroy()
-        self.negColourTexture.destroy()
-        self.auxmgr          .destroy()
+        self.auxmgr .destroy()
+        self.cmapmgr.destroy()
 
-        self.auxmgr           = None
-        self.imageTexture     = None
-        self.colourTexture    = None
-        self.negColourTexture = None
+        self.auxmgr       = None
+        self.cmapmgr      = None
+        self.imageTexture = None
 
         if self.threedee:
             self.renderTexture1.destroy()
@@ -321,6 +323,22 @@ class GLVolume(glimageobject.GLImageObject):
         return imageTexReady and self.auxmgr.texturesReady()
 
 
+    @property
+    def cmapTexture(self):
+        """Returns the :class:`.ColourMapTexture` associated with the
+        :attr:`.VolumeOpts.cmap`.
+        """
+        return self.cmapmgr.cmapTexture
+
+
+    @property
+    def negCmapTexture(self):
+        """Returns the :class:`.ColourMapTexture` associated with the
+        :attr:`.VolumeOpts.negativeCmap`.
+        """
+        return self.cmapmgr.negCmapTexture
+
+
     @property
     def clipTexture(self):
         """Returns the :class:`.ImageTexture` associated with the
@@ -368,15 +386,15 @@ class GLVolume(glimageobject.GLImageObject):
                              self._modulateImageChanged)
         opts    .addListener('invertClipping',   name,
                              self._invertClippingChanged)
-        opts    .addListener('cmap',             name, self._cmapChanged)
-        opts    .addListener('gamma',            name, self._cmapChanged)
-        opts    .addListener('logScale',         name, self._cmapChanged)
-        opts    .addListener('interpolateCmaps', name, self._cmapChanged)
-        opts    .addListener('negativeCmap',     name, self._cmapChanged)
-        opts    .addListener('cmapResolution',   name, self._cmapChanged)
+        opts    .addListener('cmap',             name, self.notify)
+        opts    .addListener('gamma',            name, self.notify)
+        opts    .addListener('logScale',         name, self.notify)
+        opts    .addListener('interpolateCmaps', name, self.notify)
+        opts    .addListener('negativeCmap',     name, self.notify)
+        opts    .addListener('cmapResolution',   name, self.notify)
+        opts    .addListener('invert',           name, self.notify)
         opts    .addListener('useNegativeCmap',  name,
                              self._useNegativeCmapChanged)
-        opts    .addListener('invert',           name, self._invertChanged)
         opts    .addListener('modulateAlpha',    name,
                              self._modulateAlphaChanged)
         opts    .addListener('invertModulateAlpha', name,
@@ -578,6 +596,7 @@ class GLVolume(glimageobject.GLImageObject):
         """
 
         opts     = self.opts
+        interp   = opts.interpolation
         texName  = self.texName
         unsynced = self.testUnsynced()
 
@@ -592,8 +611,16 @@ class GLVolume(glimageobject.GLImageObject):
             self.imageTexture.deregister(self.name)
             glresources.delete(self.imageTexture.name)
 
-        if opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                            interp = gl.GL_LINEAR
+        prefilter = kwargs.pop('prefilter', None)
+
+        if interp == 'true_spline':
+            if prefilter is None:
+                prefilter = textures.splineFilter
+            else:
+                prefilter = lambda d: textures.splineFilter(prefilter(d))
+
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
 
         if opts.enableOverrideDataRange: normRange = opts.overrideDataRange
         else:                            normRange = None
@@ -604,6 +631,7 @@ class GLVolume(glimageobject.GLImageObject):
             texName,
             self.image,
             interp=interp,
+            prefilter=prefilter,
             channel=opts.channel,
             volume=opts.index()[3:],
             normaliseRange=normRange,
@@ -628,45 +656,6 @@ class GLVolume(glimageobject.GLImageObject):
             idle.idleWhen(onReady, self.auxmgr.texturesReady)
 
 
-    def refreshColourTextures(self):
-        """Refreshes the :class:`.ColourMapTexture` instances used to colour
-        image voxels.
-        """
-
-        display  = self.display
-        opts     = self.opts
-        alpha    = display.alpha / 100.0
-        cmap     = opts.cmap
-        interp   = opts.interpolateCmaps
-        res      = opts.cmapResolution
-        logScale = opts.logScale
-        gamma    = opts.realGamma(opts.gamma)
-        negCmap  = opts.negativeCmap
-        invert   = opts.invert
-        dmin     = opts.displayRange[0]
-        dmax     = opts.displayRange[1]
-
-        if interp: interp = gl.GL_LINEAR
-        else:      interp = gl.GL_NEAREST
-
-        self.colourTexture.set(cmap=cmap,
-                               invert=invert,
-                               alpha=alpha,
-                               resolution=res,
-                               gamma=gamma,
-                               logScale=logScale,
-                               interp=interp,
-                               displayRange=(dmin, dmax))
-
-        self.negColourTexture.set(cmap=negCmap,
-                                  invert=invert,
-                                  alpha=alpha,
-                                  resolution=res,
-                                  gamma=gamma,
-                                  interp=interp,
-                                  displayRange=(dmin, dmax))
-
-
     def preDraw(self):
         """Binds the :class:`.ImageTexture` to ``GL_TEXTURE0`` and the
         :class:`.ColourMapTexture` to ``GL_TEXTURE1, and calls the
@@ -675,8 +664,8 @@ class GLVolume(glimageobject.GLImageObject):
 
         # Set up the image and colour textures
         self.imageTexture    .bindTexture(gl.GL_TEXTURE0)
-        self.colourTexture   .bindTexture(gl.GL_TEXTURE1)
-        self.negColourTexture.bindTexture(gl.GL_TEXTURE2)
+        self.cmapTexture     .bindTexture(gl.GL_TEXTURE1)
+        self.negCmapTexture  .bindTexture(gl.GL_TEXTURE2)
         self.clipTexture     .bindTexture(gl.GL_TEXTURE3)
         self.modulateTexture .bindTexture(gl.GL_TEXTURE4)
 
@@ -780,8 +769,8 @@ class GLVolume(glimageobject.GLImageObject):
         """
 
         self.imageTexture    .unbindTexture()
-        self.colourTexture   .unbindTexture()
-        self.negColourTexture.unbindTexture()
+        self.cmapTexture     .unbindTexture()
+        self.negCmapTexture  .unbindTexture()
         self.clipTexture     .unbindTexture()
         self.modulateTexture .unbindTexture()
 
@@ -863,8 +852,6 @@ class GLVolume(glimageobject.GLImageObject):
 
     def _alphaChanged(self, *a):
         """Called when the :attr:`.Display.alpha` property changes. """
-
-        self.refreshColourTextures()
         if self.threedee:
             self.updateShaderState(alwaysNotify=True)
         else:
@@ -874,7 +861,6 @@ class GLVolume(glimageobject.GLImageObject):
     def _displayRangeChanged(self, *a):
         """Called when the :attr:`.VolumeOpts.displayRange` property changes.
         """
-        self.refreshColourTextures()
         self.updateShaderState()
 
 
@@ -925,14 +911,6 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState()
 
 
-    def _cmapChanged(self, *a):
-        """Called when the :attr:`.VolumeOpts.cmap` or
-        :attr:`.VolumeOpts.negativeCmap` properties change.
-        """
-        self.refreshColourTextures()
-        self.notify()
-
-
     def _useNegativeCmapChanged(self, *a):
         """Called when the :attr:`.VolumeOpts.useNegativeCmap` property
         changes.
@@ -940,12 +918,6 @@ class GLVolume(glimageobject.GLImageObject):
         self.updateShaderState()
 
 
-    def _invertChanged(self, *a):
-        """Called when the :attr:`.VolumeOpts.invert` property changes. """
-        self.refreshColourTextures()
-        self.notify()
-
-
     def _modulateAlphaChanged(self, *a):
         """Called when the :attr:`.VolumeOpts.modulateAlpha` or
         :attr:`.VolumeOpts.invertModulateAlpha` properties change.
@@ -975,22 +947,27 @@ class GLVolume(glimageobject.GLImageObject):
         called when other properties, which require a texture refresh, change.
         """
         opts       = self.opts
+        interp     = opts.interpolation
         volRefresh = kwa.pop('volRefresh', False)
 
-        if opts.interpolation == 'none': interp = gl.GL_NEAREST
-        else:                            interp = gl.GL_LINEAR
-
         if opts.enableOverrideDataRange: normRange = opts.overrideDataRange
         else:                            normRange = None
 
+        if interp == 'true_spline': prefilter = textures.splineFilter
+        else:                       prefilter = None
+
+        if interp == 'none': interp = gl.GL_NEAREST
+        else:                interp = gl.GL_LINEAR
+
         self.imageTexture.set(volume=opts.index()[3:],
                               channel=opts.channel,
                               interp=interp,
                               volRefresh=volRefresh,
+                              prefilter=prefilter,
                               normaliseRange=normRange)
 
-        self.clipTexture    .set(interp=interp)
-        self.modulateTexture.set(interp=interp)
+        self.clipTexture    .set(interp=interp, volRefresh=False)
+        self.modulateTexture.set(interp=interp, volRefresh=False)
 
 
     def _channelChanged(self, *a, **kwa):
