@@ -16,15 +16,18 @@ import                    contextlib
 import collections.abc as abc
 import itertools       as it
 
-import OpenGL.GL       as gl
 import numpy           as np
 
-import fsl.transform.affine as affine
+from fsl.transform import affine
+from fsleyes.utils import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 def clear(bgColour):
     """Clears the current frame buffer, and does some standard setup
     operations.
@@ -132,21 +135,29 @@ def disabled(capabilities):
         yield
 
 
-def show2D(xax, yax, lo, hi, flipx=False, flipy=False):
+def show2D(xax, yax, lo, hi, flipx=False, flipy=False, expandz=True):
     """Generates a projection and model view matrix for 2D orthographic
     display for the given axes and bounds.
 
-    :arg xax:   Index (into ``lo`` and ``hi``) of the axis which
-                corresponds to the horizontal screen axis.
-    :arg yax:   Index (into ``lo`` and ``hi``) of the axis which
-                corresponds to the vertical screen axis.
-    :arg lo:    Tuple containing the mininum ``(x, y, z)`` display
-                coordinates.
-    :arg hi:    Tuple containing the maxinum ``(x, y, z)`` display
-                coordinates.
-    :arg flipx: If ``True``, the x axis is inverted.
+    :arg xax:     Index (into ``lo`` and ``hi``) of the axis which
+                  corresponds to the horizontal screen axis.
 
-    :arg flipy: If ``True``, the y axis is inverted.
+    :arg yax:     Index (into ``lo`` and ``hi``) of the axis which
+                  corresponds to the vertical screen axis.
+
+    :arg lo:      Tuple containing the mininum ``(x, y, z)`` display
+                  coordinates.
+
+    :arg hi:      Tuple containing the maxinum ``(x, y, z)`` display
+                  coordinates.
+
+    :arg flipx:   If ``True``, the x axis is inverted.
+
+    :arg flipy:   If ``True``, the y axis is inverted.
+
+    :arg expandz: If ``True`` (default), the Z range is automatically
+                  expanded to guard against anything being unintentionally
+                  clipped.
 
     Returns a tuple containing the projection and modelview matrices.
     """
@@ -159,9 +170,10 @@ def show2D(xax, yax, lo, hi, flipx=False, flipy=False):
 
     # Expand the Z range so we don't
     # accidentally clip anything
-    zdist      = max(abs(zmin), abs(zmax))
-    zmin       = -zdist
-    zmax       =  zdist
+    if expandz:
+        zdist = max(abs(zmin), abs(zmax))
+        zmin  = -zdist
+        zmax  =  zdist
 
     projmat = np.eye(4, dtype=np.float32)
 
@@ -240,6 +252,40 @@ def rotate(degrees, x, y, z):
     return xf
 
 
+def flip(xform, axes, lo, hi):
+    """Applies a flip to the specified axes of the given affine
+    transfomration.
+
+    :arg xform: Affine transformationm matrix (4x4 numpy array)
+    :arg axes:  List of axes to apply a flip to
+    :arg lo:    Sequence of three low display bounds for each axis
+    :arg hi:    Sequence of three high display bounds for each axis
+    :return:    Adjusted affine transformation
+    """
+
+    # We have to translate the slice
+    # to the origin, perform the flip
+    # there, then transform it back to
+    # its original location.
+    invert     = np.eye(4)
+    toOrigin   = np.eye(4)
+    fromOrigin = np.eye(4)
+
+    for ax in axes:
+        amin = lo[ax]
+        amax = hi[ax]
+        alen = amax - amin
+        off  = alen / 2.0 + xform[ax, 3] + amin
+
+        invert[    ax, ax] = -1
+        toOrigin[  ax, 3]  = -off
+        fromOrigin[ax, 3]  =  off
+
+    xform = affine.concat(fromOrigin, invert, toOrigin, xform)
+
+    return xform
+
+
 def ortho2D(xmin, xmax, ymin, ymax, zmin, zmax):
     """Generates an orthographic projection matrix for 2D views, for the given
     display bounds. Replacement for the ``glOrtho`` function.
@@ -281,8 +327,8 @@ def ortho3D(lo, hi, width, height, zoom):
                  for each axis.
     """
 
-    lo = np.array(lo, copy=False)
-    hi = np.array(hi, copy=False)
+    lo = np.asarray(lo)
+    hi = np.asarray(hi)
 
     xmin, ymin = lo[:2]
     xmax, ymax = hi[:2]
@@ -833,21 +879,25 @@ def voxelBox(voxel,
     scaledLo    = scaledVoxel - boxSize / 2.0
     scaledHi    = scaledVoxel + boxSize / 2.0
 
+    # Convert to voxels, and round to
+    # a sensible precision to avoid
+    # floating point errors
+    lo = np.round(scaledLo / dims, decimals=4)
+    hi = np.round(scaledHi / dims, decimals=4)
+
     # Scale the low/high bounds back
     # into voxel coordinates, and
     # round them up or down according
     # to the bias setting.
     if bias == 'low':
-        lo = np.floor(scaledLo / dims)
-        hi = np.floor(scaledHi / dims)
-
+        lo = np.floor(lo)
+        hi = np.floor(hi)
     elif bias == 'high':
-        lo = np.ceil(scaledLo / dims)
-        hi = np.ceil(scaledHi / dims)
-
+        lo = np.ceil(lo)
+        hi = np.ceil(hi)
     else:
-        lo = np.floor(scaledLo / dims)
-        hi = np.ceil( scaledHi / dims)
+        lo = np.floor(lo)
+        hi = np.ceil( hi)
 
     # Crop the box to the
     # image space if necessary
@@ -1449,6 +1499,7 @@ def lineAsPolygon(vertices,
                   width,
                   axis=None,
                   camera=None,
+                  xform=None,
                   mode='segments',
                   indices=False):
     """Returns a set of vertices which represent a line between ``start`` and
@@ -1471,7 +1522,9 @@ def lineAsPolygon(vertices,
     :arg vertices: A ``(N, 3)`` array containing the line coordinates
 
     :arg width:    Line width, in units proportional to the coordinate system
-                   that ``vertices`` are defined in.
+                   that ``vertices`` are defined in. Can be either a scalar
+                   value applied to all lines, or an array of length ``N`` with
+                   a per-line width.
 
     :arg axis:     Depth axis, if the line is being drawn on a plane orthogonal
                    to the display coordinate system.
@@ -1479,6 +1532,11 @@ def lineAsPolygon(vertices,
     :arg camera:   Camera direction vector, if the line is being drawn on an
                    arbitrary plane. Ignored if ``axis`` is specified.
 
+    :arg xform:    Matrix to transform from the display coordinate system into
+                   the coordinate system that the vertices are defined in, if
+                   it differs (specifically required if there are any rotations
+                   in the transformation).
+
     :arg mode:     Either ``'segments'`` (the default), or ``'strip'``.
                    If ``'segments'``, lines are formed from ``vertices[0]``
                    and ``vertices[1]``, ``vertices[2]`` and ``vertices[3]``,
@@ -1502,6 +1560,9 @@ def lineAsPolygon(vertices,
                    drawn.
     """
 
+    if axis is None and camera is None:
+        raise ValueError('One of axis or camera must be provided')
+
     if axis is not None:
         camera       = np.zeros(3)
         camera[axis] = 1
@@ -1509,6 +1570,11 @@ def lineAsPolygon(vertices,
     camera   = np.asarray(camera)
     vertices = np.asarray(vertices)
 
+    # Rotate the camera vector into
+    # the vertex coordinate system
+    if xform is not None:
+        camera = affine.transform(camera, xform, vector=True)
+
     rot = rotate(90, *camera)
 
     # Separate starting vertices from ending
@@ -1551,7 +1617,8 @@ def lineAsPolygon(vertices,
     # width, and create a rectangular region
     # representing the line.
     offset = affine.transform((projend - projstart), rot, vector=True)
-    offset = affine.normalise(offset) * width / 2
+    offset = affine.normalise(offset)
+    offset = ((offset.T) * width / 2).T
 
     # Each rectangle is drawn with two triangles.
     #
@@ -1589,3 +1656,21 @@ def lineAsPolygon(vertices,
         vertices[4::6] = end   + offset
         vertices[5::6] = start + offset
         return vertices
+
+
+def generateCoordinates(xend, yend, zend, xstart=0, ystart=0,zstart=0):
+    """Creates a (N, 3) array containing XYZ coordinates for every voxel
+    in a volume of size (xend-xstart, yend-ystart, zend-zstart).
+    """
+    xs           = np.arange(xstart, xend)
+    ys           = np.arange(ystart, yend)
+    zs           = np.arange(zstart, zend)
+    ys, zs, xs   = np.meshgrid(ys, zs, xs)
+    ys           = ys.flatten()
+    zs           = zs.flatten()
+    xs           = xs.flatten()
+    coords       = np.zeros((xs.shape[0], 3))
+    coords[:, 0] = xs
+    coords[:, 1] = ys
+    coords[:, 2] = zs
+    return coords
