@@ -19,24 +19,27 @@ according to the current overlay order.
 
 import logging
 
-import numpy     as np
-import OpenGL.GL as gl
+import numpy as np
 
-import fsl.data.mesh        as fslmesh
-import fsl.data.image       as fslimage
-import fsl.utils.idle       as idle
-import fsl.transform.affine as affine
+import fsl.data.mesh     as fslmesh
+import fsl.data.image    as fslimage
+from   fsl.utils     import idle
+from   fsl.transform import affine
 
-import fsleyes.data.tractogram           as fsltractogram
-import fsleyes.gl                        as fslgl
-import fsleyes.gl.routines               as glroutines
-import fsleyes.gl.globject               as globject
-import fsleyes.gl.annotations            as annotations
-import fsleyes.gl.shaders                as shaders
-import fsleyes.gl.text                   as gltext
-import fsleyes.gl.textures               as textures
-import fsleyes.displaycontext            as fsldisplay
-import fsleyes.displaycontext.canvasopts as canvasopts
+import fsleyes.data.tractogram    as fsltractogram
+import fsleyes.gl                 as fslgl
+import fsleyes.gl.routines        as glroutines
+from   fsleyes.gl             import globject
+from   fsleyes.gl             import annotations
+from   fsleyes.gl             import shaders
+from   fsleyes.gl             import textures
+import fsleyes.gl.text            as gltext
+import fsleyes.displaycontext     as fsldisplay
+from   fsleyes.displaycontext import canvasopts
+from   fsleyes.utils          import lazyimport
+
+
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
 
 
 log = logging.getLogger(__name__)
@@ -95,6 +98,8 @@ class Scene3DCanvas:
                                self.__name,
                                self.__displayBoundsChanged)
 
+        displayCtx.addListener('overlayOrder', self.__name, self.Refresh)
+
         opts = self.opts
         opts.addListener('pos',           self.__name, self.Refresh)
         opts.addListener('showCursor',    self.__name, self.Refresh)
@@ -112,7 +117,6 @@ class Scene3DCanvas:
         opts.addListener('light',         self.__name, self.Refresh)
         opts.addListener('lightPos',      self.__name, self.Refresh)
         opts.addListener('lightDistance', self.__name, self.Refresh)
-        opts.addListener('highDpi',       self.__name, self.__highDpiChanged)
 
 
     def __compileShaders(self):
@@ -527,13 +531,6 @@ class Scene3DCanvas:
             self.Refresh()
 
 
-    def __highDpiChanged(self, *a):
-        """Called when the :attr:`.Scene3DCanvasOpts.highDpi` property
-        changes. Calls the :meth:`.GLCanvasTarget.EnableHighDPI` method.
-        """
-        self.EnableHighDPI(self.opts.highDpi)
-
-
     def __displayBoundsChanged(self, *a):
         """Called when the :attr:`.DisplayContext.bounds` change. Resets
         the :attr:`.Scene3DCanvasOpts.lightPos` property.
@@ -615,7 +612,7 @@ class Scene3DCanvas:
             if overlay not in self.__glObjects:
                 return
 
-            if not self._setGLContext():
+            if not self.setGLContext():
                 self.__glObjects.pop(overlay)
                 return
 
@@ -764,7 +761,7 @@ class Scene3DCanvas:
         if self.destroyed:
             return
 
-        if not self._setGLContext():
+        if not self.setGLContext():
             return
 
         opts          = self.opts
@@ -843,7 +840,12 @@ class Scene3DCanvas:
         shader = self.__shaders.get(len(rtexs), None)
         if shader is not None:
 
-            for i, rtex in enumerate(rtexs):
+            # For fragments of equal depth, the blending shader
+            # will give precedence to the first one. This would
+            # correspond to an overlay which is earlier in the
+            # overlay list, whcih should be drawn on the bottom.
+            # So here we flip the overlay order.
+            for i, rtex in enumerate(reversed(rtexs)):
                 rtex             .bindTexture(int(gl.GL_TEXTURE0) + i * 2)
                 rtex.depthTexture.bindTexture(int(gl.GL_TEXTURE0) + i * 2 + 1)
 
