@@ -16,20 +16,23 @@ import re
 import logging
 import contextlib
 
-import numpy                          as np
+import numpy                as np
 
-import OpenGL.GL                      as gl
-import OpenGL.raw.GL._types           as gltypes
-import OpenGL.GL.ARB.fragment_program as arbfp
-import OpenGL.GL.ARB.vertex_program   as arbvp
 
-import fsl.utils.memoize              as memoize
-from . import                            parse
+from fsl.utils               import memoize
+from fsleyes.gl.shaders.arbp import parse
+from fsleyes.utils           import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl      = lazyimport('OpenGL.GL',                      f'{__name__}.gl')
+arbfp   = lazyimport('OpenGL.GL.ARB.fragment_program', f'{__name__}.arbfp')
+arbvp   = lazyimport('OpenGL.GL.ARB.vertex_program',   f'{__name__}.arbvp')
+gltypes = lazyimport('OpenGL.raw.GL._types',           f'{__name__}.gltypes')
+
+
 class ARBPShader:
     """The ``ARBPShader`` class encapsulates an OpenGL shader program
     written according to the ``ARB_vertex_program`` and
@@ -296,6 +299,11 @@ class ARBPShader:
         """
         if self.attsLoaded:
             return
+
+        # In some scenarios, GL_VERTEX_ARRAY
+        # must be enabled even if it is not
+        # used (e.g. on macOS).
+        gl.glEnableClientState(gl.GL_VERTEX_ARRAY)
         for attr in self.attrs:
             texUnit     = self.__getAttrTexUnit(attr)
             value, size = self.attCache[attr]
@@ -314,6 +322,7 @@ class ARBPShader:
             texUnit = self.__getAttrTexUnit(attr)
             gl.glClientActiveTexture(texUnit)
             gl.glDisableClientState(gl.GL_TEXTURE_COORD_ARRAY)
+        gl.glDisableClientState(gl.GL_VERTEX_ARRAY)
         self.attsLoaded = False
 
 
@@ -415,7 +424,7 @@ class ARBPShader:
             value = value.reshape(-1, 1)
 
         texUnit = self.__getAttrTexUnit(name)
-        value   = np.array(value, dtype=np.float32, copy=False)
+        value   = np.asarray(value, dtype=np.float32)
         size    = value.shape[1]
 
         log.debug('Setting vertex attribute %s [%s] = [%s * %s]',
@@ -461,7 +470,7 @@ class ARBPShader:
         if np.isscalar(value):
             value = [value]
 
-        value = np.array(value, copy=False)
+        value = np.asarray(value)
 
         # vector
         if len(value.shape) == 1:
@@ -476,7 +485,7 @@ class ARBPShader:
             padding = np.zeros((value.shape[0], 1))
             value   = np.hstack((value, padding))
 
-        value = np.array(value, dtype=np.float32, copy=False)
+        value = np.asarray(value, dtype=np.float32)
 
         if value.ndim > 2 or value.size < 4 or value.size % 4 != 0:
             raise ValueError('Invalid arbp parameter: {}'.format(value))
