@@ -15,30 +15,38 @@ import contextlib
 
 import jinja2                as j2
 import numpy                 as np
-import OpenGL.GL             as gl
-import OpenGL.raw.GL._types  as gltypes
 
-import fsleyes.gl.extensions as glexts
-import fsleyes.gl.resources  as glresources
-import fsleyes.gl            as fslgl
-import fsl.utils.memoize     as memoize
-from . import                   parse
+from   fsl.utils               import memoize
+import fsleyes.gl.extensions       as glexts
+import fsleyes.gl.resources        as glresources
+import fsleyes.gl                  as fslgl
+from   fsleyes.gl.shaders.glsl import parse
+from   fsleyes.utils           import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
-GLSL_ATTRIBUTE_TYPES = {
-    'bool'  : (gl.GL_BOOL,  1),
-    'int'   : (gl.GL_INT,   1),
-    'float' : (gl.GL_FLOAT, 1),
-    'vec2'  : (gl.GL_FLOAT, 2),
-    'vec3'  : (gl.GL_FLOAT, 3),
-    'vec4'  : (gl.GL_FLOAT, 4)
-}
-"""This dictionary contains mappings between GLSL data types, and their
-corresponding GL types and sizes.
-"""
+gl      = lazyimport('OpenGL.GL',            f'{__name__}.gl')
+gltypes = lazyimport('OpenGL.raw.GL._types', f'{__name__}.gltypes')
+
+
+def glslAttributeType(typename):
+    """Returns the GL type for a GLSL type.
+
+    Given the name fo a GLSL type, e.g. ``vec3``, returns a tuple containing
+    the corresponding GL type, and the number of values, e.g.
+    ``(OpenGL.GL.GL_FLOAT, 4)``
+    """
+
+    return {
+        'bool'  : (gl.GL_BOOL,  1),
+        'int'   : (gl.GL_INT,   1),
+        'float' : (gl.GL_FLOAT, 1),
+        'vec2'  : (gl.GL_FLOAT, 2),
+        'vec3'  : (gl.GL_FLOAT, 3),
+        'vec4'  : (gl.GL_FLOAT, 4)
+    }[typename]
 
 
 class GLSLShader:
@@ -132,10 +140,16 @@ class GLSLShader:
         if constants is None:
             constants = {}
 
-        srcs = []
-        for src in (vertSrc, fragSrc, geomSrc):
+        srcs = {'vert' : vertSrc,
+                'frag' : fragSrc,
+                'geom' : geomSrc}
+
+        for stype, src in srcs.items():
             if src is not None:
-                srcs.append(j2.Template(src).render(**constants))
+                srcs[stype] = j2.Template(src).render(**constants)
+
+        self.__srcs = srcs
+
         types      = {}
         sizes      = {}
         attributes = []
@@ -147,7 +161,7 @@ class GLSLShader:
         # declared as 'varying' or 'in', and
         # anything from any of the shaders
         # declared as 'uniform'.
-        for i, src in enumerate(srcs):
+        for stype, src in srcs.items():
             if src is None:
                 continue
             decs = parse.parseGLSL(src)
@@ -155,7 +169,7 @@ class GLSLShader:
             # get attributes/vertex inputs from
             # vertex shader. For the other shaders,
             # we only care about uniforms.
-            if i == 0:
+            if stype == 'vert':
                 atts  = decs['attribute']
                 unifs = decs['uniform']
             else:
@@ -168,7 +182,7 @@ class GLSLShader:
                 types[dname] = dtype
                 sizes[dname] = dsize
 
-        self.program     = self.__compile(*srcs)
+        self.program     = self.__compile(srcs['vert'], srcs['frag'], srcs['geom'])
         self.attDivisors = {}
         self.types       = types
         self.sizes       = sizes
@@ -222,12 +236,33 @@ class GLSLShader:
             log.debug('%s.del(%s)', type(self).__name__, id(self))
 
 
+    @property
+    def vertexSource(self):
+        """Return the vertex shader source code. """
+        return self.__srcs['vert']
+
+
+    @property
+    def fragmentSource(self):
+        """Return the fragmentshader source code. """
+        return self.__srcs['frag']
+
+
+    @property
+    def geometrySource(self):
+        """Return the geometry shader source code. Returns ``None`` for
+        programs without a geometry shader.
+        """
+        return self.__srcs['geom']
+
+
     def ready(self):
         """Checks whether every uniform and attribute has been given a value.
         Returns a tuple containing:
-          - ``True`` if every uniform/attribute has been set, ``False``
-            otherwise.
-          - A list of the names of all unset uniforms/attributes.
+
+         - ``True`` if every uniform/attribute has been set, ``False``
+           otherwise.
+         - A list of the names of all unset uniforms/attributes.
         """
         unset = []
         for n, v in self.hasValue.items():
@@ -289,7 +324,7 @@ class GLSLShader:
             aType          = self.types[          att]
             aBuf           = self.buffers[        att]
             aDivisor       = self.attDivisors.get(att)
-            glType, glSize = GLSL_ATTRIBUTE_TYPES[aType]
+            glType, glSize = glslAttributeType(aType)
 
             gl.glBindBuffer(gl.GL_ARRAY_BUFFER, aBuf)
             gl.glEnableVertexAttribArray(aPos)
@@ -424,6 +459,8 @@ class GLSLShader:
 
         if divisor is not None:
             self.attDivisors[name] = divisor
+        else:
+            self.attDivisors.pop(name, None)
 
         self.hasValue[name] = True
 
@@ -489,8 +526,6 @@ class GLSLShader:
         :returns:  A dictionary of ``{name : position}`` mappings.
         """
 
-        import OpenGL.GL as gl
-
         shaderVars = {}
 
         for v in uniforms:
@@ -543,84 +578,84 @@ class GLSLShader:
 
 
     def _attribute_bool(self, val):
-        return np.array(val, dtype=np.bool, copy=False)
+        return np.asarray(val, dtype=np.bool)
 
 
     def _attribute_int(self, val):
-        return np.array(val, dtype=np.int32, copy=False)
+        return np.asarray(val, dtype=np.int32)
 
 
     def _attribute_float(self, val):
-        return np.array(val, dtype=np.float32, copy=False)
+        return np.asarray(val, dtype=np.float32)
 
 
     def _attribute_vec2(self, val):
-        return np.array(val, dtype=np.float32, copy=False).ravel('C')
+        return np.asarray(val, dtype=np.float32).ravel('C')
 
 
     def _attribute_vec3(self, val):
-        return np.array(val, dtype=np.float32, copy=False).ravel('C')
+        return np.asarray(val, dtype=np.float32).ravel('C')
 
 
     def _attribute_vec4(self, val):
-        return np.array(val, dtype=np.float32, copy=False).ravel('C')
+        return np.asarray(val, dtype=np.float32).ravel('C')
 
 
     def _uniform_bool(self, pos, val, size):
-        val = np.array(val, dtype=np.int32)
+        val = np.asarray(val, dtype=np.int32)
         gl.glUniform1iv(pos, size, val)
 
 
     def _uniform_int(self, pos, val, size):
-        val = np.array(val, dtype=np.int32)
+        val = np.asarray(val, dtype=np.int32)
         gl.glUniform1iv(pos, size, val)
 
 
     def _uniform_float(self, pos, val, size):
-        val = np.array(val, dtype=np.float32)
+        val = np.asarray(val, dtype=np.float32)
         gl.glUniform1fv(pos, size, val)
 
 
     def _uniform_vec2(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniform2fv(pos, size, val)
 
 
     def _uniform_vec3(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniform3fv(pos, size, val)
 
 
     def _uniform_vec4(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniform4fv(pos, size, val)
 
 
     def _uniform_mat2(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniformMatrix2fv(pos, size, True, val)
 
 
     def _uniform_mat3(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniformMatrix3fv(pos, size, True, val)
 
 
     def _uniform_mat4(self, pos, val, size):
-        val = np.array(val, dtype=np.float32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.float32).ravel('C')
         gl.glUniformMatrix4fv(pos, 1, True, val)
 
 
     def _uniform_sampler1D(self, pos, val, size):
-        val = np.array(val, dtype=np.int32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.int32).ravel('C')
         gl.glUniform1iv(pos, size, val)
 
 
     def _uniform_sampler2D(self, pos, val, size):
-        val = np.array(val, dtype=np.int32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.int32).ravel('C')
         gl.glUniform1iv(pos, size, val)
 
 
     def _uniform_sampler3D(self, pos, val, size):
-        val = np.array(val, dtype=np.int32, copy=False).ravel('C')
+        val = np.asarray(val, dtype=np.int32).ravel('C')
         gl.glUniform1iv(pos, size, val)
