@@ -12,15 +12,19 @@
 import logging
 import collections.abc as abc
 
-import numpy     as np
-import OpenGL.GL as gl
+import numpy as np
 
-from . import texture
+import fsleyes_widgets.utils.colourbarbitmap as cbarbmp
+from   fsleyes.gl.textures               import texture
+from   fsleyes.utils                     import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 class ColourMapTexture(texture.Texture):
     """The ``ColourMapTexture`` class is a :class:`.Texture` which stores
     a RGB or RGBA colour map.
@@ -132,8 +136,11 @@ class ColourMapTexture(texture.Texture):
 
 
     def setInterp(self, interp):
-        """Set the interpolation used by this ``ColourMapTexture`` - either
-        ``GL_NEAREST`` or ``GL_LINEAR``.
+        """Enable/disable interpolation for this ``ColourMapTexture``.
+        Pass in ``True`` to enable interpolation, ``False`` to disable.
+
+        GL texture interpolation is not used - interpolation is performed
+        by the ``fsleyes_widgets.utils.colourbarbitmap.genColours`` function.
         """
         self.set(interp=interp)
 
@@ -228,7 +235,7 @@ class ColourMapTexture(texture.Texture):
 
         if drange   is None: drange   = [0.0, 1.0]
         if invert   is None: invert   = False
-        if interp   is None: interp   = gl.GL_NEAREST
+        if interp   is None: interp   = False
         if cmap     is None: cmap     = np.zeros((4, 4), dtype =np.float32)
         if res      is None: res      = 256
         if gamma    is None: gamma    = 1
@@ -236,38 +243,25 @@ class ColourMapTexture(texture.Texture):
 
         # The fsleyes.colourmaps module creates
         # ListedColormap instances. If the given
-        # cmap is one of these, there's no point
-        # in using a resolution greater than the
-        # number of colours in the cmap.
-        if isinstance(cmap, colors.ListedColormap):
-            res = min(res, cmap.colors.shape[0])
+        # cmap is one of these and interpolation
+        # is disabled, there's no point in using
+        # a resolution greater than the number
+        # of colours in the cmap.
+        if isinstance(cmap, colors.ListedColormap) and (not interp):
+            res = min(res, len(cmap.colors))
 
-        # If cmap is a function, assume that it accepts
-        # one or more scalar values between 0 and 1,
-        # and converts said values into a numpy array
-        # containing RGB/RGBA colours.  Crop the RGB
-        # colours, as global alpha takes precedence
+        # If cmap is a function, assume that it
+        # accepts one or more scalar values
+        # between 0 and 1, and converts said
+        # values into a numpy array containing
+        # RGB/RGBA colours.
         if isinstance(cmap, abc.Callable):
 
-            # Map display range to colour map logarithmicly
-            if logScale:
-                idxs   = np.linspace(drange[0], drange[1], res)
-                idxs   = np.log(idxs)
-                finite = np.isfinite(idxs)
-                imax   = idxs[finite].max()
-                imin   = idxs[finite].min()
-                idxs   = (idxs - imin) / (imax - imin)
-                idxs[~finite] = 0
+            if logScale: logScale = drange
+            else:        logScale = None
 
-            # Map display range to colour map
-            # linearly, applying gamma scaling to
-            # weight towards one end of the colour
-            # map
-            else:
-                idxs = np.linspace(0.0, 1.0, res) ** gamma
-
-            cmap = cmap(idxs)
-            cmap = cmap[:, :3]
+            cmap = cbarbmp.genColours(
+                cmap, res, invert, alpha, gamma, interp, logScale)
 
         # If RGB, turn into RGBA. If an RGBA cmap
         # has been provided, their alpha values take
@@ -281,10 +275,6 @@ class ColourMapTexture(texture.Texture):
             if alpha is not None:
                 cmap[:, 3] = alpha
 
-        # Reverse the colours if necessray
-        if invert:
-            cmap = cmap[::-1, :]
-
         # If border is provided and is
         # RGB, convert it to RGBA
         if border is not None and len(border) == 3:
@@ -295,7 +285,7 @@ class ColourMapTexture(texture.Texture):
 
             border = newBorder
 
-        return cmap, drange, interp, border
+        return cmap, drange, border
 
 
     def __refresh(self):
@@ -303,7 +293,7 @@ class ColourMapTexture(texture.Texture):
         Re-configures the texture.
         """
 
-        cmap, drange, interp, border = self.__prepareTextureSettings()
+        cmap, drange, border = self.__prepareTextureSettings()
 
         imin, imax = drange
 
@@ -346,10 +336,10 @@ class ColourMapTexture(texture.Texture):
 
         gl.glTexParameteri(gl.GL_TEXTURE_1D,
                            gl.GL_TEXTURE_MAG_FILTER,
-                           interp)
+                           gl.GL_NEAREST)
         gl.glTexParameteri(gl.GL_TEXTURE_1D,
                            gl.GL_TEXTURE_MIN_FILTER,
-                           interp)
+                           gl.GL_NEAREST)
 
         gl.glTexImage1D(gl.GL_TEXTURE_1D,
                         0,
