@@ -13,22 +13,27 @@ for all other FSLeyes texture types. See also the :class:`.Texture2D` and
 import              logging
 import              contextlib
 import functools as ft
+import              threading
 
-import numpy                        as np
-import OpenGL.GL                    as gl
+import numpy as np
 
-import fsl.utils.idle               as idle
+from   fsl.utils             import idle
+from   fsl.utils             import notifier
+from   fsl.transform         import affine
+from   fsleyes_widgets.utils import status
+from   fsleyes               import strings
 
-import fsl.utils.notifier           as notifier
-import fsl.transform.affine         as affine
-import fsleyes_widgets.utils.status as status
-import fsleyes.strings              as strings
-from . import data                  as texdata
+import fsleyes.gl                as fslgl
+import fsleyes.gl.textures.data  as texdata
+from   fsleyes.utils         import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 class TextureBase:
     """Base mixin class used by the :class:`Texture` class.
 
@@ -115,6 +120,22 @@ class TextureBase:
         self.__texture = None
 
 
+    @property
+    def destroyed(self):
+        """Returns ``True`` if this Texture has been destroyed, ``False``
+        otherwise.
+        """
+        return self.__texture is None
+
+
+    def recreateHandle(self):
+        """Deletes and re-creates the texture handle. The texture will need
+        to be reconfigured after a call to this method.
+        """
+        gl.glDeleteTextures(self.__texture)
+        self.__texture = int(gl.glGenTextures(1))
+
+
     @property
     def name(self):
         """Returns the name of this texture. This is not the GL texture name,
@@ -573,6 +594,12 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         self.__shape        = None
         self.__preparedData = None
 
+        # Synchronisation lock used to
+        # prevent the data reference
+        # from being cleared while a
+        # data prep task is enqueued
+        self.__dataLock     = threading.RLock()
+
         # The data is refreshed on
         # every call to set or refresh
         # These attributes are set by
@@ -609,15 +636,16 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
     def destroy(self):
         """Must be called when this ``Texture`` is no longer needed.
         """
-        TextureBase.destroy(self)
 
-        if self.__taskThread is not None:
-            self.__taskThread.stop()
+        with self.dataLock:
+            TextureBase.destroy(self)
 
-        self.__taskThread   = None
-        self.__data         = None
-        self.__preparedData = None
+            if self.__taskThread is not None:
+                self.__taskThread.stop()
 
+            self.__taskThread   = None
+            self.__data         = None
+            self.__preparedData = None
 
 
     def ready(self):
@@ -627,6 +655,16 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         return self.__ready
 
 
+    @property
+    def dataLock(self):
+        """Return a ``threading.RLock`` that is used to limit concurrent
+        access to the data. This is just used to ensure that an enqueued
+        data processing task (which takes place off the main thread) will
+        not proceed if the texture is destroyed after it has been enqeued.
+        """
+        return self.__dataLock
+
+
     @property
     def voxValXform(self):
         """Return a transformation matrix that can be used to transform
@@ -883,25 +921,37 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         # thread using the idle module.
         def genData():
 
-            # Another genData function is
-            # already queued - don't run.
-            # The TaskThreadVeto error
-            # will stop the TaskThread from
-            # calling configTexture as well.
-            if self.__taskThread is not None and \
-               self.__taskThread.isQueued(self.__taskName):
-                raise idle.TaskThreadVeto()
+            with self.dataLock:
 
-            self.__determineTextureType()
+                # The texture may be destroyed
+                # after this call was scheduled
+                if self.destroyed:
+                    return
 
-            if refreshData:
-                self.__prepareTextureData()
+                # Another genData function is
+                # already queued - don't run.
+                # The TaskThreadVeto error
+                # will stop the TaskThread from
+                # calling configTexture as well.
+                if self.__taskThread is not None and \
+                   self.__taskThread.isQueued(self.__taskName):
+                    raise idle.TaskThreadVeto()
+
+                self.__determineTextureType()
+
+                if refreshData:
+                    self.__prepareTextureData()
 
         # Once genData is finished, we pass the
         # result (see __prepareTextureData) to
         # the sub-class doRefresh method.
         def doRefresh():
 
+            # The texture may be destroyed
+            # after this call was scheduled
+            if self.destroyed:
+                return
+
             self.doRefresh()
 
             self.__ready = True
@@ -1043,9 +1093,9 @@ class Texture(notifier.Notifier, TextureBase, TextureSettingsMixin):
         log.debug('Texture (%s) is to be stored as %s/%s/%s '
                   '(normalised: %s)',
                   self.name,
-                  texdata.GL_TYPE_NAMES[texDtype],
-                  texdata.GL_TYPE_NAMES[texFmt],
-                  texdata.GL_TYPE_NAMES[intFmt],
+                  fslgl.glTypeName(texDtype),
+                  fslgl.glTypeName(texFmt),
+                  fslgl.glTypeName(intFmt),
                   normalise)
 
         self.__texFmt    = texFmt
