@@ -11,19 +11,22 @@ classes.
 
 import logging
 
-import numpy     as np
-import OpenGL.GL as gl
+import numpy as np
 
-import fsl.transform.affine        as affine
-import fsl.data.utils              as dutils
-import fsleyes.gl                  as fslgl
-import fsleyes.gl.shaders          as shaders
-import fsleyes.gl.textures.texture as texture
+from   fsl.transform       import affine
+import fsl.data.utils          as dutils
+import fsleyes.gl              as fslgl
+from   fsleyes.gl          import shaders
+from   fsleyes.gl.textures import texture
+from   fsleyes.utils       import lazyimport
 
 
 log = logging.getLogger(__name__)
 
 
+gl = lazyimport('OpenGL.GL', f'{__name__}.gl')
+
+
 class DepthTexture(texture.Texture):
     """The ``DepthTexture`` class is a 2D ``GL_DEPTH_COMPONENT24`` texture
     which is used by the :class:`.RenderTexture` class.
@@ -213,7 +216,7 @@ class Texture2D(texture.Texture):
         else:                 width, height = data.shape[1:]
 
         if data is not None:
-            data = np.array(data.ravel('F'), copy=False)
+            data = np.asarray(data.ravel('F'))
             data = dutils.makeWriteable(data)
 
         interp = self.interp
