@@ -14,15 +14,10 @@ within an existing application.
 See the :mod:`fsleyes` package documentation for more details on ``fsleyes``.
 
 
-.. note:: Even though ``fsleyes`` (this module) and :mod:`fsleyes.render` (the
-          off-screen renderer) are intended to be separate applications, the
-          current version of PyInstaller (3.x) does not support bundling of
-          multiple executables
-          (https://github.com/pyinstaller/pyinstaller/issues/1527).
-
-          So at this point in time, :mod:`.fsleyes.render` can be invoked via
-          ``fsleyes.main`` by passing ``'render'`` as the first argument,
-          e.g.::
+.. note:: ``fsleyes`` (this module) and :mod:`fsleyes.render` (the off-screen
+          renderer) are intended to be separate applications, but
+          :mod:`.fsleyes.render` can also be invoked via ``fsleyes.main`` by
+          passing ``'render'`` as the first argument, e.g.::
 
               python -m fsleyes.main render ...
 """
@@ -32,6 +27,7 @@ import functools as ft
 import os.path   as op
 import              os
 import              sys
+import              shlex
 import              signal
 import              logging
 import              textwrap
@@ -350,6 +346,8 @@ def main(args=None):
         # telling the wx.App to exit gracefully.
         except (SystemExit, Exception) as e:
             app.ExitMainLoop()
+            if not isinstance(e, SystemExit):
+                log.error(e, exc_info=True)
             exitCode[0] = getattr(e, 'code', 1)
             return
 
@@ -366,10 +364,30 @@ def main(args=None):
 
     def buildGui():
 
+        frame = None
+
+        # Called after overlays have been loaded. The
+        # SliceCanvas (upon which OrthoPanel is
+        # built) has some janky logic which tries to
+        # preserve the display bounds when new
+        # overlays are loaded. This does a reasonable
+        # job when the bounds are already valid and a
+        # new overlay is loaded, but it can mess up
+        # during initialisation. So we call
+        # resetDisplay once they're loaded and we're
+        # ready to go.
+        def onLoad(*_a):
+            from fsleyes.views.orthopanel import OrthoPanel
+            if frame is not None:
+                for vp in frame.viewPanels:
+                    if isinstance(vp, OrthoPanel):
+                        vp.resetDisplay()
+
         # Now the main stuff - create the overlay
         # list and the master display context,
         # and then create the FSLeyesFrame.
-        overlayList, displayCtx = makeDisplayContext(namespace[0], splash)
+        overlayList, displayCtx = makeDisplayContext(namespace[0], splash,
+                                                     onLoad)
         app.SetOverlayListAndDisplayContext(overlayList, displayCtx)
         frame = makeFrame(namespace[0],
                           displayCtx,
@@ -396,12 +414,22 @@ def main(args=None):
                          ignorePoint=False)
 
         # start notebook server
+        if namespace[0].noBrowser:
+            namespace[0].notebook = True
         if namespace[0].notebookFile is not None:
             namespace[0].notebook     = True
             namespace[0].notebookFile = op.abspath(namespace[0].notebookFile)
         if namespace[0].notebook:
             from fsleyes.actions.notebook import NotebookAction
-            frame.menuActions[NotebookAction](namespace[0].notebookFile)
+            nbfile  =     namespace[0].notebookFile
+            browser = not namespace[0].noBrowser
+            nbact   = frame.menuActions[NotebookAction]
+
+            nbact(nbfile, browser)
+
+            if not browser:
+                print('Connect to the FSLeyes Jupyter '
+                      f'kernel with {nbact.kernel.connfile}')
 
         # start CLI server
         if namespace[0].cliserver:
@@ -567,11 +595,6 @@ def initialise(splash, namespace, callback):
 
     fslsettings.write('loadSaveOverlayDir', curDir)
 
-    # Initialise silly things
-    if namespace.bumMode:
-        import fsleyes.icons as icons
-        icons.BUM_MODE = True
-
     # Set notebook server port
     fslsettings.write('fsleyes.notebook.port', namespace.notebookPort)
 
@@ -593,17 +616,11 @@ def initialise(splash, namespace, callback):
         sys.exit(1)
 
 
-def shutdown(ev=None):
+def shutdown():
     """Called when FSLeyes exits normally (i.e. the user closes the window).
     Does some final clean-up before exiting.
-
-    This function is used as a wxpython event handler, so it accepts an ``ev``
-    arguments, but ignores its value.
     """
 
-    if ev is not None:
-        ev.Skip()
-
     import fsl.utils.settings as fslsettings
     import fsleyes.gl         as fslgl
 
@@ -624,9 +641,29 @@ def parseArgs(argv):
     :arg argv: command line arguments for ``fsleyes``.
     """
 
-    import fsleyes.parseargs as parseargs
-    import fsleyes.layouts   as layouts
-    import fsleyes.version   as version
+    import fsl.utils.settings as fslsettings
+    import fsleyes.parseargs  as parseargs
+    import fsleyes.layouts    as layouts
+    import fsleyes.version    as version
+
+    # Apply user-defaults, stored in <fsleyes-config-dir>/default_arguments.txt
+    userArgs = fslsettings.readFile('default_arguments.txt')
+
+    # Apply site-defaults (if user defaults don't exist)
+    if userArgs is None:
+        siteDir = os.environ.get('FSLEYES_SITE_CONFIG_DIR', None)
+        if siteDir is not None:
+            siteArgs = op.join(siteDir, 'default_arguments.txt')
+            if op.exists(siteArgs):
+                with open(siteArgs, 'rt') as f:
+                    userArgs = f.read()
+
+    if userArgs is not None:
+        userArgs = [line.strip() for line in userArgs.split('\n')]
+        userArgs = [line         for line in userArgs if line != '']
+        userArgs = [line         for line in userArgs if not line[0] == '#']
+        userArgs = shlex.split(' '.join(userArgs))
+        argv     = userArgs + argv
 
     parser = parseargs.ArgumentParser(
         add_help=False,
@@ -659,6 +696,8 @@ def parseArgs(argv):
         up the scene.
         """.format(layouts=', '.join(allLayouts)))
 
+    exclude = {'LightBoxOpts' : ['nrows', 'ncols']}
+
     # Options for configuring the scene are
     # managed by the parseargs module
     return parseargs.parseArgs(parser,
@@ -666,10 +705,11 @@ def parseArgs(argv):
                                name,
                                prolog=prolog,
                                desc=description,
-                               argOpts=['-r', '--runscript'])
+                               argOpts=['-r', '--runscript'],
+                               exclude=exclude)
 
 
-def makeDisplayContext(namespace, splash):
+def makeDisplayContext(namespace, splash, onLoad=None):
     """Creates the top-level *FSLeyes* :class:`.DisplayContext` and
     :class:`.OverlayList` .
 
@@ -685,6 +725,9 @@ def makeDisplayContext(namespace, splash):
 
     :arg splash:   The :class:`.FSLeyesSplash` frame, created in :func:`init`.
 
+    :arg onLoad:   Function called when overlays have been loaded - see
+                   :func:`.loadoverlay.loadOverlays`
+
     :returns: a tuple containing:
                 - the :class:`.OverlayList`
                 - the master :class:`.DisplayContext`
@@ -720,7 +763,8 @@ def makeDisplayContext(namespace, splash):
     # Load the images - the splash screen status will
     # be updated with the currently loading overlay name.
     parseargs.applyMainArgs(   namespace, overlayList, displayCtx)
-    parseargs.applyOverlayArgs(namespace, overlayList, displayCtx)
+    parseargs.applyOverlayArgs(namespace, overlayList, displayCtx,
+                               onLoad=onLoad)
 
     return overlayList, displayCtx
 
@@ -827,6 +871,13 @@ def makeFrame(namespace, displayCtx, overlayList, splash, closeHandlers):
 
     status.update('Setting up scene...')
 
+    # Set the default SceneOpts.performance
+    # level so that all created SceneOpts
+    # instances will default to it
+    if namespace.performance is not None:
+        fsldisplay.SceneOpts.performance.setAttribute(
+            None, 'default', namespace.performance)
+
     # If a layout has been specified,
     # we load the layout
     if namespace.scene is not None:
@@ -875,7 +926,8 @@ def makeFrame(namespace, displayCtx, overlayList, splash, closeHandlers):
 def fslDirWarning(parent):
     """Checks to see if the ``$FSLDIR`` environment variable is set, or
     if a FSL installation directory has been saved previously. If not,
-    displays a warning via a :class:`.FSLDirDialog`.
+    attempts to locate ``$FSLDIR`` or, failing that, displays a warning
+    via a :class:`.FSLDirDialog`.
 
     :arg parent: A ``wx`` parent object.
     """
@@ -884,26 +936,31 @@ def fslDirWarning(parent):
         return
 
     import fsl.utils.settings as fslsettings
+    from fsleyes_widgets.dialog import FSLDirDialog
 
-    # Check settings before
-    # prompting the user
+    # Check saved settings
     fsldir = fslsettings.read('fsldir')
 
     if fsldir is not None:
         fslplatform.fsldir = fsldir
         return
 
-    from fsleyes_widgets.dialog import FSLDirDialog
+    # try to derive $FSLDIR from the python
+    # interpreter being used to run FSLeyes
+    fsldir  = None
+    basedir = op.join(op.dirname(sys.executable), '..')
 
-    dlg = FSLDirDialog(parent, 'FSLeyes', fslplatform.os == 'Darwin')
+    if op.exists(op.join(basedir, 'etc', 'fslversion')):
+        fsldir = basedir
 
-    if dlg.ShowModal() == wx.ID_OK:
-
-        fsldir = dlg.GetFSLDir()
-
-        log.debug('Setting $FSLDIR to {} (specified '
-                  'by user)'.format(fsldir))
+    # otherwise ask the user
+    else:
+        dlg = FSLDirDialog(parent, 'FSLeyes', fslplatform.os == 'Darwin')
+        if dlg.ShowModal() == wx.ID_OK:
+            fsldir = dlg.GetFSLDir()
 
+    if fsldir is not None:
+        log.debug('Setting $FSLDIR to %s', fsldir)
         fslplatform.fsldir        = fsldir
         fslsettings.write('fsldir', fsldir)
 
