@@ -16,7 +16,8 @@ def apply():
     path    = mod.__path__
     name    = mod.__name__
     submods = pkgutil.iter_modules(path, f'{name}.')
+    submods = sorted(m[1] for m in submods)
 
-    for subpath, subname, _ in  submods:
-        log.debug('Applying patch: %s', subname)
-        importlib.import_module(subname)
+    for submod in  submods:
+        log.debug('Applying patch: %s', submod)
+        importlib.import_module(submod)
