@@ -343,6 +343,7 @@ class AtlasPanel(ctrlpanel.ControlPanel):
                     return
 
                 atlas = atlases.loadAtlas(atlasID, summary, resolution=res)
+                atlas.data
 
                 # The atlas panel may be destroyed
                 # before the atlas is loaded.
@@ -492,17 +493,21 @@ class AtlasPanel(ctrlpanel.ControlPanel):
 
                 # regional statistic/probability image
                 else:
+                    # Add a small offset to the upper clipping
+                    # range, as the clipping range is inclusive
+                    # and we don't want max values to be clipped
+                    croff   = (atlasDesc.upper - atlasDesc.lower) * 0.01
                     overlay = atlas.get(index=labelIdx)
                     initprops['overlayType']   = 'volume'
                     initprops['cmap']          = 'hot'
                     initprops['displayRange']  = (atlasDesc.lower,
                                                   atlasDesc.upper)
                     initprops['clippingRange'] = (atlasDesc.lower,
-                                                  atlasDesc.upper)
+                                                  atlasDesc.upper + croff)
 
             overlay.name = overlayName
 
-            with props.suppress(self.overlayList, 'overlays', self.name):
+            with props.skip(self.overlayList, 'overlays', self.name):
                 self.overlayList.append(overlay, **initprops)
 
             self.__overlayPanel.setOverlayState(
