@@ -8,12 +8,12 @@
 panel for viewing cluster results from a FEAT analysis.
 """
 
+import itertools                     as it
 import                                  logging
 import                                  wx
-import                                  six
+import numpy                         as np
 
 import fsl.utils.idle                as idle
-import fsl.data.image                as fslimage
 import fsl.data.featimage            as featimage
 import fsl.data.featanalysis         as featanalysis
 
@@ -40,11 +40,11 @@ class ClusterPanel(ctrlpanel.ControlPanel):
 
     The ``ClusterPanel`` contains controls which allow the user to:
 
-      - Select the COPE for which cluster results are displayed
+      - Select the COPE or f-test for which cluster results are displayed
 
-      - Add a Z statistic overlay for the currently displayed COPE
+      - Add a Z statistic overlay for the currently displayed COPE/f-test
 
-      - Add a cluster mask overlay for the currently displayed COPE
+      - Add a cluster mask overlay for the currently displayed COPE/f-test
 
       - Navigate to the Z maximum location, Z centre-of-gravity location,
         or COPE maximum location, for a specific cluster.
@@ -123,12 +123,32 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         # not be in the overlay list.
         self.__featImages = {}
 
-        # A WidgetGrid is created for each
-        # contrast of a FEAT image, and cached
-        # in this dictionary. This is because
-        # it is quite expensive to create the
-        # grid widgets. This dictionary contains
-        # {FEATImage : [WidgetGrid]} mappings.
+        # The clusterData dict is a mapping of {
+        # FEATImage : clusters} pairs, containing
+        # information about the significant clusters
+        # for all COPEs/f-tests from the feat
+        # analysis.  The cluster information is
+        # loaded by the __loadClusterResults method
+        # when a FEAT image is loaded/selected, or
+        # when the user selects a contrast/f-test.
+        self.__clusterData = {}
+
+        # If more than one FEAT analysis is loaded,
+        # the cluster panel will be reset when an
+        # overlay from a different analysis is
+        # selected. This dictionary is used to store
+        # the selected statistic image for each
+        # FEAT analysis so that when an analysis is
+        # re-selected, the stat is set to its
+        # previous value.
+        self.__selectedStats = {}
+
+        # A WidgetGrid is created for each contrast/
+        # f-test of a FEAT image, and cached in this
+        # dictionary. This is because it is quite
+        # expensive to create the widget grids. This
+        # dictionary contains {FEATImage :
+        # [WidgetGrid]} mappings.
         self.__clusterGrids = {}
 
         self.__addZStats   .SetLabel(strings.labels[self, 'addZStats'])
@@ -167,7 +187,13 @@ class ClusterPanel(ctrlpanel.ControlPanel):
 
         self.SetMinSize(self.__calcMinSize())
 
-        self.__selectedOverlay = None
+        # We store a reference to the currently displayed
+        # overlay, and store a reference to the corresponding
+        # FEAT analysis in the __featImages dict. We need a
+        # ref to the overlay in case the FEATImage is not in
+        # the overlay list, and we need to do coordinate
+        # transforms via a DisplayOpts instance.
+        self.__displayedOverlay = None
         self.__selectedOverlayChanged()
 
 
@@ -182,8 +208,10 @@ class ClusterPanel(ctrlpanel.ControlPanel):
                 if grid is not None:
                     grid.Destroy()
 
-        self.__clusterGrids = None
-        self.__featImages   = None
+        self.__clusterGrids     = None
+        self.__featImages       = None
+        self.__selectedStats    = None
+        self.__displayedOverlay = None
 
         self.overlayList.removeListener('overlays',        self.name)
         self.displayCtx .removeListener('selectedOverlay', self.name)
@@ -191,6 +219,30 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         ctrlpanel.ControlPanel.destroy(self)
 
 
+    @property
+    def statSelect(self):
+        """Return a reference to the ``wx.Choice`` widget used to select
+        the current contrast/f-test.
+        """
+        return self.__statSelect
+
+
+    @property
+    def addZStats(self):
+        """Return a reference to the ``wx.Button`` widget used to add
+        the Z-statistic image for the current test to the overlay list.
+        """
+        return self.__addZStats
+
+
+    @property
+    def addClustMask(self):
+        """Return a reference to the ``wx.Button`` widget used to add
+        the cluster mask image for the current test to the overlay list.
+        """
+        return self.__addClustMask
+
+
     def __calcMinSize(self):
         """Figures out the minimum size that this ``ClusterPanel`` should
         have. Called by :meth:`__init__`.
@@ -235,14 +287,14 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         self.__sizer.Show(self.__disabledText, False)
         self.__sizer.Show(self.__mainSizer,    True)
 
-        overlay   = self.__selectedOverlay
+        overlay   = self.__displayedOverlay
         featImage = self.__featImages[overlay]
-        contrast  = self.__statSelect.GetSelection()
+        testidx   = self.__statSelect.GetSelection()
 
         for fimg, grids in self.__clusterGrids.items():
             for i, grid in enumerate(grids):
                 if grid is not None:
-                    show = fimg is featImage and i == contrast
+                    show = fimg is featImage and i == testidx
                     self.__mainSizer.Show(grid, show)
 
         self.Layout()
@@ -251,24 +303,29 @@ class ClusterPanel(ctrlpanel.ControlPanel):
     def __statSelected(self, ev=None):
         """Called when a COPE is selected. Retrieves a cached
         :class:`.WidgetGrid`, or creates a new one (via the
-        :meth:`__genClusterGrid` method) which displays information
-        about the clusters associated with the currently selected contrast.
+        :meth:`__genClusterGrid` method) which displays information about the
+        clusters associated with the currently selected contrast or f-test.
         """
-        overlay   = self.__selectedOverlay
+        overlay   = self.__displayedOverlay
         featImage = self.__featImages[overlay]
         idx       = self.__statSelect.GetSelection()
-        data      = self.__statSelect.GetClientData(idx)
+
+        # save the selection so we can
+        # restore it later on if needed
+        self.__selectedStats[featImage.dataSource] = idx
 
         self.Refresh()
         self.Update()
 
         if featImage not in self.__clusterGrids:
-            self.__clusterGrids[featImage] = [None] * featImage.numContrasts()
+            cons   = [None] * featImage.numContrasts()
+            ftests = [None] * featImage.numFTests()
+            self.__clusterGrids[featImage] = cons + ftests
 
         grid = self.__clusterGrids[featImage][idx]
 
         if grid is None:
-            grid = self.__genClusterGrid(overlay, featImage, idx, data)
+            grid = self.__genClusterGrid(overlay, featImage, idx)
             self.__clusterGrids[featImage][idx] = grid
             self.__mainSizer.Add(grid, flag=wx.EXPAND, proportion=1)
 
@@ -283,16 +340,21 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         to the :class:`.OverlayList`.
         """
 
-        overlay   = self.__selectedOverlay
+        overlay   = self.__displayedOverlay
         featImage = self.__featImages[overlay]
-        contrast  = self.__statSelect.GetSelection()
-        zstats    = featImage.getZStats(contrast)
 
-        for ol in self.overlayList:
+        # Get selected test, and convert
+        # to a contrast or f-test index
+        idx       = self.__statSelect.GetSelection()
+        ftest     = self.__clusterData[featImage][idx][1]
+        idx       = self.__clusterData[featImage][idx][0]
 
-            # Already in overlay list
-            if ol.dataSource == zstats.dataSource:
-                return
+        if ftest: zstats = featImage.getZFStats(idx)
+        else:     zstats = featImage.getZStats(idx)
+
+        # Already in overlay list
+        if self.overlayList.find(zstats.dataSource) is not None:
+            return
 
         log.debug('Adding Z-statistic %s to overlay list', zstats.name)
         self.overlayList.append(zstats, overlayType='volume')
@@ -313,22 +375,32 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         cluster mask image for the currewnt contrast (see the
         :meth:`.FEATImage.getClusterMask` method)
         """
-        overlay   = self.__selectedOverlay
+        overlay   = self.__displayedOverlay
         featImage = self.__featImages[overlay]
-        contrast  = self.__statSelect.GetSelection()
-        mask      = featImage.getClusterMask(contrast)
 
-        for ol in self.overlayList:
+        # Get selected test, and convert
+        # to a contrast or f-test index
+        idx       = self.__statSelect.GetSelection()
+        ftest     = self.__clusterData[featImage][idx][1]
+        idx       = self.__clusterData[featImage][idx][0]
 
-            # Already in overlay list
-            if ol.dataSource == mask.dataSource:
-                return
+        if ftest: mask = featImage.getFClusterMask(idx)
+        else:     mask = featImage.getClusterMask(idx)
+
+        # Already in overlay list
+        if self.overlayList.find(mask.dataSource) is not None:
+            return
 
         log.debug('Adding cluster mask %s to overlay list', mask.name)
-        self.overlayList.append(mask, overlayType='label')
+        self.overlayList.append(mask)
+
+        autodisplay.autoDisplay(
+            mask,
+            self.overlayList,
+            self.displayCtx)
 
 
-    def __genClusterGrid(self, overlay, featImage, contrast, clusters):
+    def __genClusterGrid(self, overlay, featImage, testidx):
         """Creates and returns a :class:`.WidgetGrid` which contains the given
         list of clusters, which are related to the given contrast.
 
@@ -345,11 +417,7 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         :arg featImage: The :class:`.FEATImage` to which the clusters are
                         related.
 
-        :arg contrast:  The (0-indexed) number of the contrast to which the
-                        clusters are related.
-
-        :arg clusters:  A sequence of objects, each representing one cluster.
-                        See the :meth:`.FEATImage.clusterResults` method.
+        :arg testidx:   The internal index identifying the contrast/f-test.
         """
 
         cols = {'index'         : 0,
@@ -363,18 +431,23 @@ class ClusterPanel(ctrlpanel.ControlPanel):
                 'copemaxcoords' : 8,
                 'copemean'      : 9}
 
-        grid    = widgetgrid.WidgetGrid(self)
-        conName = featImage.contrastNames()[contrast]
-        opts    = self.displayCtx.getOpts(overlay)
+        grid = widgetgrid.WidgetGrid(self)
+        opts = self.displayCtx.getOpts(overlay)
+
+        # We store contrast/f-test results in
+        # a single list, so need to look up the
+        # test type, and the actual contrast/
+        # f-test index used by the FEATImage
+        testName = self.__clusterData[featImage][testidx][2]
+        clusters = self.__clusterData[featImage][testidx][3]
 
         # We hide the grid and disable
-        # this panle while the grid is
+        # this panel while the grid is
         # being created.
         grid.Hide()
         self.Disable()
 
         grid.SetGridSize(len(clusters), 10)
-
         grid.ShowRowLabels(False)
         grid.ShowColLabels(True)
 
@@ -383,29 +456,36 @@ class ClusterPanel(ctrlpanel.ControlPanel):
 
         def makeCoordButton(coords):
 
-            label = wx.StaticText(grid, label='[{} {} {}]'.format(*coords))
-            btn   = wx.Button(grid,
-                              label=six.u('\u2192'),
-                              style=wx.BU_EXACTFIT)
+            # Cluster results for f-tests don't
+            # have copemax info - the coordinates
+            # will all be nan
+            if np.isnan(coords[0]):
+                label = wx.StaticText(grid, label='[n/a]')
+                btn   = (1, 1)
+            else:
+                x, y, z = coords
+                label   = wx.StaticText(grid, label=f'[{x} {y} {z}]')
+                btn     = wx.Button(grid, label='\u2192', style=wx.BU_EXACTFIT)
+
+                def onClick(ev):
+                    dloc = opts.transformCoords([coords], 'voxel', 'display')
+                    self.displayCtx.location = dloc[0]
+
+                btn.Bind(wx.EVT_BUTTON, onClick)
 
             sizer = wx.BoxSizer(wx.HORIZONTAL)
             sizer.Add(label, flag=wx.EXPAND, proportion=1)
             sizer.Add(btn)
 
-            def onClick(ev):
-                dloc = opts.transformCoords([coords], 'voxel', 'display')[0]
-                self.displayCtx.location = dloc
-
-            btn.Bind(wx.EVT_BUTTON, onClick)
-
             return sizer
 
         # Creating all of the widgets could
-        # take a bit of time, so we'll
-        # do it asynchronously via idle.idle
-        # display a message while doing so.
+        # take a bit of time, so we do it
+        # asynchronously via on the wx idle
+        # loop. Display a message while
+        # doing so.
         status.update(strings.messages[self, 'loadingCluster'].format(
-            contrast + 1, conName), timeout=None)
+            testName), timeout=None)
 
         def addCluster(i, clust):
 
@@ -422,19 +502,16 @@ class ClusterPanel(ctrlpanel.ControlPanel):
                                           clust.copemaxy,
                                           clust.copemaxz))
 
-            def fmt(v):
-                return '{}'.format(v)
-
-            grid.SetText(  i, cols['index'],         fmt(clust.index))
-            grid.SetText(  i, cols['nvoxels'],       fmt(clust.nvoxels))
-            grid.SetText(  i, cols['p'],             fmt(clust.p))
-            grid.SetText(  i, cols['logp'],          fmt(clust.logp))
-            grid.SetText(  i, cols['zmax'],          fmt(clust.zmax))
+            grid.SetText(  i, cols['index'],         f'{clust.index}')
+            grid.SetText(  i, cols['nvoxels'],       f'{clust.nvoxels}')
+            grid.SetText(  i, cols['p'],             f'{clust.p}')
+            grid.SetText(  i, cols['logp'],          f'{clust.logp}')
+            grid.SetText(  i, cols['zmax'],          f'{clust.zmax}')
             grid.SetWidget(i, cols['zmaxcoords'],    zmaxbtn)
             grid.SetWidget(i, cols['zcogcoords'],    zcogbtn)
-            grid.SetText(  i, cols['copemax'],       fmt(clust.copemax))
+            grid.SetText(  i, cols['copemax'],       f'{clust.copemax}')
             grid.SetWidget(i, cols['copemaxcoords'], copemaxbtn)
-            grid.SetText(  i, cols['copemean'],      fmt(clust.copemean))
+            grid.SetText(  i, cols['copemean'],      f'{clust.copemean}')
 
         # Refresh the grid widget when all
         # clusters have been added.
@@ -468,25 +545,30 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         # that have been removed from the
         # list.
         for overlay in list(self.__featImages.keys()):
-            if overlay not in self.overlayList:
+            if overlay in self.overlayList:
+                continue
 
-                featImage = self.__featImages.pop(overlay)
+            featImage = self.__featImages.pop(overlay)
 
-                # Has the feat image associated with
-                # this overlay also been removed?
-                if featImage is overlay or \
-                   featImage not in self.overlayList:
+            # Do nothing if there any other overlays
+            # associated with this FEAT analysis
+            if featImage in self.__featImages.values():
+                continue
 
-                    # The grid widgets for the feat image
-                    # associated with this overlay may
-                    # have already been destroyed.
-                    try:             grids = self.__clusterGrids.pop(featImage)
-                    except KeyError: grids = []
+            # Otherwise clear everything
+            # we know about the analysis.
+            self.__selectedStats.pop(featImage, None)
+            self.__clusterData  .pop(featImage, None)
 
-                    for grid in grids:
-                        if grid is not None:
-                            self.__mainSizer.Detach(grid)
-                            grid.Destroy()
+            grids = self.__clusterGrids.pop(featImage, [])
+            for grid in grids:
+                if grid is not None:
+                    self.__mainSizer.Detach(grid)
+                    grid.Destroy()
+
+        if self.__displayedOverlay is not None and \
+           self.__displayedOverlay not in self.overlayList:
+            self.__displayedOverlay = None
 
         self.__selectedOverlayChanged()
         self.__enableOverlayButtons()
@@ -495,24 +577,36 @@ class ClusterPanel(ctrlpanel.ControlPanel):
     def __enableOverlayButtons(self):
         """Enables/disables the *Add Z statistic* and *Add cluster mask*
         buttons depending on whether the corresponding overlays are in the
-        :class:`.OverlayList`.
+        :class:`.OverlayList`. The buttons are only enabled if the
+        corresponding overlays are *not* already loaded.
         """
 
-        if self.__selectedOverlay is None:
+        if self.__displayedOverlay is None:
             return
 
-        overlay   = self.__selectedOverlay
+        overlay   = self.__displayedOverlay
         featImage = self.__featImages[overlay]
-        contrast  = self.__statSelect.GetSelection()
+        idx       = self.__statSelect.GetSelection()
 
         # No cluster results
-        if contrast < 0:
+        if idx < 0:
             self.__addZStats   .Enable(False)
             self.__addClustMask.Enable(False)
             return
 
-        zstat     = featImage.getZStats(     contrast)
-        clustMask = featImage.getClusterMask(contrast)
+        # Convert from our list of test results
+        # to the actual contrast / f-test index.
+        # (we store all contrast/f-test results
+        # in a single list)
+        ftest = self.__clusterData[featImage][idx][1]
+        idx   = self.__clusterData[featImage][idx][0]
+
+        if ftest:
+            zstat     = featImage.getZFStats(     idx)
+            clustMask = featImage.getFClusterMask(idx)
+        else:
+            zstat     = featImage.getZStats(     idx)
+            clustMask = featImage.getClusterMask(idx)
 
         dss = [ovl.dataSource for ovl in self.overlayList]
 
@@ -529,127 +623,166 @@ class ClusterPanel(ctrlpanel.ControlPanel):
         loaded in, and displayed on a :class:`.WidgetGrid`.
         """
 
-        prevOverlay            = self.__selectedOverlay
-        self.__selectedOverlay = None
-
         # No overlays are loaded
         if len(self.overlayList) == 0:
             self.__disable(strings.messages[self, 'noOverlays'])
             return
 
-        overlay = self.displayCtx.getSelectedOverlay()
+        overlay       = self.displayCtx.getSelectedOverlay()
+        prevOverlay   = self.__displayedOverlay
+        featImage     = self.__lookupFEATImage(overlay)
+        prevFeatImage = self.__lookupFEATImage(prevOverlay)
 
-        # Overlay is in-memory
-        if overlay.dataSource is None:
-            self.__disable(strings.messages[self, 'notFEAT'])
-            return
-
-        featDir = featanalysis.getAnalysisDir(overlay.dataSource)
-
-        # No FEAT analysis, or not an Image,
-        # can't do anything with that
-        if featDir is None or not isinstance(overlay, fslimage.Nifti):
-            log.debug('Overlay %s is not part of a feat '
-                      'analysis, or is not Nifti', overlay)
-            self.__disable(strings.messages[self, 'notFEAT'])
-            return
-
-        # Selected overlay is either the
-        # same one (maybe the overlay list,
-        # rather than the selected overlay,
-        # changed) or the newly selected
-        # overlay is from the same FEAT
-        # analysis. No need to do anything.
-        if prevOverlay is not None:
-
-            prevFeatImage = self.__featImages.get(prevOverlay)
-
-            if prevOverlay is overlay or \
-               (prevFeatImage is not None and
-                    featDir == prevFeatImage.getFEATDir()):
-
-                log.debug('Overlay %s is already selected.', overlay)
-
-                # Make sure the overlay -> FEATImage
-                # mapping is present, and (re-)cache
-                # a reference to the selected overlay.
-                self.__featImages[overlay] = prevFeatImage
-                self.__selectedOverlay     = overlay
-
-                return
-
-        # We're in business. The newly selected
-        # overlay is a part of a FEAT analysis
-        # which is not currently being displayed.
-        self.__selectedOverlay = overlay
-
-        # Clear the stat selection combo box.
-        self.__statSelect.Clear()
-
-        # Get the FEATImage associated with
-        # this overlay, so we can get
-        # information about the FEAT analysis
-        featImage = self.__featImages.get(overlay)
+        # selected a non-FEAT image - clear the
+        # panel, or keep displaying the previous
+        # analysis
         if featImage is None:
-
-            # If the overlay itself is a FEATImage,
-            # then we have nothing to do.
-            if isinstance(overlay, featimage.FEATImage):
-                featImage = overlay
-            else:
-                # The FEATImage might already
-                # be in the overlay list -
-                # let's search for it.
-                for ovl in self.overlayList:
-                    if isinstance(ovl, featimage.FEATImage) and \
-                       ovl.getFEATDir() == featDir:
-                        featImage = ovl
-
-            # As a last resort, if the FEATImage is not
-            # in the overlay list, we'll create one.
-            if featImage is None:
-                featImage = featimage.FEATImage(featDir)
-
-            self.__featImages[overlay] = featImage
-
-        log.debug('Identified FEAT analysis associated with overlay '
-                  '%s: %s', overlay, featImage.getFEATDir())
-
-        # Get the contrast and cluster
-        # information for the FEAT analysis.
-        display  = self.displayCtx.getDisplay(overlay)
-        numCons  = featImage.numContrasts()
-        conNames = featImage.contrastNames()
-
-        try:
-            # clusts is a list of (contrast, clusterList) tuples
-            clusts = [(c, featImage.clusterResults(c)) for c in range(numCons)]
-            clusts = [c for c in clusts if c[1] is not None]
-
-        # Error parsing the cluster data
-        except Exception as e:
-            log.warning('Error parsing cluster data for %s: %s',
-                        featImage.name, e, exc_info=True)
-            self.__disable(strings.messages[self, 'badData'])
+            if prevFeatImage is None:
+                self.__disable(strings.messages[self, 'notFEAT'])
             return
 
-        # No cluster results exist
-        # for any contrast
-        if len(clusts) == 0:
+        # The newly selected overlay is a
+        # part of a FEAT analysis
+        self.__displayedOverlay = overlay
+
+        # New overlay from same FEAT analysis as
+        # previous overlay - keep displaying it
+        if featImage is prevFeatImage:
+            return
+
+        # Otherwise we have a new analysis to display.
+        # Get the contrast/f-test cluster information
+        # for the new FEAT analysis.
+        results = self.__loadClusterResults(featImage)
+
+        # No cluster results exist for any contrast
+        if results is None or len(results) == 0:
             self.__disable(strings.messages[self, 'noClusters'])
             return
 
         # Populate the stat selection combo box
-        for contrast, clusterList in clusts:
-            name = conNames[contrast]
-            name = strings.labels[self, 'clustName'].format(contrast + 1, name)
+        self.__statSelect.Clear()
+        for result in results:
+            name = strings.labels[self, 'clustName'].format(result[2])
+            self.__statSelect.Append(name)
 
-            self.__statSelect.Append(name, clusterList)
+        # Show the FEAT analysis name
+        self.__overlayName.SetLabel(featImage.name)
 
-        self.__overlayName.SetLabel(display.name)
+        # restore the stat selection if this
+        # analysis has been previously displayed
+        stat = self.__selectedStats.get(featImage.dataSource, 0)
+        self.__statSelect.SetSelection(stat)
 
         # Refresh the widget grid
-        self.__statSelect.SetSelection(0)
         self.__statSelected()
 
         self.Layout()
+
+
+    def __lookupFEATImage(self, overlay):
+        """Searches for a :class:`.FEATImage` for the FEAT analysis that
+        ``overlay`` is associated with. Loads and caches if if necessary.
+        Returns ``None`` if ``overlay`` is not associated with a FEAT analysis.
+        """
+
+        # accept None, makes client code easier
+        if overlay is None:
+            return None
+
+        # image has been removed
+        if overlay not in self.overlayList:
+            return None
+
+        # in-memory overlay - cannot associate
+        # with a FEAT analysis
+        if overlay.dataSource is None:
+            return None
+
+        # already loaded
+        if overlay in self.__featImages:
+            return self.__featImages[overlay]
+
+        # is this overlay from a FEAT analysis?
+        featDir = featanalysis.getAnalysisDir(overlay.dataSource)
+        if featDir is None:
+            return None
+
+        # Load the FEATImage for the FEAT analysis
+        # that the overlay comes from.  If the
+        # overlay itself is a FEATImage, then we
+        # have nothing to do.
+        if isinstance(overlay, featimage.FEATImage):
+            featImage = overlay
+        else:
+
+            featImage  = None
+            candidates = it.chain(self.__featImages.values(), self.overlayList)
+
+            # See if the FEAT image has already
+            # been loaded for another overlay,
+            # or is in the overlay list
+            for candidate in candidates:
+                if isinstance(candidate, featimage.FEATImage) and \
+                   candidate.getFEATDir() == featDir:
+                    featImage = candidate
+
+            # As a last resort, if the FEATImage is not
+            # in the overlay list, we'll load it.
+            if featImage is None:
+                featImage = featimage.FEATImage(featDir)
+
+        self.__featImages[overlay] = featImage
+
+        log.debug('Identified FEAT analysis associated with overlay '
+                  '%s: %s', overlay, featImage.getFEATDir())
+
+        return featImage
+
+
+    def __loadClusterResults(self, featImage):
+        """Loads cluster results for all contrasts and f-tests from the FEAT
+        analysis for ``featImage``.  Returns a list of tuples, with each
+        tuple containing:
+
+        - The test index, starting from 0, relative to the total number of
+          contrasts/f-tests. Contrasts and f-tests are indexed separately.
+        - A boolean indicating whether the test is a contrast or an f-test
+        - The test name
+        - A list of cluster results (see
+          :meth:`.featanalysis.loadClusterResults`). This will be ``None``
+          if the results could not be loaded for some reason.
+        """
+
+        if featImage in self.__clusterData:
+            return self.__clusterData[featImage]
+
+        ncons   = featImage.numContrasts()
+        nftests = featImage.numFTests()
+        results = []
+
+        for i in range(ncons + nftests):
+
+            ftest = i >= ncons
+
+            if ftest:
+                i    = i - ncons
+                name =  f'F-test {i + 1}'
+            else:
+                name = featImage.contrastNames()[i]
+                name = f'COPE{i + 1} ({name})'
+
+            try:
+                clusters = featImage.clusterResults(i, ftest=ftest)
+
+            # Error parsing the cluster data
+            except Exception as e:
+                log.warning('Error parsing cluster data for %s (test %i %s): '
+                            '%s', featImage.name, i, ftest, e, exc_info=True)
+                clusters = None
+
+            results.append((i, ftest, name, clusters))
+
+        self.__clusterData[featImage] = results
+
+        return results
