@@ -8,8 +8,8 @@
 to browse the contents of structured directories which are described with
 a `FileTree <https://git.fmrib.ox.ac.uk/ndcn0236/file-tree/>`_ specification.
 
-See also the :mod:`.manager` module, which contains the logic for
-generating the file list.
+See also the :mod:`~.filetreepanel.manager` module, which contains the logic
+for generating the file list.
 """
 
 
@@ -840,7 +840,7 @@ class FileListPanel(wx.Panel):
 
             thawTime = time.time()
             files    = [f for f in group.files if f is not None]
-            allin    = all([overlayList.find(f) is not None for f in files])
+            allin    = all(overlayList.find(f) is not None for f in files)
 
             return allin or ((thawTime - freezeTime) >= 5)
 
