@@ -32,10 +32,10 @@ This module handles the mechanics of generating lists of variables and files
 according to the user's settings.
 
 
-In this module, and in the :mod:`.filetreepanel` module, variables which take
-on a different value for each row are referred to as *varying*, and variables
-for which all possible values are displaed on one row are referred to as
-*fixed*.
+In this module, and in the :mod:`~.filetreepanel.filetreepanel` module,
+variables which take on a different value for each row are referred to as
+*varying*, and variables for which all possible values are displaed on one row
+are referred to as *fixed*.
 
 
 Example
@@ -112,7 +112,7 @@ FILETREE_PREFIX = '[filetree] '
 """
 
 
-class FileGroup(object):
+class FileGroup:
     """A ``FileGroup`` represents a single row in the file tree panel list.  It
     encapsulates a set of values for all varying variables, and a set of files
     and their associated fixed variable values. These are all accessible as
@@ -192,7 +192,7 @@ class FileGroup(object):
                 self.files    == other.files)
 
 
-class FileTreeManager(object):
+class FileTreeManager:
     """The ``FileTreeManager`` class handles the generation and arranging
     of varying and fixed variables, and file types, according to a
     specification of *varying* and *fixed* variables.
@@ -634,7 +634,7 @@ def filterFileGroups(filegroups, fixedcols):
     return filegroups, fixedcols
 
 
-class OverlayManager(object):
+class OverlayManager:
     """The ``OverlayManager`` is used by the :class:`FileTreeManager`. It
     manages the mechanics of displaying overlays associated with the file tree.
 
@@ -664,7 +664,7 @@ class OverlayManager(object):
         # Loaded overlays are cached to
         # reduce swap time if they are
         # re-shown.
-        self.__cache = cache.Cache(maxsize=50, lru=True)
+        self.__cache = cache.Cache(maxsize=10, lru=True)
 
         # The current list of file
         # groups - set in update
@@ -835,13 +835,20 @@ class OverlayManager(object):
         over to the new ones.
 
         All existing overlays which were previously added are removed.
+
+        :arg new:      Dict of ``{fileid : file}`` mappings, containing the
+                       files to load.
+
+        :arg old:      Dict of ``{fileid : overlay}`` mappings, containing the
+                       existing overlays which will be replaced with the new
+                       ones.
         """
 
         overlayList = self.__overlayList
         displayCtx  = self.__displayCtx
 
         # Drop old overlays that
-        # are being (re-)addd
+        # are being (re-)added
         old = collections.OrderedDict(
             [(fid, ovl) for fid, ovl in old.items()
              if ovl not in new.values()])
@@ -895,22 +902,22 @@ class OverlayManager(object):
             for group in displayCtx.overlayGroups:
                 group.removeOverlay(ovl)
 
+        newOverlayList = list(overlayList)
+        for i, ovl in enumerate(overlayList):
+            if ovl.name in new and ovl in old:
+                newOverlayList[i] = new[ovl.name]
+        newOverlayList = [o for o in newOverlayList if o not in old]
+        newOverlayList.extend([o for o in new.values() if o not in newOverlayList])
+
         # Insert the new overlays into the list.
-        overlayList.extend(new.values(), **propVals)
+        dspace = displayCtx.displaySpace
+        overlayList.replace(newOverlayList, **propVals)
 
         # preserve the display space if
         # it was set to a filegroup image
-        dspace = displayCtx.displaySpace
         if dspace in old and dspace.name in new:
             displayCtx.displaySpace = new[dspace.name]
 
-        # Remove all of the
-        # old filetree overlays
-        idxs           = range(len(overlayList))
-        idxs           = [i for i in idxs if overlayList[i] not in old]
-        overlayList[:] = [overlayList[i] for i in idxs]
-
-
 
 REPLACE = td.TypeDict({
     'MeshOpts'    : ['refImage'],
@@ -934,7 +941,7 @@ group.
 """
 
 
-class ToReplace(object):
+class ToReplace:
     """Placeholder type used by the :func:`getProperties` function when a
     property value is in the :attr:`REPLACE` dictionary, and needs to be
     explicitly handled by the :class:`OverlayManager`.
