@@ -45,9 +45,10 @@ def sampleAlongLine(data, start, end, resolution, order):
     :arg resolution: Number of points to sample
     :arg order:      Interpolation  (see ``scipy.ndimage.map_coordinates``)
     :returns:        Tuple containing:
-                       - 1D Numpy array containing the sampled values
-                       - ``(3, N)`` numpy array containing the coordinates for
-                         each sample
+
+                      - 1D Numpy array containing the sampled values
+                      - ``(3, N)`` numpy array containing the coordinates for
+                        each sample
     """
 
     start  = list(start)
@@ -335,10 +336,7 @@ class SampleLinePanel(ctrlpanel.ControlPanel):
         # Controls which allow the user to select
         # interpolation/resolution, line colour, etc
         widgets = widgetlist.WidgetList(self)
-        legend = props.makeWidget(
-            widgets, canvas, 'legend',
-            labels=strings.properties['PlotCanvas.legend'])
-
+        legend = props.makeWidget(widgets, canvas, 'legend')
         interp = props.makeWidget(
             widgets, self, 'interp', labels=strings.choices[self, 'interp'])
         resolution = props.makeWidget(
