@@ -10,6 +10,8 @@
 
 import logging
 
+import wx
+
 import fsleyes.profiles as profiles
 import fsl.utils.idle   as idle
 
@@ -37,6 +39,32 @@ class LightBoxViewProfile(profiles.Profile):
     """
 
 
+    @staticmethod
+    def supportedView():
+        """Returns the :class:`.LightBoxPanel` class. """
+        import fsleyes.views.lightboxpanel as lightboxpanel
+        return lightboxpanel.LightBoxPanel
+
+
+    @staticmethod
+    def tempModes():
+        """Returns the temporary mode map for the ``LightBoxViewProfile``,
+        which controls the use of modifier keys to temporarily enter other
+        interaction modes.
+        """
+        # Command/CTRL puts the user in zoom mode,
+        # and ALT puts the user in pan mode
+        return {('view', wx.WXK_CONTROL) : 'zoom'}
+
+
+    @staticmethod
+    def altHandlers():
+        """Returns the alternate handlers map, which allows event handlers
+        defined in one mode to be re-used whilst in another mode.
+        """
+        return { ('view', 'LeftMouseDown') :('view', 'LeftMouseDrag')}
+
+
     def __init__(self, viewPanel, overlayList, displayCtx):
         """Create a ``LightBoxViewProfile``.
 
@@ -81,12 +109,10 @@ class LightBoxViewProfile(profiles.Profile):
         elif wheel < 0: wheel =  1
         else:           return False
 
-        opts = self.__canvas.opts
-
         # See comment in OrthoViewProfile._zoomModeMouseWheel
         # about timeout
         def update():
-            opts.topRow += wheel
+            self.viewPanel.scrollpos += wheel
 
         idle.idle(update, timeout=0.1)
 
@@ -120,8 +146,8 @@ class LightBoxViewProfile(profiles.Profile):
         property.
         """
 
-        if   wheel > 0: wheel =  50
-        elif wheel < 0: wheel = -50
+        if   wheel > 0: wheel =  0.05
+        elif wheel < 0: wheel = -0.05
         else:           return False
 
         opts = self.viewPanel.sceneOpts
