@@ -16,6 +16,7 @@ actions = td.TypeDict({
     'LoadOverlayAction'        : 'Ctrl-O',
     'LoadOverlayFromDirAction' : 'Ctrl-D',
     'LoadStandardAction'       : 'Ctrl-S',
+    'NewImageAction'           : 'Ctrl-Shift-N',
     'CopyOverlayAction'        : 'Ctrl-Shift-C',
     'SaveOverlayAction'        : 'Ctrl-Shift-S',
     'ReloadOverlayAction'      : 'Ctrl-Shift-R',
@@ -23,7 +24,6 @@ actions = td.TypeDict({
 
     'FSLeyesFrame.closeFSLeyes'           : 'Ctrl-Q',
     'FSLeyesFrame.openHelp'               : 'Ctrl-?',
-    'FSLeyesFrame.removeFocusedViewPanel' : 'Ctrl-W',
 
     'FSLeyesFrame.layouts.default'        : 'Ctrl-Shift-D',
 
@@ -35,32 +35,38 @@ actions = td.TypeDict({
     'FSLeyesFrame.addScene3DPanel'        : 'Ctrl-6',
     'FSLeyesFrame.addShellPanel'          : 'Ctrl-7',
 
-    'FSLeyesFrame.selectNextOverlay'       : 'Ctrl-Up',
-    'FSLeyesFrame.selectPreviousOverlay'   : 'Ctrl-Down',
-    'FSLeyesFrame.toggleOverlayVisibility' : 'Ctrl-F',
+    'FSLeyesFrame.selectNextOverlay'        : 'Ctrl-Up',
+    'FSLeyesFrame.selectPreviousOverlay'    : 'Ctrl-Down',
+    'FSLeyesFrame.toggleOverlayVisibility'  : 'Ctrl-F',
+    'FSLeyesFrame.toggleOverlaysVisibility' : 'Ctrl-Shift-F',
 
     # Shortcuts for next/prev volume
 
-
     # ViewPanel actions must use one
     # of CTRL, ALT or Shift due to
     # hacky things in FSLeyesFrame
     # (see __onViewPanelMenuItem)
 
+    'ViewPanel.removeFromFrame'             : 'Ctrl-W',
     'ViewPanel.removeAllPanels'             : 'Ctrl-Alt-X',
 
-    'CanvasPanel.toggleOverlayList'         : 'Ctrl-Alt-1',
-    'CanvasPanel.toggleLocationPanel'       : 'Ctrl-Alt-2',
-    'CanvasPanel.toggleOverlayInfo'         : 'Ctrl-Alt-3',
-    'CanvasPanel.toggleDisplayPanel'        : 'Ctrl-Alt-4',
-    'CanvasPanel.toggleCanvasSettingsPanel' : 'Ctrl-Alt-5',
-    'CanvasPanel.toggleAtlasPanel'          : 'Ctrl-Alt-6',
+    'CanvasPanel.OverlayListPanel'          : 'Ctrl-Alt-1',
+    'CanvasPanel.LocationPanel'             : 'Ctrl-Alt-2',
+    'CanvasPanel.OverlayDisplayPanel'       : 'Ctrl-Alt-4',
+    'CanvasPanel.CanvasSettingsPanel'       : 'Ctrl-Alt-5',
 
-    'CanvasPanel.toggleDisplayToolBar'      : 'Ctrl-Alt-7',
+    'CanvasPanel.OverlayDisplayToolBar'     : 'Ctrl-Alt-7',
 
-    'OrthoPanel.toggleOrthoToolBar'         : 'Ctrl-Alt-8',
-    'LightBoxPanel.toggleLightBoxToolBar'   : 'Ctrl-Alt-8',
-    'Scene3DPanel.toggleScene3DToolBar'     : 'Ctrl-Alt-8',
+    'OrthoPanel.OrthoToolBar'               : 'Ctrl-Alt-8',
+    'LightBoxPanel.LightBoxToolBar'         : 'Ctrl-Alt-8',
+    'Scene3DPanel.Scene3DToolBar'           : 'Ctrl-Alt-8',
+
+    'CanvasPanel.OverlayInfoPanel'          : 'Ctrl-Alt-3',
+    'CanvasPanel.AtlasPanel'                : 'Ctrl-Alt-6',
+    'CanvasPanel.FileTreePanel'             : 'Ctrl-Alt-9',
+
+
+    'OrthoPanel.AnnotationPanel'            : 'Ctrl-Alt-A',
 
     'CanvasPanel.toggleMovieMode'           : 'Alt-M',
     'CanvasPanel.toggleDisplaySync'         : 'Alt-S',
@@ -74,26 +80,28 @@ actions = td.TypeDict({
     'OrthoPanel.toggleXCanvas'              : 'Alt-X',
     'OrthoPanel.toggleYCanvas'              : 'Alt-Y',
     'OrthoPanel.toggleZCanvas'              : 'Alt-Z',
-    'OrthoPanel.pearsonCorrelation'         : 'Alt-I',
 
-    'PlotPanel.toggleOverlayList'         : 'Ctrl-Alt-1',
-    'PlotPanel.togglePlotList'            : 'Ctrl-Alt-2',
+    'OrthoPanel.PearsonCorrelateAction'     : 'Alt-I',
+
+    'PlotPanel.OverlayListPanel'          : 'Ctrl-Alt-1',
+    'PlotPanel.PlotListPanel'             : 'Ctrl-Alt-2',
     'PlotPanel.importDataSeries'          : 'Ctrl-I',
     'PlotPanel.exportDataSeries'          : 'Ctrl-E',
 
-    'TimeSeriesPanel.toggleTimeSeriesToolBar'       : 'Ctrl-Alt-3',
-    'TimeSeriesPanel.toggleTimeSeriesControl'       : 'Ctrl-Alt-4',
-    'HistogramPanel.toggleHistogramToolBar'         : 'Ctrl-Alt-3',
-    'HistogramPanel.toggleHistogramControl'         : 'Ctrl-Alt-4',
-    'PowerSpectrumPanel.togglePowerSpectrumToolBar' : 'Ctrl-Alt-3',
-    'PowerSpectrumPanel.togglePowerSpectrumControl' : 'Ctrl-Alt-4',
+    'TimeSeriesPanel.TimeSeriesToolBar'            : 'Ctrl-Alt-3',
+    'TimeSeriesPanel.TimeSeriesControlPanel'       : 'Ctrl-Alt-4',
+    'HistogramPanel.HistogramToolBar'              : 'Ctrl-Alt-3',
+    'HistogramPanel.HistogramControlPanel'         : 'Ctrl-Alt-4',
+    'PowerSpectrumPanel.PowerSpectrumToolBar'      : 'Ctrl-Alt-3',
+    'PowerSpectrumPanel.PowerSpectrumControlPanel' : 'Ctrl-Alt-4',
 
-    'OrthoEditProfile.undo'           : 'Ctrl-Z',
-    'OrthoEditProfile.redo'           : 'Ctrl-Y',
-    'OrthoEditProfile.createMask'     : 'Ctrl-N',
-    'OrthoEditProfile.clearSelection' : 'Ctrl-Shift-A',
-    'OrthoEditProfile.eraseSelection' : 'Ctrl-E',
-    'OrthoEditProfile.fillSelection'  : 'Ctrl-B',
-    'OrthoEditProfile.copySelection'  : 'Ctrl-C',
-    'OrthoEditProfile.pasteSelection' : 'Ctrl-V',
+    'OrthoEditProfile.undo'               : 'Ctrl-Z',
+    'OrthoEditProfile.redo'               : 'Ctrl-Y',
+    'OrthoEditProfile.createMask'         : 'Ctrl-N',
+    'OrthoEditProfile.clearSelection'     : 'Ctrl-Shift-A',
+    'OrthoEditProfile.fillSelection'      : 'Ctrl-B',
+    'OrthoEditProfile.eraseSelection'     : 'Ctrl-E',
+    'OrthoEditProfile.invertSelection'    : 'Ctrl-I',
+    'OrthoEditProfile.copyPasteData'      : 'Ctrl-C',
+    'OrthoEditProfile.copyPasteSelection' : 'Ctrl-P',
 })
