@@ -124,7 +124,8 @@ def parseArgs(argv):
 
     mainParser.add_argument('-of',
                             '--outfile',
-                            help='Output image file name')
+                            help='Output image file name (default: out.png)',
+                            default='out.png')
     mainParser.add_argument('-c',
                             '--crop',
                             type=int,
@@ -148,6 +149,14 @@ def parseArgs(argv):
         ('ortho'), lightbox ('lightbox'), or 3D ('3d') views.
         """)
 
+    exclude = {'Main' : ['skipfslcheck',
+                         'updatecheck',
+                         'fontSize',
+                         'notebook',
+                         'notebookFile',
+                         'notebookPort',
+                         'noBrowser']}
+
     namespace = parseargs.parseArgs(
         mainParser,
         argv,
@@ -158,7 +167,8 @@ def parseArgs(argv):
         argOpts=['-of', '--outfile',
                  '-sz', '--size',
                  '-c',  '--crop'],
-        shortHelpExtra=['--outfile', '--size', '--crop'])
+        shortHelpExtra=['--outfile', '--size', '--crop'],
+        exclude=exclude)
 
     if namespace.outfile is None:
         log.error('outfile is required')
@@ -239,11 +249,16 @@ def makeDisplayContext(namespace):
     # real canvases later on, in the render
     # function below.
     if namespace.scene == 'ortho':
-        sceneOpts = orthoopts.OrthoOpts(MockCanvasPanel(3))
+        ncanvases = 3
+        optsCls   = orthoopts.OrthoOpts
     elif namespace.scene == 'lightbox':
-        sceneOpts = lightboxopts.LightBoxOpts(MockCanvasPanel(1))
+        ncanvases = 1
+        optsCls   = lightboxopts.LightBoxOpts
     elif namespace.scene == '3d':
-        sceneOpts = scene3dopts.Scene3DOpts(MockCanvasPanel(1))
+        ncanvases = 1
+        optsCls   = scene3dopts.Scene3DOpts
+
+    sceneOpts = optsCls(MockCanvasPanel(ncanvases, childDisplayCtx))
 
     # 3D views default to
     # world display space
@@ -446,23 +461,27 @@ def createLightBoxCanvas(namespace,
         displayCtx,
         zax=sceneOpts.zax,
         width=width,
-        height=height)
-
-    if sceneOpts.zrange == (0, 0):
-        sceneOpts.zrange = displayCtx.bounds.getRange(sceneOpts.zax)
+        height=height,
+        freezeOpts=True)
 
     opts                = canvas.opts
     opts.showCursor     = sceneOpts.showCursor
     opts.cursorWidth    = sceneOpts.cursorWidth
     opts.bgColour       = sceneOpts.bgColour
     opts.cursorColour   = sceneOpts.cursorColour
+    opts.renderMode     = sceneOpts.renderMode
     opts.zax            = sceneOpts.zax
     opts.sliceSpacing   = sceneOpts.sliceSpacing
+    opts.sampleSlices   = sceneOpts.sampleSlices
+    opts.reverseSlices  = sceneOpts.reverseSlices
+    opts.sliceOverlap   = sceneOpts.sliceOverlap
+    opts.reverseOverlap = sceneOpts.reverseOverlap
     opts.nrows          = sceneOpts.nrows
     opts.ncols          = sceneOpts.ncols
     opts.zrange         = sceneOpts.zrange
     opts.showGridLines  = sceneOpts.showGridLines
     opts.highlightSlice = sceneOpts.highlightSlice
+    opts.labelSpace     = sceneOpts.labelSpace
 
     return canvas
 
@@ -556,6 +575,7 @@ def createOrthoCanvases(namespace,
         opts.cursorWidth  = sceneOpts.cursorWidth
         opts.cursorGap    = sceneOpts.cursorGap
         opts.bgColour     = sceneOpts.bgColour
+        opts.renderMode   = sceneOpts.renderMode
         opts.invertX      = invertx
         opts.invertY      = inverty
 
@@ -846,8 +866,9 @@ class MockCanvasPanel:
     """Used in place of a :class:`.CanvasPanel`. This is used as a container
     for :class:`MockSliceCanvas` instances.
     """
-    def __init__(self, ncanvases):
-        self.canvases = [MockSliceCanvas() for i in range(ncanvases)]
+    def __init__(self, ncanvases, displayCtx):
+        self.canvases   = [MockSliceCanvas() for i in range(ncanvases)]
+        self.displayCtx = displayCtx
     def getGLCanvases(self):
         return self.canvases
 
@@ -857,15 +878,19 @@ class MockOrthoPanel:
     for three :class:`SliceCanvas` instances.
     """
     def __init__(self, canvases):
-        self.canvases = canvases
+        self.canvases = {}
+        for canvas in canvases:
+            zax = 'XYZ'[canvas.opts.zax]
+            self.canvases[zax] = canvas
+
     def getGLCanvases(self):
-        return self.canvases
+        return list(self.canvases.values())
     def getXCanvas(self):
-        return self.canvases[0]
+        return self.canvases.get('X', None)
     def getYCanvas(self):
-        return self.canvases[1]
+        return self.canvases.get('Y', None)
     def getZCanvas(self):
-        return self.canvases[2]
+        return self.canvases.get('Z', None)
 
 
 if __name__ == '__main__':
