@@ -83,7 +83,8 @@ messages = TypeDict({
     'loadOverlays.unknownType' : 'Unknown data type',
 
     'LocationInfoPanel.displaySpaceWarning' :
-    'Displaying images with different orientations/fields of view!',
+    'Images have different orientations/fields of view - '
+    'alignment/orientation labels may be incorrect!',
 
     'LoadColourMapAction.loadcmap'    : 'Open colour map file',
     'LoadColourMapAction.namecmap'    : 'Enter a name for the colour map.',
@@ -158,7 +159,7 @@ messages = TypeDict({
                                     'in this FEAT analysis',
     'ClusterPanel.badData'        : 'Cluster data could not be parsed - '
                                     'check your cluster_*.txt files.',
-    'ClusterPanel.loadingCluster' : 'Loading clusters for COPE{} ({}) ...',
+    'ClusterPanel.loadingCluster' : 'Loading clusters for {} ...',
 
     'MelodicClassificationPanel.disabled' :
     'Choose a melodic or other 4D image.',
@@ -505,6 +506,8 @@ titles = TypeDict({
     'SampleLinePanel.savefile' : 'Select file to save sampled data to',
     'SampleLinePanel.exportError'  : 'Error saving file',
     'ExportSampledDataDialog'  : 'Export sampled data to file',
+
+    'LightBoxSamplePanel' : 'Choose lightbox slices',
 })
 
 
@@ -545,6 +548,7 @@ actions = TypeDict({
     'SampleLineAction'            : 'Sample along line',
     'AddMaskDataSeriesAction'     : 'Add time series from ROI',
     'AddROIHistogramAction'       : 'Add histogram from ROI',
+    'LightBoxSampleAction'        : 'Choose lightbox slices',
 
     'FSLeyesFrame.addOrthoPanel'           : 'Ortho View',
     'FSLeyesFrame.addLightBoxPanel'        : 'Lightbox View',
@@ -559,6 +563,7 @@ actions = TypeDict({
     'FSLeyesFrame.selectNextOverlay'       : 'Next',
     'FSLeyesFrame.selectPreviousOverlay'   : 'Previous',
     'FSLeyesFrame.toggleOverlayVisibility' : 'Show/hide',
+    'FSLeyesFrame.toggleOverlaysVisibility' : 'Show/hide all but first image',
 
     'ViewPanel.removeAllPanels'             : 'Remove all panels',
     'ViewPanel.removeFromFrame'             : 'Close',
@@ -695,7 +700,7 @@ labels = TypeDict({
     'MagnitudePowerSpectrumSeries' : 'magnitude',
     'PhasePowerSpectrumSeries'     : 'phase',
 
-    'ClusterPanel.clustName'     : 'Z statistics for COPE{} ({})',
+    'ClusterPanel.clustName'     : 'Z statistics for {}',
 
     'ClusterPanel.index'         : 'Cluster index',
     'ClusterPanel.nvoxels'       : 'Size (voxels)',
@@ -733,10 +738,11 @@ labels = TypeDict({
 
     'OverlayDisplayPanel.clipPlane#'     : 'Clip plane #{}',
 
-    'CanvasSettingsPanel.scene'    : 'Scene settings',
-    'CanvasSettingsPanel.ortho'    : 'Ortho view settings',
-    'CanvasSettingsPanel.lightbox' : 'Lightbox settings',
-    'CanvasSettingsPanel.3d'       : '3D view settings',
+    'CanvasSettingsPanel.scene'          : 'Scene settings',
+    'CanvasSettingsPanel.ortho'          : 'Ortho view settings',
+    'CanvasSettingsPanel.lightbox'       : 'Lightbox settings',
+    'CanvasSettingsPanel.3d'             : '3D view settings',
+    'CanvasSettingsPanel.lightboxsample' : 'Choose slices',
 
     'OverlayInfoPanel.general'             : 'General information',
     'OverlayInfoPanel.overlayType'         : 'Displayed as',
@@ -817,6 +823,24 @@ labels = TypeDict({
     'OverlayInfoPanel.Tractogram.numStreamlines' : 'Total number of streamlines',
     'OverlayInfoPanel.Tractogram.numVertices'    : 'Total number of vertices',
     'OverlayInfoPanel.Tractogram.size'           : 'Bounding box size',
+    'OverlayInfoPanel.Tractogram.displaySpace'   :
+    'Display space',
+    'OverlayInfoPanel.Tractogram.refImage'       :
+    'Reference image',
+    'OverlayInfoPanel.Tractogram.coordSpace'             :
+    'Vertices defined in',
+    'OverlayInfoPanel.Tractogram.coordSpace.id'          :
+    'Voxels ({})',
+    'OverlayInfoPanel.Tractogram.coordSpace.pixdim'      :
+    'Scaled voxels ({})',
+    'OverlayInfoPanel.Tractogram.coordSpace.pixdim-flip' :
+    'Scaled voxels [FSL convention] ({})',
+    'OverlayInfoPanel.Tractogram.coordSpace.affine'      :
+    'World coordinates ({})',
+    'OverlayInfoPanel.Tractogram.coordSpace.display'     :
+    'Display coordinate system',
+    'OverlayInfoPanel.Tractogram.coordSpace.torig'       :
+    'Freesurfer coordinates',
 
     'MelodicClassificationPanel.componentTab'   : 'Components',
     'MelodicClassificationPanel.labelTab'       : 'Labels',
@@ -872,8 +896,8 @@ labels = TypeDict({
     'EditTransformPanel.cancel'      : 'Close',
 
     'EditTransformPanel.centre.options' : {
-        'volume' : 'Rotate around centre of image volume',
-        'cursor' : 'Rotate around current cursor location'
+        'volume' : 'Rotate/scale around centre of image volume',
+        'cursor' : 'Rotate/scale around current cursor location'
     },
 
     'DisplaySpaceWarning.changeDS'    : 'Change display space',
@@ -1000,6 +1024,16 @@ labels = TypeDict({
     'Which line do you want to export data for?',
     'ExportSampledDataDialog.coords' :
     'Do you want to save the sample point\ncoordinates?',
+
+    'LightBoxSamplePanel.image'     : 'Image',
+    'LightBoxSamplePanel.zax'       : 'Z axis',
+    'LightBoxSamplePanel.ok'        : 'Ok',
+    'LightBoxSamplePanel.start'     : 'Start slice',
+    'LightBoxSamplePanel.end'       : 'End slice',
+    'LightBoxSamplePanel.spacing'   : 'Slice spacing',
+    'LightBoxSamplePanel.overview'  :
+    'Configure lightbox slices in terms\n'
+    'of image voxel dimensions',
 })
 
 
@@ -1025,8 +1059,8 @@ properties = TypeDict({
     'SceneOpts.fgColour'           : 'Foreground colour',
     'SceneOpts.cursorColour'       : 'Location cursor colour',
     'SceneOpts.showColourBar'      : 'Show colour bar',
+    'SceneOpts.performance'        : 'Rendering performance',
     'SceneOpts.zoom'               : 'Zoom',
-    'SceneOpts.highDpi'            : 'Enable high-DPI rendering',
     'SceneOpts.colourBarLocation'  : 'Colour bar location',
     'SceneOpts.colourBarLabelSide' : 'Colour bar label side',
     'SceneOpts.colourBarSize'      : 'Colour bar size',
@@ -1036,7 +1070,12 @@ properties = TypeDict({
     'LightBoxOpts.highlightSlice' : 'Highlight slice',
     'LightBoxOpts.showGridLines'  : 'Show grid lines',
     'LightBoxOpts.sliceSpacing'   : 'Slice spacing',
+    'LightBoxOpts.reverseSlices'  : 'Reverse slice order',
+    'LightBoxOpts.reverseOverlap' : 'Reverse slice overlap',
+    'LightBoxOpts.sliceOverlap'   : 'Slice overlap',
     'LightBoxOpts.zrange'         : 'Z range',
+    'LightBoxOpts.labelSpace'     : 'Show slice location',
+    'LightBoxOpts.sampleSlices'   : 'Sample slices at',
 
     'OrthoOpts.showXCanvas'  : 'Show X canvas',
     'OrthoOpts.showYCanvas'  : 'Show Y canvas',
@@ -1214,22 +1253,24 @@ properties = TypeDict({
     'VectorOpts.clipImage'       : 'Clip by',
     'VectorOpts.clippingRange'   : 'Clipping range',
     'VectorOpts.modulateRange'   : 'Modulation range',
+    'VectorOpts.colourRange'     : 'Colour by range',
     'VectorOpts.modulateMode'    : 'Modulation mode',
     'VectorOpts.orientFlip'      : 'L/R orientation flip',
+    'VectorOpts.normaliseColour' : 'Normalise RGB brightness',
 
     'RGBVectorOpts.interpolation' : 'Interpolation',
-    'RGBVectorOpts.unitLength'    : 'Scale vectors to unit length',
 
     'LineVectorOpts.directed'    : 'Interpret vectors as directed',
     'LineVectorOpts.lineWidth'   : 'Line width',
     'LineVectorOpts.unitLength'  : 'Scale vectors to unit length',
     'LineVectorOpts.lengthScale' : 'Length scaling factor (%)',
 
+    'RefImageOpts.refImage'      : 'Reference image',
+    'RefImageOpts.coordSpace'    : 'Coordinate space',
+
     'MeshOpts.colour'            : 'Colour',
     'MeshOpts.outline'           : 'Show outline only',
     'MeshOpts.outlineWidth'      : 'Outline width',
-    'MeshOpts.refImage'          : 'Reference image',
-    'MeshOpts.coordSpace'        : 'Mesh coordinate space',
     'MeshOpts.custom_vertexSet'  : 'Surface definition',
     'MeshOpts.custom_vertexData' : 'Vertex data',
     'MeshOpts.vertexSet'         : 'Surface definition',
@@ -1283,8 +1324,14 @@ properties = TypeDict({
     'TractogramOpts.custom_colourMode' : 'Colour by',
     'TractogramOpts.colourMode'        : 'Colour by',
     'TractogramOpts.lineWidth'         : 'Width',
+    'TractogramOpts.sliceWidth'        : 'Slice width',
     'TractogramOpts.resolution'        : 'Resolution',
     'TractogramOpts.subsample'         : 'Sub-sample',
+    'TractogramOpts.pseudo3D'          : 'Pseudo-3D',
+    'TractogramOpts.custom_zclipping'  : 'Depth clipping',
+    'TractogramOpts.xclipdir'          : 'X clip direction',
+    'TractogramOpts.yclipdir'          : 'Y clip direction',
+    'TractogramOpts.zclipdir'          : 'Z clip direction',
 })
 
 
@@ -1292,6 +1339,7 @@ choices = TypeDict({
 
     'DisplayContext.displaySpace' : {'world'       : 'World coordinates',
                                      'scaledVoxel' : 'Scaled voxel coordinates',
+                                     'fslview'     : 'FSLView mode'
                                      },
 
     'SceneOpts.colourBarLocation'  : {'top'          : 'Top',
@@ -1301,6 +1349,10 @@ choices = TypeDict({
     'SceneOpts.colourBarLabelSide' : {'top-left'     : 'Top / Left',
                                       'bottom-right' : 'Bottom / Right'},
 
+    'SceneOpts.performance' : {1 : 'Fastest',
+                               2 : 'Faster',
+                               3 : 'Best looking'},
+
     'CanvasOpts.zax' : {0 : 'X axis',
                         1 : 'Y axis',
                         2 : 'Z axis'},
@@ -1332,6 +1384,13 @@ choices = TypeDict({
     'LightBoxOpts.zrange.min' : 'Min.',
     'LightBoxOpts.zrange.max' : 'Max.',
 
+    'LightBoxOpts.labelSpace' : {'none'  : 'No labels',
+                                 'voxel' : 'Voxel coordinates',
+                                 'world' : 'World coordinates'},
+
+    'LightBoxOpts.sampleSlices' : {'centre' : 'Slice centres',
+                                   'start'  : 'Slice starts'},
+
     'ColourMapOpts.displayRange.min' : 'Min.',
     'ColourMapOpts.displayRange.max' : 'Max.',
 
@@ -1344,26 +1403,31 @@ choices = TypeDict({
     'VectorOpts.modulateImage.none' : 'No modulation',
     'VectorOpts.clipImage.none'     : 'No clipping',
 
-    'VectorOpts.clippingRange.min' : 'Clip min.',
-    'VectorOpts.clippingRange.max' : 'Clip max.',
+    'VectorOpts.clippingRange.min' : 'Min.',
+    'VectorOpts.clippingRange.max' : 'Max.',
+    'VectorOpts.modulateRange.min' : 'Min.',
+    'VectorOpts.modulateRange.max' : 'Max.',
+    'VectorOpts.colourRange.min'   : 'Min.',
+    'VectorOpts.colourRange.max'   : 'Max.',
 
-    'VectorOpts.modulateRange.min' : 'Mod min.',
-    'VectorOpts.modulateRange.max' : 'Mod max.',
+    'VectorOpts.suppressMode' :     {'white'       : 'White',
+                                     'black'       : 'Black',
+                                     'transparent' : 'Transparent'},
+    'VectorOpts.modulateMode'     : {'brightness'  : 'Brightness',
+                                     'alpha'       : 'Transparency'},
+    'LineVectorOpts.modulateMode' : {'brightness'  : 'Brightness',
+                                     'alpha'       : 'Transparency',
+                                     'lineLength'  : 'Line length',
+                                     'lineWidth'   : 'Line width'},
 
-    'VectorOpts.suppressMode' : {'white'       : 'White',
-                                 'black'       : 'Black',
-                                 'transparent' : 'Transparent'},
-    'VectorOpts.modulateMode' : {'brightness'  : 'Brightness',
-                                 'alpha'       : 'Transparency'},
+    'RefImageOpts.refImage.none'  : 'No reference image',
 
-    'MeshOpts.refImage.none'  : 'No reference image',
-
-    'MeshOpts.coordSpace' : {'torig'       : 'Freesurfer coordinates',
-                             'affine'      : 'World coordinates',
-                             'pixdim'      : 'Scaled voxels',
-                             'pixdim-flip' : 'Scaled voxels forced to '
-                                             'radiological orientation',
-                             'id'          : 'Voxels'},
+    'RefImageOpts.coordSpace' : {'torig'       : 'Freesurfer coordinates',
+                                 'affine'      : 'World coordinates',
+                                 'pixdim'      : 'Scaled voxels',
+                                 'pixdim-flip' : 'Scaled voxels forced to '
+                                                 'radiological orientation',
+                                 'id'          : 'Voxels'},
 
     'TensorOpts.tensorResolution.min' : 'Low',
     'TensorOpts.tensorResolution.max' : 'High',
@@ -1374,10 +1438,11 @@ choices = TypeDict({
                              'id'          : 'Voxels',
                              'reference'   : 'Reference image'},
 
-    'VolumeOpts.interpolation' : {'none'    : 'No interpolation',
-                                  'nearest' : 'Nearest neighbour interpolation',
-                                  'linear'  : 'Linear interpolation',
-                                  'spline'  : 'Spline interpolation'},
+    'VolumeOpts.interpolation' : {'none'        : 'No interpolation',
+                                  'nearest'     : 'Nearest neighbour interpolation',
+                                  'linear'      : 'Linear interpolation',
+                                  'spline'      : 'Spline interpolation',
+                                  'true_spline' : 'Prefiltered spline interpolation'},
 
     'Volume3DOpts.clipMode' : {'intersection' : 'Intersection',
                                'union'        : 'Union',
@@ -1392,6 +1457,10 @@ choices = TypeDict({
                                 'phase' : 'Phase'},
 
     'TractogramOpts.colourMode' : {'orientation' : 'Orientation'},
+    'TractogramOpts.clipdir'    : {'slice' : 'Show slice',
+                                   'low'   : 'Show above',
+                                   'high'  : 'Show below',
+                                   'none'  : 'Show all'},
 
     'Display.overlayType' : {
         'volume'         : '3D/4D volume',
@@ -1455,47 +1524,53 @@ choices = TypeDict({
         'voxel' : 'Save voxel coordinates',
         'world' : 'Save world coordinates',
     },
+
+    'LightBoxSamplePanel.zax' : {0 : 'X axis',
+                                 1 : 'Y axis',
+                                 2 : 'Z axis'},
 })
 
 
 anatomy = TypeDict({
 
-    ('Nifti', 'lowlong',   constants.ORIENT_A2P)               : 'Anterior',
-    ('Nifti', 'lowlong',   constants.ORIENT_P2A)               : 'Posterior',
-    ('Nifti', 'lowlong',   constants.ORIENT_L2R)               : 'Left',
-    ('Nifti', 'lowlong',   constants.ORIENT_R2L)               : 'Right',
-    ('Nifti', 'lowlong',   constants.ORIENT_I2S)               : 'Inferior',
-    ('Nifti', 'lowlong',   constants.ORIENT_S2I)               : 'Superior',
-    ('Nifti', 'lowlong',   constants.ORIENT_UNKNOWN)           : 'Unknown',
-    ('Nifti', 'highlong',  constants.ORIENT_A2P)               : 'Posterior',
-    ('Nifti', 'highlong',  constants.ORIENT_P2A)               : 'Anterior',
-    ('Nifti', 'highlong',  constants.ORIENT_L2R)               : 'Right',
-    ('Nifti', 'highlong',  constants.ORIENT_R2L)               : 'Left',
-    ('Nifti', 'highlong',  constants.ORIENT_I2S)               : 'Superior',
-    ('Nifti', 'highlong',  constants.ORIENT_S2I)               : 'Inferior',
-    ('Nifti', 'highlong',  constants.ORIENT_UNKNOWN)           : 'Unknown',
-    ('Nifti', 'lowshort',  constants.ORIENT_A2P)               : 'A',
-    ('Nifti', 'lowshort',  constants.ORIENT_P2A)               : 'P',
-    ('Nifti', 'lowshort',  constants.ORIENT_L2R)               : 'L',
-    ('Nifti', 'lowshort',  constants.ORIENT_R2L)               : 'R',
-    ('Nifti', 'lowshort',  constants.ORIENT_I2S)               : 'I',
-    ('Nifti', 'lowshort',  constants.ORIENT_S2I)               : 'S',
-    ('Nifti', 'lowshort',  constants.ORIENT_UNKNOWN)           : '?',
-    ('Nifti', 'highshort', constants.ORIENT_A2P)               : 'P',
-    ('Nifti', 'highshort', constants.ORIENT_P2A)               : 'A',
-    ('Nifti', 'highshort', constants.ORIENT_L2R)               : 'R',
-    ('Nifti', 'highshort', constants.ORIENT_R2L)               : 'L',
-    ('Nifti', 'highshort', constants.ORIENT_I2S)               : 'S',
-    ('Nifti', 'highshort', constants.ORIENT_S2I)               : 'I',
-    ('Nifti', 'highshort', constants.ORIENT_UNKNOWN)           : '?',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_UNKNOWN)      : 'Unknown',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_SCANNER_ANAT) : 'Scanner '
-                                                                 'anatomical',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_ALIGNED_ANAT) : 'Aligned '
-                                                                 'anatomical',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_TALAIRACH)    : 'Talairach',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_MNI_152)      : 'MNI152',
-    ('Nifti', 'space',     constants.NIFTI_XFORM_ANALYZE)      : 'ANALYZE',
+    ('Nifti', 'lowlong',   constants.ORIENT_A2P)     : 'Anterior',
+    ('Nifti', 'lowlong',   constants.ORIENT_P2A)     : 'Posterior',
+    ('Nifti', 'lowlong',   constants.ORIENT_L2R)     : 'Left',
+    ('Nifti', 'lowlong',   constants.ORIENT_R2L)     : 'Right',
+    ('Nifti', 'lowlong',   constants.ORIENT_I2S)     : 'Inferior',
+    ('Nifti', 'lowlong',   constants.ORIENT_S2I)     : 'Superior',
+    ('Nifti', 'lowlong',   constants.ORIENT_UNKNOWN) : 'Unknown',
+    ('Nifti', 'highlong',  constants.ORIENT_A2P)     : 'Posterior',
+    ('Nifti', 'highlong',  constants.ORIENT_P2A)     : 'Anterior',
+    ('Nifti', 'highlong',  constants.ORIENT_L2R)     : 'Right',
+    ('Nifti', 'highlong',  constants.ORIENT_R2L)     : 'Left',
+    ('Nifti', 'highlong',  constants.ORIENT_I2S)     : 'Superior',
+    ('Nifti', 'highlong',  constants.ORIENT_S2I)     : 'Inferior',
+    ('Nifti', 'highlong',  constants.ORIENT_UNKNOWN) : 'Unknown',
+    ('Nifti', 'lowshort',  constants.ORIENT_A2P)     : 'A',
+    ('Nifti', 'lowshort',  constants.ORIENT_P2A)     : 'P',
+    ('Nifti', 'lowshort',  constants.ORIENT_L2R)     : 'L',
+    ('Nifti', 'lowshort',  constants.ORIENT_R2L)     : 'R',
+    ('Nifti', 'lowshort',  constants.ORIENT_I2S)     : 'I',
+    ('Nifti', 'lowshort',  constants.ORIENT_S2I)     : 'S',
+    ('Nifti', 'lowshort',  constants.ORIENT_UNKNOWN) : '?',
+    ('Nifti', 'highshort', constants.ORIENT_A2P)     : 'P',
+    ('Nifti', 'highshort', constants.ORIENT_P2A)     : 'A',
+    ('Nifti', 'highshort', constants.ORIENT_L2R)     : 'R',
+    ('Nifti', 'highshort', constants.ORIENT_R2L)     : 'L',
+    ('Nifti', 'highshort', constants.ORIENT_I2S)     : 'S',
+    ('Nifti', 'highshort', constants.ORIENT_S2I)     : 'I',
+    ('Nifti', 'highshort', constants.ORIENT_UNKNOWN) : '?',
+
+    ('Nifti', 'space', constants.NIFTI_XFORM_UNKNOWN)        : 'Unknown',
+    ('Nifti', 'space', constants.NIFTI_XFORM_SCANNER_ANAT)   : 'Scanner '
+                                                               'anatomical',
+    ('Nifti', 'space', constants.NIFTI_XFORM_ALIGNED_ANAT)   : 'Aligned '
+                                                               'anatomical',
+    ('Nifti', 'space', constants.NIFTI_XFORM_TALAIRACH)      : 'Talairach',
+    ('Nifti', 'space', constants.NIFTI_XFORM_MNI_152)        : 'MNI152',
+    ('Nifti', 'space', constants.NIFTI_XFORM_TEMPLATE_OTHER) : 'Other template',
+    ('Nifti', 'space', constants.NIFTI_XFORM_ANALYZE)        : 'ANALYZE',
 })
 
 
@@ -1662,12 +1737,14 @@ melodic = TypeDict({
 })
 
 layouts = {
-    'default'  : 'Default layout',
-    'melodic'  : 'MELODIC mode',
-    'feat'     : 'FEAT mode',
-    'ortho'    : 'Plain orthographic',
-    'lightbox' : 'Plain lightbox',
-    '3d'       : 'Plain 3D',
+    'default'   : 'Default layout',
+    'default3d' : 'Default 3D',
+    'defaultlb' : 'Default lightbox',
+    'melodic'   : 'MELODIC mode',
+    'feat'      : 'FEAT mode',
+    'ortho'     : 'Plain orthographic',
+    'lightbox'  : 'Plain lightbox',
+    '3d'        : 'Plain 3D'
 }
 
 
