@@ -130,7 +130,8 @@ properties = TypeDict({
     'VolumeOpts.interpolation' :
     'Interpolate the image data on the display. You can choose no  '
     'interpolation (equivalent to nearest neighbour interpolation), linear '
-    'interpolation, or third-order spline (cubic) interpolation.',
+    'interpolation, third-order spline (cubic) interpolation, or prefiltered '
+    'spline interpolation.',
 
     'VolumeOpts.enableOverrideDataRange' :
     'Override the actual data range of an image with a user-specified '
@@ -289,6 +290,9 @@ properties = TypeDict({
                                     'selected colour map. The colour image '
                                     'must have the same voxel dimensions as '
                                     'the vector image. ',
+    'VectorOpts.colourRange'      : 'The data range that is used when '
+                                    'colouring vectors by a separate '
+                                    'colour-by image.',
     'VectorOpts.clippingRange'    : 'Vector values which have a corresponding '
                                     'clipping image value that is outside of '
                                     'this range are not displayed. ',
@@ -303,6 +307,11 @@ properties = TypeDict({
     'VectorOpts.orientFlip'       : 'If checked, direction orientations '
                                     'within each voxel are flipped about '
                                     'the x axis.',
+    'VectorOpts.normaliseColour'  : 'If checked, the data is scaled so that '
+                                    'it is displayed with uniform brightness.'
+                                    'If disabled, the vector XYZ values are '
+                                    'used directly as RGB values.',
+
     'LineVectorOpts.lineWidth'    : 'The width of each vector line, in '
                                     'display pixels.',
     'LineVectorOpts.directed'     : 'If unchecked, the vector data is assumed '
@@ -322,12 +331,13 @@ properties = TypeDict({
     'LineVectorOpts.lengthScale'  : 'Scale the vector line length by this '
                                     'scaling factor (expressed as a '
                                     'percentage).',
+    'LineVectorOpts.modulateMode' : 'Either the brightness, transparency, or '
+                                    'line length of vector voxels can be '
+                                    'modulated by the modulation image.',
     'RGBVectorOpts.interpolation' : 'Interpolate the vector data for display '
                                     'purposes. You can choose none '
                                     '(equivalent to nearest-neighbour), '
                                     'linear, or spline interpolation.',
-    'RGBVectorOpts.unitLength'    : 'If checked, the data is scaled so that '
-                                    'each vector has length 1.',
 
 
     'MeshOpts.colour' :
@@ -439,12 +449,24 @@ properties = TypeDict({
     'TractogramOpts.lineWidth' :
     'Controls the thickness of streamline lines/tubes (when viewing in 3D), '
     'or the diameter of streamline vertex points (when viewing in 2D).',
+    'TractogramOpts.sliceWidth' :
+    'Controls the thickness of the slice through the tractogram when drawing '
+    '2D slices/cross-sections.',
     'TractogramOpts.resolution' :
     'Controls the quality when drawing streamline tubes (in 3D) or vertex '
     'points (in 2D).',
     'TractogramOpts.subsample' :
     'Draw a randomly selected subset of streamlines. This setting is useful '
     'when viewing very large tractograms.',
+    'TractogramOpts.pseudo3D' :
+    'Instead of drawing a 2D cross-section of the tractogram vertices, draw '
+    'a portion of the full 3D tractogram, overlaid on the 2D scene.',
+    'TractogramOpts.xclipdir' :
+    'Direction of depth clipping along the X axis.',
+    'TractogramOpts.yclipdir' :
+    'Direction of depth clipping along the Y axis.',
+    'TractogramOpts.zclipdir' :
+    'Direction of depth clipping along the Z axis.',
 
     # MIPOpts
     'MIPOpts.window'  :
@@ -484,10 +506,12 @@ properties = TypeDict({
     'SceneOpts.colourBarSize'      : 'Size of the major axis of the colour '
                                      'bar, as a proportion of the available '
                                      'space.',
-    'SceneOpts.highDpi' :
-    'If you are using a high-DPI (e.g. retina) display, FSLeyes will attempt '
-    'to display the scene at the high-DPI resolution. This will have an '
-    'effect on performance.',
+    'SceneOpts.performance'        : 'Rendering performance - 1 gives the '
+                                     'fastest, but at the cost of lower '
+                                     'display quality, and some display '
+                                     'limitations. 2 gives the best '
+                                     'display quality, but may be too slow on '
+                                     'some older systems.',
 
     'SceneOpts.labelSize'   : 'Scale the label font size.',
 
@@ -513,22 +537,34 @@ properties = TypeDict({
     'Scene3DToolBar.showCursorAndLegend' :
     'Show/hide the location cursor and anatomical legend',
 
-    'LightBoxOpts.zoom'           : 'Zoom level - this controls how many '
-                                    'slices to display.',
-    'LightBoxOpts.sliceSpacing'   : 'The spacing between adjacent slices. '
-                                    'The units and range of this setting '
-                                    'depend upon the currently selected '
-                                    'overlay.',
-    'LightBoxOpts.zrange'         : 'The start/end points of the displayed '
-                                    'range of slices. The units and range '
-                                    'of this setting depend upon the '
-                                    'currently selected overlay.',
-    'LightBoxOpts.zax'            : 'Slices along this axis will be '
-                                    'displayed.',
-    'LightBoxOpts.showGridLines'  : 'If checked, lines will be shown between '
-                                    'each slice.',
-    'LightBoxOpts.highlightSlice' : 'If checked, a box will be drawn around '
-                                    'the currently selected slice.',
+    'LightBoxOpts.zoom' :
+    'Zoom level - this controls how many slices to display.',
+    'LightBoxOpts.sliceSpacing' :
+    'The spacing between adjacent slices, specified as a percentage.',
+    'LightBoxOpts.sliceOverlap' :
+    'The overlap between adjacent slices, specified as a percentage.',
+    'LightBoxOpts.reverseSlices' :
+    'Draw slices from high Z value to low Z value, instead of low to high.',
+    'LightBoxOpts.reverseOverlap' :
+    'Overlap slices with lower Z value on top of slices with higher Z value, '
+    'instead of the default of overlapping higher on top of lower.',
+    'LightBoxOpts.zrange' :
+    'The start/end points of the displayed range of slices, specified as '
+    'percentages.',
+    'LightBoxOpts.zax' :
+    'Slices along this axis will be displayed.',
+    'LightBoxOpts.showGridLines' :
+    'If checked, lines will be shown between each slice.',
+    'LightBoxOpts.highlightSlice' :
+    'If checked, a box will be drawn around the currently selected slice.',
+    'LightBoxOpts.labelSpace' :
+    'Whether or not to display the depth axis location on each slice. If a '
+    'non-NIfTI overlay is selected, the displayed location will always be '
+    'in world coordinates.',
+    'LightBoxOpts.sampleSlices' :
+    'Whether to sample image slices from the centre or the start of each '
+    'slice region. This is useful when you have a custom slice spacing, and '
+    'want to select a specific starting slice.',
 
     'Scene3DOpts.showLegend' :
     'When selected, a legend will be displayed in the bottom right, showing '
