@@ -105,8 +105,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
                                :class:`.OrthoAnnotatePanel`.
     ========================== ==============================================
 
-    See the :class:`.ViewPanel`, and the :mod:`.profiles` package for more
-    information on interaction profiles.
+    See the :class:`.ViewPanel`, and the :mod:`fsleyes.profiles` package for
+    more information on interaction profiles.
 
 
     **Actions and control panels**
@@ -274,9 +274,9 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         yopts.bindProps('zoom',         sceneOpts, 'yzoom')
         zopts.bindProps('zoom',         sceneOpts, 'zzoom')
 
-        xopts.bindProps('highDpi',      sceneOpts)
-        yopts.bindProps('highDpi',      sceneOpts)
-        zopts.bindProps('highDpi',      sceneOpts)
+        xopts.bindProps('renderMode',   sceneOpts)
+        yopts.bindProps('renderMode',   sceneOpts)
+        zopts.bindProps('renderMode',   sceneOpts)
 
         xopts.bindProps('invertX', sceneOpts, 'invertXHorizontal')
         xopts.bindProps('invertY', sceneOpts, 'invertXVertical')
@@ -682,6 +682,8 @@ class OrthoPanel(canvaspanel.CanvasPanel):
         haveOverlays = len(self.overlayList) > 0
         selOverlay   = self.displayCtx.getSelectedOverlay()
 
+        self.__radioOrientationChanged()
+
         if selOverlay is not None:
 
             display    = self.displayCtx.getDisplay(selOverlay)
