@@ -18,11 +18,9 @@ import wx.lib.agw.aui                as aui
 import wx.lib.agw.aui.framemanager   as auifm
 
 import fsl.utils.notifier            as notifier
-import fsleyes_widgets               as fwidgets
 
 import fsleyes.panel                 as fslpanel
 import fsleyes.toolbar               as fsltoolbar
-import fsleyes.plugins               as plugins
 import fsleyes.controls.controlpanel as ctrlpanel
 import fsleyes.profiles              as profiles
 import fsleyes.strings               as strings
@@ -46,8 +44,8 @@ class ViewPanel(fslpanel.FSLeyesPanel):
     A ``ViewPanel`` class uses a ``wx.lib.agw.aui.AuiManager`` to lay out its
     children. A ``ViewPanel`` has one central panel, which contains the
     primary view; and may have one or more secondary panels, which contain
-    *controls* - see the :mod:`.controls` package. The centre panel can be set
-    via the :meth:`centrePanel` property, and secondary panels can be
+    *controls* - see the :mod:`fsleyes.controls` package. The centre panel can
+    be set via the :meth:`centrePanel` property, and secondary panels can be
     added/removed to/from with the :meth:`togglePanel` method. The current
     state of a secondary panel (i.e. whether one is open or not) can be
     queried with the :meth:`isPanelOpen` method, and existing secondary panels
@@ -69,7 +67,7 @@ class ViewPanel(fslpanel.FSLeyesPanel):
     when the control panel is toggled via the :meth:`togglePanel` method.
 
     The currently active interaction profile can be accessed with the
-    :meth:`currentProfile` method. See the :mod:`.profiles` package for
+    :meth:`currentProfile` method. See the :mod:`fsleyes.profiles` package for
     more information on interaction profiles.
 
 
@@ -140,6 +138,31 @@ class ViewPanel(fslpanel.FSLeyesPanel):
         return None
 
 
+    @staticmethod
+    def defaultLocation():
+        """May be overridden by sub-classes. Should return a sequence
+        containing:
+
+         - the preferred location of the view panel in the
+           :class:`.FSLeyesFrame` - one of ``wx.TOP``, ``wx.BOTTOM``,
+           ``wx.LEFT``, or ``wx.RIGHT``.
+         - A number between 0 and 1 indicating the preferred size of the
+           view panel, as a proportion of the total frame size.
+        """
+        return None
+
+
+    @staticmethod
+    def displayType():
+        """May be overridden by sub-classes. May return a string which
+        describes the type of display/view that this ``ViewPanel`` provides.
+        This might be used as a hint by :class:`.Display` /
+        :class:`.DisplayOpts` instances to tailor their overlay display
+        settings to different view/display types.
+        """
+        return None
+
+
     def __init__(self, parent, overlayList, displayCtx, frame):
         """Create a ``ViewPanel``.
 
@@ -254,6 +277,8 @@ class ViewPanel(fslpanel.FSLeyesPanel):
         (e.g. due to naming conflicts).
         """
 
+        import fsleyes.plugins as plugins
+
         # controls
         for ctrlTitle, ctrlType in plugins.listControls(type(self)).items():
             name = ctrlType.__name__
@@ -274,6 +299,12 @@ class ViewPanel(fslpanel.FSLeyesPanel):
                 act = toolType(self.overlayList, self.displayCtx, self)
                 setattr(self, name, act)
 
+                # This ViewPanel effectively owns this
+                # action.  Set ourselves as the action
+                # instance to ensure that the FSLeyesFrame
+                # doesn't delete it on refreshes.
+                act.instance = self
+
 
     def destroy(self):
         """Removes some property listeners, destroys all child panels,
@@ -615,8 +646,8 @@ class ViewPanel(fslpanel.FSLeyesPanel):
                           selfPos[1] + selfSize[1] * floatPos[1])
 
             paneSize = panel.GetBestSize().Get()
-            panePos  = (selfCentre[0] - paneSize[0] * 0.5,
-                        selfCentre[1] - paneSize[1] * 0.5)
+            panePos  = (int(round(selfCentre[0] - paneSize[0] * 0.5)),
+                        int(round(selfCentre[1] - paneSize[1] * 0.5)))
 
             paneInfo.Float()                 \
                     .Resizable(True)         \
@@ -703,6 +734,9 @@ class ViewPanel(fslpanel.FSLeyesPanel):
                        is added, it should be passed here.
         """
 
+        if self.destroyed:
+            return
+
         newPanel = kwargs.pop('newPanel', None)
 
         # This method makes sure that size hints
@@ -888,96 +922,3 @@ class ViewPanel(fslpanel.FSLeyesPanel):
         know about layout changes.
         """
         self.events.notify(topic='aui_perspective')
-
-
-class MyAuiFloatingFrame(auifm.AuiFloatingFrame):
-    """Here I am monkey patching the
-    ``wx.agw.aui.framemanager.AuiFloatingFrame.__init__`` method.
-
-    I am doing this because I have observed some strange behaviour when running
-    a remote instance of this application over an SSH/X11 session, with the X11
-    server (i.e. the local machine) running in OS X. When a combobox is embedded
-    in a floating frame (either a pane or a toolbar), its dropdown list appears
-    underneath the frame, meaning that the user is unable to actually select any
-    items from the list!
-
-    I have only seen this behaviour when using XQuartz on macOS.
-
-    Ultimately, this appears to be caused by the ``wx.FRAME_TOOL_WINDOW``
-    style, as passed to the ``wx.MiniFrame`` constructor (from which the
-    ``AuiFloatingFrame`` class derives). Removing this style flag fixes the
-    problem, so this is exactly what I'm doing. I haven't looked any deeper
-    into the situation.
-
-
-    This class also overrieds the ``SetPaneWindow`` method, because under gtk3,
-    the maximum size if a frame musr be set.
-    """
-
-    def __init__(self, *args, **kwargs):
-        """My new constructor, which makes sure that the ``FRAME_TOOL_WINDOW``
-        style is not passed through to the ``AuiFloatingFrame`` constructor
-        """
-
-        if 'style' in kwargs:
-            style = kwargs['style']
-
-        # This is the default style, as defined
-        # in the AuiFloatingFrame constructor
-        else:
-            style = (wx.FRAME_TOOL_WINDOW     |
-                     wx.FRAME_FLOAT_ON_PARENT |
-                     wx.FRAME_NO_TASKBAR      |
-                     wx.CLIP_CHILDREN)
-
-        if fwidgets.inSSHSession():
-            style &= ~wx.FRAME_TOOL_WINDOW
-
-        kwargs['style'] = style
-
-        super().__init__(*args, **kwargs)
-
-
-    def SetPaneWindow(self, pane):
-        """Make sure that floated toolbars are sized correctly.
-        """
-        super().SetPaneWindow(pane)
-        if isinstance(pane.window, ctrlpanel.ControlToolBar):
-            size = self.GetBestSize()
-            self.SetMaxSize(size)
-
-
-def _AuiDockingGuide_init(self, *args, **kwargs):
-    """I am also monkey-patching the
-    ``wx.lib.agw.aui.AuiDockingGuide.__init__`` method, because in this
-    instance, when running over SSH/X11, the ``wx.FRAME_TOOL_WINDOW`` style
-    seems to result in the docking guide frames being given title bars, which
-    is quite undesirable.
-
-    I cannot patch the entire class in the aui package, because it is used
-    as part of a class hierarchy. So I am just patching the method.
-    """
-
-    if 'style' in kwargs:
-        style = kwargs['style']
-
-    # This is the default style, as defined
-    # in the AuiDockingGuide constructor
-    else:
-        style = (wx.FRAME_TOOL_WINDOW |
-                 wx.FRAME_STAY_ON_TOP |
-                 wx.FRAME_NO_TASKBAR  |
-                 wx.NO_BORDER)
-
-    if fwidgets.inSSHSession():
-        style &= ~wx.FRAME_TOOL_WINDOW
-
-    kwargs['style'] = style
-
-    _AuiDockingGuide_real_init(self, *args, **kwargs)
-
-
-aui  .AuiFloatingFrame       = MyAuiFloatingFrame
-auifm.AuiFloatingFrame       = MyAuiFloatingFrame
-_AuiDockingGuide_real_init   = aui.AuiDockingGuide.__init__
-aui.AuiDockingGuide.__init__ = _AuiDockingGuide_init
